class
	P_FILENAME

inherit
	LESS_DATA
		redefine
			default_create
		end
	S_UNISTD
	S_STDLIB
	S_STRING
	S_TERMIOS
	S_SYS_IOCTL
	S_TERMCAP
	S_STDIO
	S_CTYPE
	S_LOCALE
	S_FCNTL
	S_LIBIO
	S_SYS_STAT
	S_SIGNAL
	S_TIME
	S_BITS_ERRNO
	S_REGEX

feature {NONE} -- Initialization

	default_create
		do
			create is_a_dir.make (<<16>>)
			create mchars.make
			create not_reg.make (<<42>>)
			create use_lessopen.make (<<1>>)
		end

feature {ANY} -- attributes

	is_a_dir: CE_ARRAY [INTEGER_8]

	mchars: CE_POINTER [INTEGER_8] assign set_mchars

	set_mchars (a_mchars: CE_POINTER [INTEGER_8])
		do
			mchars := a_mchars
		end

	not_reg: CE_ARRAY [INTEGER_8]

	use_lessopen: CE_ARRAY [INTEGER_32]

feature {ANY} -- routines

	bad_file (a_filename: CE_POINTER [INTEGER_8]): CE_POINTER [INTEGER_8]
		local
			l_filename: CE_POINTER [INTEGER_8]
			l_m: CE_POINTER [INTEGER_8]
			l_r: INTEGER_32
			l_statbuf: CE_ARRAY [SD_STAT]
		do
			create l_filename.make
			create l_m.make
			create l_statbuf.make (<<1>>)

			l_filename := a_filename
			l_m := create {CE_POINTER [INTEGER_8]}.make_shared (default_pointer)
			l_filename := shell_unquote (l_filename)
			if (is_dir (l_filename)).to_boolean then
				l_m := create {CE_POINTER [INTEGER_8]}.make_cast ((main_inst.ecalloc (((strlen (l_filename)) + ({PLATFORM}.Integer_8_bytes.to_natural_32 * (16).to_natural_32)).to_integer_32, {PLATFORM}.Integer_8_bytes.to_natural_32)))
				strcpy (l_m, l_filename).do_nothing
				strcat (l_m, is_a_dir).do_nothing
			else
				l_r := stat (l_filename, l_statbuf)
				if l_r < 0 then
					l_m := os_inst.errno_message (l_filename)
				elseif (edit_inst.force_open.item).to_boolean then
					l_m := create {CE_POINTER [INTEGER_8]}.make_shared (default_pointer)
				elseif ((l_statbuf.item.st_mode).bit_and ({NATURAL_32} 61_440)) /= {NATURAL_32} 32_768 then
					l_m := create {CE_POINTER [INTEGER_8]}.make_cast ((main_inst.ecalloc (((strlen (l_filename)) + ({PLATFORM}.Integer_8_bytes.to_natural_32 * (42).to_natural_32)).to_integer_32, {PLATFORM}.Integer_8_bytes.to_natural_32)))
					strcpy (l_m, l_filename).do_nothing
					strcat (l_m, not_reg).do_nothing
				end
			end
			free (create {CE_POINTER [ANY]}.make_cast (l_filename))
			Result := l_m
		end

	bin_file (a_f: INTEGER_32): INTEGER_32
		local
			l_i: INTEGER_32
			l_n: INTEGER_32
			l_data: CE_ARRAY [NATURAL_8]
			js: JUMP_STATE
		do
			create l_data.make (<<64>>)

			if not (ch_inst.seekable (a_f)).to_boolean then
				Result := 0
				js.return
			end
			if not js.return_called then
				if (lseek (a_f, (0).to_integer_32, 0)) = -1 then
					Result := 0
					js.return
				end
			end
			if not js.return_called then
				l_n := read (a_f, create {CE_POINTER [ANY]}.make_cast (l_data), ({PLATFORM}.Natural_8_bytes.to_natural_32 * (64).to_natural_32))
				l_i := 0
				from
				until
					not (not js.loop_done and then (l_i < l_n))
				loop
					if (charset_inst.binary_char (((l_data[l_i]).to_integer_32).to_natural_8)).to_boolean then
						Result := 1
						js.return
					end
					if not js.return_called then
						l_i := l_i + 1
					end
				end
			end
			if not js.return_called then
				Result := 0
			end
		end

	close_altfile (a_altfilename: CE_POINTER [INTEGER_8]; a_filename: CE_POINTER [INTEGER_8]; a_pipefd: CE_POINTER [ANY])
		local
			l_lessclose: CE_POINTER [INTEGER_8]
			l_fd: CE_POINTER [SD_IO_FILE]
			l_cmd: CE_POINTER [INTEGER_8]
			l_tmp: NATURAL_32
			l_tmp___0: NATURAL_32
			js: JUMP_STATE
		do
			create l_lessclose.make
			create l_fd.make
			create l_cmd.make

			if main_inst.secure.to_boolean then
				js.return
			end
			if not js.return_called then
				if not a_pipefd.is_void then
					pclose (create {CE_POINTER [SD_IO_FILE]}.make_cast (a_pipefd)).do_nothing
				end
				l_lessclose := decode_inst.lgetenv (ce_string ("LESSCLOSE"))
				if l_lessclose.is_void then
					js.return
				end
			end
			if not js.return_called then
				l_tmp := strlen (l_lessclose)
				l_tmp___0 := strlen (a_filename)
				l_cmd := create {CE_POINTER [INTEGER_8]}.make_cast ((main_inst.ecalloc ((((l_tmp + l_tmp___0) + (strlen (a_altfilename))) + {NATURAL_32} 2).to_integer_32, {PLATFORM}.Integer_8_bytes.to_natural_32)))
				sprintf ([l_cmd, l_lessclose, a_filename, a_altfilename]).do_nothing
				l_fd := shellcmd___0 (l_cmd)
				free (create {CE_POINTER [ANY]}.make_cast (l_cmd))
				if not l_fd.is_void then
					pclose (l_fd).do_nothing
				end
			end
		end

	dirfile (a_dirname: CE_POINTER [INTEGER_8]; a_filename: CE_POINTER [INTEGER_8]): CE_POINTER [INTEGER_8]
		local
			l_pathname: CE_POINTER [INTEGER_8]
			l_qpathname: CE_POINTER [INTEGER_8]
			l_f: INTEGER_32
			l_tmp: NATURAL_32
			js: JUMP_STATE
		do
			create l_pathname.make
			create l_qpathname.make

			if a_dirname.is_void then
				Result := (create {CE_POINTER [INTEGER_8]}.make_shared (default_pointer))
				js.return
			elseif (a_dirname[0]).to_integer_32 = 0 then
				Result := (create {CE_POINTER [INTEGER_8]}.make_shared (default_pointer))
				js.return
			end
			if not js.return_called then
				l_tmp := strlen (a_dirname)
				l_pathname := create {CE_POINTER [INTEGER_8]}.make_cast ((calloc ((l_tmp + (strlen (a_filename))) + {NATURAL_32} 2, {PLATFORM}.Integer_8_bytes.to_natural_32)))
				if l_pathname.is_void then
					Result := (create {CE_POINTER [INTEGER_8]}.make_shared (default_pointer))
					js.return
				end
			end
			if not js.return_called then
				sprintf ([l_pathname, ce_string ("%%s%%s%%s"), a_dirname, ce_string ("/"), a_filename]).do_nothing
				l_qpathname := shell_unquote (l_pathname)
				l_f := open ([l_qpathname, 0])
				if l_f < 0 then
					free (create {CE_POINTER [ANY]}.make_cast (l_pathname))
					l_pathname := create {CE_POINTER [INTEGER_8]}.make_shared (default_pointer)
				else
					close (l_f).do_nothing
				end
				free (create {CE_POINTER [ANY]}.make_cast (l_qpathname))
				Result := l_pathname
			end
		end

	fcomplete (a_s: CE_POINTER [INTEGER_8]): CE_POINTER [INTEGER_8]
		local
			l_s: CE_POINTER [INTEGER_8]
			l_fpat: CE_POINTER [INTEGER_8]
			l_qs: CE_POINTER [INTEGER_8]
			js: JUMP_STATE
		do
			create l_s.make
			create l_fpat.make
			create l_qs.make

			l_s := a_s
			if main_inst.secure.to_boolean then
				Result := (create {CE_POINTER [INTEGER_8]}.make_shared (default_pointer))
				js.return
			end
			if not js.return_called then
				l_fpat := create {CE_POINTER [INTEGER_8]}.make_cast ((main_inst.ecalloc (((strlen (l_s)) + {NATURAL_32} 2).to_integer_32, {PLATFORM}.Integer_8_bytes.to_natural_32)))
				sprintf ([l_fpat, ce_string ("%%s*"), l_s]).do_nothing
				l_qs := lglob (l_fpat)
				l_s := shell_unquote (l_qs)
				if (strcmp (l_s, l_fpat)) = 0 then
					free (create {CE_POINTER [ANY]}.make_cast (l_qs))
					l_qs := create {CE_POINTER [INTEGER_8]}.make_shared (default_pointer)
				end
				free (create {CE_POINTER [ANY]}.make_cast (l_s))
				free (create {CE_POINTER [ANY]}.make_cast (l_fpat))
				Result := l_qs
			end
		end

	fexpand (a_s: CE_POINTER [INTEGER_8]): CE_POINTER [INTEGER_8]
		local
			l_fr: CE_POINTER [INTEGER_8]
			l_to: CE_POINTER [INTEGER_8]
			l_n: INTEGER_32
			l_e: CE_POINTER [INTEGER_8]
			l_ifile: CE_POINTER [ANY]
			l_tmp: CE_POINTER [ANY]
			l_tmp___3: CE_POINTER [INTEGER_8]
			l_tmp___4: CE_POINTER [ANY]
			l_tmp___5: CE_POINTER [INTEGER_8]
			l_tmp___8: CE_POINTER [INTEGER_8]
			l_pos: INTEGER_32
		do
			create l_fr.make
			create l_to.make
			create l_e.make
			create l_ifile.make
			create l_tmp.make
			create l_tmp___3.make
			create l_tmp___4.make
			create l_tmp___5.make
			create l_tmp___8.make

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_n := 0
					l_fr := a_s
				when 1 then -- loop condition:
					if (l_fr[0]).to_integer_32 = 0 then
						l_pos := -1 + 10 -- skip loop
					end
				when 2 then -- loop body:
						inspect (l_fr[0]).to_integer_32 -- switch 1 head
						when 37 then l_pos := -1 + 3
						when 35 then l_pos := -1 + 3
						else l_pos := -1 + 8
						end
				when 3 then -- switch 1, case 37 | switch 1, case 35
						if not ((l_fr.c_pointer.to_integer_32).to_natural_32 > (a_s.c_pointer.to_integer_32).to_natural_32) then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if:
						if (l_fr[-1]).to_integer_32 = (l_fr[0]).to_integer_32 then
							l_n := l_n + 1
						else
							l_pos := -1 + 6 -- goto _L
						end
				when 5 then -- if block done, skip else block:
						l_pos := -1 + 7
				when 6 then -- else: | _L:
						if (l_fr[1]).to_integer_32 /= (l_fr[0]).to_integer_32 then
							if (l_fr[0]).to_integer_32 = 37 then
								l_ifile := main_inst.curr_ifile
							else
								if (l_fr[0]).to_integer_32 = 35 then
									l_tmp := main_inst.old_ifile
								else
									l_tmp := create {CE_POINTER [ANY]}.make_shared (default_pointer)
								end
								l_ifile := l_tmp
							end
							if l_ifile.is_void then
								l_n := l_n + 1
							else
								l_n := (l_n.to_natural_32 + (strlen ((ifile_inst.get_filename (l_ifile))))).to_integer_32
							end
						end
				when 7 then
						l_pos := -1 + 9 -- switch 1; break
				when 8 then -- switch 1, default:
						l_n := l_n + 1
						l_pos := -1 + 9 -- switch 1; break
				when 9 then -- switch 1, done:
						l_fr := l_fr + 1
						l_pos := -1 + 1 -- back to loop condition
				when 10 then -- loop done
					l_e := create {CE_POINTER [INTEGER_8]}.make_cast ((main_inst.ecalloc (l_n + 1, {PLATFORM}.Integer_8_bytes.to_natural_32)))
					l_to := l_e
					l_fr := a_s
				when 11 then -- loop condition:
					if (l_fr[0]).to_integer_32 = 0 then
						l_pos := -1 + 20 -- skip loop
					end
				when 12 then -- loop body:
						inspect (l_fr[0]).to_integer_32 -- switch 2 head
						when 37 then l_pos := -1 + 13
						when 35 then l_pos := -1 + 13
						else l_pos := -1 + 18
						end
				when 13 then -- switch 2, case 37 | switch 2, case 35
						if not ((l_fr.c_pointer.to_integer_32).to_natural_32 > (a_s.c_pointer.to_integer_32).to_natural_32) then
							l_pos := -1 + 16 -- skip if block
						end
				when 14 then -- if:
						if (l_fr[-1]).to_integer_32 = (l_fr[0]).to_integer_32 then
							l_tmp___3 := l_to
							l_to := l_to + 1
							l_tmp___3[0] := l_fr[0]
						else
							l_pos := -1 + 16 -- goto _L___0
						end
				when 15 then -- if block done, skip else block:
						l_pos := -1 + 17
				when 16 then -- else: | _L___0:
						if (l_fr[1]).to_integer_32 /= (l_fr[0]).to_integer_32 then
							if (l_fr[0]).to_integer_32 = 37 then
								l_ifile := main_inst.curr_ifile
							else
								if (l_fr[0]).to_integer_32 = 35 then
									l_tmp___4 := main_inst.old_ifile
								else
									l_tmp___4 := create {CE_POINTER [ANY]}.make_shared (default_pointer)
								end
								l_ifile := l_tmp___4
							end
							if l_ifile.is_void then
								l_tmp___5 := l_to
								l_to := l_to + 1
								l_tmp___5[0] := l_fr[0]
							else
								strcpy (l_to, (ifile_inst.get_filename (l_ifile))).do_nothing
								l_to := l_to + (strlen (l_to)).to_integer_32
							end
						end
				when 17 then
						l_pos := -1 + 19 -- switch 2; break
				when 18 then -- switch 2, default:
						l_tmp___8 := l_to
						l_to := l_to + 1
						l_tmp___8[0] := l_fr[0]
						l_pos := -1 + 19 -- switch 2; break
				when 19 then -- switch 2, done:
						l_fr := l_fr + 1
						l_pos := -1 + 11 -- back to loop condition
				when 20 then -- loop done
					l_to[0] := (('%/0/').code).to_integer_8
					Result := l_e
				when 21 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	filesize (a_f: INTEGER_32): INTEGER_32
		local
			l_statbuf: CE_ARRAY [SD_STAT]
			js: JUMP_STATE
		do
			create l_statbuf.make (<<1>>)

			if (fstat (a_f, l_statbuf)) >= 0 then
				Result := (l_statbuf.item.st_size)
				js.return
			end
			if not js.return_called then
				Result := (seek_filesize (a_f))
			end
		end

	get_meta_escape: CE_POINTER [INTEGER_8]
		local
			l_s: CE_POINTER [INTEGER_8]
		do
			create l_s.make

			l_s := decode_inst.lgetenv (ce_string ("LESSMETAESCAPE"))
			if l_s.is_void then
				l_s := ce_string ("\")
			end
			Result := l_s
		end

	homefile (a_filename: CE_POINTER [INTEGER_8]): CE_POINTER [INTEGER_8]
		local
			l_pathname: CE_POINTER [INTEGER_8]
			js: JUMP_STATE
		do
			create l_pathname.make

			l_pathname := dirfile ((decode_inst.lgetenv (ce_string ("HOME"))), a_filename)
			if not l_pathname.is_void then
				Result := l_pathname
				js.return
			end
			if not js.return_called then
				Result := (create {CE_POINTER [INTEGER_8]}.make_shared (default_pointer))
			end
		end

	is_dir (a_filename: CE_POINTER [INTEGER_8]): INTEGER_32
		local
			l_filename: CE_POINTER [INTEGER_8]
			l_isdir: INTEGER_32
			l_r: INTEGER_32
			l_statbuf: CE_ARRAY [SD_STAT]
			l_tmp: INTEGER_32
		do
			create l_filename.make
			create l_statbuf.make (<<1>>)

			l_filename := a_filename
			l_isdir := 0
			l_filename := shell_unquote (l_filename)
			l_r := stat (l_filename, l_statbuf)
			if l_r >= 0 then
				if ((l_statbuf.item.st_mode).bit_and ({NATURAL_32} 61_440)) = {NATURAL_32} 16_384 then
					l_tmp := 1
				else
					l_tmp := 0
				end
			else
				l_tmp := 0
			end
			l_isdir := l_tmp
			free (create {CE_POINTER [ANY]}.make_cast (l_filename))
			Result := l_isdir
		end

	lglob (a_filename: CE_POINTER [INTEGER_8]): CE_POINTER [INTEGER_8]
		local
			l_filename: CE_POINTER [INTEGER_8]
			l_gfilename: CE_POINTER [INTEGER_8]
			l_ofilename: CE_POINTER [INTEGER_8]
			l_fd: CE_POINTER [SD_IO_FILE]
			l_s: CE_POINTER [INTEGER_8]
			l_lessecho: CE_POINTER [INTEGER_8]
			l_cmd: CE_POINTER [INTEGER_8]
			l_esc: CE_POINTER [INTEGER_8]
			l_tmp___0: NATURAL_32
			l_tmp___1: NATURAL_32
			js: JUMP_STATE
		do
			create l_filename.make
			create l_gfilename.make
			create l_ofilename.make
			create l_fd.make
			create l_s.make
			create l_lessecho.make
			create l_cmd.make
			create l_esc.make

			l_filename := a_filename
			l_ofilename := fexpand (l_filename)
			if main_inst.secure.to_boolean then
				Result := l_ofilename
				js.return
			end
			if not js.return_called then
				l_filename := shell_unquote (l_ofilename)
				l_esc := get_meta_escape
				if (strlen (l_esc)) = {NATURAL_32} 0 then
					l_esc := ce_string ("-")
				end
				l_esc := shell_quote (l_esc)
				if l_esc.is_void then
					free (create {CE_POINTER [ANY]}.make_cast (l_filename))
					Result := l_ofilename
					js.return
				end
			end
			if not js.return_called then
				l_lessecho := decode_inst.lgetenv (ce_string ("LESSECHO"))
				if l_lessecho.is_void then
					l_lessecho := ce_string ("lessecho")
				elseif (l_lessecho[0]).to_integer_32 = 0 then
					l_lessecho := ce_string ("lessecho")
				end
				l_tmp___0 := strlen (l_lessecho)
				l_tmp___1 := strlen (l_ofilename)
				l_cmd := create {CE_POINTER [INTEGER_8]}.make_cast ((main_inst.ecalloc ((((l_tmp___0 + l_tmp___1) + {NATURAL_32} 7 * (strlen ((metachars)))) + {NATURAL_32} 24).to_integer_32, {PLATFORM}.Integer_8_bytes.to_natural_32)))
				sprintf ([l_cmd, ce_string ("%%s -p0x%%x -d0x%%x -e%%s "), l_lessecho, cmdbuf_inst.openquote.to_integer_32, cmdbuf_inst.closequote.to_integer_32, l_esc]).do_nothing
				free (create {CE_POINTER [ANY]}.make_cast (l_esc))
				l_s := metachars
				from
				until
					(l_s[0]).to_integer_32 = 0
				loop
					sprintf ([(l_cmd + (strlen (l_cmd)).to_integer_32), ce_string ("-n0x%%x "), (l_s[0]).to_integer_32]).do_nothing
					l_s := l_s + 1
				end
				sprintf ([(l_cmd + (strlen (l_cmd)).to_integer_32), ce_string ("-- %%s"), l_ofilename]).do_nothing
				l_fd := shellcmd___0 (l_cmd)
				free (create {CE_POINTER [ANY]}.make_cast (l_cmd))
				if l_fd.is_void then
					free (create {CE_POINTER [ANY]}.make_cast (l_filename))
					Result := l_ofilename
					js.return
				end
			end
			if not js.return_called then
				l_gfilename := readfd (l_fd)
				pclose (l_fd).do_nothing
				if (l_gfilename[0]).to_integer_32 = 0 then
					free (create {CE_POINTER [ANY]}.make_cast (l_gfilename))
					free (create {CE_POINTER [ANY]}.make_cast (l_filename))
					Result := l_ofilename
					js.return
				end
			end
			if not js.return_called then
				free (create {CE_POINTER [ANY]}.make_cast (l_filename))
				free (create {CE_POINTER [ANY]}.make_cast (l_ofilename))
				Result := l_gfilename
			end
		end

	metachar (a_c: INTEGER_8): INTEGER_32
		do
			Result := (not (strchr ((metachars), a_c.to_integer_32)).is_void).to_integer
		end

	metachars: CE_POINTER [INTEGER_8]
		do
			if mchars.is_void then
				mchars := decode_inst.lgetenv (ce_string ("LESSMETACHARS"))
				if mchars.is_void then
					mchars := ce_string ("; *?%T%N%'%"()<>|&^`#\")
				end
			end
			Result := mchars
		end

	open_altfile (a_filename: CE_POINTER [INTEGER_8]; a_pf: CE_POINTER [INTEGER_32]; a_pfd: CE_POINTER [CE_POINTER [ANY]]): CE_POINTER [INTEGER_8]
		local
			l_lessopen: CE_POINTER [INTEGER_8]
			l_cmd: CE_POINTER [INTEGER_8]
			l_fd: CE_POINTER [SD_IO_FILE]
			l_returnfd: INTEGER_32
			l_tmp___0: NATURAL_32
			l_f: INTEGER_32
			l_c: CE_ARRAY [INTEGER_8]
			js: JUMP_STATE
		do
			create l_lessopen.make
			create l_cmd.make
			create l_fd.make
			create l_c.make (<<1>>)

			l_returnfd := 0
			if not (use_lessopen.item).to_boolean then
				Result := (create {CE_POINTER [INTEGER_8]}.make_shared (default_pointer))
				js.return
			elseif main_inst.secure.to_boolean then
				Result := (create {CE_POINTER [INTEGER_8]}.make_shared (default_pointer))
				js.return
			end
			if not js.return_called then
				ch_inst.ch_ungetchar (-1)
				l_lessopen := decode_inst.lgetenv (ce_string ("LESSOPEN"))
				if l_lessopen.is_void then
					Result := (create {CE_POINTER [INTEGER_8]}.make_shared (default_pointer))
					js.return
				end
			end
			if not js.return_called then
				if (strcmp (a_filename, ce_string ("-"))) = 0 then
					Result := (create {CE_POINTER [INTEGER_8]}.make_shared (default_pointer))
					js.return
				end
			end
			if not js.return_called then
				if (l_lessopen[0]).to_integer_32 = 124 then
					l_lessopen := l_lessopen + 1
					l_returnfd := 1
				end
				l_tmp___0 := strlen (l_lessopen)
				l_cmd := create {CE_POINTER [INTEGER_8]}.make_cast ((main_inst.ecalloc (((l_tmp___0 + (strlen (a_filename))) + {NATURAL_32} 2).to_integer_32, {PLATFORM}.Integer_8_bytes.to_natural_32)))
				sprintf ([l_cmd, l_lessopen, a_filename]).do_nothing
				l_fd := shellcmd___0 (l_cmd)
				free (create {CE_POINTER [ANY]}.make_cast (l_cmd))
				if l_fd.is_void then
					Result := (create {CE_POINTER [INTEGER_8]}.make_shared (default_pointer))
					js.return
				end
			end
			if not js.return_called and then (l_returnfd.to_boolean) then
				l_f := fileno (l_fd)
				if (read (l_f, create {CE_POINTER [ANY]}.make_cast (l_c), (1).to_natural_32)) /= 1 then
					pclose (l_fd).do_nothing
					Result := (create {CE_POINTER [INTEGER_8]}.make_shared (default_pointer))
					js.return
				end
				if not js.return_called then
					ch_inst.ch_ungetchar ((l_c.item).to_integer_32)
					a_pfd.item := create {CE_POINTER [ANY]}.make_cast (l_fd)
					a_pf.item := l_f
					Result := (main_inst.save (ce_string ("-")))
					js.return
				end
			end
			if not js.return_called then
				l_cmd := readfd (l_fd)
				pclose (l_fd).do_nothing
				if (l_cmd[0]).to_integer_32 = 0 then
					Result := (create {CE_POINTER [INTEGER_8]}.make_shared (default_pointer))
					js.return
				end
			end
			if not js.return_called then
				Result := l_cmd
			end
		end

	readfd (a_fd: CE_POINTER [SD_IO_FILE]): CE_POINTER [INTEGER_8]
		local
			l_len: INTEGER_32
			l_ch: INTEGER_32
			l_buf___1: CE_POINTER [INTEGER_8]
			l_p: CE_POINTER [INTEGER_8]
			js: JUMP_STATE
		do
			create l_buf___1.make
			create l_p.make

			l_len := 100
			l_buf___1 := create {CE_POINTER [INTEGER_8]}.make_cast ((main_inst.ecalloc (l_len, {PLATFORM}.Integer_8_bytes.to_natural_32)))
			l_p := l_buf___1
			from
			until
				js.loop_done
			loop
				l_ch := IO_getc (a_fd)
				if l_ch = 10 then
					js.break
				elseif l_ch = -1 then
					js.break
				end
				if not js.loop_jumped then
					if l_p.distance (l_buf___1) >= l_len - 1 then
						l_len := l_len * (2)
						l_p[0] := (('%/0/').code).to_integer_8
						l_p := create {CE_POINTER [INTEGER_8]}.make_cast ((main_inst.ecalloc (l_len, {PLATFORM}.Integer_8_bytes.to_natural_32)))
						strcpy (l_p, l_buf___1).do_nothing
						free (create {CE_POINTER [ANY]}.make_cast (l_buf___1))
						l_buf___1 := l_p
						l_p := l_buf___1 + (strlen (l_buf___1)).to_integer_32
					end
					l_p[0] := l_ch.to_integer_8
					l_p := l_p + 1
				end
			end
			l_p[0] := (('%/0/').code).to_integer_8
			Result := l_buf___1
		end

	seek_filesize (a_f: INTEGER_32): INTEGER_32
		local
			l_spos: INTEGER_32
			js: JUMP_STATE
		do
			l_spos := lseek (a_f, (0).to_integer_32, 2)
			if l_spos = -1 then
				Result := (-1).to_integer_32
				js.return
			end
			if not js.return_called then
				Result := l_spos
			end
		end

	shell_coption: CE_POINTER [INTEGER_8]
		do
			Result := (ce_string ("-c"))
		end

	shell_quote (a_s: CE_POINTER [INTEGER_8]): CE_POINTER [INTEGER_8]
		local
			l_s: CE_POINTER [INTEGER_8]
			l_p: CE_POINTER [INTEGER_8]
			l_newstr: CE_POINTER [INTEGER_8]
			l_len: INTEGER_32
			l_esc: CE_POINTER [INTEGER_8]
			l_esclen: INTEGER_32
			l_use_quotes: INTEGER_32
			l_have_quotes: INTEGER_32
			l_tmp___5: CE_POINTER [INTEGER_8]
			l_tmp___6: CE_POINTER [INTEGER_8]
			js: JUMP_STATE
		do
			create l_s.make
			create l_p.make
			create l_newstr.make
			create l_esc.make
			create l_tmp___5.make
			create l_tmp___6.make

			l_s := a_s
			l_esc := (get_meta_escape)
			l_esclen := (strlen (l_esc)).to_integer_32
			l_use_quotes := 0
			l_have_quotes := 0
			l_len := 1
			l_p := l_s
			from
			until
				(l_p[0]).to_integer_32 = 0
			loop
				l_len := l_len + 1
				if (l_p[0]).to_integer_32 = cmdbuf_inst.openquote.to_integer_32 then
					l_have_quotes := 1
				elseif (l_p[0]).to_integer_32 = cmdbuf_inst.closequote.to_integer_32 then
					l_have_quotes := 1
				end
				if (metachar (l_p[0])).to_boolean then
					if l_esclen = 0 then
						l_use_quotes := 1
					else
						l_len := l_len + l_esclen
					end
				end
				l_p := l_p + 1
			end
			if l_use_quotes.to_boolean then
				if l_have_quotes.to_boolean then
					Result := (create {CE_POINTER [INTEGER_8]}.make_shared (default_pointer))
					js.return
				end
				if not js.return_called then
					l_len := ((strlen (l_s)) + {NATURAL_32} 3).to_integer_32
				end
			end
			if not js.return_called then
				l_p := create {CE_POINTER [INTEGER_8]}.make_cast ((main_inst.ecalloc (l_len, {PLATFORM}.Integer_8_bytes.to_natural_32)))
				l_newstr := l_p
				if l_use_quotes.to_boolean then
					sprintf ([l_newstr, ce_string ("%%c%%s%%c"), cmdbuf_inst.openquote.to_integer_32, l_s, cmdbuf_inst.closequote.to_integer_32]).do_nothing
				else
					from
					until
						(l_s[0]).to_integer_32 = 0
					loop
						if (metachar (l_s[0])).to_boolean then
							strcpy (l_p, l_esc).do_nothing
							l_p := l_p + l_esclen
						end
						l_tmp___5 := l_p
						l_p := l_p + 1
						l_tmp___6 := l_s
						l_s := l_s + 1
						l_tmp___5[0] := l_tmp___6[0]
					end
					l_p[0] := (('%/0/').code).to_integer_8
				end
				Result := l_newstr
			end
		end

	shell_unquote (a_str: CE_POINTER [INTEGER_8]): CE_POINTER [INTEGER_8]
		local
			l_str: CE_POINTER [INTEGER_8]
			l_name: CE_POINTER [INTEGER_8]
			l_p: CE_POINTER [INTEGER_8]
			l_tmp___1: CE_POINTER [INTEGER_8]
			l_tmp___2: CE_POINTER [INTEGER_8]
			l_esc: CE_POINTER [INTEGER_8]
			l_esclen: INTEGER_32
			l_tmp___6: CE_POINTER [INTEGER_8]
			l_tmp___7: CE_POINTER [INTEGER_8]
			js: JUMP_STATE
		do
			create l_str.make
			create l_name.make
			create l_p.make
			create l_tmp___1.make
			create l_tmp___2.make
			create l_esc.make
			create l_tmp___6.make
			create l_tmp___7.make

			l_str := a_str
			l_p := create {CE_POINTER [INTEGER_8]}.make_cast ((main_inst.ecalloc (((strlen (l_str)) + {NATURAL_32} 1).to_integer_32, {PLATFORM}.Integer_8_bytes.to_natural_32)))
			l_name := l_p
			if (l_str[0]).to_integer_32 = cmdbuf_inst.openquote.to_integer_32 then
				l_str := l_str + 1
				from
				until
					not (not js.loop_done and then ((l_str[0]).to_integer_32 /= 0))
				loop
					if (l_str[0]).to_integer_32 = cmdbuf_inst.closequote.to_integer_32 then
						if (l_str[1]).to_integer_32 /= cmdbuf_inst.closequote.to_integer_32 then
							js.break
						end
						if not js.loop_jumped then
							l_str := l_str + 1
						end
					end
					if not js.loop_jumped then
						l_tmp___1 := l_p
						l_p := l_p + 1
						l_tmp___2 := l_str
						l_str := l_str + 1
						l_tmp___1[0] := l_tmp___2[0]
					end
				end
			else
				l_esc := (get_meta_escape)
				l_esclen := (strlen (l_esc)).to_integer_32
				from
				until
					(l_str[0]).to_integer_32 = 0
				loop
					if l_esclen > 0 then
						if (strncmp (l_str, l_esc, l_esclen.to_natural_32)) = 0 then
							l_str := l_str + l_esclen
						end
					end
					l_tmp___6 := l_p
					l_p := l_p + 1
					l_tmp___7 := l_str
					l_str := l_str + 1
					l_tmp___6[0] := l_tmp___7[0]
				end
			end
			l_p[0] := (('%/0/').code).to_integer_8
			Result := l_name
		end

	shellcmd___0 (a_cmd: CE_POINTER [INTEGER_8]): CE_POINTER [SD_IO_FILE]
		local
			l_fd: CE_POINTER [SD_IO_FILE]
			l_shell: CE_POINTER [INTEGER_8]
			l_scmd: CE_POINTER [INTEGER_8]
			l_esccmd: CE_POINTER [INTEGER_8]
			l_tmp: NATURAL_32
		do
			create l_fd.make
			create l_shell.make
			create l_scmd.make
			create l_esccmd.make

			l_shell := decode_inst.lgetenv (ce_string ("SHELL"))
			if not l_shell.is_void then
				if (l_shell[0]).to_integer_32 /= 0 then
					l_esccmd := shell_quote (a_cmd)
					if l_esccmd.is_void then
						l_fd := popen (a_cmd, ce_string ("r"))
					else
						l_tmp := strlen (l_shell)
						l_scmd := create {CE_POINTER [INTEGER_8]}.make_cast ((main_inst.ecalloc (((l_tmp + (strlen (l_esccmd))) + {NATURAL_32} 5).to_integer_32, {PLATFORM}.Integer_8_bytes.to_natural_32)))
						sprintf ([l_scmd, ce_string ("%%s %%s %%s"), l_shell, (shell_coption), l_esccmd]).do_nothing
						free (create {CE_POINTER [ANY]}.make_cast (l_esccmd))
						l_fd := popen (l_scmd, ce_string ("r"))
						free (create {CE_POINTER [ANY]}.make_cast (l_scmd))
					end
				else
					l_fd := popen (a_cmd, ce_string ("r"))
				end
			else
				l_fd := popen (a_cmd, ce_string ("r"))
			end
			Result := l_fd
		end

end
