class
	PD_COOKIE

inherit
	CE_CLASS
		redefine
			make,
			copy,
			is_equal,
			bitfield_helper
		end

create
	make,
	make_shared

feature {NONE}

	make
		do
			make_memory -- has bitfields; back with memory for correct semantics
		end

feature {ANY} -- fields

	domain: CE_POINTER [INTEGER_8] assign set_domain

	port: INTEGER_32 assign set_port

	path: CE_POINTER [INTEGER_8] assign set_path

	discard_requested: NATURAL_32 assign set_discard_requested

	secure: NATURAL_32 assign set_secure

	domain_exact: NATURAL_32 assign set_domain_exact

	permanent: NATURAL_32 assign set_permanent

	expiry_time: INTEGER_32 assign set_expiry_time

	attr: CE_POINTER [INTEGER_8] assign set_attr

	value: CE_POINTER [INTEGER_8] assign set_value

	next: CE_POINTER [PD_COOKIE] assign set_next

feature {ANY} -- setters

	set_domain (a_domain: CE_POINTER [INTEGER_8])
		do
			domain := a_domain
			update_memory_field ("domain")
		end

	set_port (a_port: INTEGER_32)
		do
			port := a_port
			update_memory_field ("port")
		end

	set_path (a_path: CE_POINTER [INTEGER_8])
		do
			path := a_path
			update_memory_field ("path")
		end

	set_discard_requested (a_discard_requested: NATURAL_32)
		do
			discard_requested := a_discard_requested
			update_memory_field ("discard_requested")
		end

	set_secure (a_secure: NATURAL_32)
		do
			secure := a_secure
			update_memory_field ("secure")
		end

	set_domain_exact (a_domain_exact: NATURAL_32)
		do
			domain_exact := a_domain_exact
			update_memory_field ("domain_exact")
		end

	set_permanent (a_permanent: NATURAL_32)
		do
			permanent := a_permanent
			update_memory_field ("permanent")
		end

	set_expiry_time (a_expiry_time: INTEGER_32)
		do
			expiry_time := a_expiry_time
			update_memory_field ("expiry_time")
		end

	set_attr (a_attr: CE_POINTER [INTEGER_8])
		do
			attr := a_attr
			update_memory_field ("attr")
		end

	set_value (a_value: CE_POINTER [INTEGER_8])
		do
			value := a_value
			update_memory_field ("value")
		end

	set_next (a_next: CE_POINTER [PD_COOKIE])
		do
			next := a_next
			update_memory_field ("next")
		end

feature {ANY}

	copy (a_other: like Current)
		do
			domain := a_other.domain
			port := a_other.port
			path := a_other.path
			discard_requested := a_other.discard_requested
			secure := a_other.secure
			domain_exact := a_other.domain_exact
			permanent := a_other.permanent
			expiry_time := a_other.expiry_time
			attr := a_other.attr
			value := a_other.value
			next := a_other.next
			create_memory_copy -- has bitfields; back with memory for correct semantics
		end

	is_equal (a_other: like Current): BOOLEAN
		do
			Result :=
				domain.is_equal (a_other.domain) and
				port = a_other.port and
				path.is_equal (a_other.path) and
				discard_requested = a_other.discard_requested and
				secure = a_other.secure and
				domain_exact = a_other.domain_exact and
				permanent = a_other.permanent and
				expiry_time = a_other.expiry_time and
				attr.is_equal (a_other.attr) and
				value.is_equal (a_other.value) and
				next.is_equal (a_other.next)
		end

feature {NONE}

	structure_info: CE_STRUCTURE
		once ("PROCESS")
			create Result.make_struct (<<
					create {CE_STRUCTURE_FIELD}.make ("domain"),
					create {CE_STRUCTURE_FIELD}.make ("port"),
					create {CE_STRUCTURE_FIELD}.make ("path"),
					create {CE_STRUCTURE_FIELD}.make_bitfield ("discard_requested"),
					create {CE_STRUCTURE_FIELD}.make_bitfield ("secure"),
					create {CE_STRUCTURE_FIELD}.make_bitfield ("domain_exact"),
					create {CE_STRUCTURE_FIELD}.make_bitfield ("permanent"),
					create {CE_STRUCTURE_FIELD}.make ("expiry_time"),
					create {CE_STRUCTURE_FIELD}.make ("attr"),
					create {CE_STRUCTURE_FIELD}.make ("value"),
					create {CE_STRUCTURE_FIELD}.make ("next")
				>>)
			init_structure_info
		end

	bitfield_helper (a_operation: INTEGER_32; a_field_name, a_struct_pointer: POINTER; a_value: NATURAL_64): NATURAL_64
		external
			"C inline"
		alias
			"[
				struct cookie {
					char * domain;
					int port;
					char * path;
					unsigned int discard_requested : 1;
					unsigned int secure : 1;
					unsigned int domain_exact : 1;
					unsigned int permanent : 1;
					long expiry_time;
					char * attr;
					char * value;
					struct cookie * next;
				};
				char *field_name = (char *)$a_field_name;
				if($a_operation == 1) {
					if(field_name == (char *)0) return sizeof(struct cookie);
					if(strcmp(field_name, "expiry_time") == 0) return offsetof(struct cookie, expiry_time);
				} else {
					struct cookie *p = (struct cookie *)$a_struct_pointer;
					if($a_operation == 2) {
						     if(strcmp(field_name, "discard_requested") == 0) p->discard_requested = (unsigned int)$a_value;
						else if(strcmp(field_name, "secure") == 0) p->secure = (unsigned int)$a_value;
						else if(strcmp(field_name, "domain_exact") == 0) p->domain_exact = (unsigned int)$a_value;
						else if(strcmp(field_name, "permanent") == 0) p->permanent = (unsigned int)$a_value;
					} else if($a_operation == 3) {
						if(strcmp(field_name, "discard_requested") == 0) return p->discard_requested;
						if(strcmp(field_name, "secure") == 0) return p->secure;
						if(strcmp(field_name, "domain_exact") == 0) return p->domain_exact;
						if(strcmp(field_name, "permanent") == 0) return p->permanent;
					}
				}
			]"
		end

end
