class
	P_AF_UNIX

inherit
	LINKS_DATA
		redefine
			default_create
		end
	S_SYS_SELECT
	S_STDLIB
	S_UNISTD
	S_STRING
	S_BITS_ERRNO
	S_SYS_SOCKET
	S_STDIO
	S_FCNTL
	S_NETINET_IN
	S_SYS_STAT
	S_SYS_UTSNAME
	S_NETDB
	S_SIGNAL
	S_PWD
	S_GRP
	S_TIME
	S_DIRENT
	S_TERMIOS
	S_SYS_IOCTL
	S_SYS_TIME
	S_SYS_WAIT
	S_UTIME

feature {NONE} -- Initialization

	default_create
		do
			create s_unix.make
			create s_unix_acc.make
		end

feature {ANY} -- attributes

	s_unix: CE_POINTER [SD_SOCKADDR] assign set_s_unix

	set_s_unix (a_s_unix: CE_POINTER [SD_SOCKADDR])
		do
			s_unix := a_s_unix
		end

	s_unix_acc: CE_POINTER [SD_SOCKADDR] assign set_s_unix_acc

	set_s_unix_acc (a_s_unix_acc: CE_POINTER [SD_SOCKADDR])
		do
			s_unix_acc := a_s_unix_acc
		end

	s_unix_fd: INTEGER_32 assign set_s_unix_fd

	set_s_unix_fd (a_s_unix_fd: INTEGER_32)
		do
			s_unix_fd := a_s_unix_fd
		end

	s_unix_l: NATURAL_32 assign set_s_unix_l

	set_s_unix_l (a_s_unix_l: NATURAL_32)
		do
			s_unix_l := a_s_unix_l
		end

feature {ANY} -- routines

	af_unix_close
		do
			if s_unix_fd /= -1 then
				close (s_unix_fd).do_nothing
			end
			if (not s_unix.is_void) then
				unlink_unix
				error_inst.debug_mem_free (create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("af_unix.c")), 144, create {CE_POINTER [ANY]}.make_cast (s_unix))
				s_unix := create {CE_POINTER [SD_SOCKADDR]}.make_shared (default_pointer)
			end
			if (not s_unix_acc.is_void) then
				error_inst.debug_mem_free (create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("af_unix.c")), 145, create {CE_POINTER [ANY]}.make_cast (s_unix_acc))
				s_unix_acc := create {CE_POINTER [SD_SOCKADDR]}.make_shared (default_pointer)
			end
		end

	af_unix_connection (a_xxx: CE_POINTER [ANY])
		local
			l_l: CE_ARRAY [NATURAL_32]
			l_ns: INTEGER_32
			js: JUMP_STATE
		do
			create l_l.make (<<1>>)

			l_l.item := s_unix_l
			memset (create {CE_POINTER [ANY]}.make_cast (s_unix_acc), 0, l_l.item).do_nothing
			l_ns := accept (s_unix_fd, s_unix_acc, l_l)
			if l_ns = -1 then
				js.return
			end
			if not js.return_called then
				terminal_inst.init_term (l_ns, l_ns, session_inst.win_func_address).do_nothing
				os_dep_inst.set_highpri
			end
		end

	af_unix_connection_address: CE_ROUTINE
		once ("OBJECT")
			create Result.make_agent (agent af_unix_connection)
		end

	bind_to_af_unix: INTEGER_32
		local
			l_u: INTEGER_32
			l_a1: CE_ARRAY [INTEGER_32]
			l_cnt: INTEGER_32
			l_af: INTEGER_32
			l_tv: CE_ARRAY [SD_TIMEVAL]
			l_dummy: CE_ARRAY [SD_FD_SET]
			l___d0: CE_ARRAY [INTEGER_32]
			l___d1: CE_ARRAY [INTEGER_32]
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			create l_a1.make (<<1>>)
			create l_tv.make (<<1>>)
			create l_dummy.make (<<1>>)
			create l___d0.make (<<1>>)
			create l___d1.make (<<1>>)

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_u := 0
					l_a1.item := 1
					l_cnt := 0
					l_af := get_address
					if l_af = -1 then
						Result := (-1)
						js.return
					end
					if js.return_called then
						l_pos := -1 + 2 -- skip if block
					end
				when 1 then -- if: | again:
						s_unix_fd := socket (l_af, 1, 0)
						if s_unix_fd = -1 then
							Result := (-1)
							js.return
						end
				when 2 then -- if done
					if js.return_called then
						l_pos := -1 + 10 -- skip if block
					end
				when 3 then -- if:
						setsockopt (s_unix_fd, 1, 2, create {CE_POINTER [ANY]}.make_cast (l_a1), {PLATFORM}.Integer_32_bytes.to_natural_32).do_nothing
						if not ((bind (s_unix_fd, s_unix, s_unix_l)).to_boolean) then
							l_pos := -1 + 10 -- skip if block
						end
				when 4 then -- if:
						close (s_unix_fd).do_nothing
						s_unix_fd := socket (l_af, 1, 0)
						if s_unix_fd = -1 then
							Result := (-1)
							js.return
						end
						if js.return_called then
							l_pos := -1 + 9 -- skip if block
						end
				when 5 then -- if:
						setsockopt (s_unix_fd, 1, 2, create {CE_POINTER [ANY]}.make_cast (l_a1), {PLATFORM}.Integer_32_bytes.to_natural_32).do_nothing
						if not ((connect (s_unix_fd, s_unix, s_unix_l)).to_boolean) then
							l_pos := -1 + 9 -- skip if block
						end
				when 6 then -- if:
						l_cnt := l_cnt + 1
						if l_cnt < 3 then
							l_tv.item.tv_sec := (0).to_integer_32
							l_tv.item.tv_usec := (100_000).to_integer_32
							from
							until
								js.loop_done
							loop
								asm1 (l___d0, l___d1, l_dummy)
								js.break
							end
							select2 (0, l_dummy, l_dummy, l_dummy, l_tv).do_nothing
							close (s_unix_fd).do_nothing
							l_pos := -1 + 1 -- goto again
						end
				when 7 then
						close (s_unix_fd).do_nothing
						s_unix_fd := -1
						if not l_u.to_boolean then
							unlink_unix
							l_u := 1
							l_pos := -1 + 1 -- goto again
						end
				when 8 then
						error_inst.debug_mem_free (create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("af_unix.c")), 114, create {CE_POINTER [ANY]}.make_cast (s_unix))
						s_unix := create {CE_POINTER [SD_SOCKADDR]}.make_shared (default_pointer)
						Result := (-1)
						js.return
				when 9 then -- if done | if done
						if not js.return_called then
							error_inst.debug_mem_free (create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("af_unix.c")), 117, create {CE_POINTER [ANY]}.make_cast (s_unix))
							s_unix := create {CE_POINTER [SD_SOCKADDR]}.make_shared (default_pointer)
							Result := s_unix_fd
							js.return
						end
				when 10 then -- if done | if done
						if not js.return_called then
							if (listen (s_unix_fd, 100)).to_boolean then
								error_inst.error ([create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("ERROR: listen failed: %%d")), (errno_location).item])
								error_inst.debug_mem_free (create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("af_unix.c")), 122, create {CE_POINTER [ANY]}.make_cast (s_unix))
								s_unix := create {CE_POINTER [SD_SOCKADDR]}.make_shared (default_pointer)
								close (s_unix_fd).do_nothing
								s_unix_fd := -1
								Result := (-1)
								js.return
							end
						end
						if not js.return_called then
							select_inst.set_handlers (s_unix_fd, af_unix_connection_address, create {CE_ROUTINE}.make_shared (default_pointer), create {CE_ROUTINE}.make_shared (default_pointer), create {CE_POINTER [ANY]}.make_shared (default_pointer))
							Result := (-1)
						end
				when 11 then
						l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
			end

	get_address: INTEGER_32
		local
			l_su: CE_POINTER [SD_SOCKADDR_UN]
			l_path: CE_ARRAY [CE_POINTER [NATURAL_8]]
			js: JUMP_STATE
		do
			create l_su.make
			create l_path.make (<<1>>)

			if default_inst.links_home.is_void then
				Result := (-1)
				js.return
			end
			if not js.return_called then
				l_path.item := links_inst.debug_stracpy (create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("af_unix.c")), 34, default_inst.links_home)
				l_su := create {CE_POINTER [SD_SOCKADDR_UN]}.make_cast ((error_inst.debug_mem_alloc (create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("af_unix.c")), 35, ((create {SD_SOCKADDR_UN}.make).structure_size.to_natural_32 + (strlen (create {CE_POINTER [INTEGER_8]}.make_cast (l_path.item)))) + {NATURAL_32} 1)))
				s_unix_acc := create {CE_POINTER [SD_SOCKADDR]}.make_cast ((error_inst.debug_mem_alloc (create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("af_unix.c")), 36, ((create {SD_SOCKADDR_UN}.make).structure_size.to_natural_32 + (strlen (create {CE_POINTER [INTEGER_8]}.make_cast (l_path.item)))) + {NATURAL_32} 1)))
				memset (create {CE_POINTER [ANY]}.make_cast (l_su), 0, ((create {SD_SOCKADDR_UN}.make).structure_size.to_natural_32 + (strlen (create {CE_POINTER [INTEGER_8]}.make_cast (l_path.item)))) + {NATURAL_32} 1).do_nothing
				l_su.item.sun_family := (1).to_natural_16
				links_inst.add_to_strn (l_path, create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("socket")))
				strcpy ((l_su.item.sun_path), create {CE_POINTER [INTEGER_8]}.make_cast (l_path.item)).do_nothing
				error_inst.debug_mem_free (create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("af_unix.c")), 41, create {CE_POINTER [ANY]}.make_cast (l_path.item))
				s_unix := create {CE_POINTER [SD_SOCKADDR]}.make_cast (l_su)
				s_unix_l := (((create {CE_POINTER [INTEGER_8]}.make_cast ((create {CE_POINTER [CE_ARRAY [INTEGER_8]]}.make_shared (l_su.c_pointer + (create {SD_SOCKADDR_UN}.make).offset_of ("sun_path"))))).distance ((create {CE_POINTER [INTEGER_8]}.make_cast (l_su)))).to_natural_32 + (strlen ((l_su.item.sun_path)))) + {NATURAL_32} 1
				Result := 1
			end
		end

	unlink_unix
		do
			unlink (((create {CE_POINTER [SD_SOCKADDR_UN]}.make_cast (s_unix)).item.sun_path)).do_nothing
		end

end
