class
	PD_NBBUF_STRUCT

inherit
	CE_CLASS
		redefine
			make,
			copy,
			is_equal,
			bitfield_helper
		end

create
	make,
	make_shared

feature {NONE}

	make
		do
			make_memory -- has bitfields; back with memory for correct semantics
		end

feature {ANY} -- fields

	bufp: CE_POINTER [PD_FILE_BUFFER] assign set_bufp

	fireChanges: NATURAL_32 assign set_fireChanges

	initDone: NATURAL_32 assign set_initDone

	insertDone: NATURAL_32 assign set_insertDone

	modified: NATURAL_32 assign set_modified

	nbbuf_number: INTEGER_32 assign set_nbbuf_number

	displayname: CE_POINTER [INTEGER_8] assign set_displayname

	signmap: CE_POINTER [INTEGER_32] assign set_signmap

	signmaplen: NATURAL_16 assign set_signmaplen

	signmapused: NATURAL_16 assign set_signmapused

feature {ANY} -- setters

	set_bufp (a_bufp: CE_POINTER [PD_FILE_BUFFER])
		do
			bufp := a_bufp
			update_memory_field ("bufp")
		end

	set_fireChanges (a_fireChanges: NATURAL_32)
		do
			fireChanges := a_fireChanges
			update_memory_field ("fireChanges")
		end

	set_initDone (a_initDone: NATURAL_32)
		do
			initDone := a_initDone
			update_memory_field ("initDone")
		end

	set_insertDone (a_insertDone: NATURAL_32)
		do
			insertDone := a_insertDone
			update_memory_field ("insertDone")
		end

	set_modified (a_modified: NATURAL_32)
		do
			modified := a_modified
			update_memory_field ("modified")
		end

	set_nbbuf_number (a_nbbuf_number: INTEGER_32)
		do
			nbbuf_number := a_nbbuf_number
			update_memory_field ("nbbuf_number")
		end

	set_displayname (a_displayname: CE_POINTER [INTEGER_8])
		do
			displayname := a_displayname
			update_memory_field ("displayname")
		end

	set_signmap (a_signmap: CE_POINTER [INTEGER_32])
		do
			signmap := a_signmap
			update_memory_field ("signmap")
		end

	set_signmaplen (a_signmaplen: NATURAL_16)
		do
			signmaplen := a_signmaplen
			update_memory_field ("signmaplen")
		end

	set_signmapused (a_signmapused: NATURAL_16)
		do
			signmapused := a_signmapused
			update_memory_field ("signmapused")
		end

feature {ANY}

	copy (a_other: like Current)
		do
			bufp := a_other.bufp
			fireChanges := a_other.fireChanges
			initDone := a_other.initDone
			insertDone := a_other.insertDone
			modified := a_other.modified
			nbbuf_number := a_other.nbbuf_number
			displayname := a_other.displayname
			signmap := a_other.signmap
			signmaplen := a_other.signmaplen
			signmapused := a_other.signmapused
			create_memory_copy -- has bitfields; back with memory for correct semantics
		end

	is_equal (a_other: like Current): BOOLEAN
		do
			Result :=
				bufp.is_equal (a_other.bufp) and
				fireChanges = a_other.fireChanges and
				initDone = a_other.initDone and
				insertDone = a_other.insertDone and
				modified = a_other.modified and
				nbbuf_number = a_other.nbbuf_number and
				displayname.is_equal (a_other.displayname) and
				signmap.is_equal (a_other.signmap) and
				signmaplen = a_other.signmaplen and
				signmapused = a_other.signmapused
		end

feature {NONE}

	structure_info: CE_STRUCTURE
		once ("PROCESS")
			create Result.make_struct (<<
					create {CE_STRUCTURE_FIELD}.make ("bufp"),
					create {CE_STRUCTURE_FIELD}.make_bitfield ("fireChanges"),
					create {CE_STRUCTURE_FIELD}.make_bitfield ("initDone"),
					create {CE_STRUCTURE_FIELD}.make_bitfield ("insertDone"),
					create {CE_STRUCTURE_FIELD}.make_bitfield ("modified"),
					create {CE_STRUCTURE_FIELD}.make ("nbbuf_number"),
					create {CE_STRUCTURE_FIELD}.make ("displayname"),
					create {CE_STRUCTURE_FIELD}.make ("signmap"),
					create {CE_STRUCTURE_FIELD}.make ("signmaplen"),
					create {CE_STRUCTURE_FIELD}.make ("signmapused")
				>>)
			init_structure_info
		end

	bitfield_helper (a_operation: INTEGER_32; a_field_name, a_struct_pointer: POINTER; a_value: NATURAL_64): NATURAL_64
		external
			"C inline"
		alias
			"[
				struct nbbuf_struct {
					struct file_buffer * bufp;
					unsigned int fireChanges : 1;
					unsigned int initDone : 1;
					unsigned int insertDone : 1;
					unsigned int modified : 1;
					int nbbuf_number;
					char * displayname;
					int * signmap;
					unsigned short signmaplen;
					unsigned short signmapused;
				};
				char *field_name = (char *)$a_field_name;
				if($a_operation == 1) {
					if(field_name == (char *)0) return sizeof(struct nbbuf_struct);
					if(strcmp(field_name, "nbbuf_number") == 0) return offsetof(struct nbbuf_struct, nbbuf_number);
				} else {
					struct nbbuf_struct *p = (struct nbbuf_struct *)$a_struct_pointer;
					if($a_operation == 2) {
						     if(strcmp(field_name, "fireChanges") == 0) p->fireChanges = (unsigned int)$a_value;
						else if(strcmp(field_name, "initDone") == 0) p->initDone = (unsigned int)$a_value;
						else if(strcmp(field_name, "insertDone") == 0) p->insertDone = (unsigned int)$a_value;
						else if(strcmp(field_name, "modified") == 0) p->modified = (unsigned int)$a_value;
					} else if($a_operation == 3) {
						if(strcmp(field_name, "fireChanges") == 0) return p->fireChanges;
						if(strcmp(field_name, "initDone") == 0) return p->initDone;
						if(strcmp(field_name, "insertDone") == 0) return p->insertDone;
						if(strcmp(field_name, "modified") == 0) return p->modified;
					}
				}
			]"
		end

end
