class
	P_SHA256

inherit
	VIM_DATA
		redefine
			default_create
		end
	S_LIBIO
	S_STDIO
	S_CTYPE
	S_SYS_STAT
	S_STDLIB
	S_STRING
	S_LIBINTL
	S_WCTYPE
	S_UNISTD
	S_TIME
	S_FCNTL
	S_MATH
	S_SYS_TIME
	S_LOCALE
	S_BITS_ERRNO
	S_ICONV
	S_UTIME
	S_GLIB_OBJECT
	S_GTK_GTK
	S_SYS_SELECT
	S_X11_XLIB
	S_SIGNAL
	S_DIRENT
	S_PWD
	S_GLIB
	S_PANGO_PANGO
	S_GDK_GDK
	S_LANGINFO
	S_X11_XUTIL
	S_X11_INTRINSIC
	S_SYS_IOCTL
	S_SYS_RESOURCE
	S_SYS_WAIT
	S_SYS_UTSNAME
	S_TERMIOS
	S_DLFCN
	S_X11_ICE_ICELIB
	S_X11_SM_SMLIB
	S_X11_SHELL
	S_X11_STRINGDEFS
	S_SYS_SYSINFO
	S_TERMCAP
	S_TERM
	S_GDK_PIXBUF_GDK_PIXBUF
	S_SYS_SOCKET
	S_NETINET_IN
	S_NETDB

feature {NONE} -- Initialization

	default_create
		do
			create hexit.make (<<65>>)
			create random_data.make (<<1_000>>)
			create sha256_padding.make (<<64>>)
			create sha_self_test_msg.make (<<3>>)
			create sha_self_test_vector.make (<<3>>)
		end

feature {ANY} -- attributes

	failures: INTEGER_32 assign set_failures

	set_failures (a_failures: INTEGER_32)
		do
			failures := a_failures
		end

	hexit: CE_ARRAY [NATURAL_8]

	random_data: CE_ARRAY [NATURAL_8]

	sha256_padding: CE_ARRAY [NATURAL_8]

	sha256_self_tested: INTEGER_32 assign set_sha256_self_tested

	set_sha256_self_tested (a_sha256_self_tested: INTEGER_32)
		do
			sha256_self_tested := a_sha256_self_tested
		end

	sha_self_test_msg: CE_ARRAY [CE_POINTER [INTEGER_8]]

	sha_self_test_vector: CE_ARRAY [CE_POINTER [INTEGER_8]]

feature {ANY} -- routines

	get_some_time: NATURAL_32
		local
			l_tv: CE_ARRAY [SD_TIMEVAL]
		do
			create l_tv.make (<<1>>)

			gettimeofday (l_tv, create {CE_POINTER [SD_TIMEZONE]}.make_shared (default_pointer)).do_nothing
			Result := (l_tv.item.tv_sec + l_tv.item.tv_usec).to_natural_32
		end

	sha256_bytes (a_buf___6: CE_POINTER [NATURAL_8]; a_buf_len: INTEGER_32; a_salt: CE_POINTER [NATURAL_8]; a_salt_len: INTEGER_32): CE_POINTER [NATURAL_8]
		local
			l_sha256sum: CE_ARRAY [NATURAL_8]
			l_j: INTEGER_32
			l_ctx: CE_ARRAY [PD_CONTEXT_SHA256_T]
		do
			create l_sha256sum.make (<<32>>)
			create l_ctx.make (<<1>>)

			sha256_self_test.do_nothing
			sha256_start (l_ctx)
			sha256_update (l_ctx, a_buf___6, a_buf_len.to_natural_32)
			if not a_salt.is_void then
				sha256_update (l_ctx, a_salt, a_salt_len.to_natural_32)
			end
			sha256_finish (l_ctx, create {CE_POINTER [NATURAL_8]}.make_cast (l_sha256sum))
			l_j := 0
			from
			until
				not (l_j < 32)
			loop
				sprintf ([(create {CE_POINTER [INTEGER_8]}.make_cast (hexit) + l_j * 2), ce_string ("%%02x"), (l_sha256sum[l_j]).to_integer_32]).do_nothing
				l_j := l_j + 1
			end
			hexit[(({PLATFORM}.Natural_8_bytes.to_natural_32 * (65).to_natural_32) - {NATURAL_32} 1).to_integer_32] := (('%/0/').code).to_natural_8
			Result := hexit
		end

	sha256_finish (a_ctx: CE_POINTER [PD_CONTEXT_SHA256_T]; a_digest: CE_POINTER [NATURAL_8])
		local
			l_last: NATURAL_32
			l_padn: NATURAL_32
			l_high: NATURAL_32
			l_low: NATURAL_32
			l_msglen: CE_ARRAY [NATURAL_8]
		do
			create l_msglen.make (<<8>>)

			l_high := (((a_ctx.item.total[0]).bit_shift_right (29))).bit_or (((a_ctx.item.total[1]).bit_shift_left (3)))
			l_low := (a_ctx.item.total[0]).bit_shift_left (3)
			l_msglen[0] := ((l_high).bit_shift_right (24)).to_natural_8
			l_msglen[1] := ((l_high).bit_shift_right (16)).to_natural_8
			l_msglen[2] := ((l_high).bit_shift_right (8)).to_natural_8
			l_msglen[3] := l_high.to_natural_8
			l_msglen[4] := ((l_low).bit_shift_right (24)).to_natural_8
			l_msglen[5] := ((l_low).bit_shift_right (16)).to_natural_8
			l_msglen[6] := ((l_low).bit_shift_right (8)).to_natural_8
			l_msglen[7] := l_low.to_natural_8
			l_last := (a_ctx.item.total[0]).bit_and ({NATURAL_32} 63)
			if l_last < {NATURAL_32} 56 then
				l_padn := {NATURAL_32} 56 - l_last
			else
				l_padn := {NATURAL_32} 120 - l_last
			end
			sha256_update (a_ctx, sha256_padding, l_padn)
			sha256_update (a_ctx, l_msglen, (8).to_natural_32)
			a_digest[0] := ((a_ctx.item.state[0]).bit_shift_right (24)).to_natural_8
			a_digest[1] := ((a_ctx.item.state[0]).bit_shift_right (16)).to_natural_8
			a_digest[2] := ((a_ctx.item.state[0]).bit_shift_right (8)).to_natural_8
			a_digest[3] := (a_ctx.item.state[0]).to_natural_8
			a_digest[4] := ((a_ctx.item.state[1]).bit_shift_right (24)).to_natural_8
			a_digest[5] := ((a_ctx.item.state[1]).bit_shift_right (16)).to_natural_8
			a_digest[6] := ((a_ctx.item.state[1]).bit_shift_right (8)).to_natural_8
			a_digest[7] := (a_ctx.item.state[1]).to_natural_8
			a_digest[8] := ((a_ctx.item.state[2]).bit_shift_right (24)).to_natural_8
			a_digest[9] := ((a_ctx.item.state[2]).bit_shift_right (16)).to_natural_8
			a_digest[10] := ((a_ctx.item.state[2]).bit_shift_right (8)).to_natural_8
			a_digest[11] := (a_ctx.item.state[2]).to_natural_8
			a_digest[12] := ((a_ctx.item.state[3]).bit_shift_right (24)).to_natural_8
			a_digest[13] := ((a_ctx.item.state[3]).bit_shift_right (16)).to_natural_8
			a_digest[14] := ((a_ctx.item.state[3]).bit_shift_right (8)).to_natural_8
			a_digest[15] := (a_ctx.item.state[3]).to_natural_8
			a_digest[16] := ((a_ctx.item.state[4]).bit_shift_right (24)).to_natural_8
			a_digest[17] := ((a_ctx.item.state[4]).bit_shift_right (16)).to_natural_8
			a_digest[18] := ((a_ctx.item.state[4]).bit_shift_right (8)).to_natural_8
			a_digest[19] := (a_ctx.item.state[4]).to_natural_8
			a_digest[20] := ((a_ctx.item.state[5]).bit_shift_right (24)).to_natural_8
			a_digest[21] := ((a_ctx.item.state[5]).bit_shift_right (16)).to_natural_8
			a_digest[22] := ((a_ctx.item.state[5]).bit_shift_right (8)).to_natural_8
			a_digest[23] := (a_ctx.item.state[5]).to_natural_8
			a_digest[24] := ((a_ctx.item.state[6]).bit_shift_right (24)).to_natural_8
			a_digest[25] := ((a_ctx.item.state[6]).bit_shift_right (16)).to_natural_8
			a_digest[26] := ((a_ctx.item.state[6]).bit_shift_right (8)).to_natural_8
			a_digest[27] := (a_ctx.item.state[6]).to_natural_8
			a_digest[28] := ((a_ctx.item.state[7]).bit_shift_right (24)).to_natural_8
			a_digest[29] := ((a_ctx.item.state[7]).bit_shift_right (16)).to_natural_8
			a_digest[30] := ((a_ctx.item.state[7]).bit_shift_right (8)).to_natural_8
			a_digest[31] := (a_ctx.item.state[7]).to_natural_8
		end

	sha256_key (a_buf___6: CE_POINTER [NATURAL_8]; a_salt: CE_POINTER [NATURAL_8]; a_salt_len: INTEGER_32): CE_POINTER [NATURAL_8]
		local
			js: JUMP_STATE
		do
			if a_buf___6.is_void then
				Result := (create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("")))
				js.return
			elseif (a_buf___6.item).to_integer_32 = 0 then
				Result := (create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("")))
				js.return
			end
			if not js.return_called then
				Result := (sha256_bytes (a_buf___6, (strlen (create {CE_POINTER [INTEGER_8]}.make_cast (a_buf___6))).to_integer_32, a_salt, a_salt_len))
			end
		end

	sha256_process (a_ctx: CE_POINTER [PD_CONTEXT_SHA256_T]; a_data: CE_POINTER [NATURAL_8])
		local
			l_temp1: NATURAL_32
			l_temp2: NATURAL_32
			l_W: CE_ARRAY [NATURAL_32]
			l_A: NATURAL_32
			l_B: NATURAL_32
			l_C: NATURAL_32
			l_D: NATURAL_32
			l_E: NATURAL_32
			l_F: NATURAL_32
			l_G: NATURAL_32
			l_H: NATURAL_32
		do
			create l_W.make (<<64>>)

			l_W[0] := ((((((((a_data[0]).to_natural_32).bit_shift_left (24))).bit_or ((((a_data[1]).to_natural_32).bit_shift_left (16))))).bit_or ((((a_data[2]).to_natural_32).bit_shift_left (8))))).bit_or ((a_data[3]).to_natural_32)
			l_W[1] := ((((((((a_data[4]).to_natural_32).bit_shift_left (24))).bit_or ((((a_data[5]).to_natural_32).bit_shift_left (16))))).bit_or ((((a_data[6]).to_natural_32).bit_shift_left (8))))).bit_or ((a_data[7]).to_natural_32)
			l_W[2] := ((((((((a_data[8]).to_natural_32).bit_shift_left (24))).bit_or ((((a_data[9]).to_natural_32).bit_shift_left (16))))).bit_or ((((a_data[10]).to_natural_32).bit_shift_left (8))))).bit_or ((a_data[11]).to_natural_32)
			l_W[3] := ((((((((a_data[12]).to_natural_32).bit_shift_left (24))).bit_or ((((a_data[13]).to_natural_32).bit_shift_left (16))))).bit_or ((((a_data[14]).to_natural_32).bit_shift_left (8))))).bit_or ((a_data[15]).to_natural_32)
			l_W[4] := ((((((((a_data[16]).to_natural_32).bit_shift_left (24))).bit_or ((((a_data[17]).to_natural_32).bit_shift_left (16))))).bit_or ((((a_data[18]).to_natural_32).bit_shift_left (8))))).bit_or ((a_data[19]).to_natural_32)
			l_W[5] := ((((((((a_data[20]).to_natural_32).bit_shift_left (24))).bit_or ((((a_data[21]).to_natural_32).bit_shift_left (16))))).bit_or ((((a_data[22]).to_natural_32).bit_shift_left (8))))).bit_or ((a_data[23]).to_natural_32)
			l_W[6] := ((((((((a_data[24]).to_natural_32).bit_shift_left (24))).bit_or ((((a_data[25]).to_natural_32).bit_shift_left (16))))).bit_or ((((a_data[26]).to_natural_32).bit_shift_left (8))))).bit_or ((a_data[27]).to_natural_32)
			l_W[7] := ((((((((a_data[28]).to_natural_32).bit_shift_left (24))).bit_or ((((a_data[29]).to_natural_32).bit_shift_left (16))))).bit_or ((((a_data[30]).to_natural_32).bit_shift_left (8))))).bit_or ((a_data[31]).to_natural_32)
			l_W[8] := ((((((((a_data[32]).to_natural_32).bit_shift_left (24))).bit_or ((((a_data[33]).to_natural_32).bit_shift_left (16))))).bit_or ((((a_data[34]).to_natural_32).bit_shift_left (8))))).bit_or ((a_data[35]).to_natural_32)
			l_W[9] := ((((((((a_data[36]).to_natural_32).bit_shift_left (24))).bit_or ((((a_data[37]).to_natural_32).bit_shift_left (16))))).bit_or ((((a_data[38]).to_natural_32).bit_shift_left (8))))).bit_or ((a_data[39]).to_natural_32)
			l_W[10] := ((((((((a_data[40]).to_natural_32).bit_shift_left (24))).bit_or ((((a_data[41]).to_natural_32).bit_shift_left (16))))).bit_or ((((a_data[42]).to_natural_32).bit_shift_left (8))))).bit_or ((a_data[43]).to_natural_32)
			l_W[11] := ((((((((a_data[44]).to_natural_32).bit_shift_left (24))).bit_or ((((a_data[45]).to_natural_32).bit_shift_left (16))))).bit_or ((((a_data[46]).to_natural_32).bit_shift_left (8))))).bit_or ((a_data[47]).to_natural_32)
			l_W[12] := ((((((((a_data[48]).to_natural_32).bit_shift_left (24))).bit_or ((((a_data[49]).to_natural_32).bit_shift_left (16))))).bit_or ((((a_data[50]).to_natural_32).bit_shift_left (8))))).bit_or ((a_data[51]).to_natural_32)
			l_W[13] := ((((((((a_data[52]).to_natural_32).bit_shift_left (24))).bit_or ((((a_data[53]).to_natural_32).bit_shift_left (16))))).bit_or ((((a_data[54]).to_natural_32).bit_shift_left (8))))).bit_or ((a_data[55]).to_natural_32)
			l_W[14] := ((((((((a_data[56]).to_natural_32).bit_shift_left (24))).bit_or ((((a_data[57]).to_natural_32).bit_shift_left (16))))).bit_or ((((a_data[58]).to_natural_32).bit_shift_left (8))))).bit_or ((a_data[59]).to_natural_32)
			l_W[15] := ((((((((a_data[60]).to_natural_32).bit_shift_left (24))).bit_or ((((a_data[61]).to_natural_32).bit_shift_left (16))))).bit_or ((((a_data[62]).to_natural_32).bit_shift_left (8))))).bit_or ((a_data[63]).to_natural_32)
			l_A := a_ctx.item.state[0]
			l_B := a_ctx.item.state[1]
			l_C := a_ctx.item.state[2]
			l_D := a_ctx.item.state[3]
			l_E := a_ctx.item.state[4]
			l_F := a_ctx.item.state[5]
			l_G := a_ctx.item.state[6]
			l_H := a_ctx.item.state[7]
			l_temp1 := (((l_H + ((((((((((l_E).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_E).bit_shift_left (26))))).bit_xor (((((((l_E).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_E).bit_shift_left (21))))))).bit_xor (((((((l_E).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_E).bit_shift_left (7))))))) + ((l_G).bit_xor (((l_E).bit_and (((l_F).bit_xor (l_G))))))) + {NATURAL_32} 1_116_352_408) + l_W[0]
			l_temp2 := ((((((((((l_A).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_A).bit_shift_left (30))))).bit_xor (((((((l_A).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_A).bit_shift_left (19))))))).bit_xor (((((((l_A).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_A).bit_shift_left (10)))))) + ((((l_A).bit_and (l_B))).bit_or (((l_C).bit_and (((l_A).bit_or (l_B))))))
			l_D := l_D + l_temp1
			l_H := l_temp1 + l_temp2
			l_temp1 := (((l_G + ((((((((((l_D).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_D).bit_shift_left (26))))).bit_xor (((((((l_D).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_D).bit_shift_left (21))))))).bit_xor (((((((l_D).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_D).bit_shift_left (7))))))) + ((l_F).bit_xor (((l_D).bit_and (((l_E).bit_xor (l_F))))))) + {NATURAL_32} 1_899_447_441) + l_W[1]
			l_temp2 := ((((((((((l_H).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_H).bit_shift_left (30))))).bit_xor (((((((l_H).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_H).bit_shift_left (19))))))).bit_xor (((((((l_H).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_H).bit_shift_left (10)))))) + ((((l_H).bit_and (l_A))).bit_or (((l_B).bit_and (((l_H).bit_or (l_A))))))
			l_C := l_C + l_temp1
			l_G := l_temp1 + l_temp2
			l_temp1 := (((l_F + ((((((((((l_C).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_C).bit_shift_left (26))))).bit_xor (((((((l_C).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_C).bit_shift_left (21))))))).bit_xor (((((((l_C).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_C).bit_shift_left (7))))))) + ((l_E).bit_xor (((l_C).bit_and (((l_D).bit_xor (l_E))))))) + {NATURAL_32} 3_049_323_471) + l_W[2]
			l_temp2 := ((((((((((l_G).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_G).bit_shift_left (30))))).bit_xor (((((((l_G).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_G).bit_shift_left (19))))))).bit_xor (((((((l_G).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_G).bit_shift_left (10)))))) + ((((l_G).bit_and (l_H))).bit_or (((l_A).bit_and (((l_G).bit_or (l_H))))))
			l_B := l_B + l_temp1
			l_F := l_temp1 + l_temp2
			l_temp1 := (((l_E + ((((((((((l_B).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_B).bit_shift_left (26))))).bit_xor (((((((l_B).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_B).bit_shift_left (21))))))).bit_xor (((((((l_B).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_B).bit_shift_left (7))))))) + ((l_D).bit_xor (((l_B).bit_and (((l_C).bit_xor (l_D))))))) + {NATURAL_32} 3_921_009_573) + l_W[3]
			l_temp2 := ((((((((((l_F).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_F).bit_shift_left (30))))).bit_xor (((((((l_F).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_F).bit_shift_left (19))))))).bit_xor (((((((l_F).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_F).bit_shift_left (10)))))) + ((((l_F).bit_and (l_G))).bit_or (((l_H).bit_and (((l_F).bit_or (l_G))))))
			l_A := l_A + l_temp1
			l_E := l_temp1 + l_temp2
			l_temp1 := (((l_D + ((((((((((l_A).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_A).bit_shift_left (26))))).bit_xor (((((((l_A).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_A).bit_shift_left (21))))))).bit_xor (((((((l_A).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_A).bit_shift_left (7))))))) + ((l_C).bit_xor (((l_A).bit_and (((l_B).bit_xor (l_C))))))) + {NATURAL_32} 961_987_163) + l_W[4]
			l_temp2 := ((((((((((l_E).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_E).bit_shift_left (30))))).bit_xor (((((((l_E).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_E).bit_shift_left (19))))))).bit_xor (((((((l_E).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_E).bit_shift_left (10)))))) + ((((l_E).bit_and (l_F))).bit_or (((l_G).bit_and (((l_E).bit_or (l_F))))))
			l_H := l_H + l_temp1
			l_D := l_temp1 + l_temp2
			l_temp1 := (((l_C + ((((((((((l_H).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_H).bit_shift_left (26))))).bit_xor (((((((l_H).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_H).bit_shift_left (21))))))).bit_xor (((((((l_H).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_H).bit_shift_left (7))))))) + ((l_B).bit_xor (((l_H).bit_and (((l_A).bit_xor (l_B))))))) + {NATURAL_32} 1_508_970_993) + l_W[5]
			l_temp2 := ((((((((((l_D).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_D).bit_shift_left (30))))).bit_xor (((((((l_D).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_D).bit_shift_left (19))))))).bit_xor (((((((l_D).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_D).bit_shift_left (10)))))) + ((((l_D).bit_and (l_E))).bit_or (((l_F).bit_and (((l_D).bit_or (l_E))))))
			l_G := l_G + l_temp1
			l_C := l_temp1 + l_temp2
			l_temp1 := (((l_B + ((((((((((l_G).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_G).bit_shift_left (26))))).bit_xor (((((((l_G).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_G).bit_shift_left (21))))))).bit_xor (((((((l_G).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_G).bit_shift_left (7))))))) + ((l_A).bit_xor (((l_G).bit_and (((l_H).bit_xor (l_A))))))) + {NATURAL_32} 2_453_635_748) + l_W[6]
			l_temp2 := ((((((((((l_C).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_C).bit_shift_left (30))))).bit_xor (((((((l_C).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_C).bit_shift_left (19))))))).bit_xor (((((((l_C).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_C).bit_shift_left (10)))))) + ((((l_C).bit_and (l_D))).bit_or (((l_E).bit_and (((l_C).bit_or (l_D))))))
			l_F := l_F + l_temp1
			l_B := l_temp1 + l_temp2
			l_temp1 := (((l_A + ((((((((((l_F).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_F).bit_shift_left (26))))).bit_xor (((((((l_F).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_F).bit_shift_left (21))))))).bit_xor (((((((l_F).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_F).bit_shift_left (7))))))) + ((l_H).bit_xor (((l_F).bit_and (((l_G).bit_xor (l_H))))))) + {NATURAL_32} 2_870_763_221) + l_W[7]
			l_temp2 := ((((((((((l_B).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_B).bit_shift_left (30))))).bit_xor (((((((l_B).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_B).bit_shift_left (19))))))).bit_xor (((((((l_B).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_B).bit_shift_left (10)))))) + ((((l_B).bit_and (l_C))).bit_or (((l_D).bit_and (((l_B).bit_or (l_C))))))
			l_E := l_E + l_temp1
			l_A := l_temp1 + l_temp2
			l_temp1 := (((l_H + ((((((((((l_E).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_E).bit_shift_left (26))))).bit_xor (((((((l_E).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_E).bit_shift_left (21))))))).bit_xor (((((((l_E).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_E).bit_shift_left (7))))))) + ((l_G).bit_xor (((l_E).bit_and (((l_F).bit_xor (l_G))))))) + {NATURAL_32} 3_624_381_080) + l_W[8]
			l_temp2 := ((((((((((l_A).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_A).bit_shift_left (30))))).bit_xor (((((((l_A).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_A).bit_shift_left (19))))))).bit_xor (((((((l_A).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_A).bit_shift_left (10)))))) + ((((l_A).bit_and (l_B))).bit_or (((l_C).bit_and (((l_A).bit_or (l_B))))))
			l_D := l_D + l_temp1
			l_H := l_temp1 + l_temp2
			l_temp1 := (((l_G + ((((((((((l_D).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_D).bit_shift_left (26))))).bit_xor (((((((l_D).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_D).bit_shift_left (21))))))).bit_xor (((((((l_D).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_D).bit_shift_left (7))))))) + ((l_F).bit_xor (((l_D).bit_and (((l_E).bit_xor (l_F))))))) + {NATURAL_32} 310_598_401) + l_W[9]
			l_temp2 := ((((((((((l_H).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_H).bit_shift_left (30))))).bit_xor (((((((l_H).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_H).bit_shift_left (19))))))).bit_xor (((((((l_H).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_H).bit_shift_left (10)))))) + ((((l_H).bit_and (l_A))).bit_or (((l_B).bit_and (((l_H).bit_or (l_A))))))
			l_C := l_C + l_temp1
			l_G := l_temp1 + l_temp2
			l_temp1 := (((l_F + ((((((((((l_C).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_C).bit_shift_left (26))))).bit_xor (((((((l_C).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_C).bit_shift_left (21))))))).bit_xor (((((((l_C).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_C).bit_shift_left (7))))))) + ((l_E).bit_xor (((l_C).bit_and (((l_D).bit_xor (l_E))))))) + {NATURAL_32} 607_225_278) + l_W[10]
			l_temp2 := ((((((((((l_G).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_G).bit_shift_left (30))))).bit_xor (((((((l_G).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_G).bit_shift_left (19))))))).bit_xor (((((((l_G).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_G).bit_shift_left (10)))))) + ((((l_G).bit_and (l_H))).bit_or (((l_A).bit_and (((l_G).bit_or (l_H))))))
			l_B := l_B + l_temp1
			l_F := l_temp1 + l_temp2
			l_temp1 := (((l_E + ((((((((((l_B).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_B).bit_shift_left (26))))).bit_xor (((((((l_B).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_B).bit_shift_left (21))))))).bit_xor (((((((l_B).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_B).bit_shift_left (7))))))) + ((l_D).bit_xor (((l_B).bit_and (((l_C).bit_xor (l_D))))))) + {NATURAL_32} 1_426_881_987) + l_W[11]
			l_temp2 := ((((((((((l_F).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_F).bit_shift_left (30))))).bit_xor (((((((l_F).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_F).bit_shift_left (19))))))).bit_xor (((((((l_F).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_F).bit_shift_left (10)))))) + ((((l_F).bit_and (l_G))).bit_or (((l_H).bit_and (((l_F).bit_or (l_G))))))
			l_A := l_A + l_temp1
			l_E := l_temp1 + l_temp2
			l_temp1 := (((l_D + ((((((((((l_A).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_A).bit_shift_left (26))))).bit_xor (((((((l_A).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_A).bit_shift_left (21))))))).bit_xor (((((((l_A).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_A).bit_shift_left (7))))))) + ((l_C).bit_xor (((l_A).bit_and (((l_B).bit_xor (l_C))))))) + {NATURAL_32} 1_925_078_388) + l_W[12]
			l_temp2 := ((((((((((l_E).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_E).bit_shift_left (30))))).bit_xor (((((((l_E).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_E).bit_shift_left (19))))))).bit_xor (((((((l_E).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_E).bit_shift_left (10)))))) + ((((l_E).bit_and (l_F))).bit_or (((l_G).bit_and (((l_E).bit_or (l_F))))))
			l_H := l_H + l_temp1
			l_D := l_temp1 + l_temp2
			l_temp1 := (((l_C + ((((((((((l_H).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_H).bit_shift_left (26))))).bit_xor (((((((l_H).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_H).bit_shift_left (21))))))).bit_xor (((((((l_H).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_H).bit_shift_left (7))))))) + ((l_B).bit_xor (((l_H).bit_and (((l_A).bit_xor (l_B))))))) + {NATURAL_32} 2_162_078_206) + l_W[13]
			l_temp2 := ((((((((((l_D).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_D).bit_shift_left (30))))).bit_xor (((((((l_D).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_D).bit_shift_left (19))))))).bit_xor (((((((l_D).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_D).bit_shift_left (10)))))) + ((((l_D).bit_and (l_E))).bit_or (((l_F).bit_and (((l_D).bit_or (l_E))))))
			l_G := l_G + l_temp1
			l_C := l_temp1 + l_temp2
			l_temp1 := (((l_B + ((((((((((l_G).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_G).bit_shift_left (26))))).bit_xor (((((((l_G).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_G).bit_shift_left (21))))))).bit_xor (((((((l_G).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_G).bit_shift_left (7))))))) + ((l_A).bit_xor (((l_G).bit_and (((l_H).bit_xor (l_A))))))) + {NATURAL_32} 2_614_888_103) + l_W[14]
			l_temp2 := ((((((((((l_C).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_C).bit_shift_left (30))))).bit_xor (((((((l_C).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_C).bit_shift_left (19))))))).bit_xor (((((((l_C).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_C).bit_shift_left (10)))))) + ((((l_C).bit_and (l_D))).bit_or (((l_E).bit_and (((l_C).bit_or (l_D))))))
			l_F := l_F + l_temp1
			l_B := l_temp1 + l_temp2
			l_temp1 := (((l_A + ((((((((((l_F).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_F).bit_shift_left (26))))).bit_xor (((((((l_F).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_F).bit_shift_left (21))))))).bit_xor (((((((l_F).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_F).bit_shift_left (7))))))) + ((l_H).bit_xor (((l_F).bit_and (((l_G).bit_xor (l_H))))))) + {NATURAL_32} 3_248_222_580) + l_W[15]
			l_temp2 := ((((((((((l_B).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_B).bit_shift_left (30))))).bit_xor (((((((l_B).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_B).bit_shift_left (19))))))).bit_xor (((((((l_B).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_B).bit_shift_left (10)))))) + ((((l_B).bit_and (l_C))).bit_or (((l_D).bit_and (((l_B).bit_or (l_C))))))
			l_E := l_E + l_temp1
			l_A := l_temp1 + l_temp2
			l_W[16] := ((((((((((((l_W[14]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (17))).bit_or (((l_W[14]).bit_shift_left (15))))).bit_xor (((((((l_W[14]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (19))).bit_or (((l_W[14]).bit_shift_left (13))))))).bit_xor (((((l_W[14]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (10)))) + l_W[9]) + ((((((((((l_W[1]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (7))).bit_or (((l_W[1]).bit_shift_left (25))))).bit_xor (((((((l_W[1]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (18))).bit_or (((l_W[1]).bit_shift_left (14))))))).bit_xor (((((l_W[1]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (3))))) + l_W[0]
			l_temp1 := (((l_H + ((((((((((l_E).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_E).bit_shift_left (26))))).bit_xor (((((((l_E).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_E).bit_shift_left (21))))))).bit_xor (((((((l_E).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_E).bit_shift_left (7))))))) + ((l_G).bit_xor (((l_E).bit_and (((l_F).bit_xor (l_G))))))) + {NATURAL_32} 3_835_390_401) + l_W[16]
			l_temp2 := ((((((((((l_A).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_A).bit_shift_left (30))))).bit_xor (((((((l_A).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_A).bit_shift_left (19))))))).bit_xor (((((((l_A).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_A).bit_shift_left (10)))))) + ((((l_A).bit_and (l_B))).bit_or (((l_C).bit_and (((l_A).bit_or (l_B))))))
			l_D := l_D + l_temp1
			l_H := l_temp1 + l_temp2
			l_W[17] := ((((((((((((l_W[15]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (17))).bit_or (((l_W[15]).bit_shift_left (15))))).bit_xor (((((((l_W[15]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (19))).bit_or (((l_W[15]).bit_shift_left (13))))))).bit_xor (((((l_W[15]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (10)))) + l_W[10]) + ((((((((((l_W[2]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (7))).bit_or (((l_W[2]).bit_shift_left (25))))).bit_xor (((((((l_W[2]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (18))).bit_or (((l_W[2]).bit_shift_left (14))))))).bit_xor (((((l_W[2]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (3))))) + l_W[1]
			l_temp1 := (((l_G + ((((((((((l_D).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_D).bit_shift_left (26))))).bit_xor (((((((l_D).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_D).bit_shift_left (21))))))).bit_xor (((((((l_D).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_D).bit_shift_left (7))))))) + ((l_F).bit_xor (((l_D).bit_and (((l_E).bit_xor (l_F))))))) + {NATURAL_32} 4_022_224_774) + l_W[17]
			l_temp2 := ((((((((((l_H).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_H).bit_shift_left (30))))).bit_xor (((((((l_H).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_H).bit_shift_left (19))))))).bit_xor (((((((l_H).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_H).bit_shift_left (10)))))) + ((((l_H).bit_and (l_A))).bit_or (((l_B).bit_and (((l_H).bit_or (l_A))))))
			l_C := l_C + l_temp1
			l_G := l_temp1 + l_temp2
			l_W[18] := ((((((((((((l_W[16]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (17))).bit_or (((l_W[16]).bit_shift_left (15))))).bit_xor (((((((l_W[16]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (19))).bit_or (((l_W[16]).bit_shift_left (13))))))).bit_xor (((((l_W[16]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (10)))) + l_W[11]) + ((((((((((l_W[3]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (7))).bit_or (((l_W[3]).bit_shift_left (25))))).bit_xor (((((((l_W[3]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (18))).bit_or (((l_W[3]).bit_shift_left (14))))))).bit_xor (((((l_W[3]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (3))))) + l_W[2]
			l_temp1 := (((l_F + ((((((((((l_C).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_C).bit_shift_left (26))))).bit_xor (((((((l_C).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_C).bit_shift_left (21))))))).bit_xor (((((((l_C).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_C).bit_shift_left (7))))))) + ((l_E).bit_xor (((l_C).bit_and (((l_D).bit_xor (l_E))))))) + {NATURAL_32} 264_347_078) + l_W[18]
			l_temp2 := ((((((((((l_G).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_G).bit_shift_left (30))))).bit_xor (((((((l_G).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_G).bit_shift_left (19))))))).bit_xor (((((((l_G).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_G).bit_shift_left (10)))))) + ((((l_G).bit_and (l_H))).bit_or (((l_A).bit_and (((l_G).bit_or (l_H))))))
			l_B := l_B + l_temp1
			l_F := l_temp1 + l_temp2
			l_W[19] := ((((((((((((l_W[17]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (17))).bit_or (((l_W[17]).bit_shift_left (15))))).bit_xor (((((((l_W[17]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (19))).bit_or (((l_W[17]).bit_shift_left (13))))))).bit_xor (((((l_W[17]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (10)))) + l_W[12]) + ((((((((((l_W[4]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (7))).bit_or (((l_W[4]).bit_shift_left (25))))).bit_xor (((((((l_W[4]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (18))).bit_or (((l_W[4]).bit_shift_left (14))))))).bit_xor (((((l_W[4]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (3))))) + l_W[3]
			l_temp1 := (((l_E + ((((((((((l_B).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_B).bit_shift_left (26))))).bit_xor (((((((l_B).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_B).bit_shift_left (21))))))).bit_xor (((((((l_B).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_B).bit_shift_left (7))))))) + ((l_D).bit_xor (((l_B).bit_and (((l_C).bit_xor (l_D))))))) + {NATURAL_32} 604_807_628) + l_W[19]
			l_temp2 := ((((((((((l_F).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_F).bit_shift_left (30))))).bit_xor (((((((l_F).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_F).bit_shift_left (19))))))).bit_xor (((((((l_F).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_F).bit_shift_left (10)))))) + ((((l_F).bit_and (l_G))).bit_or (((l_H).bit_and (((l_F).bit_or (l_G))))))
			l_A := l_A + l_temp1
			l_E := l_temp1 + l_temp2
			l_W[20] := ((((((((((((l_W[18]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (17))).bit_or (((l_W[18]).bit_shift_left (15))))).bit_xor (((((((l_W[18]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (19))).bit_or (((l_W[18]).bit_shift_left (13))))))).bit_xor (((((l_W[18]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (10)))) + l_W[13]) + ((((((((((l_W[5]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (7))).bit_or (((l_W[5]).bit_shift_left (25))))).bit_xor (((((((l_W[5]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (18))).bit_or (((l_W[5]).bit_shift_left (14))))))).bit_xor (((((l_W[5]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (3))))) + l_W[4]
			l_temp1 := (((l_D + ((((((((((l_A).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_A).bit_shift_left (26))))).bit_xor (((((((l_A).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_A).bit_shift_left (21))))))).bit_xor (((((((l_A).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_A).bit_shift_left (7))))))) + ((l_C).bit_xor (((l_A).bit_and (((l_B).bit_xor (l_C))))))) + {NATURAL_32} 770_255_983) + l_W[20]
			l_temp2 := ((((((((((l_E).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_E).bit_shift_left (30))))).bit_xor (((((((l_E).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_E).bit_shift_left (19))))))).bit_xor (((((((l_E).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_E).bit_shift_left (10)))))) + ((((l_E).bit_and (l_F))).bit_or (((l_G).bit_and (((l_E).bit_or (l_F))))))
			l_H := l_H + l_temp1
			l_D := l_temp1 + l_temp2
			l_W[21] := ((((((((((((l_W[19]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (17))).bit_or (((l_W[19]).bit_shift_left (15))))).bit_xor (((((((l_W[19]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (19))).bit_or (((l_W[19]).bit_shift_left (13))))))).bit_xor (((((l_W[19]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (10)))) + l_W[14]) + ((((((((((l_W[6]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (7))).bit_or (((l_W[6]).bit_shift_left (25))))).bit_xor (((((((l_W[6]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (18))).bit_or (((l_W[6]).bit_shift_left (14))))))).bit_xor (((((l_W[6]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (3))))) + l_W[5]
			l_temp1 := (((l_C + ((((((((((l_H).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_H).bit_shift_left (26))))).bit_xor (((((((l_H).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_H).bit_shift_left (21))))))).bit_xor (((((((l_H).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_H).bit_shift_left (7))))))) + ((l_B).bit_xor (((l_H).bit_and (((l_A).bit_xor (l_B))))))) + {NATURAL_32} 1_249_150_122) + l_W[21]
			l_temp2 := ((((((((((l_D).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_D).bit_shift_left (30))))).bit_xor (((((((l_D).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_D).bit_shift_left (19))))))).bit_xor (((((((l_D).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_D).bit_shift_left (10)))))) + ((((l_D).bit_and (l_E))).bit_or (((l_F).bit_and (((l_D).bit_or (l_E))))))
			l_G := l_G + l_temp1
			l_C := l_temp1 + l_temp2
			l_W[22] := ((((((((((((l_W[20]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (17))).bit_or (((l_W[20]).bit_shift_left (15))))).bit_xor (((((((l_W[20]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (19))).bit_or (((l_W[20]).bit_shift_left (13))))))).bit_xor (((((l_W[20]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (10)))) + l_W[15]) + ((((((((((l_W[7]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (7))).bit_or (((l_W[7]).bit_shift_left (25))))).bit_xor (((((((l_W[7]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (18))).bit_or (((l_W[7]).bit_shift_left (14))))))).bit_xor (((((l_W[7]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (3))))) + l_W[6]
			l_temp1 := (((l_B + ((((((((((l_G).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_G).bit_shift_left (26))))).bit_xor (((((((l_G).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_G).bit_shift_left (21))))))).bit_xor (((((((l_G).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_G).bit_shift_left (7))))))) + ((l_A).bit_xor (((l_G).bit_and (((l_H).bit_xor (l_A))))))) + {NATURAL_32} 1_555_081_692) + l_W[22]
			l_temp2 := ((((((((((l_C).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_C).bit_shift_left (30))))).bit_xor (((((((l_C).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_C).bit_shift_left (19))))))).bit_xor (((((((l_C).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_C).bit_shift_left (10)))))) + ((((l_C).bit_and (l_D))).bit_or (((l_E).bit_and (((l_C).bit_or (l_D))))))
			l_F := l_F + l_temp1
			l_B := l_temp1 + l_temp2
			l_W[23] := ((((((((((((l_W[21]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (17))).bit_or (((l_W[21]).bit_shift_left (15))))).bit_xor (((((((l_W[21]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (19))).bit_or (((l_W[21]).bit_shift_left (13))))))).bit_xor (((((l_W[21]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (10)))) + l_W[16]) + ((((((((((l_W[8]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (7))).bit_or (((l_W[8]).bit_shift_left (25))))).bit_xor (((((((l_W[8]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (18))).bit_or (((l_W[8]).bit_shift_left (14))))))).bit_xor (((((l_W[8]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (3))))) + l_W[7]
			l_temp1 := (((l_A + ((((((((((l_F).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_F).bit_shift_left (26))))).bit_xor (((((((l_F).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_F).bit_shift_left (21))))))).bit_xor (((((((l_F).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_F).bit_shift_left (7))))))) + ((l_H).bit_xor (((l_F).bit_and (((l_G).bit_xor (l_H))))))) + {NATURAL_32} 1_996_064_986) + l_W[23]
			l_temp2 := ((((((((((l_B).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_B).bit_shift_left (30))))).bit_xor (((((((l_B).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_B).bit_shift_left (19))))))).bit_xor (((((((l_B).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_B).bit_shift_left (10)))))) + ((((l_B).bit_and (l_C))).bit_or (((l_D).bit_and (((l_B).bit_or (l_C))))))
			l_E := l_E + l_temp1
			l_A := l_temp1 + l_temp2
			l_W[24] := ((((((((((((l_W[22]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (17))).bit_or (((l_W[22]).bit_shift_left (15))))).bit_xor (((((((l_W[22]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (19))).bit_or (((l_W[22]).bit_shift_left (13))))))).bit_xor (((((l_W[22]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (10)))) + l_W[17]) + ((((((((((l_W[9]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (7))).bit_or (((l_W[9]).bit_shift_left (25))))).bit_xor (((((((l_W[9]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (18))).bit_or (((l_W[9]).bit_shift_left (14))))))).bit_xor (((((l_W[9]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (3))))) + l_W[8]
			l_temp1 := (((l_H + ((((((((((l_E).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_E).bit_shift_left (26))))).bit_xor (((((((l_E).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_E).bit_shift_left (21))))))).bit_xor (((((((l_E).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_E).bit_shift_left (7))))))) + ((l_G).bit_xor (((l_E).bit_and (((l_F).bit_xor (l_G))))))) + {NATURAL_32} 2_554_220_882) + l_W[24]
			l_temp2 := ((((((((((l_A).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_A).bit_shift_left (30))))).bit_xor (((((((l_A).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_A).bit_shift_left (19))))))).bit_xor (((((((l_A).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_A).bit_shift_left (10)))))) + ((((l_A).bit_and (l_B))).bit_or (((l_C).bit_and (((l_A).bit_or (l_B))))))
			l_D := l_D + l_temp1
			l_H := l_temp1 + l_temp2
			l_W[25] := ((((((((((((l_W[23]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (17))).bit_or (((l_W[23]).bit_shift_left (15))))).bit_xor (((((((l_W[23]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (19))).bit_or (((l_W[23]).bit_shift_left (13))))))).bit_xor (((((l_W[23]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (10)))) + l_W[18]) + ((((((((((l_W[10]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (7))).bit_or (((l_W[10]).bit_shift_left (25))))).bit_xor (((((((l_W[10]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (18))).bit_or (((l_W[10]).bit_shift_left (14))))))).bit_xor (((((l_W[10]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (3))))) + l_W[9]
			l_temp1 := (((l_G + ((((((((((l_D).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_D).bit_shift_left (26))))).bit_xor (((((((l_D).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_D).bit_shift_left (21))))))).bit_xor (((((((l_D).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_D).bit_shift_left (7))))))) + ((l_F).bit_xor (((l_D).bit_and (((l_E).bit_xor (l_F))))))) + {NATURAL_32} 2_821_834_349) + l_W[25]
			l_temp2 := ((((((((((l_H).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_H).bit_shift_left (30))))).bit_xor (((((((l_H).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_H).bit_shift_left (19))))))).bit_xor (((((((l_H).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_H).bit_shift_left (10)))))) + ((((l_H).bit_and (l_A))).bit_or (((l_B).bit_and (((l_H).bit_or (l_A))))))
			l_C := l_C + l_temp1
			l_G := l_temp1 + l_temp2
			l_W[26] := ((((((((((((l_W[24]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (17))).bit_or (((l_W[24]).bit_shift_left (15))))).bit_xor (((((((l_W[24]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (19))).bit_or (((l_W[24]).bit_shift_left (13))))))).bit_xor (((((l_W[24]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (10)))) + l_W[19]) + ((((((((((l_W[11]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (7))).bit_or (((l_W[11]).bit_shift_left (25))))).bit_xor (((((((l_W[11]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (18))).bit_or (((l_W[11]).bit_shift_left (14))))))).bit_xor (((((l_W[11]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (3))))) + l_W[10]
			l_temp1 := (((l_F + ((((((((((l_C).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_C).bit_shift_left (26))))).bit_xor (((((((l_C).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_C).bit_shift_left (21))))))).bit_xor (((((((l_C).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_C).bit_shift_left (7))))))) + ((l_E).bit_xor (((l_C).bit_and (((l_D).bit_xor (l_E))))))) + {NATURAL_32} 2_952_996_808) + l_W[26]
			l_temp2 := ((((((((((l_G).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_G).bit_shift_left (30))))).bit_xor (((((((l_G).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_G).bit_shift_left (19))))))).bit_xor (((((((l_G).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_G).bit_shift_left (10)))))) + ((((l_G).bit_and (l_H))).bit_or (((l_A).bit_and (((l_G).bit_or (l_H))))))
			l_B := l_B + l_temp1
			l_F := l_temp1 + l_temp2
			l_W[27] := ((((((((((((l_W[25]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (17))).bit_or (((l_W[25]).bit_shift_left (15))))).bit_xor (((((((l_W[25]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (19))).bit_or (((l_W[25]).bit_shift_left (13))))))).bit_xor (((((l_W[25]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (10)))) + l_W[20]) + ((((((((((l_W[12]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (7))).bit_or (((l_W[12]).bit_shift_left (25))))).bit_xor (((((((l_W[12]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (18))).bit_or (((l_W[12]).bit_shift_left (14))))))).bit_xor (((((l_W[12]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (3))))) + l_W[11]
			l_temp1 := (((l_E + ((((((((((l_B).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_B).bit_shift_left (26))))).bit_xor (((((((l_B).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_B).bit_shift_left (21))))))).bit_xor (((((((l_B).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_B).bit_shift_left (7))))))) + ((l_D).bit_xor (((l_B).bit_and (((l_C).bit_xor (l_D))))))) + {NATURAL_32} 3_210_313_671) + l_W[27]
			l_temp2 := ((((((((((l_F).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_F).bit_shift_left (30))))).bit_xor (((((((l_F).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_F).bit_shift_left (19))))))).bit_xor (((((((l_F).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_F).bit_shift_left (10)))))) + ((((l_F).bit_and (l_G))).bit_or (((l_H).bit_and (((l_F).bit_or (l_G))))))
			l_A := l_A + l_temp1
			l_E := l_temp1 + l_temp2
			l_W[28] := ((((((((((((l_W[26]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (17))).bit_or (((l_W[26]).bit_shift_left (15))))).bit_xor (((((((l_W[26]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (19))).bit_or (((l_W[26]).bit_shift_left (13))))))).bit_xor (((((l_W[26]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (10)))) + l_W[21]) + ((((((((((l_W[13]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (7))).bit_or (((l_W[13]).bit_shift_left (25))))).bit_xor (((((((l_W[13]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (18))).bit_or (((l_W[13]).bit_shift_left (14))))))).bit_xor (((((l_W[13]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (3))))) + l_W[12]
			l_temp1 := (((l_D + ((((((((((l_A).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_A).bit_shift_left (26))))).bit_xor (((((((l_A).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_A).bit_shift_left (21))))))).bit_xor (((((((l_A).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_A).bit_shift_left (7))))))) + ((l_C).bit_xor (((l_A).bit_and (((l_B).bit_xor (l_C))))))) + {NATURAL_32} 3_336_571_891) + l_W[28]
			l_temp2 := ((((((((((l_E).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_E).bit_shift_left (30))))).bit_xor (((((((l_E).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_E).bit_shift_left (19))))))).bit_xor (((((((l_E).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_E).bit_shift_left (10)))))) + ((((l_E).bit_and (l_F))).bit_or (((l_G).bit_and (((l_E).bit_or (l_F))))))
			l_H := l_H + l_temp1
			l_D := l_temp1 + l_temp2
			l_W[29] := ((((((((((((l_W[27]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (17))).bit_or (((l_W[27]).bit_shift_left (15))))).bit_xor (((((((l_W[27]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (19))).bit_or (((l_W[27]).bit_shift_left (13))))))).bit_xor (((((l_W[27]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (10)))) + l_W[22]) + ((((((((((l_W[14]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (7))).bit_or (((l_W[14]).bit_shift_left (25))))).bit_xor (((((((l_W[14]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (18))).bit_or (((l_W[14]).bit_shift_left (14))))))).bit_xor (((((l_W[14]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (3))))) + l_W[13]
			l_temp1 := (((l_C + ((((((((((l_H).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_H).bit_shift_left (26))))).bit_xor (((((((l_H).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_H).bit_shift_left (21))))))).bit_xor (((((((l_H).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_H).bit_shift_left (7))))))) + ((l_B).bit_xor (((l_H).bit_and (((l_A).bit_xor (l_B))))))) + {NATURAL_32} 3_584_528_711) + l_W[29]
			l_temp2 := ((((((((((l_D).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_D).bit_shift_left (30))))).bit_xor (((((((l_D).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_D).bit_shift_left (19))))))).bit_xor (((((((l_D).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_D).bit_shift_left (10)))))) + ((((l_D).bit_and (l_E))).bit_or (((l_F).bit_and (((l_D).bit_or (l_E))))))
			l_G := l_G + l_temp1
			l_C := l_temp1 + l_temp2
			l_W[30] := ((((((((((((l_W[28]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (17))).bit_or (((l_W[28]).bit_shift_left (15))))).bit_xor (((((((l_W[28]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (19))).bit_or (((l_W[28]).bit_shift_left (13))))))).bit_xor (((((l_W[28]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (10)))) + l_W[23]) + ((((((((((l_W[15]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (7))).bit_or (((l_W[15]).bit_shift_left (25))))).bit_xor (((((((l_W[15]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (18))).bit_or (((l_W[15]).bit_shift_left (14))))))).bit_xor (((((l_W[15]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (3))))) + l_W[14]
			l_temp1 := (((l_B + ((((((((((l_G).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_G).bit_shift_left (26))))).bit_xor (((((((l_G).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_G).bit_shift_left (21))))))).bit_xor (((((((l_G).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_G).bit_shift_left (7))))))) + ((l_A).bit_xor (((l_G).bit_and (((l_H).bit_xor (l_A))))))) + {NATURAL_32} 113_926_993) + l_W[30]
			l_temp2 := ((((((((((l_C).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_C).bit_shift_left (30))))).bit_xor (((((((l_C).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_C).bit_shift_left (19))))))).bit_xor (((((((l_C).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_C).bit_shift_left (10)))))) + ((((l_C).bit_and (l_D))).bit_or (((l_E).bit_and (((l_C).bit_or (l_D))))))
			l_F := l_F + l_temp1
			l_B := l_temp1 + l_temp2
			l_W[31] := ((((((((((((l_W[29]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (17))).bit_or (((l_W[29]).bit_shift_left (15))))).bit_xor (((((((l_W[29]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (19))).bit_or (((l_W[29]).bit_shift_left (13))))))).bit_xor (((((l_W[29]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (10)))) + l_W[24]) + ((((((((((l_W[16]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (7))).bit_or (((l_W[16]).bit_shift_left (25))))).bit_xor (((((((l_W[16]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (18))).bit_or (((l_W[16]).bit_shift_left (14))))))).bit_xor (((((l_W[16]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (3))))) + l_W[15]
			l_temp1 := (((l_A + ((((((((((l_F).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_F).bit_shift_left (26))))).bit_xor (((((((l_F).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_F).bit_shift_left (21))))))).bit_xor (((((((l_F).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_F).bit_shift_left (7))))))) + ((l_H).bit_xor (((l_F).bit_and (((l_G).bit_xor (l_H))))))) + {NATURAL_32} 338_241_895) + l_W[31]
			l_temp2 := ((((((((((l_B).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_B).bit_shift_left (30))))).bit_xor (((((((l_B).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_B).bit_shift_left (19))))))).bit_xor (((((((l_B).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_B).bit_shift_left (10)))))) + ((((l_B).bit_and (l_C))).bit_or (((l_D).bit_and (((l_B).bit_or (l_C))))))
			l_E := l_E + l_temp1
			l_A := l_temp1 + l_temp2
			l_W[32] := ((((((((((((l_W[30]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (17))).bit_or (((l_W[30]).bit_shift_left (15))))).bit_xor (((((((l_W[30]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (19))).bit_or (((l_W[30]).bit_shift_left (13))))))).bit_xor (((((l_W[30]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (10)))) + l_W[25]) + ((((((((((l_W[17]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (7))).bit_or (((l_W[17]).bit_shift_left (25))))).bit_xor (((((((l_W[17]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (18))).bit_or (((l_W[17]).bit_shift_left (14))))))).bit_xor (((((l_W[17]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (3))))) + l_W[16]
			l_temp1 := (((l_H + ((((((((((l_E).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_E).bit_shift_left (26))))).bit_xor (((((((l_E).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_E).bit_shift_left (21))))))).bit_xor (((((((l_E).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_E).bit_shift_left (7))))))) + ((l_G).bit_xor (((l_E).bit_and (((l_F).bit_xor (l_G))))))) + {NATURAL_32} 666_307_205) + l_W[32]
			l_temp2 := ((((((((((l_A).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_A).bit_shift_left (30))))).bit_xor (((((((l_A).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_A).bit_shift_left (19))))))).bit_xor (((((((l_A).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_A).bit_shift_left (10)))))) + ((((l_A).bit_and (l_B))).bit_or (((l_C).bit_and (((l_A).bit_or (l_B))))))
			l_D := l_D + l_temp1
			l_H := l_temp1 + l_temp2
			l_W[33] := ((((((((((((l_W[31]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (17))).bit_or (((l_W[31]).bit_shift_left (15))))).bit_xor (((((((l_W[31]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (19))).bit_or (((l_W[31]).bit_shift_left (13))))))).bit_xor (((((l_W[31]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (10)))) + l_W[26]) + ((((((((((l_W[18]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (7))).bit_or (((l_W[18]).bit_shift_left (25))))).bit_xor (((((((l_W[18]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (18))).bit_or (((l_W[18]).bit_shift_left (14))))))).bit_xor (((((l_W[18]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (3))))) + l_W[17]
			l_temp1 := (((l_G + ((((((((((l_D).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_D).bit_shift_left (26))))).bit_xor (((((((l_D).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_D).bit_shift_left (21))))))).bit_xor (((((((l_D).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_D).bit_shift_left (7))))))) + ((l_F).bit_xor (((l_D).bit_and (((l_E).bit_xor (l_F))))))) + {NATURAL_32} 773_529_912) + l_W[33]
			l_temp2 := ((((((((((l_H).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_H).bit_shift_left (30))))).bit_xor (((((((l_H).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_H).bit_shift_left (19))))))).bit_xor (((((((l_H).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_H).bit_shift_left (10)))))) + ((((l_H).bit_and (l_A))).bit_or (((l_B).bit_and (((l_H).bit_or (l_A))))))
			l_C := l_C + l_temp1
			l_G := l_temp1 + l_temp2
			l_W[34] := ((((((((((((l_W[32]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (17))).bit_or (((l_W[32]).bit_shift_left (15))))).bit_xor (((((((l_W[32]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (19))).bit_or (((l_W[32]).bit_shift_left (13))))))).bit_xor (((((l_W[32]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (10)))) + l_W[27]) + ((((((((((l_W[19]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (7))).bit_or (((l_W[19]).bit_shift_left (25))))).bit_xor (((((((l_W[19]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (18))).bit_or (((l_W[19]).bit_shift_left (14))))))).bit_xor (((((l_W[19]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (3))))) + l_W[18]
			l_temp1 := (((l_F + ((((((((((l_C).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_C).bit_shift_left (26))))).bit_xor (((((((l_C).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_C).bit_shift_left (21))))))).bit_xor (((((((l_C).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_C).bit_shift_left (7))))))) + ((l_E).bit_xor (((l_C).bit_and (((l_D).bit_xor (l_E))))))) + {NATURAL_32} 1_294_757_372) + l_W[34]
			l_temp2 := ((((((((((l_G).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_G).bit_shift_left (30))))).bit_xor (((((((l_G).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_G).bit_shift_left (19))))))).bit_xor (((((((l_G).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_G).bit_shift_left (10)))))) + ((((l_G).bit_and (l_H))).bit_or (((l_A).bit_and (((l_G).bit_or (l_H))))))
			l_B := l_B + l_temp1
			l_F := l_temp1 + l_temp2
			l_W[35] := ((((((((((((l_W[33]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (17))).bit_or (((l_W[33]).bit_shift_left (15))))).bit_xor (((((((l_W[33]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (19))).bit_or (((l_W[33]).bit_shift_left (13))))))).bit_xor (((((l_W[33]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (10)))) + l_W[28]) + ((((((((((l_W[20]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (7))).bit_or (((l_W[20]).bit_shift_left (25))))).bit_xor (((((((l_W[20]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (18))).bit_or (((l_W[20]).bit_shift_left (14))))))).bit_xor (((((l_W[20]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (3))))) + l_W[19]
			l_temp1 := (((l_E + ((((((((((l_B).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_B).bit_shift_left (26))))).bit_xor (((((((l_B).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_B).bit_shift_left (21))))))).bit_xor (((((((l_B).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_B).bit_shift_left (7))))))) + ((l_D).bit_xor (((l_B).bit_and (((l_C).bit_xor (l_D))))))) + {NATURAL_32} 1_396_182_291) + l_W[35]
			l_temp2 := ((((((((((l_F).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_F).bit_shift_left (30))))).bit_xor (((((((l_F).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_F).bit_shift_left (19))))))).bit_xor (((((((l_F).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_F).bit_shift_left (10)))))) + ((((l_F).bit_and (l_G))).bit_or (((l_H).bit_and (((l_F).bit_or (l_G))))))
			l_A := l_A + l_temp1
			l_E := l_temp1 + l_temp2
			l_W[36] := ((((((((((((l_W[34]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (17))).bit_or (((l_W[34]).bit_shift_left (15))))).bit_xor (((((((l_W[34]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (19))).bit_or (((l_W[34]).bit_shift_left (13))))))).bit_xor (((((l_W[34]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (10)))) + l_W[29]) + ((((((((((l_W[21]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (7))).bit_or (((l_W[21]).bit_shift_left (25))))).bit_xor (((((((l_W[21]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (18))).bit_or (((l_W[21]).bit_shift_left (14))))))).bit_xor (((((l_W[21]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (3))))) + l_W[20]
			l_temp1 := (((l_D + ((((((((((l_A).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_A).bit_shift_left (26))))).bit_xor (((((((l_A).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_A).bit_shift_left (21))))))).bit_xor (((((((l_A).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_A).bit_shift_left (7))))))) + ((l_C).bit_xor (((l_A).bit_and (((l_B).bit_xor (l_C))))))) + {NATURAL_32} 1_695_183_700) + l_W[36]
			l_temp2 := ((((((((((l_E).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_E).bit_shift_left (30))))).bit_xor (((((((l_E).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_E).bit_shift_left (19))))))).bit_xor (((((((l_E).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_E).bit_shift_left (10)))))) + ((((l_E).bit_and (l_F))).bit_or (((l_G).bit_and (((l_E).bit_or (l_F))))))
			l_H := l_H + l_temp1
			l_D := l_temp1 + l_temp2
			l_W[37] := ((((((((((((l_W[35]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (17))).bit_or (((l_W[35]).bit_shift_left (15))))).bit_xor (((((((l_W[35]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (19))).bit_or (((l_W[35]).bit_shift_left (13))))))).bit_xor (((((l_W[35]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (10)))) + l_W[30]) + ((((((((((l_W[22]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (7))).bit_or (((l_W[22]).bit_shift_left (25))))).bit_xor (((((((l_W[22]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (18))).bit_or (((l_W[22]).bit_shift_left (14))))))).bit_xor (((((l_W[22]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (3))))) + l_W[21]
			l_temp1 := (((l_C + ((((((((((l_H).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_H).bit_shift_left (26))))).bit_xor (((((((l_H).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_H).bit_shift_left (21))))))).bit_xor (((((((l_H).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_H).bit_shift_left (7))))))) + ((l_B).bit_xor (((l_H).bit_and (((l_A).bit_xor (l_B))))))) + {NATURAL_32} 1_986_661_051) + l_W[37]
			l_temp2 := ((((((((((l_D).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_D).bit_shift_left (30))))).bit_xor (((((((l_D).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_D).bit_shift_left (19))))))).bit_xor (((((((l_D).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_D).bit_shift_left (10)))))) + ((((l_D).bit_and (l_E))).bit_or (((l_F).bit_and (((l_D).bit_or (l_E))))))
			l_G := l_G + l_temp1
			l_C := l_temp1 + l_temp2
			l_W[38] := ((((((((((((l_W[36]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (17))).bit_or (((l_W[36]).bit_shift_left (15))))).bit_xor (((((((l_W[36]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (19))).bit_or (((l_W[36]).bit_shift_left (13))))))).bit_xor (((((l_W[36]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (10)))) + l_W[31]) + ((((((((((l_W[23]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (7))).bit_or (((l_W[23]).bit_shift_left (25))))).bit_xor (((((((l_W[23]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (18))).bit_or (((l_W[23]).bit_shift_left (14))))))).bit_xor (((((l_W[23]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (3))))) + l_W[22]
			l_temp1 := (((l_B + ((((((((((l_G).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_G).bit_shift_left (26))))).bit_xor (((((((l_G).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_G).bit_shift_left (21))))))).bit_xor (((((((l_G).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_G).bit_shift_left (7))))))) + ((l_A).bit_xor (((l_G).bit_and (((l_H).bit_xor (l_A))))))) + {NATURAL_32} 2_177_026_350) + l_W[38]
			l_temp2 := ((((((((((l_C).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_C).bit_shift_left (30))))).bit_xor (((((((l_C).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_C).bit_shift_left (19))))))).bit_xor (((((((l_C).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_C).bit_shift_left (10)))))) + ((((l_C).bit_and (l_D))).bit_or (((l_E).bit_and (((l_C).bit_or (l_D))))))
			l_F := l_F + l_temp1
			l_B := l_temp1 + l_temp2
			l_W[39] := ((((((((((((l_W[37]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (17))).bit_or (((l_W[37]).bit_shift_left (15))))).bit_xor (((((((l_W[37]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (19))).bit_or (((l_W[37]).bit_shift_left (13))))))).bit_xor (((((l_W[37]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (10)))) + l_W[32]) + ((((((((((l_W[24]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (7))).bit_or (((l_W[24]).bit_shift_left (25))))).bit_xor (((((((l_W[24]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (18))).bit_or (((l_W[24]).bit_shift_left (14))))))).bit_xor (((((l_W[24]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (3))))) + l_W[23]
			l_temp1 := (((l_A + ((((((((((l_F).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_F).bit_shift_left (26))))).bit_xor (((((((l_F).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_F).bit_shift_left (21))))))).bit_xor (((((((l_F).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_F).bit_shift_left (7))))))) + ((l_H).bit_xor (((l_F).bit_and (((l_G).bit_xor (l_H))))))) + {NATURAL_32} 2_456_956_037) + l_W[39]
			l_temp2 := ((((((((((l_B).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_B).bit_shift_left (30))))).bit_xor (((((((l_B).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_B).bit_shift_left (19))))))).bit_xor (((((((l_B).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_B).bit_shift_left (10)))))) + ((((l_B).bit_and (l_C))).bit_or (((l_D).bit_and (((l_B).bit_or (l_C))))))
			l_E := l_E + l_temp1
			l_A := l_temp1 + l_temp2
			l_W[40] := ((((((((((((l_W[38]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (17))).bit_or (((l_W[38]).bit_shift_left (15))))).bit_xor (((((((l_W[38]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (19))).bit_or (((l_W[38]).bit_shift_left (13))))))).bit_xor (((((l_W[38]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (10)))) + l_W[33]) + ((((((((((l_W[25]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (7))).bit_or (((l_W[25]).bit_shift_left (25))))).bit_xor (((((((l_W[25]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (18))).bit_or (((l_W[25]).bit_shift_left (14))))))).bit_xor (((((l_W[25]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (3))))) + l_W[24]
			l_temp1 := (((l_H + ((((((((((l_E).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_E).bit_shift_left (26))))).bit_xor (((((((l_E).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_E).bit_shift_left (21))))))).bit_xor (((((((l_E).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_E).bit_shift_left (7))))))) + ((l_G).bit_xor (((l_E).bit_and (((l_F).bit_xor (l_G))))))) + {NATURAL_32} 2_730_485_921) + l_W[40]
			l_temp2 := ((((((((((l_A).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_A).bit_shift_left (30))))).bit_xor (((((((l_A).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_A).bit_shift_left (19))))))).bit_xor (((((((l_A).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_A).bit_shift_left (10)))))) + ((((l_A).bit_and (l_B))).bit_or (((l_C).bit_and (((l_A).bit_or (l_B))))))
			l_D := l_D + l_temp1
			l_H := l_temp1 + l_temp2
			l_W[41] := ((((((((((((l_W[39]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (17))).bit_or (((l_W[39]).bit_shift_left (15))))).bit_xor (((((((l_W[39]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (19))).bit_or (((l_W[39]).bit_shift_left (13))))))).bit_xor (((((l_W[39]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (10)))) + l_W[34]) + ((((((((((l_W[26]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (7))).bit_or (((l_W[26]).bit_shift_left (25))))).bit_xor (((((((l_W[26]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (18))).bit_or (((l_W[26]).bit_shift_left (14))))))).bit_xor (((((l_W[26]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (3))))) + l_W[25]
			l_temp1 := (((l_G + ((((((((((l_D).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_D).bit_shift_left (26))))).bit_xor (((((((l_D).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_D).bit_shift_left (21))))))).bit_xor (((((((l_D).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_D).bit_shift_left (7))))))) + ((l_F).bit_xor (((l_D).bit_and (((l_E).bit_xor (l_F))))))) + {NATURAL_32} 2_820_302_411) + l_W[41]
			l_temp2 := ((((((((((l_H).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_H).bit_shift_left (30))))).bit_xor (((((((l_H).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_H).bit_shift_left (19))))))).bit_xor (((((((l_H).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_H).bit_shift_left (10)))))) + ((((l_H).bit_and (l_A))).bit_or (((l_B).bit_and (((l_H).bit_or (l_A))))))
			l_C := l_C + l_temp1
			l_G := l_temp1 + l_temp2
			l_W[42] := ((((((((((((l_W[40]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (17))).bit_or (((l_W[40]).bit_shift_left (15))))).bit_xor (((((((l_W[40]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (19))).bit_or (((l_W[40]).bit_shift_left (13))))))).bit_xor (((((l_W[40]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (10)))) + l_W[35]) + ((((((((((l_W[27]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (7))).bit_or (((l_W[27]).bit_shift_left (25))))).bit_xor (((((((l_W[27]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (18))).bit_or (((l_W[27]).bit_shift_left (14))))))).bit_xor (((((l_W[27]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (3))))) + l_W[26]
			l_temp1 := (((l_F + ((((((((((l_C).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_C).bit_shift_left (26))))).bit_xor (((((((l_C).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_C).bit_shift_left (21))))))).bit_xor (((((((l_C).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_C).bit_shift_left (7))))))) + ((l_E).bit_xor (((l_C).bit_and (((l_D).bit_xor (l_E))))))) + {NATURAL_32} 3_259_730_800) + l_W[42]
			l_temp2 := ((((((((((l_G).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_G).bit_shift_left (30))))).bit_xor (((((((l_G).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_G).bit_shift_left (19))))))).bit_xor (((((((l_G).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_G).bit_shift_left (10)))))) + ((((l_G).bit_and (l_H))).bit_or (((l_A).bit_and (((l_G).bit_or (l_H))))))
			l_B := l_B + l_temp1
			l_F := l_temp1 + l_temp2
			l_W[43] := ((((((((((((l_W[41]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (17))).bit_or (((l_W[41]).bit_shift_left (15))))).bit_xor (((((((l_W[41]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (19))).bit_or (((l_W[41]).bit_shift_left (13))))))).bit_xor (((((l_W[41]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (10)))) + l_W[36]) + ((((((((((l_W[28]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (7))).bit_or (((l_W[28]).bit_shift_left (25))))).bit_xor (((((((l_W[28]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (18))).bit_or (((l_W[28]).bit_shift_left (14))))))).bit_xor (((((l_W[28]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (3))))) + l_W[27]
			l_temp1 := (((l_E + ((((((((((l_B).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_B).bit_shift_left (26))))).bit_xor (((((((l_B).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_B).bit_shift_left (21))))))).bit_xor (((((((l_B).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_B).bit_shift_left (7))))))) + ((l_D).bit_xor (((l_B).bit_and (((l_C).bit_xor (l_D))))))) + {NATURAL_32} 3_345_764_771) + l_W[43]
			l_temp2 := ((((((((((l_F).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_F).bit_shift_left (30))))).bit_xor (((((((l_F).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_F).bit_shift_left (19))))))).bit_xor (((((((l_F).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_F).bit_shift_left (10)))))) + ((((l_F).bit_and (l_G))).bit_or (((l_H).bit_and (((l_F).bit_or (l_G))))))
			l_A := l_A + l_temp1
			l_E := l_temp1 + l_temp2
			l_W[44] := ((((((((((((l_W[42]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (17))).bit_or (((l_W[42]).bit_shift_left (15))))).bit_xor (((((((l_W[42]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (19))).bit_or (((l_W[42]).bit_shift_left (13))))))).bit_xor (((((l_W[42]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (10)))) + l_W[37]) + ((((((((((l_W[29]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (7))).bit_or (((l_W[29]).bit_shift_left (25))))).bit_xor (((((((l_W[29]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (18))).bit_or (((l_W[29]).bit_shift_left (14))))))).bit_xor (((((l_W[29]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (3))))) + l_W[28]
			l_temp1 := (((l_D + ((((((((((l_A).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_A).bit_shift_left (26))))).bit_xor (((((((l_A).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_A).bit_shift_left (21))))))).bit_xor (((((((l_A).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_A).bit_shift_left (7))))))) + ((l_C).bit_xor (((l_A).bit_and (((l_B).bit_xor (l_C))))))) + {NATURAL_32} 3_516_065_817) + l_W[44]
			l_temp2 := ((((((((((l_E).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_E).bit_shift_left (30))))).bit_xor (((((((l_E).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_E).bit_shift_left (19))))))).bit_xor (((((((l_E).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_E).bit_shift_left (10)))))) + ((((l_E).bit_and (l_F))).bit_or (((l_G).bit_and (((l_E).bit_or (l_F))))))
			l_H := l_H + l_temp1
			l_D := l_temp1 + l_temp2
			l_W[45] := ((((((((((((l_W[43]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (17))).bit_or (((l_W[43]).bit_shift_left (15))))).bit_xor (((((((l_W[43]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (19))).bit_or (((l_W[43]).bit_shift_left (13))))))).bit_xor (((((l_W[43]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (10)))) + l_W[38]) + ((((((((((l_W[30]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (7))).bit_or (((l_W[30]).bit_shift_left (25))))).bit_xor (((((((l_W[30]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (18))).bit_or (((l_W[30]).bit_shift_left (14))))))).bit_xor (((((l_W[30]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (3))))) + l_W[29]
			l_temp1 := (((l_C + ((((((((((l_H).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_H).bit_shift_left (26))))).bit_xor (((((((l_H).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_H).bit_shift_left (21))))))).bit_xor (((((((l_H).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_H).bit_shift_left (7))))))) + ((l_B).bit_xor (((l_H).bit_and (((l_A).bit_xor (l_B))))))) + {NATURAL_32} 3_600_352_804) + l_W[45]
			l_temp2 := ((((((((((l_D).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_D).bit_shift_left (30))))).bit_xor (((((((l_D).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_D).bit_shift_left (19))))))).bit_xor (((((((l_D).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_D).bit_shift_left (10)))))) + ((((l_D).bit_and (l_E))).bit_or (((l_F).bit_and (((l_D).bit_or (l_E))))))
			l_G := l_G + l_temp1
			l_C := l_temp1 + l_temp2
			l_W[46] := ((((((((((((l_W[44]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (17))).bit_or (((l_W[44]).bit_shift_left (15))))).bit_xor (((((((l_W[44]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (19))).bit_or (((l_W[44]).bit_shift_left (13))))))).bit_xor (((((l_W[44]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (10)))) + l_W[39]) + ((((((((((l_W[31]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (7))).bit_or (((l_W[31]).bit_shift_left (25))))).bit_xor (((((((l_W[31]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (18))).bit_or (((l_W[31]).bit_shift_left (14))))))).bit_xor (((((l_W[31]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (3))))) + l_W[30]
			l_temp1 := (((l_B + ((((((((((l_G).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_G).bit_shift_left (26))))).bit_xor (((((((l_G).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_G).bit_shift_left (21))))))).bit_xor (((((((l_G).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_G).bit_shift_left (7))))))) + ((l_A).bit_xor (((l_G).bit_and (((l_H).bit_xor (l_A))))))) + {NATURAL_32} 4_094_571_909) + l_W[46]
			l_temp2 := ((((((((((l_C).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_C).bit_shift_left (30))))).bit_xor (((((((l_C).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_C).bit_shift_left (19))))))).bit_xor (((((((l_C).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_C).bit_shift_left (10)))))) + ((((l_C).bit_and (l_D))).bit_or (((l_E).bit_and (((l_C).bit_or (l_D))))))
			l_F := l_F + l_temp1
			l_B := l_temp1 + l_temp2
			l_W[47] := ((((((((((((l_W[45]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (17))).bit_or (((l_W[45]).bit_shift_left (15))))).bit_xor (((((((l_W[45]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (19))).bit_or (((l_W[45]).bit_shift_left (13))))))).bit_xor (((((l_W[45]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (10)))) + l_W[40]) + ((((((((((l_W[32]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (7))).bit_or (((l_W[32]).bit_shift_left (25))))).bit_xor (((((((l_W[32]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (18))).bit_or (((l_W[32]).bit_shift_left (14))))))).bit_xor (((((l_W[32]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (3))))) + l_W[31]
			l_temp1 := (((l_A + ((((((((((l_F).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_F).bit_shift_left (26))))).bit_xor (((((((l_F).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_F).bit_shift_left (21))))))).bit_xor (((((((l_F).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_F).bit_shift_left (7))))))) + ((l_H).bit_xor (((l_F).bit_and (((l_G).bit_xor (l_H))))))) + {NATURAL_32} 275_423_344) + l_W[47]
			l_temp2 := ((((((((((l_B).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_B).bit_shift_left (30))))).bit_xor (((((((l_B).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_B).bit_shift_left (19))))))).bit_xor (((((((l_B).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_B).bit_shift_left (10)))))) + ((((l_B).bit_and (l_C))).bit_or (((l_D).bit_and (((l_B).bit_or (l_C))))))
			l_E := l_E + l_temp1
			l_A := l_temp1 + l_temp2
			l_W[48] := ((((((((((((l_W[46]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (17))).bit_or (((l_W[46]).bit_shift_left (15))))).bit_xor (((((((l_W[46]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (19))).bit_or (((l_W[46]).bit_shift_left (13))))))).bit_xor (((((l_W[46]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (10)))) + l_W[41]) + ((((((((((l_W[33]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (7))).bit_or (((l_W[33]).bit_shift_left (25))))).bit_xor (((((((l_W[33]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (18))).bit_or (((l_W[33]).bit_shift_left (14))))))).bit_xor (((((l_W[33]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (3))))) + l_W[32]
			l_temp1 := (((l_H + ((((((((((l_E).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_E).bit_shift_left (26))))).bit_xor (((((((l_E).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_E).bit_shift_left (21))))))).bit_xor (((((((l_E).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_E).bit_shift_left (7))))))) + ((l_G).bit_xor (((l_E).bit_and (((l_F).bit_xor (l_G))))))) + {NATURAL_32} 430_227_734) + l_W[48]
			l_temp2 := ((((((((((l_A).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_A).bit_shift_left (30))))).bit_xor (((((((l_A).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_A).bit_shift_left (19))))))).bit_xor (((((((l_A).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_A).bit_shift_left (10)))))) + ((((l_A).bit_and (l_B))).bit_or (((l_C).bit_and (((l_A).bit_or (l_B))))))
			l_D := l_D + l_temp1
			l_H := l_temp1 + l_temp2
			l_W[49] := ((((((((((((l_W[47]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (17))).bit_or (((l_W[47]).bit_shift_left (15))))).bit_xor (((((((l_W[47]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (19))).bit_or (((l_W[47]).bit_shift_left (13))))))).bit_xor (((((l_W[47]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (10)))) + l_W[42]) + ((((((((((l_W[34]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (7))).bit_or (((l_W[34]).bit_shift_left (25))))).bit_xor (((((((l_W[34]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (18))).bit_or (((l_W[34]).bit_shift_left (14))))))).bit_xor (((((l_W[34]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (3))))) + l_W[33]
			l_temp1 := (((l_G + ((((((((((l_D).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_D).bit_shift_left (26))))).bit_xor (((((((l_D).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_D).bit_shift_left (21))))))).bit_xor (((((((l_D).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_D).bit_shift_left (7))))))) + ((l_F).bit_xor (((l_D).bit_and (((l_E).bit_xor (l_F))))))) + {NATURAL_32} 506_948_616) + l_W[49]
			l_temp2 := ((((((((((l_H).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_H).bit_shift_left (30))))).bit_xor (((((((l_H).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_H).bit_shift_left (19))))))).bit_xor (((((((l_H).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_H).bit_shift_left (10)))))) + ((((l_H).bit_and (l_A))).bit_or (((l_B).bit_and (((l_H).bit_or (l_A))))))
			l_C := l_C + l_temp1
			l_G := l_temp1 + l_temp2
			l_W[50] := ((((((((((((l_W[48]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (17))).bit_or (((l_W[48]).bit_shift_left (15))))).bit_xor (((((((l_W[48]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (19))).bit_or (((l_W[48]).bit_shift_left (13))))))).bit_xor (((((l_W[48]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (10)))) + l_W[43]) + ((((((((((l_W[35]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (7))).bit_or (((l_W[35]).bit_shift_left (25))))).bit_xor (((((((l_W[35]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (18))).bit_or (((l_W[35]).bit_shift_left (14))))))).bit_xor (((((l_W[35]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (3))))) + l_W[34]
			l_temp1 := (((l_F + ((((((((((l_C).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_C).bit_shift_left (26))))).bit_xor (((((((l_C).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_C).bit_shift_left (21))))))).bit_xor (((((((l_C).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_C).bit_shift_left (7))))))) + ((l_E).bit_xor (((l_C).bit_and (((l_D).bit_xor (l_E))))))) + {NATURAL_32} 659_060_556) + l_W[50]
			l_temp2 := ((((((((((l_G).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_G).bit_shift_left (30))))).bit_xor (((((((l_G).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_G).bit_shift_left (19))))))).bit_xor (((((((l_G).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_G).bit_shift_left (10)))))) + ((((l_G).bit_and (l_H))).bit_or (((l_A).bit_and (((l_G).bit_or (l_H))))))
			l_B := l_B + l_temp1
			l_F := l_temp1 + l_temp2
			l_W[51] := ((((((((((((l_W[49]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (17))).bit_or (((l_W[49]).bit_shift_left (15))))).bit_xor (((((((l_W[49]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (19))).bit_or (((l_W[49]).bit_shift_left (13))))))).bit_xor (((((l_W[49]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (10)))) + l_W[44]) + ((((((((((l_W[36]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (7))).bit_or (((l_W[36]).bit_shift_left (25))))).bit_xor (((((((l_W[36]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (18))).bit_or (((l_W[36]).bit_shift_left (14))))))).bit_xor (((((l_W[36]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (3))))) + l_W[35]
			l_temp1 := (((l_E + ((((((((((l_B).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_B).bit_shift_left (26))))).bit_xor (((((((l_B).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_B).bit_shift_left (21))))))).bit_xor (((((((l_B).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_B).bit_shift_left (7))))))) + ((l_D).bit_xor (((l_B).bit_and (((l_C).bit_xor (l_D))))))) + {NATURAL_32} 883_997_877) + l_W[51]
			l_temp2 := ((((((((((l_F).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_F).bit_shift_left (30))))).bit_xor (((((((l_F).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_F).bit_shift_left (19))))))).bit_xor (((((((l_F).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_F).bit_shift_left (10)))))) + ((((l_F).bit_and (l_G))).bit_or (((l_H).bit_and (((l_F).bit_or (l_G))))))
			l_A := l_A + l_temp1
			l_E := l_temp1 + l_temp2
			l_W[52] := ((((((((((((l_W[50]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (17))).bit_or (((l_W[50]).bit_shift_left (15))))).bit_xor (((((((l_W[50]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (19))).bit_or (((l_W[50]).bit_shift_left (13))))))).bit_xor (((((l_W[50]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (10)))) + l_W[45]) + ((((((((((l_W[37]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (7))).bit_or (((l_W[37]).bit_shift_left (25))))).bit_xor (((((((l_W[37]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (18))).bit_or (((l_W[37]).bit_shift_left (14))))))).bit_xor (((((l_W[37]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (3))))) + l_W[36]
			l_temp1 := (((l_D + ((((((((((l_A).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_A).bit_shift_left (26))))).bit_xor (((((((l_A).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_A).bit_shift_left (21))))))).bit_xor (((((((l_A).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_A).bit_shift_left (7))))))) + ((l_C).bit_xor (((l_A).bit_and (((l_B).bit_xor (l_C))))))) + {NATURAL_32} 958_139_571) + l_W[52]
			l_temp2 := ((((((((((l_E).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_E).bit_shift_left (30))))).bit_xor (((((((l_E).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_E).bit_shift_left (19))))))).bit_xor (((((((l_E).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_E).bit_shift_left (10)))))) + ((((l_E).bit_and (l_F))).bit_or (((l_G).bit_and (((l_E).bit_or (l_F))))))
			l_H := l_H + l_temp1
			l_D := l_temp1 + l_temp2
			l_W[53] := ((((((((((((l_W[51]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (17))).bit_or (((l_W[51]).bit_shift_left (15))))).bit_xor (((((((l_W[51]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (19))).bit_or (((l_W[51]).bit_shift_left (13))))))).bit_xor (((((l_W[51]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (10)))) + l_W[46]) + ((((((((((l_W[38]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (7))).bit_or (((l_W[38]).bit_shift_left (25))))).bit_xor (((((((l_W[38]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (18))).bit_or (((l_W[38]).bit_shift_left (14))))))).bit_xor (((((l_W[38]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (3))))) + l_W[37]
			l_temp1 := (((l_C + ((((((((((l_H).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_H).bit_shift_left (26))))).bit_xor (((((((l_H).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_H).bit_shift_left (21))))))).bit_xor (((((((l_H).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_H).bit_shift_left (7))))))) + ((l_B).bit_xor (((l_H).bit_and (((l_A).bit_xor (l_B))))))) + {NATURAL_32} 1_322_822_218) + l_W[53]
			l_temp2 := ((((((((((l_D).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_D).bit_shift_left (30))))).bit_xor (((((((l_D).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_D).bit_shift_left (19))))))).bit_xor (((((((l_D).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_D).bit_shift_left (10)))))) + ((((l_D).bit_and (l_E))).bit_or (((l_F).bit_and (((l_D).bit_or (l_E))))))
			l_G := l_G + l_temp1
			l_C := l_temp1 + l_temp2
			l_W[54] := ((((((((((((l_W[52]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (17))).bit_or (((l_W[52]).bit_shift_left (15))))).bit_xor (((((((l_W[52]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (19))).bit_or (((l_W[52]).bit_shift_left (13))))))).bit_xor (((((l_W[52]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (10)))) + l_W[47]) + ((((((((((l_W[39]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (7))).bit_or (((l_W[39]).bit_shift_left (25))))).bit_xor (((((((l_W[39]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (18))).bit_or (((l_W[39]).bit_shift_left (14))))))).bit_xor (((((l_W[39]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (3))))) + l_W[38]
			l_temp1 := (((l_B + ((((((((((l_G).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_G).bit_shift_left (26))))).bit_xor (((((((l_G).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_G).bit_shift_left (21))))))).bit_xor (((((((l_G).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_G).bit_shift_left (7))))))) + ((l_A).bit_xor (((l_G).bit_and (((l_H).bit_xor (l_A))))))) + {NATURAL_32} 1_537_002_063) + l_W[54]
			l_temp2 := ((((((((((l_C).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_C).bit_shift_left (30))))).bit_xor (((((((l_C).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_C).bit_shift_left (19))))))).bit_xor (((((((l_C).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_C).bit_shift_left (10)))))) + ((((l_C).bit_and (l_D))).bit_or (((l_E).bit_and (((l_C).bit_or (l_D))))))
			l_F := l_F + l_temp1
			l_B := l_temp1 + l_temp2
			l_W[55] := ((((((((((((l_W[53]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (17))).bit_or (((l_W[53]).bit_shift_left (15))))).bit_xor (((((((l_W[53]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (19))).bit_or (((l_W[53]).bit_shift_left (13))))))).bit_xor (((((l_W[53]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (10)))) + l_W[48]) + ((((((((((l_W[40]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (7))).bit_or (((l_W[40]).bit_shift_left (25))))).bit_xor (((((((l_W[40]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (18))).bit_or (((l_W[40]).bit_shift_left (14))))))).bit_xor (((((l_W[40]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (3))))) + l_W[39]
			l_temp1 := (((l_A + ((((((((((l_F).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_F).bit_shift_left (26))))).bit_xor (((((((l_F).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_F).bit_shift_left (21))))))).bit_xor (((((((l_F).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_F).bit_shift_left (7))))))) + ((l_H).bit_xor (((l_F).bit_and (((l_G).bit_xor (l_H))))))) + {NATURAL_32} 1_747_873_779) + l_W[55]
			l_temp2 := ((((((((((l_B).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_B).bit_shift_left (30))))).bit_xor (((((((l_B).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_B).bit_shift_left (19))))))).bit_xor (((((((l_B).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_B).bit_shift_left (10)))))) + ((((l_B).bit_and (l_C))).bit_or (((l_D).bit_and (((l_B).bit_or (l_C))))))
			l_E := l_E + l_temp1
			l_A := l_temp1 + l_temp2
			l_W[56] := ((((((((((((l_W[54]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (17))).bit_or (((l_W[54]).bit_shift_left (15))))).bit_xor (((((((l_W[54]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (19))).bit_or (((l_W[54]).bit_shift_left (13))))))).bit_xor (((((l_W[54]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (10)))) + l_W[49]) + ((((((((((l_W[41]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (7))).bit_or (((l_W[41]).bit_shift_left (25))))).bit_xor (((((((l_W[41]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (18))).bit_or (((l_W[41]).bit_shift_left (14))))))).bit_xor (((((l_W[41]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (3))))) + l_W[40]
			l_temp1 := (((l_H + ((((((((((l_E).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_E).bit_shift_left (26))))).bit_xor (((((((l_E).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_E).bit_shift_left (21))))))).bit_xor (((((((l_E).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_E).bit_shift_left (7))))))) + ((l_G).bit_xor (((l_E).bit_and (((l_F).bit_xor (l_G))))))) + {NATURAL_32} 1_955_562_222) + l_W[56]
			l_temp2 := ((((((((((l_A).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_A).bit_shift_left (30))))).bit_xor (((((((l_A).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_A).bit_shift_left (19))))))).bit_xor (((((((l_A).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_A).bit_shift_left (10)))))) + ((((l_A).bit_and (l_B))).bit_or (((l_C).bit_and (((l_A).bit_or (l_B))))))
			l_D := l_D + l_temp1
			l_H := l_temp1 + l_temp2
			l_W[57] := ((((((((((((l_W[55]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (17))).bit_or (((l_W[55]).bit_shift_left (15))))).bit_xor (((((((l_W[55]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (19))).bit_or (((l_W[55]).bit_shift_left (13))))))).bit_xor (((((l_W[55]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (10)))) + l_W[50]) + ((((((((((l_W[42]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (7))).bit_or (((l_W[42]).bit_shift_left (25))))).bit_xor (((((((l_W[42]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (18))).bit_or (((l_W[42]).bit_shift_left (14))))))).bit_xor (((((l_W[42]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (3))))) + l_W[41]
			l_temp1 := (((l_G + ((((((((((l_D).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_D).bit_shift_left (26))))).bit_xor (((((((l_D).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_D).bit_shift_left (21))))))).bit_xor (((((((l_D).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_D).bit_shift_left (7))))))) + ((l_F).bit_xor (((l_D).bit_and (((l_E).bit_xor (l_F))))))) + {NATURAL_32} 2_024_104_815) + l_W[57]
			l_temp2 := ((((((((((l_H).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_H).bit_shift_left (30))))).bit_xor (((((((l_H).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_H).bit_shift_left (19))))))).bit_xor (((((((l_H).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_H).bit_shift_left (10)))))) + ((((l_H).bit_and (l_A))).bit_or (((l_B).bit_and (((l_H).bit_or (l_A))))))
			l_C := l_C + l_temp1
			l_G := l_temp1 + l_temp2
			l_W[58] := ((((((((((((l_W[56]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (17))).bit_or (((l_W[56]).bit_shift_left (15))))).bit_xor (((((((l_W[56]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (19))).bit_or (((l_W[56]).bit_shift_left (13))))))).bit_xor (((((l_W[56]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (10)))) + l_W[51]) + ((((((((((l_W[43]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (7))).bit_or (((l_W[43]).bit_shift_left (25))))).bit_xor (((((((l_W[43]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (18))).bit_or (((l_W[43]).bit_shift_left (14))))))).bit_xor (((((l_W[43]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (3))))) + l_W[42]
			l_temp1 := (((l_F + ((((((((((l_C).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_C).bit_shift_left (26))))).bit_xor (((((((l_C).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_C).bit_shift_left (21))))))).bit_xor (((((((l_C).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_C).bit_shift_left (7))))))) + ((l_E).bit_xor (((l_C).bit_and (((l_D).bit_xor (l_E))))))) + {NATURAL_32} 2_227_730_452) + l_W[58]
			l_temp2 := ((((((((((l_G).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_G).bit_shift_left (30))))).bit_xor (((((((l_G).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_G).bit_shift_left (19))))))).bit_xor (((((((l_G).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_G).bit_shift_left (10)))))) + ((((l_G).bit_and (l_H))).bit_or (((l_A).bit_and (((l_G).bit_or (l_H))))))
			l_B := l_B + l_temp1
			l_F := l_temp1 + l_temp2
			l_W[59] := ((((((((((((l_W[57]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (17))).bit_or (((l_W[57]).bit_shift_left (15))))).bit_xor (((((((l_W[57]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (19))).bit_or (((l_W[57]).bit_shift_left (13))))))).bit_xor (((((l_W[57]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (10)))) + l_W[52]) + ((((((((((l_W[44]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (7))).bit_or (((l_W[44]).bit_shift_left (25))))).bit_xor (((((((l_W[44]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (18))).bit_or (((l_W[44]).bit_shift_left (14))))))).bit_xor (((((l_W[44]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (3))))) + l_W[43]
			l_temp1 := (((l_E + ((((((((((l_B).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_B).bit_shift_left (26))))).bit_xor (((((((l_B).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_B).bit_shift_left (21))))))).bit_xor (((((((l_B).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_B).bit_shift_left (7))))))) + ((l_D).bit_xor (((l_B).bit_and (((l_C).bit_xor (l_D))))))) + {NATURAL_32} 2_361_852_424) + l_W[59]
			l_temp2 := ((((((((((l_F).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_F).bit_shift_left (30))))).bit_xor (((((((l_F).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_F).bit_shift_left (19))))))).bit_xor (((((((l_F).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_F).bit_shift_left (10)))))) + ((((l_F).bit_and (l_G))).bit_or (((l_H).bit_and (((l_F).bit_or (l_G))))))
			l_A := l_A + l_temp1
			l_E := l_temp1 + l_temp2
			l_W[60] := ((((((((((((l_W[58]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (17))).bit_or (((l_W[58]).bit_shift_left (15))))).bit_xor (((((((l_W[58]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (19))).bit_or (((l_W[58]).bit_shift_left (13))))))).bit_xor (((((l_W[58]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (10)))) + l_W[53]) + ((((((((((l_W[45]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (7))).bit_or (((l_W[45]).bit_shift_left (25))))).bit_xor (((((((l_W[45]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (18))).bit_or (((l_W[45]).bit_shift_left (14))))))).bit_xor (((((l_W[45]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (3))))) + l_W[44]
			l_temp1 := (((l_D + ((((((((((l_A).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_A).bit_shift_left (26))))).bit_xor (((((((l_A).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_A).bit_shift_left (21))))))).bit_xor (((((((l_A).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_A).bit_shift_left (7))))))) + ((l_C).bit_xor (((l_A).bit_and (((l_B).bit_xor (l_C))))))) + {NATURAL_32} 2_428_436_474) + l_W[60]
			l_temp2 := ((((((((((l_E).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_E).bit_shift_left (30))))).bit_xor (((((((l_E).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_E).bit_shift_left (19))))))).bit_xor (((((((l_E).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_E).bit_shift_left (10)))))) + ((((l_E).bit_and (l_F))).bit_or (((l_G).bit_and (((l_E).bit_or (l_F))))))
			l_H := l_H + l_temp1
			l_D := l_temp1 + l_temp2
			l_W[61] := ((((((((((((l_W[59]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (17))).bit_or (((l_W[59]).bit_shift_left (15))))).bit_xor (((((((l_W[59]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (19))).bit_or (((l_W[59]).bit_shift_left (13))))))).bit_xor (((((l_W[59]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (10)))) + l_W[54]) + ((((((((((l_W[46]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (7))).bit_or (((l_W[46]).bit_shift_left (25))))).bit_xor (((((((l_W[46]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (18))).bit_or (((l_W[46]).bit_shift_left (14))))))).bit_xor (((((l_W[46]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (3))))) + l_W[45]
			l_temp1 := (((l_C + ((((((((((l_H).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_H).bit_shift_left (26))))).bit_xor (((((((l_H).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_H).bit_shift_left (21))))))).bit_xor (((((((l_H).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_H).bit_shift_left (7))))))) + ((l_B).bit_xor (((l_H).bit_and (((l_A).bit_xor (l_B))))))) + {NATURAL_32} 2_756_734_187) + l_W[61]
			l_temp2 := ((((((((((l_D).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_D).bit_shift_left (30))))).bit_xor (((((((l_D).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_D).bit_shift_left (19))))))).bit_xor (((((((l_D).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_D).bit_shift_left (10)))))) + ((((l_D).bit_and (l_E))).bit_or (((l_F).bit_and (((l_D).bit_or (l_E))))))
			l_G := l_G + l_temp1
			l_C := l_temp1 + l_temp2
			l_W[62] := ((((((((((((l_W[60]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (17))).bit_or (((l_W[60]).bit_shift_left (15))))).bit_xor (((((((l_W[60]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (19))).bit_or (((l_W[60]).bit_shift_left (13))))))).bit_xor (((((l_W[60]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (10)))) + l_W[55]) + ((((((((((l_W[47]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (7))).bit_or (((l_W[47]).bit_shift_left (25))))).bit_xor (((((((l_W[47]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (18))).bit_or (((l_W[47]).bit_shift_left (14))))))).bit_xor (((((l_W[47]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (3))))) + l_W[46]
			l_temp1 := (((l_B + ((((((((((l_G).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_G).bit_shift_left (26))))).bit_xor (((((((l_G).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_G).bit_shift_left (21))))))).bit_xor (((((((l_G).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_G).bit_shift_left (7))))))) + ((l_A).bit_xor (((l_G).bit_and (((l_H).bit_xor (l_A))))))) + {NATURAL_32} 3_204_031_479) + l_W[62]
			l_temp2 := ((((((((((l_C).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_C).bit_shift_left (30))))).bit_xor (((((((l_C).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_C).bit_shift_left (19))))))).bit_xor (((((((l_C).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_C).bit_shift_left (10)))))) + ((((l_C).bit_and (l_D))).bit_or (((l_E).bit_and (((l_C).bit_or (l_D))))))
			l_F := l_F + l_temp1
			l_B := l_temp1 + l_temp2
			l_W[63] := ((((((((((((l_W[61]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (17))).bit_or (((l_W[61]).bit_shift_left (15))))).bit_xor (((((((l_W[61]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (19))).bit_or (((l_W[61]).bit_shift_left (13))))))).bit_xor (((((l_W[61]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (10)))) + l_W[56]) + ((((((((((l_W[48]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (7))).bit_or (((l_W[48]).bit_shift_left (25))))).bit_xor (((((((l_W[48]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (18))).bit_or (((l_W[48]).bit_shift_left (14))))))).bit_xor (((((l_W[48]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (3))))) + l_W[47]
			l_temp1 := (((l_A + ((((((((((l_F).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_F).bit_shift_left (26))))).bit_xor (((((((l_F).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_F).bit_shift_left (21))))))).bit_xor (((((((l_F).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_F).bit_shift_left (7))))))) + ((l_H).bit_xor (((l_F).bit_and (((l_G).bit_xor (l_H))))))) + {NATURAL_32} 3_329_325_298) + l_W[63]
			l_temp2 := ((((((((((l_B).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_B).bit_shift_left (30))))).bit_xor (((((((l_B).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_B).bit_shift_left (19))))))).bit_xor (((((((l_B).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_B).bit_shift_left (10)))))) + ((((l_B).bit_and (l_C))).bit_or (((l_D).bit_and (((l_B).bit_or (l_C))))))
			l_E := l_E + l_temp1
			l_A := l_temp1 + l_temp2
			a_ctx.item.state[0] := a_ctx.item.state[0] + l_A
			a_ctx.item.state[1] := a_ctx.item.state[1] + l_B
			a_ctx.item.state[2] := a_ctx.item.state[2] + l_C
			a_ctx.item.state[3] := a_ctx.item.state[3] + l_D
			a_ctx.item.state[4] := a_ctx.item.state[4] + l_E
			a_ctx.item.state[5] := a_ctx.item.state[5] + l_F
			a_ctx.item.state[6] := a_ctx.item.state[6] + l_G
			a_ctx.item.state[7] := a_ctx.item.state[7] + l_H
		end

	sha256_self_test: INTEGER_32
		local
			l_i___0: INTEGER_32
			l_j: INTEGER_32
			l_output: CE_ARRAY [INTEGER_8]
			l_ctx: CE_ARRAY [PD_CONTEXT_SHA256_T]
			l_buf___6: CE_ARRAY [NATURAL_8]
			l_sha256sum: CE_ARRAY [NATURAL_8]
			l_hexit___0: CE_POINTER [NATURAL_8]
			l_tmp: INTEGER_32
			l_tmp___2: INTEGER_32
			js: JUMP_STATE
		do
			create l_output.make (<<65>>)
			create l_ctx.make (<<1>>)
			create l_buf___6.make (<<1_000>>)
			create l_sha256sum.make (<<32>>)
			create l_hexit___0.make

			if sha256_self_tested > 0 then
				if failures > 0 then
					l_tmp := 0
				else
					l_tmp := 1
				end
				Result := l_tmp
				js.return
			end
			if not js.return_called then
				sha256_self_tested := 1
				l_i___0 := 0
				from
				until
					not (not js.loop_done and then (l_i___0 < 3))
				loop
					if l_i___0 < 2 then
						l_hexit___0 := sha256_bytes (create {CE_POINTER [NATURAL_8]}.make_cast (sha_self_test_msg[l_i___0]), (strlen (sha_self_test_msg[l_i___0])).to_integer_32, create {CE_POINTER [NATURAL_8]}.make_shared (default_pointer), 0)
						strcpy (l_output, create {CE_POINTER [INTEGER_8]}.make_cast (l_hexit___0)).do_nothing
					else
						sha256_start (l_ctx)
						memset (create {CE_POINTER [ANY]}.make_cast (l_buf___6), ('a').code, (1_000).to_natural_32).do_nothing
						l_j := 0
						from
						until
							not (l_j < 1_000)
						loop
							sha256_update (l_ctx, l_buf___6, (1_000).to_natural_32)
							l_j := l_j + 1
						end
						sha256_finish (l_ctx, create {CE_POINTER [NATURAL_8]}.make_cast (l_sha256sum))
						l_j := 0
						from
						until
							not (l_j < 32)
						loop
							sprintf ([(l_output + l_j * 2), ce_string ("%%02x"), (l_sha256sum[l_j]).to_integer_32]).do_nothing
							l_j := l_j + 1
						end
					end
					if (memcmp (create {CE_POINTER [ANY]}.make_cast (l_output), create {CE_POINTER [ANY]}.make_cast (sha_self_test_vector[l_i___0]), (64).to_natural_32)).to_boolean then
						failures := failures + 1
						l_output[(({PLATFORM}.Integer_8_bytes.to_natural_32 * (65).to_natural_32) - {NATURAL_32} 1).to_integer_32] := (('%/0/').code).to_integer_8
					end
					l_i___0 := l_i___0 + 1
				end
				if failures > 0 then
					l_tmp___2 := 0
				else
					l_tmp___2 := 1
				end
				Result := l_tmp___2
			end
		end

	sha256_start (a_ctx: CE_POINTER [PD_CONTEXT_SHA256_T])
		do
			a_ctx.item.total[0] := (0).to_natural_32
			a_ctx.item.total[1] := (0).to_natural_32
			a_ctx.item.state[0] := (1_779_033_703).to_natural_32
			a_ctx.item.state[1] := {NATURAL_32} 3_144_134_277
			a_ctx.item.state[2] := (1_013_904_242).to_natural_32
			a_ctx.item.state[3] := {NATURAL_32} 2_773_480_762
			a_ctx.item.state[4] := (1_359_893_119).to_natural_32
			a_ctx.item.state[5] := {NATURAL_32} 2_600_822_924
			a_ctx.item.state[6] := (528_734_635).to_natural_32
			a_ctx.item.state[7] := (1_541_459_225).to_natural_32
		end

	sha256_update (a_ctx: CE_POINTER [PD_CONTEXT_SHA256_T]; a_input: CE_POINTER [NATURAL_8]; a_length: NATURAL_32)
		local
			l_input: CE_POINTER [NATURAL_8]
			l_length: NATURAL_32
			l_left: NATURAL_32
			l_fill: NATURAL_32
			js: JUMP_STATE
		do
			create l_input.make

			l_input := a_input
			l_length := a_length
			if l_length = {NATURAL_32} 0 then
				js.return
			end
			if not js.return_called then
				l_left := (a_ctx.item.total[0]).bit_and ({NATURAL_32} 63)
				l_fill := {NATURAL_32} 64 - l_left
				a_ctx.item.total[0] := a_ctx.item.total[0] + l_length
				a_ctx.item.total[0] := (a_ctx.item.total[0]).bit_and (({NATURAL_32} 4_294_967_295))
				if a_ctx.item.total[0] < l_length then
					a_ctx.item.total[1] := (a_ctx.item.total[1]) + 1
				end
				if l_left.to_boolean then
					if l_length >= l_fill then
						memcpy (create {CE_POINTER [ANY]}.make_cast ((a_ctx.item.buffer + l_left.to_integer_32)), create {CE_POINTER [ANY]}.make_cast (l_input), l_fill).do_nothing
						sha256_process (a_ctx, create {CE_POINTER [NATURAL_8]}.make_cast ((a_ctx.item.buffer)))
						l_length := l_length - l_fill
						l_input := l_input + l_fill.to_integer_32
						l_left := (0).to_natural_32
					end
				end
				from
				until
					not (l_length >= {NATURAL_32} 64)
				loop
					sha256_process (a_ctx, l_input)
					l_length := l_length - ({NATURAL_32} 64)
					l_input := l_input + (64)
				end
				if l_length.to_boolean then
					memcpy (create {CE_POINTER [ANY]}.make_cast ((a_ctx.item.buffer + l_left.to_integer_32)), create {CE_POINTER [ANY]}.make_cast (l_input), l_length).do_nothing
				end
			end
		end

	sha2_seed (a_header: CE_POINTER [NATURAL_8]; a_header_len: INTEGER_32; a_salt: CE_POINTER [NATURAL_8]; a_salt_len: INTEGER_32)
		local
			l_i___0: INTEGER_32
			l_sha256sum: CE_ARRAY [NATURAL_8]
			l_ctx: CE_ARRAY [PD_CONTEXT_SHA256_T]
			l_tmp___0: NATURAL_32
		do
			create l_sha256sum.make (<<32>>)
			create l_ctx.make (<<1>>)

			srand ((get_some_time))
			l_i___0 := 0
			from
			until
				not (l_i___0 < ({PLATFORM}.Natural_8_bytes.to_natural_32 * (1_000).to_natural_32).to_integer_32 - 1)
			loop
				l_tmp___0 := get_some_time
				random_data[l_i___0] := ((((l_tmp___0).bit_xor ((rand).to_natural_32))).bit_and ({NATURAL_32} 255)).to_natural_8
				l_i___0 := l_i___0 + 1
			end
			sha256_start (l_ctx)
			sha256_update (l_ctx, random_data, ({PLATFORM}.Natural_8_bytes.to_natural_32 * (1_000).to_natural_32))
			sha256_finish (l_ctx, create {CE_POINTER [NATURAL_8]}.make_cast (l_sha256sum))
			l_i___0 := 0
			from
			until
				not (l_i___0 < a_header_len)
			loop
				a_header[l_i___0] := l_sha256sum[(l_i___0.to_natural_32 \\ ({PLATFORM}.Natural_8_bytes.to_natural_32 * (32).to_natural_32)).to_integer_32]
				l_i___0 := l_i___0 + 1
			end
			if not a_salt.is_void then
				l_i___0 := 0
				from
				until
					not (l_i___0 < a_salt_len)
				loop
					a_salt[l_i___0] := l_sha256sum[((l_i___0 + a_header_len).to_natural_32 \\ ({PLATFORM}.Natural_8_bytes.to_natural_32 * (32).to_natural_32)).to_integer_32]
					l_i___0 := l_i___0 + 1
				end
			end
		end

end
