class
	SD_D

inherit
	CE_CLASS
		redefine
			make,
			copy,
			is_equal
		end

create
	make,
	make_shared

feature {NONE}

	make
		do
			make_memory -- union; back with memory for correct semantics
		end

feature {ANY} -- fields

	char_data: INTEGER_8 assign set_char_data

	uchar_data: NATURAL_8 assign set_uchar_data

	bool_data: INTEGER_32 assign set_bool_data

	int_data: INTEGER_32 assign set_int_data

	uint_data: NATURAL_32 assign set_uint_data

	long_data: INTEGER_32 assign set_long_data

	ulong_data: NATURAL_32 assign set_ulong_data

	float_data: REAL_32 assign set_float_data

	double_data: REAL_64 assign set_double_data

	string_data: CE_POINTER [INTEGER_8] assign set_string_data

	object_data: CE_POINTER [SD_GTKOBJECT] assign set_object_data

	pointer_data: CE_POINTER [ANY] assign set_pointer_data

	signal_data: SD_SIGNAL_DATA assign set_signal_data

feature {ANY} -- setters

	set_char_data (a_char_data: INTEGER_8)
		do
			char_data := a_char_data
			update_memory_field ("char_data")
		end

	set_uchar_data (a_uchar_data: NATURAL_8)
		do
			uchar_data := a_uchar_data
			update_memory_field ("uchar_data")
		end

	set_bool_data (a_bool_data: INTEGER_32)
		do
			bool_data := a_bool_data
			update_memory_field ("bool_data")
		end

	set_int_data (a_int_data: INTEGER_32)
		do
			int_data := a_int_data
			update_memory_field ("int_data")
		end

	set_uint_data (a_uint_data: NATURAL_32)
		do
			uint_data := a_uint_data
			update_memory_field ("uint_data")
		end

	set_long_data (a_long_data: INTEGER_32)
		do
			long_data := a_long_data
			update_memory_field ("long_data")
		end

	set_ulong_data (a_ulong_data: NATURAL_32)
		do
			ulong_data := a_ulong_data
			update_memory_field ("ulong_data")
		end

	set_float_data (a_float_data: REAL_32)
		do
			float_data := a_float_data
			update_memory_field ("float_data")
		end

	set_double_data (a_double_data: REAL_64)
		do
			double_data := a_double_data
			update_memory_field ("double_data")
		end

	set_string_data (a_string_data: CE_POINTER [INTEGER_8])
		do
			string_data := a_string_data
			update_memory_field ("string_data")
		end

	set_object_data (a_object_data: CE_POINTER [SD_GTKOBJECT])
		do
			object_data := a_object_data
			update_memory_field ("object_data")
		end

	set_pointer_data (a_pointer_data: CE_POINTER [ANY])
		do
			pointer_data := a_pointer_data
			update_memory_field ("pointer_data")
		end

	set_signal_data (a_signal_data: SD_SIGNAL_DATA)
		do
			signal_data := a_signal_data
			update_memory_field ("signal_data")
		end

feature {ANY}

	copy (a_other: like Current)
		do
			char_data := a_other.char_data
			uchar_data := a_other.uchar_data
			bool_data := a_other.bool_data
			int_data := a_other.int_data
			uint_data := a_other.uint_data
			long_data := a_other.long_data
			ulong_data := a_other.ulong_data
			float_data := a_other.float_data
			double_data := a_other.double_data
			string_data := a_other.string_data
			object_data := a_other.object_data
			pointer_data := a_other.pointer_data
			signal_data := a_other.signal_data.twin
			create_memory_copy -- union; back with memory for correct semantics
		end

	is_equal (a_other: like Current): BOOLEAN
		do
			Result :=
				char_data = a_other.char_data and
				uchar_data = a_other.uchar_data and
				bool_data = a_other.bool_data and
				int_data = a_other.int_data and
				uint_data = a_other.uint_data and
				long_data = a_other.long_data and
				ulong_data = a_other.ulong_data and
				float_data = a_other.float_data and
				double_data = a_other.double_data and
				string_data.is_equal (a_other.string_data) and
				object_data.is_equal (a_other.object_data) and
				pointer_data.is_equal (a_other.pointer_data) and
				signal_data.is_equal (a_other.signal_data)
		end

feature {NONE}

	structure_info: CE_STRUCTURE
		once ("PROCESS")
			create Result.make_union (<<
					create {CE_STRUCTURE_FIELD}.make ("char_data"),
					create {CE_STRUCTURE_FIELD}.make ("uchar_data"),
					create {CE_STRUCTURE_FIELD}.make ("bool_data"),
					create {CE_STRUCTURE_FIELD}.make ("int_data"),
					create {CE_STRUCTURE_FIELD}.make ("uint_data"),
					create {CE_STRUCTURE_FIELD}.make ("long_data"),
					create {CE_STRUCTURE_FIELD}.make ("ulong_data"),
					create {CE_STRUCTURE_FIELD}.make ("float_data"),
					create {CE_STRUCTURE_FIELD}.make ("double_data"),
					create {CE_STRUCTURE_FIELD}.make ("string_data"),
					create {CE_STRUCTURE_FIELD}.make ("object_data"),
					create {CE_STRUCTURE_FIELD}.make ("pointer_data"),
					create {CE_STRUCTURE_FIELD}.make ("signal_data")
				>>)
			init_structure_info
		end

end
