class
	SD_GTKPLUG

inherit
	CE_CLASS
		redefine
			make,
			copy,
			is_equal,
			bitfield_helper
		end

create
	make,
	make_shared

feature {NONE}

	make
		do
			make_memory -- has bitfields; back with memory for correct semantics
		end

feature {ANY} -- fields

	window: SD_GTKWINDOW assign set_window

	socket_window: CE_POINTER [SD_GDKDRAWABLE] assign set_socket_window

	modality_window: CE_POINTER [SD_GTKWIDGET] assign set_modality_window

	modality_group: CE_POINTER [SD_GTKWINDOWGROUP] assign set_modality_group

	grabbed_keys: CE_POINTER [ANY] assign set_grabbed_keys

	same_app: NATURAL_32 assign set_same_app

feature {ANY} -- setters

	set_window (a_window: SD_GTKWINDOW)
		do
			window := a_window
			update_memory_field ("window")
		end

	set_socket_window (a_socket_window: CE_POINTER [SD_GDKDRAWABLE])
		do
			socket_window := a_socket_window
			update_memory_field ("socket_window")
		end

	set_modality_window (a_modality_window: CE_POINTER [SD_GTKWIDGET])
		do
			modality_window := a_modality_window
			update_memory_field ("modality_window")
		end

	set_modality_group (a_modality_group: CE_POINTER [SD_GTKWINDOWGROUP])
		do
			modality_group := a_modality_group
			update_memory_field ("modality_group")
		end

	set_grabbed_keys (a_grabbed_keys: CE_POINTER [ANY])
		do
			grabbed_keys := a_grabbed_keys
			update_memory_field ("grabbed_keys")
		end

	set_same_app (a_same_app: NATURAL_32)
		do
			same_app := a_same_app
			update_memory_field ("same_app")
		end

feature {ANY}

	copy (a_other: like Current)
		do
			window := a_other.window.twin
			socket_window := a_other.socket_window
			modality_window := a_other.modality_window
			modality_group := a_other.modality_group
			grabbed_keys := a_other.grabbed_keys
			same_app := a_other.same_app
			create_memory_copy -- has bitfields; back with memory for correct semantics
		end

	is_equal (a_other: like Current): BOOLEAN
		do
			Result :=
				window.is_equal (a_other.window) and
				socket_window.is_equal (a_other.socket_window) and
				modality_window.is_equal (a_other.modality_window) and
				modality_group.is_equal (a_other.modality_group) and
				grabbed_keys.is_equal (a_other.grabbed_keys) and
				same_app = a_other.same_app
		end

feature {NONE}

	structure_info: CE_STRUCTURE
		once ("PROCESS")
			create Result.make_struct (<<
					create {CE_STRUCTURE_FIELD}.make ("window"),
					create {CE_STRUCTURE_FIELD}.make ("socket_window"),
					create {CE_STRUCTURE_FIELD}.make ("modality_window"),
					create {CE_STRUCTURE_FIELD}.make ("modality_group"),
					create {CE_STRUCTURE_FIELD}.make ("grabbed_keys"),
					create {CE_STRUCTURE_FIELD}.make_bitfield ("same_app")
				>>)
			init_structure_info
		end

	bitfield_helper (a_operation: INTEGER_32; a_field_name, a_struct_pointer: POINTER; a_value: NATURAL_64): NATURAL_64
		external
			"C inline use <gtk/gtk.h>"
		alias
			"[
				char *field_name = (char *)$a_field_name;
				if($a_operation == 1) {
					if(field_name == (char *)0) return sizeof(struct _GtkPlug);
				} else {
					struct _GtkPlug *p = (struct _GtkPlug *)$a_struct_pointer;
					if($a_operation == 2) {
						     if(strcmp(field_name, "same_app") == 0) p->same_app = (unsigned int)$a_value;
					} else if($a_operation == 3) {
						if(strcmp(field_name, "same_app") == 0) return p->same_app;
					}
				}
			]"
		end

end
