class
	PD_SESSIONHANDLE

inherit
	CE_CLASS
		redefine
			make,
			copy,
			is_equal
		end

create
	make,
	make_shared

feature {NONE}

	make
		do
			create dns.make
			create multi.make
			create multi_pos.make
			create share.make
			create req.make
			create set.make
			create change.make
			create cookies.make
			create progress.make
			create state.make
			create wildcard.make
			create info.make
		end

feature {ANY} -- fields

	dns: PD_NAMES assign set_dns

	multi: CE_POINTER [PD_CURL_MULTI] assign set_multi

	multi_pos: CE_POINTER [PD_CURL_ONE_EASY] assign set_multi_pos

	share: CE_POINTER [PD_CURL_SHARE] assign set_share

	req: PD_SINGLEREQUEST assign set_req

	set: PD_USERDEFINED assign set_set

	change: PD_DYNAMICSTATIC assign set_change

	cookies: CE_POINTER [PD_COOKIEINFO] assign set_cookies

	progress: PD_PROGRESS assign set_progress

	state: PD_URLSTATE assign set_state

	wildcard: PD_WILDCARDDATA assign set_wildcard

	info: PD_PUREINFO assign set_info

	magic: NATURAL_32 assign set_magic

feature {ANY} -- setters

	set_dns (a_dns: PD_NAMES)
		do
			dns := a_dns
			update_memory_field ("dns")
		end

	set_multi (a_multi: CE_POINTER [PD_CURL_MULTI])
		do
			multi := a_multi
			update_memory_field ("multi")
		end

	set_multi_pos (a_multi_pos: CE_POINTER [PD_CURL_ONE_EASY])
		do
			multi_pos := a_multi_pos
			update_memory_field ("multi_pos")
		end

	set_share (a_share: CE_POINTER [PD_CURL_SHARE])
		do
			share := a_share
			update_memory_field ("share")
		end

	set_req (a_req: PD_SINGLEREQUEST)
		do
			req := a_req
			update_memory_field ("req")
		end

	set_set (a_set: PD_USERDEFINED)
		do
			set := a_set
			update_memory_field ("set")
		end

	set_change (a_change: PD_DYNAMICSTATIC)
		do
			change := a_change
			update_memory_field ("change")
		end

	set_cookies (a_cookies: CE_POINTER [PD_COOKIEINFO])
		do
			cookies := a_cookies
			update_memory_field ("cookies")
		end

	set_progress (a_progress: PD_PROGRESS)
		do
			progress := a_progress
			update_memory_field ("progress")
		end

	set_state (a_state: PD_URLSTATE)
		do
			state := a_state
			update_memory_field ("state")
		end

	set_wildcard (a_wildcard: PD_WILDCARDDATA)
		do
			wildcard := a_wildcard
			update_memory_field ("wildcard")
		end

	set_info (a_info: PD_PUREINFO)
		do
			info := a_info
			update_memory_field ("info")
		end

	set_magic (a_magic: NATURAL_32)
		do
			magic := a_magic
			update_memory_field ("magic")
		end

feature {ANY}

	copy (a_other: like Current)
		do
			dns := a_other.dns.twin
			multi := a_other.multi
			multi_pos := a_other.multi_pos
			share := a_other.share
			req := a_other.req.twin
			set := a_other.set.twin
			change := a_other.change.twin
			cookies := a_other.cookies
			progress := a_other.progress.twin
			state := a_other.state.twin
			wildcard := a_other.wildcard.twin
			info := a_other.info.twin
			magic := a_other.magic
		end

	is_equal (a_other: like Current): BOOLEAN
		do
			Result :=
				dns.is_equal (a_other.dns) and
				multi.is_equal (a_other.multi) and
				multi_pos.is_equal (a_other.multi_pos) and
				share.is_equal (a_other.share) and
				req.is_equal (a_other.req) and
				set.is_equal (a_other.set) and
				change.is_equal (a_other.change) and
				cookies.is_equal (a_other.cookies) and
				progress.is_equal (a_other.progress) and
				state.is_equal (a_other.state) and
				wildcard.is_equal (a_other.wildcard) and
				info.is_equal (a_other.info) and
				magic = a_other.magic
		end

feature {NONE}

	structure_info: CE_STRUCTURE
		once ("PROCESS")
			create Result.make_struct (<<
					create {CE_STRUCTURE_FIELD}.make ("dns"),
					create {CE_STRUCTURE_FIELD}.make ("multi"),
					create {CE_STRUCTURE_FIELD}.make ("multi_pos"),
					create {CE_STRUCTURE_FIELD}.make ("share"),
					create {CE_STRUCTURE_FIELD}.make ("req"),
					create {CE_STRUCTURE_FIELD}.make ("set"),
					create {CE_STRUCTURE_FIELD}.make ("change"),
					create {CE_STRUCTURE_FIELD}.make ("cookies"),
					create {CE_STRUCTURE_FIELD}.make ("progress"),
					create {CE_STRUCTURE_FIELD}.make ("state"),
					create {CE_STRUCTURE_FIELD}.make ("wildcard"),
					create {CE_STRUCTURE_FIELD}.make ("info"),
					create {CE_STRUCTURE_FIELD}.make ("magic")
				>>)
			init_structure_info
		end

end
