class
	P_CONNECT

inherit
	LIBCURL_DATA
	S_STDIO
	S_STDLIB
	S_STRING
	S_CTYPE
	S_SYS_STAT
	S_TIME
	S_UNISTD
	S_FCNTL
	S_SYS_TIME
	S_ARPA_INET
	S_SIGNAL
	S_LIBGEN
	S_SYS_SOCKET
	S_BITS_ERRNO
	S_NETINET_IN
	S_IFADDRS
	S_PWD
	S_SYS_POLL
	S_NETDB

feature {ANY} -- routines

	Curl_connecthost (a_conn: CE_POINTER [PD_CONNECTDATA]; a_remotehost: CE_POINTER [PD_CURL_DNS_ENTRY]; a_sockconn: CE_POINTER [INTEGER_32]; a_addr: CE_POINTER [CE_POINTER [PD_CURL_ADDRINFO]]; a_connected: CE_POINTER [BOOLEAN]): NATURAL_32
		local
			l_data: CE_POINTER [PD_SESSIONHANDLE]
			l_sockfd: CE_ARRAY [INTEGER_32]
			l_aliasindex: INTEGER_32
			l_num_addr: INTEGER_32
			l_ai: CE_POINTER [PD_CURL_ADDRINFO]
			l_curr_addr: CE_POINTER [PD_CURL_ADDRINFO]
			l_after: SD_TIMEVAL
			l_before: CE_ARRAY [SD_TIMEVAL]
			l_timeout_ms: INTEGER_32
			l_timeout_per_addr: INTEGER_32
			l_res: NATURAL_32
			js: JUMP_STATE
		do
			create l_data.make
			create l_sockfd.make (<<1>>)
			create l_ai.make
			create l_curr_addr.make
			create l_after.make
			create l_before.make (<<1>>)

			l_data := a_conn.item.data
			l_sockfd.item := -1
			l_before.item := (timeval_inst.curlx_tvnow).twin
			a_connected.item := (0).to_boolean
			l_timeout_ms := Curl_timeleft (a_conn, l_before, (1).to_boolean)
			if l_timeout_ms < 0 then
				sendf_inst.Curl_failf ([l_data, ce_string ("Connection time-out")])
				Result := ({PC_CURLCODE}.CURLE_OPERATION_TIMEDOUT)
				js.return
			end
			if not js.return_called then
				l_num_addr := hostip_inst.Curl_num_addresses (a_remotehost.item.addr)
				l_timeout_per_addr := l_timeout_ms // l_num_addr.to_integer_32
				l_ai := a_remotehost.item.addr
				if l_data.item.state.used_interface = {NATURAL_32} 2 then
					l_timeout_per_addr := 0
				end
				l_curr_addr := l_ai
				l_aliasindex := 0
				from
				until
					not (not js.loop_done and then (not l_curr_addr.is_void))
				loop
					l_res := (singleipconnect (a_conn, l_curr_addr, l_timeout_per_addr, l_sockfd, a_connected))
					if l_res.to_boolean then
						Result := l_res
						js.return
					end
					if not js.return_called then
						if l_sockfd.item /= -1 then
							js.break
						end
						if not js.loop_jumped then
							l_after := (timeval_inst.curlx_tvnow).twin
							l_timeout_ms := l_timeout_ms - (timeval_inst.curlx_tvdiff (l_after.twin, (l_before.item).twin))
							if l_timeout_ms < 0 then
								sendf_inst.Curl_failf ([l_data, ce_string ("connect() timed out!")])
								Result := ({PC_CURLCODE}.CURLE_OPERATION_TIMEDOUT)
								js.return
							end
							if not js.return_called then
								l_before.item := l_after.twin
								l_curr_addr := l_curr_addr.item.ai_next
								l_aliasindex := l_aliasindex + 1
							end
						end
					end
				end
			end
			if not js.return_called then
				a_sockconn.item := l_sockfd.item
				if l_sockfd.item = -1 then
					sendf_inst.Curl_failf ([l_data, ce_string ("couldn%'t connect to host")])
					Result := ({PC_CURLCODE}.CURLE_COULDNT_CONNECT)
					js.return
				end
			end
			if not js.return_called then
				if (not a_addr.is_void) then
					a_addr.item := l_curr_addr
				end
				l_data.item.info.numconnects := (l_data.item.info.numconnects) + 1
				Result := ({PC_CURLCODE}.CURLE_OK)
			end
		end

	Curl_getconnectinfo (a_data: CE_POINTER [PD_SESSIONHANDLE]; a_connp: CE_POINTER [CE_POINTER [PD_CONNECTDATA]]): INTEGER_32
		local
			l_sockfd: INTEGER_32
			l_c: CE_POINTER [PD_CONNECTDATA]
			l_buf: CE_ARRAY [INTEGER_8]
			js: JUMP_STATE
		do
			create l_c.make
			create l_buf.make (<<1>>)

			if a_data.item.state.lastconnect /= -1 then
				if not ((a_data.item.state.connc).item.connects[a_data.item.state.lastconnect]).is_void then
					l_c := (a_data.item.state.connc).item.connects[a_data.item.state.lastconnect]
					if (not a_connp.is_void) then
						a_connp.item := l_c
					end
					l_sockfd := l_c.item.sock[0]
					if l_c.item.ssl[0].use then
						Result := (-1)
						js.return
					elseif (recv (l_c.item.sock[0], create {CE_POINTER [ANY]}.make_cast (l_buf), (1).to_natural_32, 2)) = 0 then
						Result := (-1)
						js.return
					end
				else
					Result := (-1)
					js.return
				end
			else
				Result := (-1)
				js.return
			end
			if not js.return_called then
				Result := l_sockfd
			end
		end

	Curl_is_connected (a_conn: CE_POINTER [PD_CONNECTDATA]; a_sockindex: INTEGER_32; a_connected: CE_POINTER [BOOLEAN]): NATURAL_32
		local
			l_rc: INTEGER_32
			l_data: CE_POINTER [PD_SESSIONHANDLE]
			l_code: NATURAL_32
			l_sockfd: INTEGER_32
			l_allow: INTEGER_32
			l_error: CE_ARRAY [INTEGER_32]
			js: JUMP_STATE
		do
			create l_data.make
			create l_error.make (<<1>>)

			l_data := a_conn.item.data
			l_code := {PC_CURLCODE}.CURLE_OK
			l_sockfd := a_conn.item.sock[a_sockindex]
			l_allow := 300_000
			l_error.item := 0
			a_connected.item := (0).to_boolean
			if a_conn.item.bits.tcpconnect then
				a_connected.item := (1).to_boolean
				Result := ({PC_CURLCODE}.CURLE_OK)
				js.return
			end
			if not js.return_called then
				l_allow := Curl_timeleft (a_conn, create {CE_POINTER [SD_TIMEVAL]}.make_shared (default_pointer), (1).to_boolean)
				if l_allow < 0 then
					sendf_inst.Curl_failf ([l_data, ce_string ("Connection time-out")])
					Result := ({PC_CURLCODE}.CURLE_OPERATION_TIMEDOUT)
					js.return
				end
			end
			if not js.return_called then
				l_rc := waitconnect (a_conn, l_sockfd, 0)
				if 1 = l_rc then
					Result := l_code
					js.return
				end
			end
			if not js.return_called then
				if 0 = l_rc then
					if (verifyconnect (l_sockfd, l_error)) then
						a_conn.item.bits.tcpconnect := (1).to_boolean
						a_connected.item := (1).to_boolean
						progress_inst.Curl_pgrsTime (l_data, {PC_TIMERID}.TIMER_CONNECT)
						url_inst.Curl_verboseconnect (a_conn)
						Curl_updateconninfo (a_conn, l_sockfd)
						Result := ({PC_CURLCODE}.CURLE_OK)
						js.return
					end
				elseif 2 = l_rc then
					verifyconnect (l_sockfd, l_error).do_nothing
					sendf_inst.Curl_infof ([l_data, ce_string ("%%s%N"), (strerror_inst.Curl_strerror (a_conn, l_error.item))])
				else
					sendf_inst.Curl_infof ([l_data, ce_string ("Connection failed%N")])
				end
			end
			if not js.return_called then
				if (l_error.item).to_boolean then
					l_data.item.state.os_errno := l_error.item;
					(errno_location).item := l_error.item
				end
				l_code := trynextip (a_conn, a_sockindex, a_connected)
				if l_code.to_boolean then
					l_error.item := (errno_location).item
					l_data.item.state.os_errno := l_error.item
					sendf_inst.Curl_failf ([l_data, ce_string ("Failed connect to %%s:%%ld; %%s"), a_conn.item.host.name, a_conn.item.port, (strerror_inst.Curl_strerror (a_conn, l_error.item))])
				end
				Result := l_code
			end
		end

	Curl_timeleft (a_conn: CE_POINTER [PD_CONNECTDATA]; a_nowp: CE_POINTER [SD_TIMEVAL]; a_duringconnect: BOOLEAN): INTEGER_32
		local
			l_nowp: CE_POINTER [SD_TIMEVAL]
			l_data: CE_POINTER [PD_SESSIONHANDLE]
			l_timeout_set: INTEGER_32
			l_timeout_ms: INTEGER_32
			l_tmp: INTEGER_32
			l_now: CE_ARRAY [SD_TIMEVAL]
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			create l_nowp.make
			create l_data.make
			create l_now.make (<<1>>)

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_nowp := a_nowp
					l_data := a_conn.item.data
					l_timeout_set := 0
					if a_duringconnect then
						l_tmp := 300_000
					else
						l_tmp := 0
					end
					l_timeout_ms := l_tmp.to_integer_32
					if l_data.item.set.timeout > 0 then
						l_timeout_set := (l_timeout_set).bit_or ((1))
					end
					if a_duringconnect then
						if l_data.item.set.connecttimeout > 0 then
							l_timeout_set := (l_timeout_set).bit_or ((2))
						end
					end
					inspect l_timeout_set -- switch 1 head
					when 1 then l_pos := -1 + 1
					when 2 then l_pos := -1 + 2
					when 3 then l_pos := -1 + 3
					else l_pos := -1 + 4
					end
				when 1 then -- switch 1, case 1
					l_timeout_ms := l_data.item.set.timeout
					l_pos := -1 + 5 -- switch 1; break
				when 2 then -- switch 1, case 2
					l_timeout_ms := l_data.item.set.connecttimeout
					l_pos := -1 + 5 -- switch 1; break
				when 3 then -- switch 1, case 3
					if l_data.item.set.timeout < l_data.item.set.connecttimeout then
						l_timeout_ms := l_data.item.set.timeout
					else
						l_timeout_ms := l_data.item.set.connecttimeout
					end
					l_pos := -1 + 5 -- switch 1; break
				when 4 then -- switch 1, default:
					if not a_duringconnect then
						Result := 0
						js.return
					end
					l_pos := -1 + 5 -- switch 1; break
				when 5 then -- switch 1, done:
					if not js.return_called then
						if l_nowp.is_void then
							l_now.item := (timeval_inst.curlx_tvnow).twin
							l_nowp := l_now
						end
						l_timeout_ms := l_timeout_ms - (timeval_inst.curlx_tvdiff ((l_nowp.item).twin, (l_data.item.progress.t_startsingle).twin))
						if not l_timeout_ms.to_boolean then
							Result := (-1)
							js.return
						end
					end
					if not js.return_called then
						Result := l_timeout_ms
					end
				when 6 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	Curl_updateconninfo (a_conn: CE_POINTER [PD_CONNECTDATA]; a_sockfd: INTEGER_32)
		local
			l_error: INTEGER_32
			l_len: CE_ARRAY [NATURAL_32]
			l_ssrem: CE_ARRAY [PD_CURL_SOCKADDR_STORAGE]
			l_ssloc: CE_ARRAY [PD_CURL_SOCKADDR_STORAGE]
			l_data: CE_POINTER [PD_SESSIONHANDLE]
			l_info: CE_POINTER [PD_PUREINFO]
			js: JUMP_STATE
		do
			create l_len.make (<<1>>)
			create l_ssrem.make (<<1>>)
			create l_ssloc.make (<<1>>)
			create l_data.make
			create l_info.make

			l_data := a_conn.item.data
			l_info := create {CE_POINTER [PD_PUREINFO]}.make_shared ((a_conn.item.data).c_pointer + (create {PD_SESSIONHANDLE}.make).offset_of ("info"))
			if a_conn.item.bits.reuse then
				js.return
			end
			if not js.return_called then
				l_len.item := (create {PD_CURL_SOCKADDR_STORAGE}.make).structure_size.to_natural_32
				if (getpeername (a_sockfd, create {CE_POINTER [SD_SOCKADDR]}.make_cast (l_ssrem), l_len)).to_boolean then
					l_error := (errno_location).item
					sendf_inst.Curl_failf ([l_data, ce_string ("getpeername() failed with errno %%d: %%s"), l_error, (strerror_inst.Curl_strerror (a_conn, l_error))])
					js.return
				end
			end
			if not js.return_called then
				l_len.item := (create {PD_CURL_SOCKADDR_STORAGE}.make).structure_size.to_natural_32
				if (getsockname (a_sockfd, create {CE_POINTER [SD_SOCKADDR]}.make_cast (l_ssloc), l_len)).to_boolean then
					l_error := (errno_location).item
					sendf_inst.Curl_failf ([l_data, ce_string ("getsockname() failed with errno %%d: %%s"), l_error, (strerror_inst.Curl_strerror (a_conn, l_error))])
					js.return
				end
			end
			if not js.return_called then
				if not (getaddressinfo (create {CE_POINTER [SD_SOCKADDR]}.make_cast (l_ssrem), l_info.item.ip, create {CE_POINTER [INTEGER_32]}.make_shared (l_info.c_pointer + (create {PD_PUREINFO}.make).offset_of ("port")))) then
					l_error := (errno_location).item
					sendf_inst.Curl_failf ([l_data, ce_string ("ssrem inet_ntop() failed with errno %%d: %%s"), l_error, (strerror_inst.Curl_strerror (a_conn, l_error))])
					js.return
				end
			end
			if not js.return_called then
				if not (getaddressinfo (create {CE_POINTER [SD_SOCKADDR]}.make_cast (l_ssloc), l_info.item.localip, create {CE_POINTER [INTEGER_32]}.make_shared (l_info.c_pointer + (create {PD_PUREINFO}.make).offset_of ("localport")))) then
					l_error := (errno_location).item
					sendf_inst.Curl_failf ([l_data, ce_string ("ssloc inet_ntop() failed with errno %%d: %%s"), l_error, (strerror_inst.Curl_strerror (a_conn, l_error))])
					js.return
				end
			end
			if not js.return_called then
			end
		end

	bindlocal (a_conn: CE_POINTER [PD_CONNECTDATA]; a_sockfd: INTEGER_32; a_af: INTEGER_32): NATURAL_32
		local
			l_data: CE_POINTER [PD_SESSIONHANDLE]
			l_sa: CE_ARRAY [PD_CURL_SOCKADDR_STORAGE]
			l_sock: CE_POINTER [SD_SOCKADDR]
			l_sizeof_sa: NATURAL_32
			l_si4: CE_POINTER [SD_SOCKADDR_IN]
			l_si6: CE_POINTER [SD_SOCKADDR_IN6]
			l_h: CE_ARRAY [CE_POINTER [PD_CURL_DNS_ENTRY]]
			l_port: NATURAL_16
			l_portnum: INTEGER_32
			l_dev: CE_POINTER [INTEGER_8]
			l_error: INTEGER_32
			l_myhost: CE_ARRAY [INTEGER_8]
			l_done: INTEGER_32
			l_ipver: INTEGER_32
			l_rc: INTEGER_32
			l_add: CE_ARRAY [PD_CURL_SOCKADDR_STORAGE]
			l_size: CE_ARRAY [NATURAL_32]
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			create l_data.make
			create l_sa.make (<<1>>)
			create l_sock.make
			create l_si4.make
			create l_si6.make
			create l_h.make (<<1>>)
			create l_dev.make
			create l_myhost.make (<<256>>)
			create l_add.make (<<1>>)
			create l_size.make (<<1>>)

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_data := a_conn.item.data
					l_sock := create {CE_POINTER [SD_SOCKADDR]}.make_cast (l_sa)
					l_sizeof_sa := (0).to_natural_32
					l_si4 := create {CE_POINTER [SD_SOCKADDR_IN]}.make_cast (l_sa)
					l_si6 := create {CE_POINTER [SD_SOCKADDR_IN6]}.make_cast (l_sa)
					l_h.item := create {CE_POINTER [PD_CURL_DNS_ENTRY]}.make_shared (default_pointer)
					l_port := l_data.item.set.localport
					l_portnum := l_data.item.set.localportrange
					l_dev := l_data.item.set.str[5]
					l_myhost[0] := (('%/0/').code).to_integer_8
					l_done := 0
					if l_dev.is_void then
						if not l_port.to_boolean then
							Result := ({PC_CURLCODE}.CURLE_OK)
							js.return
						end
					end
					if js.return_called then
						l_pos := -1 + 13 -- skip if block
					end
				when 1 then -- if:
						memset (create {CE_POINTER [ANY]}.make_cast (l_sa), 0, (create {PD_CURL_SOCKADDR_STORAGE}.make).structure_size.to_natural_32).do_nothing
						if l_dev.is_void then
							l_pos := -1 + 12 -- skip if block
						end
				when 2 then -- if:
						if not ((strlen (l_dev)) < {NATURAL_32} 255) then
							l_pos := -1 + 10 -- skip if block
						end
				when 3 then -- if:
						if (not (if2ip_inst.Curl_if2ip (a_af, l_dev, l_myhost, ({PLATFORM}.Integer_8_bytes.to_natural_32 * (256).to_natural_32).to_integer_32)).is_void) then
							sendf_inst.Curl_infof ([l_data, ce_string ("Local Interface %%s is ip %%s using address family %%i%N"), l_dev, l_myhost, a_af])
							l_done := 1
							if (setsockopt (a_sockfd, 1, 25, create {CE_POINTER [ANY]}.make_cast (l_dev), (strlen (l_dev)) + {NATURAL_32} 1)) /= 0 then
								l_error := (errno_location).item
								sendf_inst.Curl_infof ([l_data, ce_string ("SO_BINDTODEVICE %%s failed with errno %%d: %%s; will do regular bind%N"), l_dev, l_error, (strerror_inst.Curl_strerror (a_conn, l_error))])
							end
						else
							l_ipver := l_data.item.set.ip_version
							if a_af = 2 then
								l_data.item.set.ip_version := 1
							elseif a_af = 10 then
								l_data.item.set.ip_version := 2
							end
							l_rc := hostip_inst.Curl_resolv (a_conn, l_dev, 0, l_h)
							if l_rc = 1 then
								hostsyn_inst.Curl_wait_for_resolv (a_conn, l_h).do_nothing
							end
							l_data.item.set.ip_version := l_ipver
							if (not (l_h.item).is_void) then
								hostip_inst.Curl_printable_address (l_h.item.item.addr, l_myhost, ({PLATFORM}.Integer_8_bytes.to_natural_32 * (256).to_natural_32)).do_nothing
								sendf_inst.Curl_infof ([l_data, ce_string ("Name %'%%s%' family %%i resolved to %'%%s%' family %%i%N"), l_dev, a_af, l_myhost, (l_h.item.item.addr).item.ai_family])
								hostip_inst.Curl_resolv_unlock (l_data, l_h.item)
								l_done := 1
							else
								l_done := -1
							end
						end
						if not (l_done > 0) then
							l_pos := -1 + 8 -- skip if block
						end
				when 4 then -- if:
						if a_af /= 10 then
							l_pos := -1 + 7 -- skip if block
						end
				when 5 then -- if:
						if (inet_pton (10, l_myhost, create {CE_POINTER [ANY]}.make_cast ((create {CE_POINTER [SD_IN6_ADDR]}.make_shared (l_si6.c_pointer + (create {SD_SOCKADDR_IN6}.make).offset_of ("sin6_addr")))))) > 0 then
							l_si6.item.sin6_family := (10).to_natural_16
							l_si6.item.sin6_port := htons (l_port)
							l_sizeof_sa := (create {SD_SOCKADDR_IN6}.make).structure_size.to_natural_32
						else
							l_pos := -1 + 7 -- goto _L
						end
				when 6 then -- if block done, skip else block:
						l_pos := -1 + 8
				when 7 then -- else: | _L:
						if a_af = 2 then
							if (inet_pton (2, l_myhost, create {CE_POINTER [ANY]}.make_cast ((create {CE_POINTER [SD_IN_ADDR]}.make_shared (l_si4.c_pointer + (create {SD_SOCKADDR_IN}.make).offset_of ("sin_addr")))))) > 0 then
								l_si4.item.sin_family := (2).to_natural_16
								l_si4.item.sin_port := htons (l_port)
								l_sizeof_sa := (create {SD_SOCKADDR_IN}.make).structure_size.to_natural_32
							end
						end
				when 8 then -- if done
						if l_done < 1 then
							sendf_inst.Curl_failf ([l_data, ce_string ("Couldn%'t bind to %'%%s%'"), l_dev])
							Result := ({PC_CURLCODE}.CURLE_INTERFACE_FAILED)
							js.return
						end
				when 9 then -- if block done, skip else block:
						l_pos := -1 + 11
				when 10 then -- else:
						l_pos := -1 + 12 -- goto _L___0
				when 11 then -- if done | if block done, skip else block:
						l_pos := -1 + 13
				when 12 then -- else: | _L___0:
						if a_af = 10 then
							l_si6.item.sin6_family := (10).to_natural_16
							l_si6.item.sin6_port := htons (l_port)
							l_sizeof_sa := (create {SD_SOCKADDR_IN6}.make).structure_size.to_natural_32
						elseif a_af = 2 then
							l_si4.item.sin_family := (2).to_natural_16
							l_si4.item.sin_port := htons (l_port)
							l_sizeof_sa := (create {SD_SOCKADDR_IN}.make).structure_size.to_natural_32
						end
				when 13 then -- if done
						if not js.return_called then
							from
							until
								js.loop_done
							loop
								if (bind (a_sockfd, l_sock, l_sizeof_sa)) >= 0 then
									l_size.item := (create {PD_CURL_SOCKADDR_STORAGE}.make).structure_size.to_natural_32
									memset (create {CE_POINTER [ANY]}.make_cast (l_add), 0, (create {PD_CURL_SOCKADDR_STORAGE}.make).structure_size.to_natural_32).do_nothing
									if (getsockname (a_sockfd, create {CE_POINTER [SD_SOCKADDR]}.make_cast (l_add), l_size)) < 0 then
										l_error := (errno_location).item
										l_data.item.state.os_errno := l_error
										sendf_inst.Curl_failf ([l_data, ce_string ("getsockname() failed with errno %%d: %%s"), l_error, (strerror_inst.Curl_strerror (a_conn, l_error))])
										Result := ({PC_CURLCODE}.CURLE_INTERFACE_FAILED)
										js.return
									end
									if not js.return_called then
										sendf_inst.Curl_infof ([l_data, ce_string ("Local port: %%hu%N"), l_port.to_integer_32])
										a_conn.item.bits.bound := (1).to_boolean
										Result := ({PC_CURLCODE}.CURLE_OK)
										js.return
									end
								end
								if not js.return_called then
									l_portnum := l_portnum - 1
									if l_portnum > 0 then
										sendf_inst.Curl_infof ([l_data, ce_string ("Bind to local port %%hu failed, trying next%N"), l_port.to_integer_32])
										l_port := (l_port.to_integer_32 + 1).to_natural_16
										if (l_sock.item.sa_family).to_integer_32 = 2 then
											l_si4.item.sin_port := ntohs (l_port)
										else
											l_si6.item.sin6_port := ntohs (l_port)
										end
									else
										js.break
									end
								end
							end
						end
						if not js.return_called then
							l_error := (errno_location).item
							l_data.item.state.os_errno := l_error
							sendf_inst.Curl_failf ([l_data, ce_string ("bind failed with errno %%d: %%s"), l_error, (strerror_inst.Curl_strerror (a_conn, l_error))])
							Result := ({PC_CURLCODE}.CURLE_INTERFACE_FAILED)
						end
				when 14 then
						l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
			end

	getaddressinfo (a_sa: CE_POINTER [SD_SOCKADDR]; a_addr: CE_POINTER [INTEGER_8]; a_port: CE_POINTER [INTEGER_32]): BOOLEAN
		local
			l_us_port: NATURAL_16
			l_si: CE_POINTER [SD_SOCKADDR_IN]
			l_si6: CE_POINTER [SD_SOCKADDR_IN6]
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			create l_si.make
			create l_si6.make

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_si := create {CE_POINTER [SD_SOCKADDR_IN]}.make_shared (default_pointer)
					l_si6 := create {CE_POINTER [SD_SOCKADDR_IN6]}.make_shared (default_pointer)
					inspect (a_sa.item.sa_family).to_integer_32 -- switch 1 head
					when 2 then l_pos := -1 + 1
					when 10 then l_pos := -1 + 2
					else l_pos := -1 + 3
					end
				when 1 then -- switch 1, case 2
					l_si := create {CE_POINTER [SD_SOCKADDR_IN]}.make_cast (a_sa)
					if (inet_ntop ((a_sa.item.sa_family).to_integer_32, create {CE_POINTER [ANY]}.make_cast ((create {CE_POINTER [SD_IN_ADDR]}.make_shared (l_si.c_pointer + (create {SD_SOCKADDR_IN}.make).offset_of ("sin_addr")))), a_addr, (("ffff:ffff:ffff:ffff:ffff:ffff:255.255.255.255").count + 1).to_natural_32)).is_void then
						Result := (0).to_boolean
						js.return
					end
					if not js.return_called then
						l_us_port := ntohs (l_si.item.sin_port)
						a_port.item := l_us_port.to_integer_32
					end
					l_pos := -1 + 4 -- switch 1; break
				when 2 then -- switch 1, case 10
					l_si6 := create {CE_POINTER [SD_SOCKADDR_IN6]}.make_cast (a_sa)
					if (inet_ntop ((a_sa.item.sa_family).to_integer_32, create {CE_POINTER [ANY]}.make_cast ((create {CE_POINTER [SD_IN6_ADDR]}.make_shared (l_si6.c_pointer + (create {SD_SOCKADDR_IN6}.make).offset_of ("sin6_addr")))), a_addr, (("ffff:ffff:ffff:ffff:ffff:ffff:255.255.255.255").count + 1).to_natural_32)).is_void then
						Result := (0).to_boolean
						js.return
					end
					if not js.return_called then
						l_us_port := ntohs (l_si6.item.sin6_port)
						a_port.item := l_us_port.to_integer_32
					end
					l_pos := -1 + 4 -- switch 1; break
				when 3 then -- switch 1, default:
					a_addr[0] := (('%/0/').code).to_integer_8
					a_port.item := 0
				when 4 then -- switch 1, done:
					if not js.return_called then
						Result := (1).to_boolean
					end
				when 5 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	singleipconnect (a_conn: CE_POINTER [PD_CONNECTDATA]; a_ai: CE_POINTER [PD_CURL_ADDRINFO]; a_timeout_ms: INTEGER_32; a_sockp: CE_POINTER [INTEGER_32]; a_connected: CE_POINTER [BOOLEAN]): NATURAL_32
		local
			l_addr: CE_ARRAY [PD_CURL_SOCKADDR_EX]
			l_addr_buf: CE_ARRAY [INTEGER_8]
			l_rc: INTEGER_32
			l_error: CE_ARRAY [INTEGER_32]
			l_isconnected: BOOLEAN
			l_data: CE_POINTER [PD_SESSIONHANDLE]
			l_sockfd: INTEGER_32
			l_res: NATURAL_32
			l_iptoprint: CE_POINTER [ANY]
			l_sa4: CE_POINTER [SD_SOCKADDR_IN]
			l_sa6: CE_POINTER [SD_SOCKADDR_IN6]
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			create l_addr.make (<<1>>)
			create l_addr_buf.make (<<128>>)
			create l_error.make (<<1>>)
			create l_data.make
			create l_iptoprint.make
			create l_sa4.make
			create l_sa6.make

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_data := a_conn.item.data
					l_res := {PC_CURLCODE}.CURLE_OK
					l_sa4 := create {CE_POINTER [SD_SOCKADDR_IN]}.make_cast ((create {CE_POINTER [SD_SOCKADDR]}.make_shared (l_addr.item.c_pointer + (create {PD_CURL_SOCKADDR_EX}.make).offset_of ("sa_ex_u") + (create {PD_SA_EX_U}.make).offset_of ("addr"))))
					l_sa6 := create {CE_POINTER [SD_SOCKADDR_IN6]}.make_cast ((create {CE_POINTER [SD_SOCKADDR]}.make_shared (l_addr.item.c_pointer + (create {PD_CURL_SOCKADDR_EX}.make).offset_of ("sa_ex_u") + (create {PD_SA_EX_U}.make).offset_of ("addr"))))
					a_sockp.item := -1
					l_addr.item.family := a_ai.item.ai_family
					l_addr.item.socktype := a_conn.item.socktype
					l_addr.item.protocol := a_ai.item.ai_protocol
					l_addr.item.addrlen := a_ai.item.ai_addrlen
					if l_addr.item.addrlen > (create {PD_CURL_SOCKADDR_STORAGE}.make).structure_size.to_natural_32 then
						l_addr.item.addrlen := (create {PD_CURL_SOCKADDR_STORAGE}.make).structure_size.to_natural_32
					end
					memcpy (create {CE_POINTER [ANY]}.make_cast ((create {CE_POINTER [SD_SOCKADDR]}.make_shared (l_addr.item.c_pointer + (create {PD_CURL_SOCKADDR_EX}.make).offset_of ("sa_ex_u") + (create {PD_SA_EX_U}.make).offset_of ("addr")))), create {CE_POINTER [ANY]}.make_cast (a_ai.item.ai_addr), l_addr.item.addrlen).do_nothing
					a_connected.item := (0).to_boolean
					if (not (l_data.item.set.fopensocket).is_void) then
						l_sockfd := (l_data.item.set.fopensocket).integer_32_item ([l_data.item.set.opensocket_client, {PC_CURLSOCKTYPE}.CURLSOCKTYPE_IPCXN, create {CE_POINTER [PD_CURL_SOCKADDR]}.make_cast (l_addr)])
					else
						l_sockfd := socket (l_addr.item.family, l_addr.item.socktype, l_addr.item.protocol)
					end
					if l_sockfd = -1 then
						Result := ({PC_CURLCODE}.CURLE_OK)
						js.return
					end
					if not js.return_called then
						if (a_conn.item.scope).to_boolean then
							if l_addr.item.family = 10 then
								l_sa6.item.sin6_scope_id := a_conn.item.scope
							end
						end
						if l_addr.item.family = 1 then
							sendf_inst.Curl_infof ([l_data, ce_string ("  Trying %%s... "), (create {CE_POINTER [SD_SOCKADDR_UN]}.make_cast ((create {CE_POINTER [SD_SOCKADDR]}.make_shared (l_addr.item.c_pointer + (create {PD_CURL_SOCKADDR_EX}.make).offset_of ("sa_ex_u") + (create {PD_SA_EX_U}.make).offset_of ("addr"))))).item.sun_path])
							mprintf_inst.curl_msnprintf ([l_data.item.info.ip, (("ffff:ffff:ffff:ffff:ffff:ffff:255.255.255.255").count + 1).to_natural_32, ce_string ("%%s"), (create {CE_POINTER [SD_SOCKADDR_UN]}.make_cast ((create {CE_POINTER [SD_SOCKADDR]}.make_shared (l_addr.item.c_pointer + (create {PD_CURL_SOCKADDR_EX}.make).offset_of ("sa_ex_u") + (create {PD_SA_EX_U}.make).offset_of ("addr"))))).item.sun_path]).do_nothing
							strcpy ((a_conn.item.ip_addr_str), (l_data.item.info.ip)).do_nothing
						else
							if l_addr.item.family = 10 then
								l_iptoprint := create {CE_POINTER [ANY]}.make_cast ((create {CE_POINTER [SD_IN6_ADDR]}.make_shared (l_sa6.c_pointer + (create {SD_SOCKADDR_IN6}.make).offset_of ("sin6_addr"))))
								a_conn.item.bits.ipv6 := (1).to_boolean
							else
								l_iptoprint := create {CE_POINTER [ANY]}.make_cast ((create {CE_POINTER [SD_IN_ADDR]}.make_shared (l_sa4.c_pointer + (create {SD_SOCKADDR_IN}.make).offset_of ("sin_addr"))))
							end
							if not (inet_ntop (l_addr.item.family, l_iptoprint, l_addr_buf, ({PLATFORM}.Integer_8_bytes.to_natural_32 * (128).to_natural_32))).is_void then
								sendf_inst.Curl_infof ([l_data, ce_string ("  Trying %%s... "), l_addr_buf])
								mprintf_inst.curl_msnprintf ([l_data.item.info.ip, (("ffff:ffff:ffff:ffff:ffff:ffff:255.255.255.255").count + 1).to_natural_32, ce_string ("%%s"), l_addr_buf]).do_nothing
								strcpy ((a_conn.item.ip_addr_str), (l_data.item.info.ip)).do_nothing
							end
						end
						if l_data.item.set.tcp_nodelay then
							tcpnodelay (a_conn, l_sockfd)
						end
						if (not (l_data.item.set.fsockopt).is_void) then
							l_error.item := (l_data.item.set.fsockopt).integer_32_item ([l_data.item.set.sockopt_client, l_sockfd, {PC_CURLSOCKTYPE}.CURLSOCKTYPE_IPCXN])
							if (l_error.item).to_boolean then
								close (l_sockfd).do_nothing
								Result := l_res
								js.return
							end
						end
					end
					if not js.return_called then
						l_res := bindlocal (a_conn, l_sockfd, l_addr.item.family)
						if l_res.to_boolean then
							close (l_sockfd).do_nothing
							Result := l_res
							js.return
						end
					end
					if js.return_called then
						l_pos := -1 + 5 -- skip if block
					end
				when 1 then -- if:
						nonblock_inst.curlx_nonblock (l_sockfd, 1).do_nothing
						if a_conn.item.socktype = 1 then
							l_rc := connect (l_sockfd, (create {CE_POINTER [SD_SOCKADDR]}.make_shared (l_addr.item.c_pointer + (create {PD_CURL_SOCKADDR_EX}.make).offset_of ("sa_ex_u") + (create {PD_SA_EX_U}.make).offset_of ("addr"))), l_addr.item.addrlen)
						else
							l_rc := 0
						end
						if -1 /= l_rc then
							l_pos := -1 + 5 -- skip if block
						end
				when 2 then -- if:
						l_error.item := (errno_location).item
						inspect l_error.item -- switch 1 head
						when 115 then l_pos := -1 + 3
						when 11 then l_pos := -1 + 3
						else l_pos := -1 + 4
						end
				when 3 then -- switch 1, case 115 | switch 1, case 11
						l_rc := waitconnect (a_conn, l_sockfd, a_timeout_ms)
						if 3 = l_rc then
							close (l_sockfd).do_nothing
							Result := ({PC_CURLCODE}.CURLE_ABORTED_BY_CALLBACK)
							js.return
						end
						l_pos := -1 + 5 -- switch 1; break
				when 4 then -- switch 1, default:
						sendf_inst.Curl_failf ([l_data, ce_string ("Failed to connect to %%s: %%s"), l_addr_buf, (strerror_inst.Curl_strerror (a_conn, l_error.item))])
						l_data.item.state.os_errno := l_error.item
						l_pos := -1 + 5 -- switch 1; break
				when 5 then -- switch 1, done: | if done | if done
						if not js.return_called and then (1 = l_rc) then
							if l_data.item.state.used_interface = {NATURAL_32} 2 then
								a_sockp.item := l_sockfd
								Result := ({PC_CURLCODE}.CURLE_OK)
								js.return
							end
						end
						if js.return_called then
							l_pos := -1 + 10 -- skip if block
						end
				when 6 then -- if:
							l_isconnected := verifyconnect (l_sockfd, l_error)
							if l_rc.to_boolean then
								l_pos := -1 + 9 -- skip if block
							end
				when 7 then -- if:
							if l_isconnected then
								a_connected.item := (1).to_boolean
								sendf_inst.Curl_infof ([l_data, ce_string ("connected%N")])
								Curl_updateconninfo (a_conn, l_sockfd)
								a_sockp.item := l_sockfd
								Result := ({PC_CURLCODE}.CURLE_OK)
								js.return
							else
								l_pos := -1 + 9 -- goto _L
							end
				when 8 then -- if block done, skip else block:
							l_pos := -1 + 10
				when 9 then -- else: | _L:
							if 1 = l_rc then
								sendf_inst.Curl_infof ([l_data, ce_string ("Timeout%N")])
							else
								l_data.item.state.os_errno := l_error.item
								sendf_inst.Curl_infof ([l_data, ce_string ("%%s%N"), (strerror_inst.Curl_strerror (a_conn, l_error.item))])
							end
				when 10 then -- if done
							if not js.return_called then
								close (l_sockfd).do_nothing
								Result := ({PC_CURLCODE}.CURLE_OK)
							end
				when 11 then
							l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
				end

	tcpnodelay (a_conn: CE_POINTER [PD_CONNECTDATA]; a_sockfd: INTEGER_32)
		local
			l_data: CE_POINTER [PD_SESSIONHANDLE]
			l_onoff: CE_ARRAY [NATURAL_32]
			l_proto: INTEGER_32
		do
			create l_data.make
			create l_onoff.make (<<1>>)

			l_data := a_conn.item.data
			l_onoff.item := (l_data.item.set.tcp_nodelay).to_integer.to_natural_32
			l_proto := 6
			if (setsockopt (a_sockfd, l_proto, 1, create {CE_POINTER [ANY]}.make_cast (l_onoff), {PLATFORM}.Natural_32_bytes.to_natural_32)) < 0 then
				sendf_inst.Curl_infof ([l_data, ce_string ("Could not set TCP_NODELAY: %%s%N"), (strerror_inst.Curl_strerror (a_conn, (errno_location).item))])
			else
				sendf_inst.Curl_infof ([l_data, ce_string ("TCP_NODELAY set%N")])
			end
		end

	trynextip (a_conn: CE_POINTER [PD_CONNECTDATA]; a_sockindex: INTEGER_32; a_connected: CE_POINTER [BOOLEAN]): NATURAL_32
		local
			l_sockfd: CE_ARRAY [INTEGER_32]
			l_ai: CE_POINTER [PD_CURL_ADDRINFO]
			l_fd_to_close: INTEGER_32
			l_res: NATURAL_32
			js: JUMP_STATE
		do
			create l_sockfd.make (<<1>>)
			create l_ai.make

			l_fd_to_close := a_conn.item.sock[a_sockindex]
			a_conn.item.sock[a_sockindex] := -1
			a_connected.item := (0).to_boolean
			if a_sockindex /= 0 then
				close (l_fd_to_close).do_nothing
				Result := ({PC_CURLCODE}.CURLE_COULDNT_CONNECT)
				js.return
			end
			if not js.return_called then
				l_ai := (a_conn.item.ip_addr).item.ai_next
				from
				until
					not (not js.loop_done and then (not l_ai.is_void))
				loop
					l_res := (singleipconnect (a_conn, l_ai, 0, l_sockfd, a_connected))
					if l_res.to_boolean then
						Result := l_res
						js.return
					end
					if not js.return_called and then (l_sockfd.item /= -1) then
						a_conn.item.sock[a_sockindex] := l_sockfd.item
						a_conn.item.ip_addr := l_ai
						close (l_fd_to_close).do_nothing
						Result := ({PC_CURLCODE}.CURLE_OK)
						js.return
					end
					if not js.return_called then
						l_ai := l_ai.item.ai_next
					end
				end
			end
			if not js.return_called then
				close (l_fd_to_close).do_nothing
				Result := ({PC_CURLCODE}.CURLE_COULDNT_CONNECT)
			end
		end

	verifyconnect (a_sockfd: INTEGER_32; a_error: CE_POINTER [INTEGER_32]): BOOLEAN
		local
			l_rc: BOOLEAN
			l_err: CE_ARRAY [INTEGER_32]
			l_errSize: CE_ARRAY [NATURAL_32]
		do
			create l_err.make (<<1>>)
			create l_errSize.make (<<1>>)

			l_rc := (1).to_boolean
			l_err.item := 0
			l_errSize.item := {PLATFORM}.Integer_32_bytes.to_natural_32
			if 0 /= (getsockopt (a_sockfd, 1, 4, create {CE_POINTER [ANY]}.make_cast (l_err), l_errSize)) then
				l_err.item := (errno_location).item
			end
			if 0 = l_err.item then
				l_rc := (1).to_boolean
			elseif 106 = l_err.item then
				l_rc := (1).to_boolean
			else
				l_rc := (0).to_boolean
			end
			if (not a_error.is_void) then
				a_error.item := l_err.item
			end
			Result := l_rc
		end

	waitconnect (a_conn: CE_POINTER [PD_CONNECTDATA]; a_sockfd: INTEGER_32; a_timeout_msec: INTEGER_32): INTEGER_32
		local
			l_timeout_msec: INTEGER_32
			l_rc: INTEGER_32
			l_tmp: INTEGER_32
			js: JUMP_STATE
		do
			l_timeout_msec := a_timeout_msec
			from
			until
				js.loop_done
			loop
				if l_timeout_msec > 1_000 then
					l_tmp := 1_000
				else
					l_tmp := l_timeout_msec
				end
				l_rc := select_inst.Curl_socket_ready (-1, a_sockfd, l_tmp.to_integer_32)
				if (progress_inst.Curl_pgrsUpdate (a_conn)).to_boolean then
					Result := 3
					js.return
				end
				if not js.return_called then
					if -1 = l_rc then
						Result := (-1)
						js.return
					elseif 0 = l_rc then
						l_timeout_msec := l_timeout_msec - (1_000)
						if l_timeout_msec <= 0 then
							Result := 1
							js.return
						end
						if not js.return_called then
							js.continue
						end
					end
				end
				if not js.return_called then
					if not js.loop_jumped then
						if ((l_rc).bit_and (4)).to_boolean then
							Result := 2
							js.return
						end
						if not js.return_called then
							js.break
						end
					end
					if js.loop_increment then
						
					end
				end
			end
			if not js.return_called then
				Result := 0
			end
		end

end
