class
	P_SHARE

inherit
	LIBCURL_DATA
	S_STDIO
	S_STDLIB
	S_STRING
	S_CTYPE
	S_SYS_STAT
	S_TIME
	S_UNISTD
	S_FCNTL
	S_SYS_TIME
	S_ARPA_INET
	S_SIGNAL
	S_LIBGEN
	S_SYS_SOCKET
	S_BITS_ERRNO
	S_NETINET_IN
	S_IFADDRS
	S_PWD
	S_SYS_POLL
	S_NETDB

feature {ANY} -- routines

	Curl_share_lock (a_data: CE_POINTER [PD_SESSIONHANDLE]; a_type: NATURAL_32; a_accesstype: NATURAL_32): NATURAL_32
		local
			l_share: CE_POINTER [PD_CURL_SHARE]
			js: JUMP_STATE
		do
			create l_share.make

			l_share := a_data.item.share
			if l_share.is_void then
				Result := ({PC_CURLSHCODE}.CURLSHE_INVALID)
				js.return
			end
			if not js.return_called then
				if ((l_share.item.specifier).bit_and (((1).bit_shift_left (a_type.to_integer_32)).to_natural_32)).to_boolean then
					if (not (l_share.item.lockfunc).is_void) then
						(l_share.item.lockfunc).call ([create {CE_POINTER [ANY]}.make_cast (a_data), a_type, a_accesstype, l_share.item.clientdata])
					end
				end
				Result := ({PC_CURLSHCODE}.CURLSHE_OK)
			end
		end

	Curl_share_unlock (a_data: CE_POINTER [PD_SESSIONHANDLE]; a_type: NATURAL_32): NATURAL_32
		local
			l_share: CE_POINTER [PD_CURL_SHARE]
			js: JUMP_STATE
		do
			create l_share.make

			l_share := a_data.item.share
			if l_share.is_void then
				Result := ({PC_CURLSHCODE}.CURLSHE_INVALID)
				js.return
			end
			if not js.return_called then
				if ((l_share.item.specifier).bit_and (((1).bit_shift_left (a_type.to_integer_32)).to_natural_32)).to_boolean then
					if (not (l_share.item.unlockfunc).is_void) then
						(l_share.item.unlockfunc).call ([create {CE_POINTER [ANY]}.make_cast (a_data), a_type, l_share.item.clientdata])
					end
				end
				Result := ({PC_CURLSHCODE}.CURLSHE_OK)
			end
		end

	curl_share_cleanup (a_sh: CE_POINTER [ANY]): NATURAL_32
		local
			l_share: CE_POINTER [PD_CURL_SHARE]
			js: JUMP_STATE
		do
			create l_share.make

			l_share := create {CE_POINTER [PD_CURL_SHARE]}.make_cast (a_sh)
			if l_share.is_void then
				Result := ({PC_CURLSHCODE}.CURLSHE_INVALID)
				js.return
			end
			if not js.return_called then
				if (not (l_share.item.lockfunc).is_void) then
					(l_share.item.lockfunc).call ([create {CE_POINTER [ANY]}.make_shared (default_pointer), {PC_CURL_LOCK_DATA}.CURL_LOCK_DATA_SHARE, {PC_CURL_LOCK_ACCESS}.CURL_LOCK_ACCESS_SINGLE, l_share.item.clientdata])
				end
				if (l_share.item.dirty).to_boolean then
					if (not (l_share.item.unlockfunc).is_void) then
						(l_share.item.unlockfunc).call ([create {CE_POINTER [ANY]}.make_shared (default_pointer), {PC_CURL_LOCK_DATA}.CURL_LOCK_DATA_SHARE, l_share.item.clientdata])
					end
					Result := ({PC_CURLSHCODE}.CURLSHE_IN_USE)
					js.return
				end
			end
			if not js.return_called then
				if (not (l_share.item.hostcache).is_void) then
					hash_inst.Curl_hash_destroy (l_share.item.hostcache)
					l_share.item.hostcache := create {CE_POINTER [PD_CURL_HASH]}.make_shared (default_pointer)
				end
				if (not (l_share.item.cookies).is_void) then
					cookie_inst.Curl_cookie_cleanup (l_share.item.cookies)
				end
				if (not (l_share.item.unlockfunc).is_void) then
					(l_share.item.unlockfunc).call ([create {CE_POINTER [ANY]}.make_shared (default_pointer), {PC_CURL_LOCK_DATA}.CURL_LOCK_DATA_SHARE, l_share.item.clientdata])
				end
				easy_inst.Curl_cfree.call ([create {CE_POINTER [ANY]}.make_cast (l_share)])
				Result := ({PC_CURLSHCODE}.CURLSHE_OK)
			end
		end

	curl_share_init: CE_POINTER [ANY]
		local
			l_share: CE_POINTER [PD_CURL_SHARE]
		do
			create l_share.make

			l_share := create {CE_POINTER [PD_CURL_SHARE]}.make_cast ((create {CE_POINTER [ANY]}.make_cast (easy_inst.Curl_ccalloc.pointer_item ([(1).to_natural_32, (create {PD_CURL_SHARE}.make).structure_size.to_natural_32]))))
			if (not l_share.is_void) then
				l_share.item.specifier := (l_share.item.specifier).bit_or (((1).bit_shift_left (1)).to_natural_32)
			end
			Result := (create {CE_POINTER [ANY]}.make_cast (l_share))
		end

	curl_share_setopt (a_args: TUPLE [a_sh: CE_POINTER [ANY]; a_option: NATURAL_32]): NATURAL_32
		local
			l_share: CE_POINTER [PD_CURL_SHARE]
			l_param: CE_VA_LIST
			l_type: INTEGER_32
			l_lockfunc: CE_ROUTINE
			l_unlockfunc: CE_ROUTINE
			l_ptr: CE_POINTER [ANY]
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			create l_share.make
			create l_param.make_shared (default_pointer)
			create l_lockfunc.make
			create l_unlockfunc.make
			create l_ptr.make

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_share := create {CE_POINTER [PD_CURL_SHARE]}.make_cast (a_args.a_sh)
					if (l_share.item.dirty).to_boolean then
						Result := ({PC_CURLSHCODE}.CURLSHE_IN_USE)
						js.return
					end
					if js.return_called then
						l_pos := -1 + 18 -- skip if block
					end
				when 1 then -- if:
						l_param := create {CE_VA_LIST}.make (a_args, 3)
						inspect a_args.a_option.to_integer_32 -- switch 1 head
						when 1 then l_pos := -1 + 2
						when 2 then l_pos := -1 + 7
						when 3 then l_pos := -1 + 14
						when 4 then l_pos := -1 + 15
						when 5 then l_pos := -1 + 16
						else l_pos := -1 + 17
						end
				when 2 then -- switch 1, case 1
						l_type := l_param.integer_32_item
						l_share.item.specifier := (l_share.item.specifier).bit_or (((1).bit_shift_left (l_type)).to_natural_32)
						inspect l_type -- switch 2 head
						when 3 then l_pos := -1 + 3
						when 2 then l_pos := -1 + 4
						else l_pos := -1 + 5
						end
				when 3 then -- switch 2, case 3
						if (l_share.item.hostcache).is_void then
							l_share.item.hostcache := hostip_inst.Curl_mk_dnscache
							if (l_share.item.hostcache).is_void then
								Result := ({PC_CURLSHCODE}.CURLSHE_NOMEM)
								js.return
							end
						end
						l_pos := -1 + 6 -- switch 2; break
				when 4 then -- switch 2, case 2
						if (l_share.item.cookies).is_void then
							l_share.item.cookies := cookie_inst.Curl_cookie_init (create {CE_POINTER [PD_SESSIONHANDLE]}.make_shared (default_pointer), create {CE_POINTER [INTEGER_8]}.make_shared (default_pointer), create {CE_POINTER [PD_COOKIEINFO]}.make_shared (default_pointer), (1).to_boolean)
							if (l_share.item.cookies).is_void then
								Result := ({PC_CURLSHCODE}.CURLSHE_NOMEM)
								js.return
							end
						end
						l_pos := -1 + 6 -- switch 2; break
				when 5 then -- switch 2, default:
						Result := ({PC_CURLSHCODE}.CURLSHE_BAD_OPTION)
						js.return
				when 6 then -- switch 2, done:
						l_pos := -1 + 18 -- switch 1; break
				when 7 then -- switch 1, case 2
						l_type := l_param.integer_32_item
						l_share.item.specifier := (l_share.item.specifier).bit_and (((((1).bit_shift_left (l_type))).bit_not).to_natural_32)
						inspect l_type -- switch 3 head
						when 3 then l_pos := -1 + 8
						when 2 then l_pos := -1 + 9
						when 4 then l_pos := -1 + 10
						when 5 then l_pos := -1 + 11
						else l_pos := -1 + 12
						end
				when 8 then -- switch 3, case 3
						if (not (l_share.item.hostcache).is_void) then
							hash_inst.Curl_hash_destroy (l_share.item.hostcache)
							l_share.item.hostcache := create {CE_POINTER [PD_CURL_HASH]}.make_shared (default_pointer)
						end
						l_pos := -1 + 13 -- switch 3; break
				when 9 then -- switch 3, case 2
						if (not (l_share.item.cookies).is_void) then
							cookie_inst.Curl_cookie_cleanup (l_share.item.cookies)
							l_share.item.cookies := create {CE_POINTER [PD_COOKIEINFO]}.make_shared (default_pointer)
						end
						l_pos := -1 + 13 -- switch 3; break
				when 10 then -- switch 3, case 4
						l_pos := -1 + 13 -- switch 3; break
				when 11 then -- switch 3, case 5
						l_pos := -1 + 13 -- switch 3; break
				when 12 then -- switch 3, default:
						Result := ({PC_CURLSHCODE}.CURLSHE_BAD_OPTION)
						js.return
				when 13 then -- switch 3, done:
						l_pos := -1 + 18 -- switch 1; break
				when 14 then -- switch 1, case 3
						l_lockfunc := (create {CE_ROUTINE}.make_cast (l_param.pointer_item))
						l_share.item.lockfunc := l_lockfunc
						l_pos := -1 + 18 -- switch 1; break
				when 15 then -- switch 1, case 4
						l_unlockfunc := (create {CE_ROUTINE}.make_cast (l_param.pointer_item))
						l_share.item.unlockfunc := l_unlockfunc
						l_pos := -1 + 18 -- switch 1; break
				when 16 then -- switch 1, case 5
						l_ptr := (create {CE_POINTER [ANY]}.make_cast (l_param.pointer_item))
						l_share.item.clientdata := l_ptr
						l_pos := -1 + 18 -- switch 1; break
				when 17 then -- switch 1, default:
						Result := ({PC_CURLSHCODE}.CURLSHE_BAD_OPTION)
						js.return
				when 18 then -- switch 1, done: | if done
						if not js.return_called then
							Result := ({PC_CURLSHCODE}.CURLSHE_OK)
						end
				when 19 then
						l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
			end

end
