class
	P_MUL_FFT

inherit
	LIBGMP_DATA
		redefine
			default_create
		end
	S_GCC_BUILTIN
	S_STDIO
	S_STDLIB
	S_SIGNAL
	S_STRING
	S_CTYPE
	S_LANGINFO
	S_LIBIO
	S_OBSTACK

feature {NONE} -- Initialization

	default_create
		do
			create fft_tab.make
			create mpn_fft_table3.make (<<2, 146>>)
			create tab.make
		end

feature {ANY} -- attributes

	fft_tab: CE_POINTER [PD_FFT_TABLE_NK] assign set_fft_tab

	set_fft_tab (a_fft_tab: CE_POINTER [PD_FFT_TABLE_NK])
		do
			fft_tab := a_fft_tab
		end

	mpn_fft_table3: CE_ARRAY [CE_ARRAY [PD_FFT_TABLE_NK]]

	tab: CE_POINTER [PD_FFT_TABLE_NK] assign set_tab

	set_tab (a_tab: CE_POINTER [PD_FFT_TABLE_NK])
		do
			tab := a_tab
		end

feature {ANY} -- routines

	gmpn_fft_best_k (a_n: INTEGER_32; a_sqr: INTEGER_32): INTEGER_32
		local
			l_tab_n: INTEGER_32
			l_thres: INTEGER_32
			l_last_k: INTEGER_32
			js: JUMP_STATE
		do
			fft_tab := mpn_fft_table3[a_sqr]
			l_last_k := (fft_tab.item.k).to_integer_32
			tab := fft_tab + 1
			from
			until
				js.loop_done
			loop
				l_tab_n := (tab.item.n).to_integer_32
				l_thres := (l_tab_n).bit_shift_left (l_last_k)
				if a_n <= l_thres then
					js.break
				end
				if not js.loop_jumped then
					l_last_k := (tab.item.k).to_integer_32
					tab := tab + 1
				end
			end
			Result := l_last_k
		end

	gmpn_fft_next_size (a_pl: INTEGER_32; a_k: INTEGER_32): INTEGER_32
		local
			l_pl: INTEGER_32
		do
			l_pl := a_pl
			l_pl := 1 + (((l_pl - 1)).bit_shift_right (a_k))
			Result := ((l_pl).bit_shift_left (a_k))
		end

	gmpn_mul_fft (a_op: CE_POINTER [NATURAL_32]; a_pl: INTEGER_32; a_n: CE_POINTER [NATURAL_32]; a_nl: INTEGER_32; a_m: CE_POINTER [NATURAL_32]; a_ml: INTEGER_32; a_k: INTEGER_32): NATURAL_32
		local
			l_K: INTEGER_32
			l_maxLK: INTEGER_32
			l_i: INTEGER_32
			l_N: INTEGER_32
			l_Nprime: INTEGER_32
			l_nprime2: INTEGER_32
			l_M: INTEGER_32
			l_Mp: INTEGER_32
			l_l: INTEGER_32
			l_Ap: CE_POINTER [CE_POINTER [NATURAL_32]]
			l_Bp: CE_POINTER [CE_POINTER [NATURAL_32]]
			l_A: CE_POINTER [NATURAL_32]
			l_T: CE_POINTER [NATURAL_32]
			l_B: CE_POINTER [NATURAL_32]
			l_fft_l: CE_POINTER [CE_POINTER [INTEGER_32]]
			l_sqr: INTEGER_32
			l_tmp: INTEGER_32
			l_h: NATURAL_32
			l___tmp_marker: CE_ARRAY [CE_POINTER [PD_TMP_REENTRANT_T]]
			l_tmp___4: CE_POINTER [ANY]
			l_tmp___9: CE_POINTER [ANY]
			l_K2: NATURAL_32
			l_tmp___13: INTEGER_32
			l_tmp___17: CE_POINTER [ANY]
			l_tmp___22: CE_POINTER [ANY]
			l_tmp___27: CE_POINTER [ANY]
			l_pla: INTEGER_32
			l_tmp___32: CE_POINTER [ANY]
			l_tmp___37: CE_POINTER [ANY]
			l_tmp___42: CE_POINTER [ANY]
			l_tmp___47: CE_POINTER [ANY]
			js: JUMP_STATE
			l_aaf: CE_ALLOCA_FACTORY
		do
			create l_Ap.make
			create l_Bp.make
			create l_A.make
			create l_T.make
			create l_B.make
			create l_fft_l.make
			create l___tmp_marker.make (<<1>>)
			create l_tmp___4.make
			create l_tmp___9.make
			create l_tmp___17.make
			create l_tmp___22.make
			create l_tmp___27.make
			create l_tmp___32.make
			create l_tmp___37.make
			create l_tmp___42.make
			create l_tmp___47.make

			if a_n.is_equal (a_m) then
				if a_nl = a_ml then
					l_tmp := 1
				else
					l_tmp := 0
				end
			else
				l_tmp := 0
			end
			l_sqr := l_tmp
			from
			until
				js.loop_done
			loop
				if (gmpn_fft_next_size (a_pl, a_k)) /= a_pl then
					assert_inst.gmp_assert_fail (ce_string ("mul_fft.c"), 824, ce_string ("__gmpn_fft_next_size (pl, k) == pl"))
				end
				js.break
			end
			l___tmp_marker.item := create {CE_POINTER [PD_TMP_REENTRANT_T]}.make_shared (default_pointer)
			l_N := a_pl * 32
			if (builtin_expect (((a_k + 1).to_natural_32 * {PLATFORM}.Pointer_bytes.to_natural_32 < {NATURAL_32} 65_536).to_integer, 1)).to_boolean then
				l_tmp___4 := (l_aaf.new ((a_k + 1).to_natural_32 * {PLATFORM}.Pointer_bytes.to_natural_32))
			else
				l_tmp___4 := (tal_reent_inst.gmp_tmp_reentrant_alloc (l___tmp_marker, (a_k + 1).to_natural_32 * {PLATFORM}.Pointer_bytes.to_natural_32))
			end
			l_fft_l := create {CE_POINTER [CE_POINTER [INTEGER_32]]}.make_cast (l_tmp___4)
			l_i := 0
			from
			until
				not (l_i <= a_k)
			loop
				if (builtin_expect ((((1).bit_shift_left (l_i)).to_natural_32 * {PLATFORM}.Integer_32_bytes.to_natural_32 < {NATURAL_32} 65_536).to_integer, 1)).to_boolean then
					l_tmp___9 := (l_aaf.new (((1).bit_shift_left (l_i)).to_natural_32 * {PLATFORM}.Integer_32_bytes.to_natural_32))
				else
					l_tmp___9 := (tal_reent_inst.gmp_tmp_reentrant_alloc (l___tmp_marker, ((1).bit_shift_left (l_i)).to_natural_32 * {PLATFORM}.Integer_32_bytes.to_natural_32))
				end
				l_fft_l[l_i] := create {CE_POINTER [INTEGER_32]}.make_cast (l_tmp___9)
				l_i := l_i + 1
			end
			mpn_fft_initl (l_fft_l, a_k)
			l_K := (1).bit_shift_left (a_k)
			l_M := (l_N).bit_shift_right (a_k)
			l_l := 1 + (l_M - 1) // 32
			l_maxLK := (mpn_mul_fft_lcm ({NATURAL_32} 32, a_k.to_natural_32)).to_integer_32
			l_Nprime := (1 + ((2 * l_M + a_k.to_integer_32) + 2) // l_maxLK.to_integer_32) * l_maxLK.to_integer_32
			l_nprime2 := l_Nprime // 32
			if l_sqr.to_boolean then
				l_tmp___13 := 472
			else
				l_tmp___13 := 565
			end
			if l_nprime2 >= l_tmp___13.to_integer_32 then
				from
				until
					js.loop_done
				loop
					l_K2 := ((1).bit_shift_left ((gmpn_fft_best_k (l_nprime2, l_sqr)))).to_natural_32
					if ((l_nprime2.to_natural_32).bit_and ((l_K2 - {NATURAL_32} 1))) = {NATURAL_32} 0 then
						js.break
					end
					if not js.loop_jumped then
						l_nprime2 := ((((l_nprime2.to_natural_32 + l_K2) - {NATURAL_32} 1)).bit_and ((-l_K2.to_integer_32).to_natural_32)).to_integer_32
						l_Nprime := l_nprime2 * 32
					end
				end
			end
			from
			until
				js.loop_done
			loop
				if not (l_nprime2 < a_pl) then
					assert_inst.gmp_assert_fail (ce_string ("mul_fft.c"), 857, ce_string ("nprime < pl"))
				end
				js.break
			end
			if (builtin_expect (((2 * (l_nprime2 + 1)).to_natural_32 * ({PLATFORM}.Natural_32_bytes.to_natural_32).to_natural_32 < {NATURAL_32} 65_536).to_integer, 1)).to_boolean then
				l_tmp___17 := (l_aaf.new (((2 * (l_nprime2 + 1)).to_natural_32 * ({PLATFORM}.Natural_32_bytes.to_natural_32).to_natural_32).to_natural_32))
			else
				l_tmp___17 := (tal_reent_inst.gmp_tmp_reentrant_alloc (l___tmp_marker, ((2 * (l_nprime2 + 1)).to_natural_32 * ({PLATFORM}.Natural_32_bytes.to_natural_32).to_natural_32).to_natural_32))
			end
			l_T := create {CE_POINTER [NATURAL_32]}.make_cast (l_tmp___17)
			l_Mp := (l_Nprime).bit_shift_right (a_k)
			if (builtin_expect (((l_K.to_integer_32 * (l_nprime2 + 1)).to_natural_32 * ({PLATFORM}.Natural_32_bytes.to_natural_32).to_natural_32 < {NATURAL_32} 65_536).to_integer, 1)).to_boolean then
				l_tmp___22 := (l_aaf.new (((l_K.to_integer_32 * (l_nprime2 + 1)).to_natural_32 * ({PLATFORM}.Natural_32_bytes.to_natural_32).to_natural_32).to_natural_32))
			else
				l_tmp___22 := (tal_reent_inst.gmp_tmp_reentrant_alloc (l___tmp_marker, ((l_K.to_integer_32 * (l_nprime2 + 1)).to_natural_32 * ({PLATFORM}.Natural_32_bytes.to_natural_32).to_natural_32).to_natural_32))
			end
			l_A := create {CE_POINTER [NATURAL_32]}.make_cast (l_tmp___22)
			if (builtin_expect ((l_K.to_natural_32 * {PLATFORM}.Pointer_bytes.to_natural_32 < {NATURAL_32} 65_536).to_integer, 1)).to_boolean then
				l_tmp___27 := (l_aaf.new (l_K.to_natural_32 * {PLATFORM}.Pointer_bytes.to_natural_32))
			else
				l_tmp___27 := (tal_reent_inst.gmp_tmp_reentrant_alloc (l___tmp_marker, l_K.to_natural_32 * {PLATFORM}.Pointer_bytes.to_natural_32))
			end
			l_Ap := create {CE_POINTER [CE_POINTER [NATURAL_32]]}.make_cast (l_tmp___27)
			mpn_mul_fft_decompose (l_A, l_Ap, l_K, l_nprime2.to_integer_32, a_n, a_nl, l_l.to_integer_32, l_Mp.to_integer_32, l_T)
			if l_sqr.to_boolean then
				l_pla := (l_l * (l_K - 1).to_integer_32 + l_nprime2) + 1
				if (builtin_expect ((l_pla.to_natural_32 * ({PLATFORM}.Natural_32_bytes.to_natural_32).to_natural_32 < {NATURAL_32} 65_536).to_integer, 1)).to_boolean then
					l_tmp___32 := (l_aaf.new ((l_pla.to_natural_32 * ({PLATFORM}.Natural_32_bytes.to_natural_32).to_natural_32).to_natural_32))
				else
					l_tmp___32 := (tal_reent_inst.gmp_tmp_reentrant_alloc (l___tmp_marker, (l_pla.to_natural_32 * ({PLATFORM}.Natural_32_bytes.to_natural_32).to_natural_32).to_natural_32))
				end
				l_B := create {CE_POINTER [NATURAL_32]}.make_cast (l_tmp___32)
				if (builtin_expect ((l_K.to_natural_32 * {PLATFORM}.Pointer_bytes.to_natural_32 < {NATURAL_32} 65_536).to_integer, 1)).to_boolean then
					l_tmp___37 := (l_aaf.new (l_K.to_natural_32 * {PLATFORM}.Pointer_bytes.to_natural_32))
				else
					l_tmp___37 := (tal_reent_inst.gmp_tmp_reentrant_alloc (l___tmp_marker, l_K.to_natural_32 * {PLATFORM}.Pointer_bytes.to_natural_32))
				end
				l_Bp := create {CE_POINTER [CE_POINTER [NATURAL_32]]}.make_cast (l_tmp___37)
			else
				if (builtin_expect (((l_K.to_integer_32 * (l_nprime2 + 1)).to_natural_32 * ({PLATFORM}.Natural_32_bytes.to_natural_32).to_natural_32 < {NATURAL_32} 65_536).to_integer, 1)).to_boolean then
					l_tmp___42 := (l_aaf.new (((l_K.to_integer_32 * (l_nprime2 + 1)).to_natural_32 * ({PLATFORM}.Natural_32_bytes.to_natural_32).to_natural_32).to_natural_32))
				else
					l_tmp___42 := (tal_reent_inst.gmp_tmp_reentrant_alloc (l___tmp_marker, ((l_K.to_integer_32 * (l_nprime2 + 1)).to_natural_32 * ({PLATFORM}.Natural_32_bytes.to_natural_32).to_natural_32).to_natural_32))
				end
				l_B := create {CE_POINTER [NATURAL_32]}.make_cast (l_tmp___42)
				if (builtin_expect ((l_K.to_natural_32 * {PLATFORM}.Pointer_bytes.to_natural_32 < {NATURAL_32} 65_536).to_integer, 1)).to_boolean then
					l_tmp___47 := (l_aaf.new (l_K.to_natural_32 * {PLATFORM}.Pointer_bytes.to_natural_32))
				else
					l_tmp___47 := (tal_reent_inst.gmp_tmp_reentrant_alloc (l___tmp_marker, l_K.to_natural_32 * {PLATFORM}.Pointer_bytes.to_natural_32))
				end
				l_Bp := create {CE_POINTER [CE_POINTER [NATURAL_32]]}.make_cast (l_tmp___47)
				mpn_mul_fft_decompose (l_B, l_Bp, l_K, l_nprime2.to_integer_32, a_m, a_ml, l_l.to_integer_32, l_Mp.to_integer_32, l_T)
			end
			l_h := mpn_mul_fft_internal (a_op, a_pl, a_k, l_Ap, l_Bp, l_A, l_B, l_nprime2, l_l, l_Mp, l_fft_l, l_T, l_sqr)
			from
			until
				js.loop_done
			loop
				if (builtin_expect ((not (l___tmp_marker.item).is_void).to_integer, 0)).to_boolean then
					tal_reent_inst.gmp_tmp_reentrant_free (l___tmp_marker.item)
				end
				js.break
			end
			Result := l_h
		end

	mpn_fft_add_modF (a_r: CE_POINTER [NATURAL_32]; a_a: CE_POINTER [NATURAL_32]; a_b: CE_POINTER [NATURAL_32]; a_n: INTEGER_32)
		local
			l_r: CE_ARRAY [CE_POINTER [NATURAL_32]]
			l_c: NATURAL_32
			l_x___2: CE_ARRAY [NATURAL_32]
			l___ptr_dummy: CE_ARRAY [CE_POINTER [NATURAL_32]]
			js: JUMP_STATE
		do
			create l_r.make (<<1>>)
			create l_x___2.make (<<1>>)
			create l___ptr_dummy.make (<<1>>)

			l_r.item := a_r
			l_c := ((a_a[a_n] + a_b[a_n]) + (gmp_inst.gmpn_add_n (l_r.item, a_a, a_b, a_n.to_integer_32)))
			l_x___2.item := ((l_c - {NATURAL_32} 1)).bit_and ((-(l_c /= {NATURAL_32} 0).to_integer).to_integer.to_natural_32)
			l_r.item[a_n] := l_c - l_x___2.item
			from
			until
				js.loop_done
			loop
				asm97 (l___ptr_dummy, l_r, l_x___2)
				js.break
			end
		end

	mpn_fft_div_2exp_modF (a_r: CE_POINTER [NATURAL_32]; a_a: CE_POINTER [NATURAL_32]; a_k: INTEGER_32; a_n: INTEGER_32)
		local
			l_i: INTEGER_32
		do
			l_i := ((2 * a_n) * 32 - a_k.to_integer_32).to_integer_32
			mpn_fft_mul_2exp_modF (a_r, a_a, l_i.to_natural_32, a_n)
			mpn_fft_normalize (a_r, a_n)
		end

	mpn_fft_fft (a_Ap: CE_POINTER [CE_POINTER [NATURAL_32]]; a_K: INTEGER_32; a_ll: CE_POINTER [CE_POINTER [INTEGER_32]]; a_omega: INTEGER_32; a_n: INTEGER_32; a_inc: INTEGER_32; a_tp: CE_POINTER [NATURAL_32])
		local
			l_Ap: CE_POINTER [CE_POINTER [NATURAL_32]]
			l_cy: NATURAL_32
			l_j: INTEGER_32
			l_lk: CE_POINTER [INTEGER_32]
			js: JUMP_STATE
		do
			create l_Ap.make
			create l_lk.make

			l_Ap := a_Ap
			if a_K = 2 then
				from
				until
					js.loop_done
				loop
					from
					until
						js.loop_done
					loop
						gmp_inst.gmpn_copyi (a_tp, l_Ap[0], a_n + 1)
						js.break
					end
					js.break
				end
				gmp_inst.gmpn_add_n (l_Ap[0], l_Ap[0], l_Ap[a_inc], a_n + 1).do_nothing
				l_cy := gmp_inst.gmpn_sub_n (l_Ap[a_inc], a_tp, l_Ap[a_inc], a_n + 1)
				if (l_Ap[0])[a_n] > {NATURAL_32} 1 then
					(l_Ap[0])[a_n] := {NATURAL_32} 1 - (gmp_inst.gmpn_sub_1 (l_Ap[0], l_Ap[0], a_n, (l_Ap[0])[a_n] - {NATURAL_32} 1))
				end
				if l_cy.to_boolean then
					(l_Ap[a_inc])[a_n] := gmp_inst.gmpn_add_1 (l_Ap[a_inc], l_Ap[a_inc], a_n, ((l_Ap[a_inc])[a_n]).bit_not + {NATURAL_32} 1)
				end
			else
				l_lk := a_ll.item
				mpn_fft_fft (l_Ap, (a_K).bit_shift_right (1), a_ll - 1, 2 * a_omega, a_n, a_inc * 2, a_tp)
				mpn_fft_fft (l_Ap + a_inc, (a_K).bit_shift_right (1), a_ll - 1, 2 * a_omega, a_n, a_inc * 2, a_tp)
				l_j := 0
				from
				until
					not (l_j.to_integer_32 < (a_K).bit_shift_right (1))
				loop
					mpn_fft_mul_2exp_modF (a_tp, l_Ap[a_inc], ((l_lk[0]).to_integer_32 * a_omega).to_natural_32, a_n)
					mpn_fft_sub_modF (l_Ap[a_inc], l_Ap[0], a_tp, a_n.to_integer_32)
					mpn_fft_add_modF (l_Ap[0], l_Ap[0], a_tp, a_n.to_integer_32)
					l_j := l_j + 1
					l_lk := l_lk + (2)
					l_Ap := l_Ap + (2 * a_inc)
				end
			end
		end

	mpn_fft_fftinv (a_Ap: CE_POINTER [CE_POINTER [NATURAL_32]]; a_K: INTEGER_32; a_omega: INTEGER_32; a_n: INTEGER_32; a_tp: CE_POINTER [NATURAL_32])
		local
			l_Ap: CE_POINTER [CE_POINTER [NATURAL_32]]
			l_cy: NATURAL_32
			l_j: INTEGER_32
			l_K2: INTEGER_32
			js: JUMP_STATE
		do
			create l_Ap.make

			l_Ap := a_Ap
			if a_K = 2 then
				from
				until
					js.loop_done
				loop
					from
					until
						js.loop_done
					loop
						gmp_inst.gmpn_copyi (a_tp, l_Ap[0], a_n + 1)
						js.break
					end
					js.break
				end
				gmp_inst.gmpn_add_n (l_Ap[0], l_Ap[0], l_Ap[1], a_n + 1).do_nothing
				l_cy := gmp_inst.gmpn_sub_n (l_Ap[1], a_tp, l_Ap[1], a_n + 1)
				if (l_Ap[0])[a_n] > {NATURAL_32} 1 then
					(l_Ap[0])[a_n] := {NATURAL_32} 1 - (gmp_inst.gmpn_sub_1 (l_Ap[0], l_Ap[0], a_n, (l_Ap[0])[a_n] - {NATURAL_32} 1))
				end
				if l_cy.to_boolean then
					(l_Ap[1])[a_n] := gmp_inst.gmpn_add_1 (l_Ap[1], l_Ap[1], a_n, ((l_Ap[1])[a_n]).bit_not + {NATURAL_32} 1)
				end
			else
				l_K2 := (a_K).bit_shift_right (1)
				mpn_fft_fftinv (l_Ap, l_K2, 2 * a_omega, a_n, a_tp)
				mpn_fft_fftinv (l_Ap + l_K2, l_K2, 2 * a_omega, a_n, a_tp)
				l_j := 0
				from
				until
					not (l_j < l_K2)
				loop
					mpn_fft_mul_2exp_modF (a_tp, l_Ap[l_K2], (l_j.to_integer_32 * a_omega).to_natural_32, a_n)
					mpn_fft_sub_modF (l_Ap[l_K2], l_Ap[0], a_tp, a_n.to_integer_32)
					mpn_fft_add_modF (l_Ap[0], l_Ap[0], a_tp, a_n.to_integer_32)
					l_j := l_j + 1
					l_Ap := l_Ap + 1
				end
			end
		end

	mpn_fft_initl (a_l: CE_POINTER [CE_POINTER [INTEGER_32]]; a_k: INTEGER_32)
		local
			l_i: INTEGER_32
			l_j: INTEGER_32
			l_K: INTEGER_32
			l_li: CE_POINTER [INTEGER_32]
			js: JUMP_STATE
		do
			create l_li.make;

			(a_l[0])[0] := 0
			l_i := 1
			l_K := 1
			from
			until
				not (not js.loop_done and then (l_i <= a_k))
			loop
				l_li := a_l[l_i]
				l_j := 0
				from
				until
					not (l_j < l_K)
				loop
					l_li[l_j] := 2 * (a_l[(l_i - 1)])[l_j]
					l_li[(l_K + l_j)] := 1 + l_li[l_j]
					l_j := l_j + 1
				end
				l_i := l_i + 1
				l_K := l_K * (2)
			end
		end

	mpn_fft_mul_2exp_modF (a_r: CE_POINTER [NATURAL_32]; a_a: CE_POINTER [NATURAL_32]; a_d: NATURAL_32; a_n: INTEGER_32)
		local
			l_r: CE_ARRAY [CE_POINTER [NATURAL_32]]
			l_d: NATURAL_32
			l_sh: INTEGER_32
			l_cc: CE_ARRAY [NATURAL_32]
			l_rd: NATURAL_32
			l___d: CE_POINTER [NATURAL_32]
			l___s: CE_POINTER [NATURAL_32]
			l___n: INTEGER_32
			l_tmp: CE_POINTER [NATURAL_32]
			l_tmp___0: CE_POINTER [NATURAL_32]
			l___ptr_dummy: CE_ARRAY [CE_POINTER [NATURAL_32]]
			l___ptr_dummy___0: CE_ARRAY [CE_POINTER [NATURAL_32]]
			l___d___0: CE_POINTER [NATURAL_32]
			l___s___0: CE_POINTER [NATURAL_32]
			l___n___0: INTEGER_32
			l_tmp___1: CE_POINTER [NATURAL_32]
			l_tmp___2: CE_POINTER [NATURAL_32]
			l_tmp___3: NATURAL_32
			js: JUMP_STATE
		do
			create l_r.make (<<1>>)
			create l_cc.make (<<1>>)
			create l___d.make
			create l___s.make
			create l_tmp.make
			create l_tmp___0.make
			create l___ptr_dummy.make (<<1>>)
			create l___ptr_dummy___0.make (<<1>>)
			create l___d___0.make
			create l___s___0.make
			create l_tmp___1.make
			create l_tmp___2.make

			l_r.item := a_r
			l_d := a_d
			l_sh := (l_d \\ {NATURAL_32} 32).to_integer_32
			l_d := l_d // ({NATURAL_32} 32)
			if l_d.to_natural_32 >= a_n.to_natural_32 then
				l_d := (l_d.to_natural_32 - a_n.to_natural_32).to_natural_32
				if l_sh /= 0 then
					gmp_inst.gmpn_lshift (l_r.item, (a_a + a_n) - l_d.to_integer_32, (l_d + {NATURAL_32} 1).to_integer_32, l_sh.to_natural_32).do_nothing
					l_rd := l_r.item[l_d.to_integer_32]
					l_cc.item := lshiftc_inst.gmpn_lshiftc (l_r.item + l_d.to_integer_32, a_a, (a_n.to_natural_32 - l_d.to_natural_32).to_integer_32, l_sh.to_natural_32)
				else
					from
					until
						js.loop_done
					loop
						from
						until
							js.loop_done
						loop
							gmp_inst.gmpn_copyi (l_r.item, (a_a + a_n) - l_d.to_integer_32, l_d.to_integer_32)
							js.break
						end
						js.break
					end
					l_rd := a_a[a_n]
					from
					until
						js.loop_done
					loop
						l___d := l_r.item + l_d.to_integer_32
						l___s := a_a
						l___n := (a_n.to_natural_32 - l_d.to_natural_32).to_integer_32
						from
						until
							js.loop_done
						loop
							l_tmp := l___d
							l___d := l___d + 1
							l_tmp___0 := l___s
							l___s := l___s + 1
							l_tmp.item := (((l_tmp___0.item).bit_not).bit_and ({NATURAL_32} 4_294_967_295))
							l___n := l___n - 1
							if not l___n.to_boolean then
								js.break
							end
						end
						js.break
					end
					l_cc.item := (0).to_natural_32
				end
				l_r.item[a_n] := (0).to_natural_32
				l_cc.item := l_cc.item + 1
				from
				until
					js.loop_done
				loop
					asm95 (l___ptr_dummy, l_r, l_cc)
					js.break
				end
				l_rd := l_rd + 1
				if l_rd = {NATURAL_32} 0 then
					l_cc.item := (1).to_natural_32
				else
					l_cc.item := l_rd
				end
				l_r.item := (l_r.item + l_d.to_integer_32) + (l_rd = {NATURAL_32} 0).to_integer
				from
				until
					js.loop_done
				loop
					asm96 (l___ptr_dummy___0, l_r, l_cc)
					js.break
				end
			else
				if l_sh /= 0 then
					lshiftc_inst.gmpn_lshiftc (l_r.item, (a_a + a_n) - l_d.to_integer_32, (l_d + {NATURAL_32} 1).to_integer_32, l_sh.to_natural_32).do_nothing
					l_rd := (l_r.item[l_d.to_integer_32]).bit_not
					l_cc.item := gmp_inst.gmpn_lshift (l_r.item + l_d.to_integer_32, a_a, (a_n.to_natural_32 - l_d.to_natural_32).to_integer_32, l_sh.to_natural_32)
				else
					from
					until
						js.loop_done
					loop
						l___d___0 := l_r.item
						l___s___0 := (a_a + a_n) - l_d.to_integer_32
						l___n___0 := (l_d + {NATURAL_32} 1).to_integer_32
						from
						until
							js.loop_done
						loop
							l_tmp___1 := l___d___0
							l___d___0 := l___d___0 + 1
							l_tmp___2 := l___s___0
							l___s___0 := l___s___0 + 1
							l_tmp___1.item := (((l_tmp___2.item).bit_not).bit_and ({NATURAL_32} 4_294_967_295))
							l___n___0 := l___n___0 - 1
							if not l___n___0.to_boolean then
								js.break
							end
						end
						js.break
					end
					l_rd := a_a[a_n]
					from
					until
						js.loop_done
					loop
						from
						until
							js.loop_done
						loop
							gmp_inst.gmpn_copyi (l_r.item + l_d.to_integer_32, a_a, (a_n.to_natural_32 - l_d.to_natural_32).to_integer_32)
							js.break
						end
						js.break
					end
					l_cc.item := (0).to_natural_32
				end
				if l_d /= {NATURAL_32} 0 then
					l_tmp___3 := l_cc.item
					l_cc.item := l_cc.item - 1
					if l_tmp___3 = {NATURAL_32} 0 then
						l_cc.item := gmp_inst.gmpn_add_1 (l_r.item, l_r.item, a_n, (1).to_natural_32)
					end
					l_cc.item := (gmp_inst.gmpn_sub_1 (l_r.item, l_r.item, l_d.to_integer_32, l_cc.item)) + {NATURAL_32} 1
				end
				l_r.item[a_n] := (-(gmp_inst.gmpn_sub_1 (l_r.item + l_d.to_integer_32, (l_r.item + l_d.to_integer_32), (a_n.to_natural_32 - l_d.to_natural_32).to_integer_32, l_cc.item)).to_integer_32).to_natural_32
				l_r.item[a_n] := l_r.item[a_n] - (gmp_inst.gmpn_sub_1 (l_r.item + l_d.to_integer_32, (l_r.item + l_d.to_integer_32), (a_n.to_natural_32 - l_d.to_natural_32).to_integer_32, l_rd))
				if ((l_r.item[a_n]).bit_and ((({NATURAL_32} 4_294_967_295).bit_xor ((({NATURAL_32} 4_294_967_295).bit_shift_right (1)))))).to_boolean then
					l_r.item[a_n] := gmp_inst.gmpn_add_1 (l_r.item, l_r.item, a_n, (1).to_natural_32)
				end
			end
		end

	mpn_fft_mul_modF_K (a_ap: CE_POINTER [CE_POINTER [NATURAL_32]]; a_bp: CE_POINTER [CE_POINTER [NATURAL_32]]; a_n: INTEGER_32; a_K: INTEGER_32)
		local
			l_ap: CE_POINTER [CE_POINTER [NATURAL_32]]
			l_bp: CE_POINTER [CE_POINTER [NATURAL_32]]
			l_i: INTEGER_32
			l_sqr: INTEGER_32
			l___tmp_marker: CE_ARRAY [CE_POINTER [PD_TMP_REENTRANT_T]]
			l_k: INTEGER_32
			l_K2: INTEGER_32
			l_nprime2: INTEGER_32
			l_Nprime22: INTEGER_32
			l_M2: INTEGER_32
			l_maxLK: INTEGER_32
			l_l: INTEGER_32
			l_Mp2: INTEGER_32
			l_fft_l: CE_POINTER [CE_POINTER [INTEGER_32]]
			l_Ap2: CE_POINTER [CE_POINTER [NATURAL_32]]
			l_Bp2: CE_POINTER [CE_POINTER [NATURAL_32]]
			l_A: CE_POINTER [NATURAL_32]
			l_B: CE_POINTER [NATURAL_32]
			l_T: CE_POINTER [NATURAL_32]
			l_K3: NATURAL_32
			l_tmp___0: INTEGER_32
			l_tmp___4: CE_POINTER [ANY]
			l_tmp___9: CE_POINTER [ANY]
			l_tmp___14: CE_POINTER [ANY]
			l_tmp___19: CE_POINTER [ANY]
			l_tmp___24: CE_POINTER [ANY]
			l_tmp___29: CE_POINTER [ANY]
			l_cy: NATURAL_32
			l_a2: CE_POINTER [NATURAL_32]
			l_b2: CE_POINTER [NATURAL_32]
			l_tp: CE_POINTER [NATURAL_32]
			l_tpn: CE_POINTER [NATURAL_32]
			l_cc: NATURAL_32
			l_n2: INTEGER_32
			l_tmp___34: CE_POINTER [ANY]
			l_tmp___36: CE_POINTER [CE_POINTER [NATURAL_32]]
			l_tmp___37: CE_POINTER [CE_POINTER [NATURAL_32]]
			l_tmp___41: INTEGER_32
			l_tmp___42: INTEGER_32
			js: JUMP_STATE
			l_aaf: CE_ALLOCA_FACTORY
		do
			create l_ap.make
			create l_bp.make
			create l___tmp_marker.make (<<1>>)
			create l_fft_l.make
			create l_Ap2.make
			create l_Bp2.make
			create l_A.make
			create l_B.make
			create l_T.make
			create l_tmp___4.make
			create l_tmp___9.make
			create l_tmp___14.make
			create l_tmp___19.make
			create l_tmp___24.make
			create l_tmp___29.make
			create l_a2.make
			create l_b2.make
			create l_tp.make
			create l_tpn.make
			create l_tmp___34.make
			create l_tmp___36.make
			create l_tmp___37.make

			l_ap := a_ap
			l_bp := a_bp
			l_sqr := (l_ap.is_equal (l_bp)).to_integer
			l___tmp_marker.item := create {CE_POINTER [PD_TMP_REENTRANT_T]}.make_shared (default_pointer)
			if l_sqr.to_boolean then
				l_tmp___42 := 472
			else
				l_tmp___42 := 565
			end
			if a_n >= l_tmp___42.to_integer_32 then
				l_k := gmpn_fft_best_k (a_n, l_sqr)
				l_K2 := (1).bit_shift_left (l_k)
				from
				until
					js.loop_done
				loop
					if ((a_n).bit_and ((l_K2 - 1).to_integer_32)) /= 0 then
						assert_inst.gmp_assert_fail (ce_string ("mul_fft.c"), 437, ce_string ("(n & (K2 - 1)) == 0"))
					end
					js.break
				end
				if l_K2 > 32 then
					l_maxLK := l_K2
				else
					l_maxLK := 32
				end
				l_M2 := ((a_n * 32).bit_shift_right (l_k)).to_integer_32
				l_l := ((a_n).bit_shift_right (l_k)).to_integer_32
				l_Nprime22 := ((((2 * l_M2 + l_k) + 2) + l_maxLK) // l_maxLK) * l_maxLK
				l_nprime2 := l_Nprime22 // 32
				if l_sqr.to_boolean then
					l_tmp___0 := 472
				else
					l_tmp___0 := 565
				end
				if l_nprime2 >= l_tmp___0 then
					from
					until
						js.loop_done
					loop
						l_K3 := ((1).bit_shift_left ((gmpn_fft_best_k (l_nprime2.to_integer_32, l_sqr)))).to_natural_32
						if ((l_nprime2.to_natural_32).bit_and ((l_K3 - {NATURAL_32} 1))) = {NATURAL_32} 0 then
							js.break
						end
						if not js.loop_jumped then
							l_nprime2 := ((((l_nprime2.to_natural_32 + l_K3) - {NATURAL_32} 1)).bit_and ((-l_K3.to_integer_32).to_natural_32)).to_integer_32
							l_Nprime22 := l_nprime2 * 32
						end
					end
				end
				from
				until
					js.loop_done
				loop
					if not (l_nprime2.to_integer_32 < a_n) then
						assert_inst.gmp_assert_fail (ce_string ("mul_fft.c"), 459, ce_string ("nprime2 < n"))
					end
					js.break
				end
				l_Mp2 := (l_Nprime22).bit_shift_right (l_k)
				if (builtin_expect ((l_K2.to_natural_32 * {PLATFORM}.Pointer_bytes.to_natural_32 < {NATURAL_32} 65_536).to_integer, 1)).to_boolean then
					l_tmp___4 := (l_aaf.new (l_K2.to_natural_32 * {PLATFORM}.Pointer_bytes.to_natural_32))
				else
					l_tmp___4 := (tal_reent_inst.gmp_tmp_reentrant_alloc (l___tmp_marker, l_K2.to_natural_32 * {PLATFORM}.Pointer_bytes.to_natural_32))
				end
				l_Ap2 := create {CE_POINTER [CE_POINTER [NATURAL_32]]}.make_cast (l_tmp___4)
				if (builtin_expect ((l_K2.to_natural_32 * {PLATFORM}.Pointer_bytes.to_natural_32 < {NATURAL_32} 65_536).to_integer, 1)).to_boolean then
					l_tmp___9 := (l_aaf.new (l_K2.to_natural_32 * {PLATFORM}.Pointer_bytes.to_natural_32))
				else
					l_tmp___9 := (tal_reent_inst.gmp_tmp_reentrant_alloc (l___tmp_marker, l_K2.to_natural_32 * {PLATFORM}.Pointer_bytes.to_natural_32))
				end
				l_Bp2 := create {CE_POINTER [CE_POINTER [NATURAL_32]]}.make_cast (l_tmp___9)
				if (builtin_expect ((((2 * (l_nprime2 + 1)).bit_shift_left (l_k)).to_natural_32 * {PLATFORM}.Natural_32_bytes.to_natural_32 < {NATURAL_32} 65_536).to_integer, 1)).to_boolean then
					l_tmp___14 := (l_aaf.new (((2 * (l_nprime2 + 1)).bit_shift_left (l_k)).to_natural_32 * {PLATFORM}.Natural_32_bytes.to_natural_32))
				else
					l_tmp___14 := (tal_reent_inst.gmp_tmp_reentrant_alloc (l___tmp_marker, ((2 * (l_nprime2 + 1)).bit_shift_left (l_k)).to_natural_32 * {PLATFORM}.Natural_32_bytes.to_natural_32))
				end
				l_A := create {CE_POINTER [NATURAL_32]}.make_cast (l_tmp___14)
				if (builtin_expect (((2 * (l_nprime2 + 1)).to_natural_32 * {PLATFORM}.Natural_32_bytes.to_natural_32 < {NATURAL_32} 65_536).to_integer, 1)).to_boolean then
					l_tmp___19 := (l_aaf.new ((2 * (l_nprime2 + 1)).to_natural_32 * {PLATFORM}.Natural_32_bytes.to_natural_32))
				else
					l_tmp___19 := (tal_reent_inst.gmp_tmp_reentrant_alloc (l___tmp_marker, (2 * (l_nprime2 + 1)).to_natural_32 * {PLATFORM}.Natural_32_bytes.to_natural_32))
				end
				l_T := create {CE_POINTER [NATURAL_32]}.make_cast (l_tmp___19)
				l_B := l_A + (((l_nprime2 + 1)).bit_shift_left (l_k))
				if (builtin_expect (((l_k + 1).to_natural_32 * {PLATFORM}.Pointer_bytes.to_natural_32 < {NATURAL_32} 65_536).to_integer, 1)).to_boolean then
					l_tmp___24 := (l_aaf.new ((l_k + 1).to_natural_32 * {PLATFORM}.Pointer_bytes.to_natural_32))
				else
					l_tmp___24 := (tal_reent_inst.gmp_tmp_reentrant_alloc (l___tmp_marker, (l_k + 1).to_natural_32 * {PLATFORM}.Pointer_bytes.to_natural_32))
				end
				l_fft_l := create {CE_POINTER [CE_POINTER [INTEGER_32]]}.make_cast (l_tmp___24)
				l_i := 0
				from
				until
					not (l_i <= l_k)
				loop
					if (builtin_expect ((((1).bit_shift_left (l_i)).to_natural_32 * {PLATFORM}.Integer_32_bytes.to_natural_32 < {NATURAL_32} 65_536).to_integer, 1)).to_boolean then
						l_tmp___29 := (l_aaf.new (((1).bit_shift_left (l_i)).to_natural_32 * {PLATFORM}.Integer_32_bytes.to_natural_32))
					else
						l_tmp___29 := (tal_reent_inst.gmp_tmp_reentrant_alloc (l___tmp_marker, ((1).bit_shift_left (l_i)).to_natural_32 * {PLATFORM}.Integer_32_bytes.to_natural_32))
					end
					l_fft_l[l_i] := create {CE_POINTER [INTEGER_32]}.make_cast (l_tmp___29)
					l_i := l_i + 1
				end
				mpn_fft_initl (l_fft_l, l_k)
				l_i := 0
				from
				until
					not (l_i < a_K)
				loop
					mpn_fft_normalize (l_ap.item, a_n)
					if not l_sqr.to_boolean then
						mpn_fft_normalize (l_bp.item, a_n)
					end
					mpn_mul_fft_decompose (l_A, l_Ap2, l_K2, l_nprime2, l_ap.item, (((l_l).bit_shift_left (l_k)) + 1).to_integer_32, l_l, l_Mp2, l_T)
					if not l_sqr.to_boolean then
						mpn_mul_fft_decompose (l_B, l_Bp2, l_K2, l_nprime2, l_bp.item, (((l_l).bit_shift_left (l_k)) + 1).to_integer_32, l_l, l_Mp2, l_T)
					end
					l_cy := mpn_mul_fft_internal (l_ap.item, a_n, l_k, l_Ap2, l_Bp2, l_A, l_B, l_nprime2.to_integer_32, l_l.to_integer_32, l_Mp2.to_integer_32, l_fft_l, l_T, l_sqr)
					l_ap.item[a_n] := l_cy
					l_i := l_i + 1
					l_ap := l_ap + 1
					l_bp := l_bp + 1
				end
			else
				l_n2 := (2 * a_n).to_integer_32
				if (builtin_expect ((l_n2.to_natural_32 * {PLATFORM}.Natural_32_bytes.to_natural_32 < {NATURAL_32} 65_536).to_integer, 1)).to_boolean then
					l_tmp___34 := (l_aaf.new (l_n2.to_natural_32 * {PLATFORM}.Natural_32_bytes.to_natural_32))
				else
					l_tmp___34 := (tal_reent_inst.gmp_tmp_reentrant_alloc (l___tmp_marker, l_n2.to_natural_32 * {PLATFORM}.Natural_32_bytes.to_natural_32))
				end
				l_tp := create {CE_POINTER [NATURAL_32]}.make_cast (l_tmp___34)
				l_tpn := l_tp + a_n
				l_i := 0
				from
				until
					not (not js.loop_done and then (l_i < a_K))
				loop
					l_tmp___36 := l_ap
					l_ap := l_ap + 1
					l_a2 := l_tmp___36.item
					l_tmp___37 := l_bp
					l_bp := l_bp + 1
					l_b2 := l_tmp___37.item
					if l_sqr.to_boolean then
						sqr_inst.gmpn_sqr (l_tp, l_a2, a_n)
					else
						mul_n_inst.gmpn_mul_n (l_tp, l_b2, l_a2, a_n)
					end
					if l_a2[a_n] /= {NATURAL_32} 0 then
						l_cc := gmp_inst.gmpn_add_n (l_tpn, l_tpn, l_b2, a_n)
					else
						l_cc := (0).to_natural_32
					end
					if l_b2[a_n] /= {NATURAL_32} 0 then
						l_cc := l_cc + ((gmp_inst.gmpn_add_n (l_tpn, l_tpn, l_a2, a_n)) + l_a2[a_n])
					end
					if l_cc /= {NATURAL_32} 0 then
						l_cc := gmp_inst.gmpn_add_1 (l_tp, l_tp, l_n2.to_integer_32, l_cc)
					end
					if (gmp_inst.gmpn_sub_n (l_a2, l_tp, l_tpn, a_n)).to_boolean then
						if (gmp_inst.gmpn_add_1 (l_a2, l_a2, a_n, (1).to_natural_32)).to_boolean then
							l_tmp___41 := 1
						else
							l_tmp___41 := 0
						end
					else
						l_tmp___41 := 0
					end
					l_a2[a_n] := l_tmp___41.to_natural_32
					l_i := l_i + 1
				end
			end
			from
			until
				js.loop_done
			loop
				if (builtin_expect ((not (l___tmp_marker.item).is_void).to_integer, 0)).to_boolean then
					tal_reent_inst.gmp_tmp_reentrant_free (l___tmp_marker.item)
				end
				js.break
			end
		end

	mpn_fft_norm_modF (a_rp: CE_POINTER [NATURAL_32]; a_n: INTEGER_32; a_ap: CE_POINTER [NATURAL_32]; a_an: INTEGER_32): INTEGER_32
		local
			l_l: INTEGER_32
			l_m: INTEGER_32
			l_cc: NATURAL_32
			l_rpn: INTEGER_32
			js: JUMP_STATE
		do
			l_m := a_an - 2 * a_n
			if l_m > 0 then
				l_l := a_n
				l_cc := gmp_inst.gmpn_add_n (a_rp, a_ap, (a_ap + 2 * a_n), l_m)
				l_rpn := (gmp_inst.gmpn_add_1 (a_rp + l_m, (a_ap + l_m), a_n - l_m, l_cc)).to_integer_32
			else
				l_l := a_an - a_n
				from
				until
					js.loop_done
				loop
					from
					until
						js.loop_done
					loop
						gmp_inst.gmpn_copyi (a_rp, a_ap, a_n)
						js.break
					end
					js.break
				end
				l_rpn := 0
			end
			l_cc := gmp_inst.gmpn_sub_n (a_rp, a_rp, (a_ap + a_n), l_l)
			l_rpn := (l_rpn.to_natural_32 - (gmp_inst.gmpn_sub_1 (a_rp + l_l, (a_rp + l_l), a_n - l_l, l_cc))).to_integer_32
			if l_rpn < 0 then
				l_rpn := (gmp_inst.gmpn_add_1 (a_rp, a_rp, a_n, (1).to_natural_32)).to_integer_32
			end
			Result := l_rpn
		end

	mpn_fft_normalize (a_ap: CE_POINTER [NATURAL_32]; a_n: INTEGER_32)
		local
			l_ap: CE_ARRAY [CE_POINTER [NATURAL_32]]
			l___ptr_dummy: CE_ARRAY [CE_POINTER [NATURAL_32]]
			l___dst: CE_POINTER [NATURAL_32]
			l___n: INTEGER_32
			l_tmp: CE_POINTER [NATURAL_32]
			js: JUMP_STATE
		do
			create l_ap.make (<<1>>)
			create l___ptr_dummy.make (<<1>>)
			create l___dst.make
			create l_tmp.make

			l_ap.item := a_ap
			if l_ap.item[a_n] /= {NATURAL_32} 0 then
				from
				until
					js.loop_done
				loop
					asm99 (l___ptr_dummy, l_ap)
					js.break
				end
				if l_ap.item[a_n] = {NATURAL_32} 0 then
					from
					until
						js.loop_done
					loop
						if a_n /= 0 then
							l___dst := l_ap.item
							l___n := a_n
							from
							until
								js.loop_done
							loop
								l_tmp := l___dst
								l___dst := l___dst + 1
								l_tmp.item := (0).to_natural_32
								l___n := l___n - 1
								if not l___n.to_boolean then
									js.break
								end
							end
						end
						js.break
					end
					l_ap.item[a_n] := (1).to_natural_32
				else
					l_ap.item[a_n] := (0).to_natural_32
				end
			end
		end

	mpn_fft_sub_modF (a_r: CE_POINTER [NATURAL_32]; a_a: CE_POINTER [NATURAL_32]; a_b: CE_POINTER [NATURAL_32]; a_n: INTEGER_32)
		local
			l_r: CE_ARRAY [CE_POINTER [NATURAL_32]]
			l_c: NATURAL_32
			l_x___2: CE_ARRAY [NATURAL_32]
			l___ptr_dummy: CE_ARRAY [CE_POINTER [NATURAL_32]]
			js: JUMP_STATE
		do
			create l_r.make (<<1>>)
			create l_x___2.make (<<1>>)
			create l___ptr_dummy.make (<<1>>)

			l_r.item := a_r
			l_c := ((a_a[a_n] - a_b[a_n]) - (gmp_inst.gmpn_sub_n (l_r.item, a_a, a_b, a_n.to_integer_32)))
			l_x___2.item := ((-l_c.to_integer_32).to_natural_32).bit_and ((-(((l_c).bit_and ((({NATURAL_32} 4_294_967_295).bit_xor ((({NATURAL_32} 4_294_967_295).bit_shift_right (1)))))) /= {NATURAL_32} 0).to_integer).to_integer.to_natural_32)
			l_r.item[a_n] := l_x___2.item + l_c
			from
			until
				js.loop_done
			loop
				asm98 (l___ptr_dummy, l_r, l_x___2)
				js.break
			end
		end

	mpn_mul_fft_decompose (a_A: CE_POINTER [NATURAL_32]; a_Ap: CE_POINTER [CE_POINTER [NATURAL_32]]; a_K: INTEGER_32; a_nprime: INTEGER_32; a_n: CE_POINTER [NATURAL_32]; a_nl: INTEGER_32; a_l: INTEGER_32; a_Mp: INTEGER_32; a_T: CE_POINTER [NATURAL_32])
		local
			l_A: CE_POINTER [NATURAL_32]
			l_n: CE_POINTER [NATURAL_32]
			l_nl: INTEGER_32
			l_i: INTEGER_32
			l_j: INTEGER_32
			l_tmp: CE_POINTER [NATURAL_32]
			l_Kl: INTEGER_32
			l___tmp_marker: CE_ARRAY [CE_POINTER [PD_TMP_REENTRANT_T]]
			l_dif: INTEGER_32
			l_cy: INTEGER_32
			l_tmp___3: CE_POINTER [ANY]
			l_subp: INTEGER_32
			l___dst: CE_POINTER [NATURAL_32]
			l___n: INTEGER_32
			l_tmp___14: CE_POINTER [NATURAL_32]
			l___dst___0: CE_POINTER [NATURAL_32]
			l___n___0: INTEGER_32
			l_tmp___15: CE_POINTER [NATURAL_32]
			js: JUMP_STATE
			l_aaf: CE_ALLOCA_FACTORY
		do
			create l_A.make
			create l_n.make
			create l_tmp.make
			create l___tmp_marker.make (<<1>>)
			create l_tmp___3.make
			create l___dst.make
			create l_tmp___14.make
			create l___dst___0.make
			create l_tmp___15.make

			l_A := a_A
			l_n := a_n
			l_nl := a_nl
			l_Kl := (a_K * a_l).to_integer_32
			l___tmp_marker.item := create {CE_POINTER [PD_TMP_REENTRANT_T]}.make_shared (default_pointer)
			if l_nl > l_Kl then
				l_dif := l_nl - l_Kl
				if (builtin_expect (((l_Kl + 1).to_natural_32 * ({PLATFORM}.Natural_32_bytes.to_natural_32).to_natural_32 < {NATURAL_32} 65_536).to_integer, 1)).to_boolean then
					l_tmp___3 := (l_aaf.new (((l_Kl + 1).to_natural_32 * ({PLATFORM}.Natural_32_bytes.to_natural_32).to_natural_32).to_natural_32))
				else
					l_tmp___3 := (tal_reent_inst.gmp_tmp_reentrant_alloc (l___tmp_marker, ((l_Kl + 1).to_natural_32 * ({PLATFORM}.Natural_32_bytes.to_natural_32).to_natural_32).to_natural_32))
				end
				l_tmp := create {CE_POINTER [NATURAL_32]}.make_cast (l_tmp___3)
				if l_dif > l_Kl then
					l_subp := 0
					l_cy := (gmp_inst.gmpn_sub_n (l_tmp, l_n, l_n + l_Kl, l_Kl)).to_integer_32
					l_n := l_n + (2 * l_Kl)
					l_dif := l_dif - l_Kl
					from
					until
						not (l_dif > l_Kl)
					loop
						if l_subp.to_boolean then
							l_cy := (l_cy.to_natural_32 + (gmp_inst.gmpn_sub_n (l_tmp, l_tmp, l_n, l_Kl))).to_integer_32
						else
							l_cy := (l_cy.to_natural_32 - (gmp_inst.gmpn_add_n (l_tmp, l_tmp, l_n, l_Kl))).to_integer_32
						end
						l_subp := (l_subp).bit_xor ((1))
						l_n := l_n + l_Kl
						l_dif := l_dif - l_Kl
					end
					if l_subp.to_boolean then
						l_cy := (l_cy.to_natural_32 + (gmp_inst.gmpn_sub (l_tmp, l_tmp, l_Kl, l_n, l_dif))).to_integer_32
					else
						l_cy := (l_cy.to_natural_32 - (gmp_inst.gmpn_add (l_tmp, l_tmp, l_Kl, l_n, l_dif))).to_integer_32
					end
					if l_cy >= 0 then
						l_cy := (gmp_inst.gmpn_add_1 (l_tmp, l_tmp, l_Kl, l_cy.to_natural_32)).to_integer_32
					else
						l_cy := (gmp_inst.gmpn_sub_1 (l_tmp, l_tmp, l_Kl, (-l_cy).to_natural_32)).to_integer_32
					end
				else
					l_cy := (gmp_inst.gmpn_sub (l_tmp, l_n, l_Kl, l_n + l_Kl, l_dif)).to_integer_32
					l_cy := (gmp_inst.gmpn_add_1 (l_tmp, l_tmp, l_Kl, l_cy.to_natural_32)).to_integer_32
				end
				l_tmp[l_Kl] := l_cy.to_natural_32
				l_nl := l_Kl + 1
				l_n := l_tmp
			end
			l_i := 0
			from
			until
				not (not js.loop_done and then (l_i < a_K))
			loop
				a_Ap[l_i] := l_A
				if l_nl > 0 then
					if a_l.to_integer_32 <= l_nl then
						if l_i < a_K - 1 then
							l_j := a_l
						else
							l_j := l_nl.to_integer_32
						end
					else
						l_j := l_nl.to_integer_32
					end
					l_nl := l_nl - l_j.to_integer_32
					from
					until
						js.loop_done
					loop
						from
						until
							js.loop_done
						loop
							gmp_inst.gmpn_copyi (a_T, l_n, l_j.to_integer_32)
							js.break
						end
						js.break
					end
					from
					until
						js.loop_done
					loop
						if (a_nprime + 1) - l_j /= 0 then
							l___dst := a_T + l_j
							l___n := ((a_nprime + 1) - l_j).to_integer_32
							from
							until
								js.loop_done
							loop
								l_tmp___14 := l___dst
								l___dst := l___dst + 1
								l_tmp___14.item := (0).to_natural_32
								l___n := l___n - 1
								if not l___n.to_boolean then
									js.break
								end
							end
						end
						js.break
					end
					l_n := l_n + a_l
					mpn_fft_mul_2exp_modF (l_A, a_T, (l_i * a_Mp).to_natural_32, a_nprime.to_integer_32)
				else
					from
					until
						js.loop_done
					loop
						if a_nprime + 1 /= 0 then
							l___dst___0 := l_A
							l___n___0 := (a_nprime + 1).to_integer_32
							from
							until
								js.loop_done
							loop
								l_tmp___15 := l___dst___0
								l___dst___0 := l___dst___0 + 1
								l_tmp___15.item := (0).to_natural_32
								l___n___0 := l___n___0 - 1
								if not l___n___0.to_boolean then
									js.break
								end
							end
						end
						js.break
					end
				end
				l_A := l_A + (a_nprime + 1)
				l_i := l_i + 1
			end
			from
			until
				js.loop_done
			loop
				if l_nl /= 0 then
					assert_inst.gmp_assert_fail (ce_string ("mul_fft.c"), 699, ce_string ("nl == 0"))
				end
				js.break
			end
			from
			until
				js.loop_done
			loop
				if (builtin_expect ((not (l___tmp_marker.item).is_void).to_integer, 0)).to_boolean then
					tal_reent_inst.gmp_tmp_reentrant_free (l___tmp_marker.item)
				end
				js.break
			end
		end

	mpn_mul_fft_internal (a_op: CE_POINTER [NATURAL_32]; a_pl: INTEGER_32; a_k: INTEGER_32; a_Ap: CE_POINTER [CE_POINTER [NATURAL_32]]; a_Bp: CE_POINTER [CE_POINTER [NATURAL_32]]; a_A: CE_POINTER [NATURAL_32]; a_B: CE_POINTER [NATURAL_32]; a_nprime: INTEGER_32; a_l: INTEGER_32; a_Mp: INTEGER_32; a_fft_l: CE_POINTER [CE_POINTER [INTEGER_32]]; a_T: CE_POINTER [NATURAL_32]; a_sqr: INTEGER_32): NATURAL_32
		local
			l_K: INTEGER_32
			l_i: INTEGER_32
			l_pla: INTEGER_32
			l_lo: INTEGER_32
			l_sh: INTEGER_32
			l_j: INTEGER_32
			l_p: CE_POINTER [NATURAL_32]
			l_cc: NATURAL_32
			l_tmp: CE_POINTER [CE_POINTER [NATURAL_32]]
			l___dst: CE_POINTER [NATURAL_32]
			l___n: INTEGER_32
			l_tmp___0: CE_POINTER [NATURAL_32]
			l___dst___0: CE_POINTER [NATURAL_32]
			l___n___0: INTEGER_32
			l_tmp___1: CE_POINTER [NATURAL_32]
			l_n: CE_POINTER [NATURAL_32]
			js: JUMP_STATE
		do
			create l_p.make
			create l_tmp.make
			create l___dst.make
			create l_tmp___0.make
			create l___dst___0.make
			create l_tmp___1.make
			create l_n.make

			l_K := (1).bit_shift_left (a_k)
			mpn_fft_fft (a_Ap, l_K.to_integer_32, a_fft_l + a_k, 2 * a_Mp, a_nprime, (1).to_integer_32, a_T)
			if not a_sqr.to_boolean then
				mpn_fft_fft (a_Bp, l_K.to_integer_32, a_fft_l + a_k, 2 * a_Mp, a_nprime, (1).to_integer_32, a_T)
			end
			if a_sqr.to_boolean then
				l_tmp := a_Ap
			else
				l_tmp := a_Bp
			end
			mpn_fft_mul_modF_K (a_Ap, l_tmp, a_nprime, l_K)
			mpn_fft_fftinv (a_Ap, l_K, 2 * a_Mp, a_nprime, a_T)
			a_Bp[0] := (a_T + a_nprime) + 1
			mpn_fft_div_2exp_modF (a_Bp[0], a_Ap[0], a_k, a_nprime)
			l_i := 1
			from
			until
				not (l_i < l_K)
			loop
				a_Bp[l_i] := a_Ap[(l_i - 1)]
				mpn_fft_div_2exp_modF (a_Bp[l_i], a_Ap[l_i], (a_k.to_integer_32 + (l_K - l_i).to_integer_32 * a_Mp).to_integer_32, a_nprime)
				l_i := l_i + 1
			end
			from
			until
				js.loop_done
			loop
				if a_nprime + 1 /= 0 then
					l___dst := a_T
					l___n := a_nprime + 1
					from
					until
						js.loop_done
					loop
						l_tmp___0 := l___dst
						l___dst := l___dst + 1
						l_tmp___0.item := (0).to_natural_32
						l___n := l___n - 1
						if not l___n.to_boolean then
							js.break
						end
					end
				end
				js.break
			end
			l_pla := ((a_l * (l_K - 1).to_integer_32 + a_nprime) + 1).to_integer_32
			l_p := a_B
			from
			until
				js.loop_done
			loop
				if l_pla /= 0 then
					l___dst___0 := l_p
					l___n___0 := l_pla.to_integer_32
					from
					until
						js.loop_done
					loop
						l_tmp___1 := l___dst___0
						l___dst___0 := l___dst___0 + 1
						l_tmp___1.item := (0).to_natural_32
						l___n___0 := l___n___0 - 1
						if not l___n___0.to_boolean then
							js.break
						end
					end
				end
				js.break
			end
			l_cc := (0).to_natural_32
			l_i := l_K - 1
			l_lo := (a_l * l_i.to_integer_32 + a_nprime).to_integer_32
			l_sh := (a_l * l_i.to_integer_32).to_integer_32
			from
			until
				not (l_i >= 0)
			loop
				l_n := l_p + l_sh
				l_j := ((l_K - l_i)).bit_and ((l_K - 1))
				if (gmp_inst.gmpn_add_n (l_n, l_n, a_Bp[l_j], a_nprime + 1)).to_boolean then
					l_cc := l_cc + (gmp_inst.gmpn_add_1 ((l_n + a_nprime) + 1, ((l_n + a_nprime) + 1), ((l_pla - l_sh).to_integer_32 - a_nprime) - 1, (1).to_natural_32))
				end
				a_T[2 * a_l] := (l_i + 1).to_natural_32
				if (gmp_inst.gmpn_cmp (a_Bp[l_j], a_T, a_nprime + 1)) > 0 then
					l_cc := l_cc - (gmp_inst.gmpn_sub_1 (l_n, l_n, (l_pla - l_sh).to_integer_32, (1).to_natural_32))
					l_cc := l_cc - (gmp_inst.gmpn_sub_1 (l_p + l_lo, (l_p + l_lo), (l_pla - l_lo).to_integer_32, (1).to_natural_32))
				end
				l_i := l_i - 1
				l_lo := (l_lo.to_integer_32 - a_l).to_integer_32
				l_sh := (l_sh.to_integer_32 - a_l).to_integer_32
			end
			if l_cc = {NATURAL_32} 4_294_967_295 then
				l_cc := gmp_inst.gmpn_add_1 ((l_p + l_pla) - a_pl, ((l_p + l_pla) - a_pl), a_pl, (1).to_natural_32)
				if l_cc.to_boolean then
					gmp_inst.gmpn_sub_1 (((l_p + l_pla) - a_pl) - 1, (((l_p + l_pla) - a_pl) - 1), a_pl + 1, (1).to_natural_32).do_nothing
					gmp_inst.gmpn_sub_1 ((l_p + l_pla) - 1, ((l_p + l_pla) - 1), (1).to_integer_32, (1).to_natural_32).do_nothing
				end
			elseif l_cc = {NATURAL_32} 1 then
				if l_pla.to_integer_32 >= 2 * a_pl then
					from
					until
						js.loop_done
					loop
						l_cc := gmp_inst.gmpn_add_1 ((l_p + l_pla) - 2 * a_pl, ((l_p + l_pla) - 2 * a_pl), 2 * a_pl, l_cc)
						if not l_cc.to_boolean then
							js.break
						end
					end
				else
					l_cc := gmp_inst.gmpn_sub_1 ((l_p + l_pla) - a_pl, ((l_p + l_pla) - a_pl), a_pl, l_cc)
				end
			else
			end
			Result := (mpn_fft_norm_modF (a_op, a_pl, l_p, l_pla.to_integer_32)).to_natural_32
		end

	mpn_mul_fft_lcm (a_a: NATURAL_32; a_k: NATURAL_32): NATURAL_32
		local
			l_a: NATURAL_32
			l_k: NATURAL_32
			l_l: NATURAL_32
			js: JUMP_STATE
		do
			l_a := a_a
			l_k := a_k
			l_l := l_k.to_natural_32
			from
			until
				js.loop_done
			loop
				if l_a \\ {NATURAL_32} 2 = {NATURAL_32} 0 then
					if not (l_k > {NATURAL_32} 0) then
						js.break
					end
				else
					js.break
				end
				if not js.loop_jumped then
					l_a := (l_a).bit_shift_right ((1))
					l_k := l_k - 1
				end
			end
			Result := ((l_a).bit_shift_left (l_l.to_integer_32))
		end

end
