class
	P_BESSEL_I

inherit
	LIBGSL_DATA
	S_GCC_BUILTIN
	S_LIBIO
	S_STDIO
	S_STDLIB
	S_MATH
	S_STRING
	S_ASSERT

feature {ANY} -- routines

	bessel_il_CF1 (a_l: INTEGER_32; a_x: REAL_64; a_threshold: REAL_64; a_ratio: CE_POINTER [REAL_64]): INTEGER_32
		local
			l_kmax___0: INTEGER_32
			l_tk: REAL_64
			l_sum: REAL_64
			l_rhok: REAL_64
			l_k: INTEGER_32
			l_ak: REAL_64
			js: JUMP_STATE
		do
			l_kmax___0 := 2_000
			l_tk := 1.0
			l_sum := 1.0
			l_rhok := 0.0
			l_k := 1
			from
			until
				not (not js.loop_done and then (l_k <= l_kmax___0))
			loop
				l_ak := (a_x / ((2.0 * a_l.to_double + 1.0) + 2.0 * l_k.to_double)) * (a_x / ((2.0 * a_l.to_double + 3.0) + 2.0 * l_k.to_double))
				l_rhok := (-l_ak * (1.0 + l_rhok)) / (1.0 + l_ak * (1.0 + l_rhok))
				l_tk := l_tk * l_rhok
				l_sum := l_sum + l_tk
				if (fabs (l_tk / l_sum)) < a_threshold then
					js.break
				end
				if not js.loop_jumped then
					l_k := l_k + 1
				end
			end
			a_ratio.item := (a_x / (2.0 * a_l.to_double + 3.0)) * l_sum
			if l_k = l_kmax___0 then
				from
				until
					js.loop_done
				loop
					error_inst.gsl_error (ce_string ("error"), ce_string ("bessel_i.c"), 56, 11)
					Result := 11
					js.return
					if not js.return_called then
						js.break
					end
				end
			else
				Result := 0
				js.return
			end
			if not js.return_called then
				Result := 0
			end
		end

	gsl_sf_bessel_i0_scaled (a_x: REAL_64): REAL_64
		local
			l_result: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_status: INTEGER_32
			js: JUMP_STATE
		do
			create l_result.make (<<1>>)

			l_status := (gsl_sf_bessel_i0_scaled_e (a_x, l_result))
			if l_status /= 0 then
				from
				until
					js.loop_done
				loop
					error_inst.gsl_error (ce_string ("gsl_sf_bessel_i0_scaled_e(x, &result)"), ce_string ("bessel_i.c"), 312, l_status)
					Result := (l_result.item.val)
					js.return
					if not js.return_called then
						js.break
					end
				end
			end
			if not js.return_called then
				Result := (l_result.item.val)
			end
		end

	gsl_sf_bessel_i0_scaled_e (a_x: REAL_64; a_result: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]): INTEGER_32
		local
			l_ax: REAL_64
			l_eax: REAL_64
			l_y: REAL_64
			l_c1___2: REAL_64
			l_c2___0: REAL_64
			l_c3___3: REAL_64
			l_c4___2: REAL_64
			l_c5___1: REAL_64
			l_sum: REAL_64
			js: JUMP_STATE
		do
			l_ax := (fabs (a_x))
			if a_x = 0.0 then
				a_result.item.val := 1.0
				a_result.item.err := 0.0
				Result := 0
				js.return
			elseif l_ax < 0.2 then
				l_eax := (exp (-l_ax))
				l_y := l_ax * l_ax
				l_c1___2 := 1.0 / 6.0
				l_c2___0 := 1.0 / 120.0
				l_c3___3 := 1.0 / 5040.0
				l_c4___2 := 1.0 / 362880.0
				l_c5___1 := 1.0 / 39916800.0
				l_sum := 1.0 + l_y * (l_c1___2 + l_y * (l_c2___0 + l_y * (l_c3___3 + l_y * (l_c4___2 + l_y * l_c5___1))))
				a_result.item.val := l_eax * l_sum
				a_result.item.err := (2.0 * 2.2204460492503131e-16) * a_result.item.val
			elseif l_ax < -0.5 * -3.6043653389117154e+01 then
				a_result.item.val := (1.0 - (exp (-2.0 * l_ax))) / (2.0 * l_ax)
				a_result.item.err := (2.0 * 2.2204460492503131e-16) * a_result.item.val
			else
				a_result.item.val := 1.0 / (2.0 * l_ax)
				a_result.item.err := (2.0 * 2.2204460492503131e-16) * a_result.item.val
			end
			if not js.return_called then
				Result := 0
			end
		end

	gsl_sf_bessel_i1_scaled (a_x: REAL_64): REAL_64
		local
			l_result: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_status: INTEGER_32
			js: JUMP_STATE
		do
			create l_result.make (<<1>>)

			l_status := (gsl_sf_bessel_i1_scaled_e (a_x, l_result))
			if l_status /= 0 then
				from
				until
					js.loop_done
				loop
					error_inst.gsl_error (ce_string ("gsl_sf_bessel_i1_scaled_e(x, &result)"), ce_string ("bessel_i.c"), 317, l_status)
					Result := (l_result.item.val)
					js.return
					if not js.return_called then
						js.break
					end
				end
			end
			if not js.return_called then
				Result := (l_result.item.val)
			end
		end

	gsl_sf_bessel_i1_scaled_e (a_x: REAL_64; a_result: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]): INTEGER_32
		local
			l_ax: REAL_64
			l_eax: REAL_64
			l_y: REAL_64
			l_c1___2: REAL_64
			l_c2___0: REAL_64
			l_c3___3: REAL_64
			l_c4___2: REAL_64
			l_c5___1: REAL_64
			l_sum: REAL_64
			l_ex: REAL_64
			js: JUMP_STATE
		do
			l_ax := (fabs (a_x))
			if a_x = 0.0 then
				a_result.item.val := 0.0
				a_result.item.err := 0.0
				Result := 0
				js.return
			elseif l_ax < 3.0 * 2.2250738585072014e-308 then
				from
				until
					js.loop_done
				loop
					a_result.item.val := 0.0
					a_result.item.err := 2.2250738585072014e-308
					from
					until
						js.loop_done
					loop
						error_inst.gsl_error (ce_string ("underflow"), ce_string ("bessel_i.c"), 111, 15)
						Result := 15
						js.return
						if not js.return_called then
							js.break
						end
					end
					if not js.return_called then
						js.break
					end
				end
			elseif l_ax < 0.25 then
				l_eax := (exp (-l_ax))
				l_y := a_x * a_x
				l_c1___2 := 1.0 / 10.0
				l_c2___0 := 1.0 / 280.0
				l_c3___3 := 1.0 / 15120.0
				l_c4___2 := 1.0 / 1330560.0
				l_c5___1 := 1.0 / 172972800.0
				l_sum := 1.0 + l_y * (l_c1___2 + l_y * (l_c2___0 + l_y * (l_c3___3 + l_y * (l_c4___2 + l_y * l_c5___1))))
				a_result.item.val := ((l_eax * a_x) / 3.0) * l_sum
				a_result.item.err := (2.0 * 2.2204460492503131e-16) * (fabs (a_result.item.val))
				Result := 0
				js.return
			else
				l_ex := (exp (-2.0 * l_ax))
				a_result.item.val := (0.5 * (l_ax * (1.0 + l_ex) - (1.0 - l_ex))) / (l_ax * l_ax)
				a_result.item.err := (2.0 * 2.2204460492503131e-16) * (fabs (a_result.item.val))
				if a_x < 0.0 then
					a_result.item.val := -a_result.item.val
				end
				Result := 0
				js.return
			end
			if not js.return_called then
				Result := 0
			end
		end

	gsl_sf_bessel_i2_scaled (a_x: REAL_64): REAL_64
		local
			l_result: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_status: INTEGER_32
			js: JUMP_STATE
		do
			create l_result.make (<<1>>)

			l_status := (gsl_sf_bessel_i2_scaled_e (a_x, l_result))
			if l_status /= 0 then
				from
				until
					js.loop_done
				loop
					error_inst.gsl_error (ce_string ("gsl_sf_bessel_i2_scaled_e(x, &result)"), ce_string ("bessel_i.c"), 322, l_status)
					Result := (l_result.item.val)
					js.return
					if not js.return_called then
						js.break
					end
				end
			end
			if not js.return_called then
				Result := (l_result.item.val)
			end
		end

	gsl_sf_bessel_i2_scaled_e (a_x: REAL_64; a_result: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]): INTEGER_32
		local
			l_ax: REAL_64
			l_y: REAL_64
			l_c1___2: REAL_64
			l_c2___0: REAL_64
			l_c3___3: REAL_64
			l_c4___2: REAL_64
			l_c5___1: REAL_64
			l_sum: REAL_64
			l_pre: REAL_64
			l_ex: REAL_64
			l_x2___1: REAL_64
			js: JUMP_STATE
		do
			l_ax := (fabs (a_x))
			if a_x = 0.0 then
				a_result.item.val := 0.0
				a_result.item.err := 0.0
				Result := 0
				js.return
			elseif l_ax < 4.0 * 1.4916681462400413e-154 then
				from
				until
					js.loop_done
				loop
					a_result.item.val := 0.0
					a_result.item.err := 2.2250738585072014e-308
					from
					until
						js.loop_done
					loop
						error_inst.gsl_error (ce_string ("underflow"), ce_string ("bessel_i.c"), 148, 15)
						Result := 15
						js.return
						if not js.return_called then
							js.break
						end
					end
					if not js.return_called then
						js.break
					end
				end
			elseif l_ax < 0.25 then
				l_y := a_x * a_x
				l_c1___2 := 1.0 / 14.0
				l_c2___0 := 1.0 / 504.0
				l_c3___3 := 1.0 / 33264.0
				l_c4___2 := 1.0 / 3459456.0
				l_c5___1 := 1.0 / 518918400.0
				l_sum := 1.0 + l_y * (l_c1___2 + l_y * (l_c2___0 + l_y * (l_c3___3 + l_y * (l_c4___2 + l_y * l_c5___1))))
				l_pre := (((exp (-l_ax)) * a_x) * a_x) / 15.0
				a_result.item.val := l_pre * l_sum
				a_result.item.err := (2.0 * 2.2204460492503131e-16) * (fabs (a_result.item.val))
				Result := 0
				js.return
			else
				l_ex := (exp (-2.0 * l_ax))
				l_x2___1 := a_x * a_x
				a_result.item.val := (0.5 * ((3.0 + l_x2___1) * (1.0 - l_ex) - (3.0 * l_ax) * (1.0 + l_ex))) / ((l_ax * l_ax) * l_ax)
				a_result.item.err := (2.0 * 2.2204460492503131e-16) * (fabs (a_result.item.val))
				Result := 0
				js.return
			end
			if not js.return_called then
				Result := 0
			end
		end

	gsl_sf_bessel_il_scaled (a_l: INTEGER_32; a_x: REAL_64): REAL_64
		local
			l_result: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_status: INTEGER_32
			js: JUMP_STATE
		do
			create l_result.make (<<1>>)

			l_status := (gsl_sf_bessel_il_scaled_e (a_l, a_x, l_result))
			if l_status /= 0 then
				from
				until
					js.loop_done
				loop
					error_inst.gsl_error (ce_string ("gsl_sf_bessel_il_scaled_e(l, x, &result)"), ce_string ("bessel_i.c"), 327, l_status)
					Result := (l_result.item.val)
					js.return
					if not js.return_called then
						js.break
					end
				end
			end
			if not js.return_called then
				Result := (l_result.item.val)
			end
		end

	gsl_sf_bessel_il_scaled_array (a_lmax: INTEGER_32; a_x: REAL_64; a_result_array: CE_POINTER [REAL_64]): INTEGER_32
		local
			l_ell: INTEGER_32
			l_ell___0: INTEGER_32
			l_r_iellp1: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_r_iell: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_stat_0: INTEGER_32
			l_stat_1: INTEGER_32
			l_iellp1: REAL_64
			l_iell: REAL_64
			l_iellm1: REAL_64
			l_tmp___1: INTEGER_32
			l_tmp___2: INTEGER_32
		do
			create l_r_iellp1.make (<<1>>)
			create l_r_iell.make (<<1>>)

			if a_x = 0.0 then
				a_result_array[0] := 1.0
				l_ell := a_lmax
				from
				until
					not (l_ell >= 1)
				loop
					a_result_array[l_ell] := 0.0
					l_ell := l_ell - 1
				end
				Result := 0
			else
				l_stat_0 := (gsl_sf_bessel_il_scaled_e (a_lmax + 1, a_x, l_r_iellp1))
				l_stat_1 := (gsl_sf_bessel_il_scaled_e (a_lmax, a_x, l_r_iell))
				l_iellp1 := l_r_iellp1.item.val
				l_iell := l_r_iell.item.val
				a_result_array[a_lmax] := l_iell
				l_ell___0 := a_lmax
				from
				until
					not (l_ell___0 >= 1)
				loop
					l_iellm1 := l_iellp1 + ((2 * l_ell___0 + 1).to_double / a_x) * l_iell
					l_iellp1 := l_iell
					l_iell := l_iellm1
					a_result_array[(l_ell___0 - 1)] := l_iellm1
					l_ell___0 := l_ell___0 - 1
				end
				if l_stat_0 /= 0 then
					l_tmp___2 := l_stat_0
				else
					if l_stat_1 /= 0 then
						l_tmp___1 := l_stat_1
					else
						l_tmp___1 := 0
					end
					l_tmp___2 := l_tmp___1
				end
				Result := l_tmp___2
			end
		end

	gsl_sf_bessel_il_scaled_e (a_l: INTEGER_32; a_x: REAL_64; a_result: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]): INTEGER_32
		local
			l_x: REAL_64
			l_sgn: REAL_64
			l_ax: REAL_64
			l_il: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_stat_il: INTEGER_32
			l_il___0: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_stat_il___0: INTEGER_32
			l_il___1: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_stat_il___1: INTEGER_32
			l_b___0: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_stat: INTEGER_32
			l_pre: REAL_64
			l_tmp___6: REAL_64
			l_i0_scaled: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_stat_i0: INTEGER_32
			l_rat: CE_ARRAY [REAL_64]
			l_stat_CF1: INTEGER_32
			l_iellp1: REAL_64
			l_iell: REAL_64
			l_iellm1: REAL_64
			l_ell: INTEGER_32
			l_tmp___12: INTEGER_32
			l_tmp___13: INTEGER_32
			l_status: INTEGER_32
			l_pre___0: REAL_64
			l_rt_term: REAL_64
			l_LMAX: INTEGER_32
			l_r_iellp1: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_r_iell: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_stat_a1: INTEGER_32
			l_stat_a2: INTEGER_32
			l_iellp1___0: REAL_64
			l_iell___0: REAL_64
			l_iellm1___0: REAL_64
			l_ell___0: INTEGER_32
			l_tmp___19: REAL_64
			l_tmp___20: REAL_64
			l_tmp___23: INTEGER_32
			l_tmp___24: INTEGER_32
			l_tmp___25: REAL_64
			js: JUMP_STATE
		do
			create l_il.make (<<1>>)
			create l_il___0.make (<<1>>)
			create l_il___1.make (<<1>>)
			create l_b___0.make (<<1>>)
			create l_i0_scaled.make (<<1>>)
			create l_rat.make (<<1>>)
			create l_r_iellp1.make (<<1>>)
			create l_r_iell.make (<<1>>)

			l_x := a_x
			l_sgn := 1.0
			l_ax := (fabs (l_x))
			if l_x < 0.0 then
				if ((a_l).bit_and (1)).to_boolean then
					l_sgn := -1.0
				else
					l_sgn := 1.0
				end
				l_x := -l_x
			end
			if a_l < 0 then
				from
				until
					js.loop_done
				loop
					a_result.item.val := (builtin_nanf (ce_string (""))).to_double
					a_result.item.err := (builtin_nanf (ce_string (""))).to_double
					from
					until
						js.loop_done
					loop
						error_inst.gsl_error (ce_string ("domain error"), ce_string ("bessel_i.c"), 185, 1)
						Result := 1
						js.return
						if not js.return_called then
							js.break
						end
					end
					if not js.return_called then
						js.break
					end
				end
			elseif l_x = 0.0 then
				if a_l = 0 then
					a_result.item.val := 1.0
				else
					a_result.item.val := 0.0
				end
				a_result.item.err := 0.0
				Result := 0
				js.return
			elseif a_l = 0 then
				l_stat_il := (gsl_sf_bessel_i0_scaled_e (l_x, l_il))
				a_result.item.val := l_sgn * l_il.item.val
				a_result.item.err := l_il.item.err
				Result := l_stat_il
				js.return
			elseif a_l = 1 then
				l_stat_il___0 := (gsl_sf_bessel_i1_scaled_e (l_x, l_il___0))
				a_result.item.val := l_sgn * l_il___0.item.val
				a_result.item.err := l_il___0.item.err
				Result := l_stat_il___0
				js.return
			elseif a_l = 2 then
				l_stat_il___1 := (gsl_sf_bessel_i2_scaled_e (l_x, l_il___1))
				a_result.item.val := l_sgn * l_il___1.item.val
				a_result.item.err := l_il___1.item.err
				Result := l_stat_il___1
				js.return
			elseif l_x * l_x < (10.0 * (a_l.to_double + 1.5)) / 2.7182818284590452354 then
				l_stat := (bessel_inst.gsl_sf_bessel_IJ_taylor_e (a_l.to_double + 0.5, l_x, 1, 50, 2.2204460492503131e-16, l_b___0))
				l_tmp___6 := exp (-l_ax)
				l_pre := l_tmp___6 * (sqrt ((0.5 * 3.14159265358979323846) / l_x))
				a_result.item.val := (l_sgn * l_pre) * l_b___0.item.val
				a_result.item.err := l_pre * l_b___0.item.err
				a_result.item.err := a_result.item.err + ((2.0 * 2.2204460492503131e-16) * (fabs (a_result.item.val)))
				Result := l_stat
				js.return
			elseif a_l < 150 then
				l_stat_i0 := (gsl_sf_bessel_i0_scaled_e (l_ax, l_i0_scaled))
				l_stat_CF1 := (bessel_il_CF1 (a_l, l_ax, 2.2204460492503131e-16, l_rat))
				l_iellp1 := l_rat.item * 1.4916681462400413e-154
				l_iell := 1.4916681462400413e-154
				l_ell := a_l
				from
				until
					not (l_ell >= 1)
				loop
					l_iellm1 := l_iellp1 + ((2 * l_ell + 1).to_double / l_x) * l_iell
					l_iellp1 := l_iell
					l_iell := l_iellm1
					l_ell := l_ell - 1
				end
				a_result.item.val := (l_sgn * l_i0_scaled.item.val) * (1.4916681462400413e-154 / l_iell)
				a_result.item.err := l_i0_scaled.item.err * (1.4916681462400413e-154 / l_iell)
				a_result.item.err := a_result.item.err + ((2.0 * 2.2204460492503131e-16) * (fabs (a_result.item.val)))
				if l_stat_i0 /= 0 then
					l_tmp___13 := l_stat_i0
				else
					if l_stat_CF1 /= 0 then
						l_tmp___12 := l_stat_CF1
					else
						l_tmp___12 := 0
					end
					l_tmp___13 := l_tmp___12
				end
				Result := l_tmp___13
				js.return
			else
				if 0.29 / ((a_l * a_l).to_double + 1.0) < 0.5 / (((a_l * a_l).to_double + 1.0) + l_x * l_x) then
					l_tmp___25 := 0.29 / ((a_l * a_l).to_double + 1.0)
				else
					l_tmp___25 := 0.5 / (((a_l * a_l).to_double + 1.0) + l_x * l_x)
				end
				if l_tmp___25 < 0.5 * 6.0554544523933429e-06 then
					l_status := (bessel_inst.gsl_sf_bessel_Inu_scaled_asymp_unif_e (a_l.to_double + 0.5, l_x, a_result))
					l_pre___0 := (sqrt ((0.5 * 3.14159265358979323846) / l_x))
					a_result.item.val := a_result.item.val * (l_sgn * l_pre___0)
					a_result.item.err := a_result.item.err * l_pre___0
					Result := l_status
					js.return
				else
					l_rt_term := (sqrt ((0.5 * 3.14159265358979323846) / l_x))
					l_LMAX := 2 + (1.2 / 2.4607833005759251e-03).truncated_to_integer
					l_stat_a1 := (bessel_inst.gsl_sf_bessel_Inu_scaled_asymp_unif_e ((l_LMAX + 1).to_double + 0.5, l_x, l_r_iellp1))
					l_stat_a2 := (bessel_inst.gsl_sf_bessel_Inu_scaled_asymp_unif_e (l_LMAX.to_double + 0.5, l_x, l_r_iell))
					l_iellp1___0 := l_r_iellp1.item.val
					l_iell___0 := l_r_iell.item.val
					l_iellm1___0 := 0.0
					l_iellp1___0 := l_iellp1___0 * l_rt_term
					l_iell___0 := l_iell___0 * l_rt_term
					l_ell___0 := l_LMAX
					from
					until
						not (l_ell___0 >= a_l + 1)
					loop
						l_iellm1___0 := l_iellp1___0 + ((2 * l_ell___0 + 1).to_double / l_x) * l_iell___0
						l_iellp1___0 := l_iell___0
						l_iell___0 := l_iellm1___0
						l_ell___0 := l_ell___0 - 1
					end
					a_result.item.val := l_sgn * l_iellm1___0
					l_tmp___19 := fabs (a_result.item.val)
					l_tmp___20 := fabs (l_r_iellp1.item.err / l_r_iellp1.item.val)
					a_result.item.err := l_tmp___19 * ((2.2204460492503131e-16 + l_tmp___20) + (fabs (l_r_iell.item.err / l_r_iell.item.val)))
					a_result.item.err := a_result.item.err + ((2.0 * 2.2204460492503131e-16) * (fabs (a_result.item.val)))
					if l_stat_a1 /= 0 then
						l_tmp___24 := l_stat_a1
					else
						if l_stat_a2 /= 0 then
							l_tmp___23 := l_stat_a2
						else
							l_tmp___23 := 0
						end
						l_tmp___24 := l_tmp___23
					end
					Result := l_tmp___24
					js.return
				end
			end
			if not js.return_called then
				Result := 0
			end
		end

end
