class
	P_BESSEL_J0

inherit
	LIBGSL_DATA
		redefine
			default_create
		end
	S_GCC_BUILTIN
	S_LIBIO
	S_STDIO
	S_STDLIB
	S_MATH
	S_STRING
	S_ASSERT

feature {NONE} -- Initialization

	default_create
		do
			create bj0_cs.make (<<1>>)
			create bj0_data.make (<<13>>)
		end

feature {ANY} -- attributes

	bj0_cs: CE_ARRAY [PD_CHEB_SERIES_STRUCT]

	bj0_data: CE_ARRAY [REAL_64]

feature {ANY} -- routines

	gsl_sf_bessel_J02 (a_x: REAL_64): REAL_64
		local
			l_result: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_status: INTEGER_32
			js: JUMP_STATE
		do
			create l_result.make (<<1>>)

			l_status := (gsl_sf_bessel_J0_e2 (a_x, l_result))
			if l_status /= 0 then
				from
				until
					js.loop_done
				loop
					error_inst.gsl_error (ce_string ("gsl_sf_bessel_J0_e(x, &result)"), ce_string ("bessel_J0.c"), 110, l_status)
					Result := (l_result.item.val)
					js.return
					if not js.return_called then
						js.break
					end
				end
			end
			if not js.return_called then
				Result := (l_result.item.val)
			end
		end

	gsl_sf_bessel_J0_e2 (a_x: REAL_64; a_result: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]): INTEGER_32
		local
			l_y: REAL_64
			l_z: REAL_64
			l_ca: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_ct: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_cp: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_stat_ca: INTEGER_32
			l_stat_ct: INTEGER_32
			l_stat_cp: INTEGER_32
			l_sqrty: REAL_64
			l_ampl: REAL_64
			l_tmp___5: REAL_64
			l_tmp___8: INTEGER_32
			l_tmp___9: INTEGER_32
			l_tmp___10: INTEGER_32
		do
			create l_ca.make (<<1>>)
			create l_ct.make (<<1>>)
			create l_cp.make (<<1>>)

			l_y := (fabs (a_x))
			if l_y < 2.0 * 1.4901161193847656e-08 then
				a_result.item.val := 1.0
				a_result.item.err := l_y * l_y
				Result := 0
			elseif l_y <= 4.0 then
				Result := (cheb_eval_inst.cheb_eval_e (bj0_cs, (0.125 * l_y) * l_y - 1.0, a_result))
			else
				l_z := 32.0 / (l_y * l_y) - 1.0
				l_stat_ca := (cheb_eval_inst.cheb_eval_e (bessel_amp_phase_inst.gsl_sf_bessel_amp_phase_bm0_cs, l_z, l_ca))
				l_stat_ct := (cheb_eval_inst.cheb_eval_e (bessel_amp_phase_inst.gsl_sf_bessel_amp_phase_bth0_cs, l_z, l_ct))
				l_stat_cp := (bessel_inst.gsl_sf_bessel_cos_pi4_e (l_y, l_ct.item.val / l_y, l_cp))
				l_sqrty := (sqrt (l_y))
				l_ampl := (0.75 + l_ca.item.val) / l_sqrty
				a_result.item.val := l_ampl * l_cp.item.val
				l_tmp___5 := fabs (l_cp.item.val)
				a_result.item.err := (l_tmp___5 * l_ca.item.err) / l_sqrty + (fabs (l_ampl)) * l_cp.item.err
				a_result.item.err := a_result.item.err + (2.2204460492503131e-16 * (fabs (a_result.item.val)))
				if l_stat_ca /= 0 then
					l_tmp___10 := l_stat_ca
				else
					if l_stat_ct /= 0 then
						l_tmp___9 := l_stat_ct
					else
						if l_stat_cp /= 0 then
							l_tmp___8 := l_stat_cp
						else
							l_tmp___8 := 0
						end
						l_tmp___9 := l_tmp___8
					end
					l_tmp___10 := l_tmp___9
				end
				Result := l_tmp___10
			end
		end

end
