class
	P_BESSEL_JN

inherit
	LIBGSL_DATA
	S_GCC_BUILTIN
	S_LIBIO
	S_STDIO
	S_STDLIB
	S_MATH
	S_STRING
	S_ASSERT

feature {ANY} -- routines

	gsl_sf_bessel_Jn (a_n: INTEGER_32; a_x: REAL_64): REAL_64
		local
			l_result: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_status: INTEGER_32
			js: JUMP_STATE
		do
			create l_result.make (<<1>>)

			l_status := (gsl_sf_bessel_Jn_e (a_n, a_x, l_result))
			if l_status /= 0 then
				from
				until
					js.loop_done
				loop
					error_inst.gsl_error (ce_string ("gsl_sf_bessel_Jn_e(n, x, &result)"), ce_string ("bessel_Jn.c"), 198, l_status)
					Result := (l_result.item.val)
					js.return
					if not js.return_called then
						js.break
					end
				end
			end
			if not js.return_called then
				Result := (l_result.item.val)
			end
		end

	gsl_sf_bessel_Jn_array (a_nmin: INTEGER_32; a_nmax: INTEGER_32; a_x: REAL_64; a_result_array: CE_POINTER [REAL_64]): INTEGER_32
		local
			l_n: INTEGER_32
			l_n___0: INTEGER_32
			l_r_Jnp1: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_r_Jn: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_stat_np1: INTEGER_32
			l_stat_n: INTEGER_32
			l_stat: INTEGER_32
			l_tmp___1: INTEGER_32
			l_tmp___2: INTEGER_32
			l_Jnp1: REAL_64
			l_Jn: REAL_64
			l_Jnm1: REAL_64
			l_n___1: INTEGER_32
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			create l_r_Jnp1.make (<<1>>)
			create l_r_Jn.make (<<1>>)

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					if not (a_nmin < 0) then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						l_pos := -1 + 4 -- goto _L
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 7
				when 3 then -- else:
						if not (a_nmax < a_nmin) then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if: | _L:
						l_n := a_nmax
						from
						until
							not (l_n >= a_nmin)
						loop
							a_result_array[(l_n - a_nmin)] := 0.0
							l_n := l_n - 1
						end
						from
						until
							js.loop_done
						loop
							error_inst.gsl_error (ce_string ("domain error"), ce_string ("bessel_Jn.c"), 152, 1)
							Result := 1
							js.return
							if not js.return_called then
								js.break
							end
						end
				when 5 then -- if block done, skip else block:
						l_pos := -1 + 7
				when 6 then -- else:
						if a_x = 0.0 then
							l_n___0 := a_nmax
							from
							until
								not (l_n___0 >= a_nmin)
							loop
								a_result_array[(l_n___0 - a_nmin)] := 0.0
								l_n___0 := l_n___0 - 1
							end
							if a_nmin = 0 then
								a_result_array[0] := 1.0
							end
							Result := 0
							js.return
						else
							l_stat_np1 := (gsl_sf_bessel_Jn_e (a_nmax + 1, a_x, l_r_Jnp1))
							l_stat_n := (gsl_sf_bessel_Jn_e (a_nmax, a_x, l_r_Jn))
							if l_stat_np1 /= 0 then
								l_tmp___2 := l_stat_np1
							else
								if l_stat_n /= 0 then
									l_tmp___1 := l_stat_n
								else
									l_tmp___1 := 0
								end
								l_tmp___2 := l_tmp___1
							end
							l_stat := l_tmp___2
							l_Jnp1 := l_r_Jnp1.item.val
							l_Jn := l_r_Jn.item.val
							if l_stat = 0 then
								l_n___1 := a_nmax
								from
								until
									not (l_n___1 >= a_nmin)
								loop
									a_result_array[(l_n___1 - a_nmin)] := l_Jn
									l_Jnm1 := -l_Jnp1 + ((2.0 * l_n___1.to_double) / a_x) * l_Jn
									l_Jnp1 := l_Jn
									l_Jn := l_Jnm1
									l_n___1 := l_n___1 - 1
								end
							else
								l_n___1 := a_nmax
								from
								until
									not (l_n___1 >= a_nmin)
								loop
									a_result_array[(l_n___1 - a_nmin)] := 0.0
									l_n___1 := l_n___1 - 1
								end
							end
							Result := l_stat
							js.return
						end
				when 7 then -- if done
						if not js.return_called then
							Result := 0
						end
				when 8 then
						l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
			end

	gsl_sf_bessel_Jn_e (a_n: INTEGER_32; a_x: REAL_64; a_result: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]): INTEGER_32
		local
			l_n: INTEGER_32
			l_x: REAL_64
			l_sign: INTEGER_32
			l_b0: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_stat_J0: INTEGER_32
			l_b1___0: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_stat_J1: INTEGER_32
			l_b___0: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_status: INTEGER_32
			l_status___0: INTEGER_32
			l_status___1: INTEGER_32
			l_status___2: INTEGER_32
			l_ans: REAL_64
			l_err: REAL_64
			l_ratio: CE_ARRAY [REAL_64]
			l_sgn: CE_ARRAY [REAL_64]
			l_stat_b: INTEGER_32
			l_stat_CF1: INTEGER_32
			l_Jkp1: REAL_64
			l_Jk: REAL_64
			l_Jkm1: REAL_64
			l_k: INTEGER_32
			l_b1___1: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_b0___0: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_tmp___7: REAL_64
			l_tmp___9: INTEGER_32
			l_tmp___10: INTEGER_32
		do
			create l_b0.make (<<1>>)
			create l_b1___0.make (<<1>>)
			create l_b___0.make (<<1>>)
			create l_ratio.make (<<1>>)
			create l_sgn.make (<<1>>)
			create l_b1___1.make (<<1>>)
			create l_b0___0.make (<<1>>)

			l_n := a_n
			l_x := a_x
			l_sign := 1
			if l_n < 0 then
				l_n := -l_n
				if ((l_n).bit_and (1)).to_boolean then
					l_sign := -l_sign
				end
			end
			if l_x < 0.0 then
				l_x := -l_x
				if ((l_n).bit_and (1)).to_boolean then
					l_sign := -l_sign
				end
			end
			if l_n = 0 then
				l_stat_J0 := (bessel_j0_inst.gsl_sf_bessel_J0_e2 (l_x, l_b0))
				a_result.item.val := l_sign.to_double * l_b0.item.val
				a_result.item.err := l_b0.item.err
				Result := l_stat_J0
			elseif l_n = 1 then
				l_stat_J1 := (bessel_j1_inst.gsl_sf_bessel_J1_e (l_x, l_b1___0))
				a_result.item.val := l_sign.to_double * l_b1___0.item.val
				a_result.item.err := l_b1___0.item.err
				Result := l_stat_J1
			elseif l_x = 0.0 then
				a_result.item.val := 0.0
				a_result.item.err := 0.0
				Result := 0
			elseif l_x * l_x < (10.0 * (l_n.to_double + 1.0)) * 7.4009597974140505e-04 then
				l_status := (bessel_inst.gsl_sf_bessel_IJ_taylor_e (l_n.to_double, l_x, -1, 50, 2.2204460492503131e-16, l_b___0))
				a_result.item.val := l_sign.to_double * l_b___0.item.val
				a_result.item.err := l_b___0.item.err
				a_result.item.err := a_result.item.err + (2.2204460492503131e-16 * (fabs (a_result.item.val)))
				Result := l_status
			elseif 1.2207031250000000e-04 * l_x > (l_n * l_n).to_double + 1.0 then
				l_status___0 := (bessel_inst.gsl_sf_bessel_Jnu_asympx_e (l_n.to_double, l_x, a_result))
				a_result.item.val := a_result.item.val * l_sign.to_double
				Result := l_status___0
			elseif l_n > 50 then
				l_status___1 := (bessel_olver_inst.gsl_sf_bessel_Jnu_asymp_Olver_e (l_n.to_double, l_x, a_result))
				a_result.item.val := a_result.item.val * l_sign.to_double
				Result := l_status___1
			elseif l_x > 1000.0 then
				l_status___2 := (bessel_inst.gsl_sf_bessel_Jnu_asympx_e (l_n.to_double, l_x, a_result))
				a_result.item.val := a_result.item.val * l_sign.to_double
				Result := l_status___2
			else
				l_stat_CF1 := (bessel_inst.gsl_sf_bessel_J_CF1 (l_n.to_double, l_x, l_ratio, l_sgn))
				l_Jkp1 := 1.4916681462400413e-154 * l_ratio.item
				l_Jk := 1.4916681462400413e-154
				l_k := l_n
				from
				until
					not (l_k > 0)
				loop
					l_Jkm1 := ((2.0 * l_k.to_double) / l_x) * l_Jk - l_Jkp1
					l_Jkp1 := l_Jk
					l_Jk := l_Jkm1
					l_k := l_k - 1
				end
				l_tmp___7 := fabs (l_Jkp1)
				if l_tmp___7 > (fabs (l_Jk)) then
					l_stat_b := bessel_j1_inst.gsl_sf_bessel_J1_e (l_x, l_b1___1)
					l_ans := (l_b1___1.item.val / l_Jkp1) * 1.4916681462400413e-154
					l_err := (l_b1___1.item.err / l_Jkp1) * 1.4916681462400413e-154
				else
					l_stat_b := bessel_j0_inst.gsl_sf_bessel_J0_e2 (l_x, l_b0___0)
					l_ans := (l_b0___0.item.val / l_Jk) * 1.4916681462400413e-154
					l_err := (l_b0___0.item.err / l_Jk) * 1.4916681462400413e-154
				end
				a_result.item.val := l_sign.to_double * l_ans
				a_result.item.err := fabs (l_err)
				if l_stat_CF1 /= 0 then
					l_tmp___10 := l_stat_CF1
				else
					if l_stat_b /= 0 then
						l_tmp___9 := l_stat_b
					else
						l_tmp___9 := 0
					end
					l_tmp___10 := l_tmp___9
				end
				Result := l_tmp___10
			end
		end

end
