class
	P_BESSEL_KNU

inherit
	LIBGSL_DATA
	S_GCC_BUILTIN
	S_LIBIO
	S_STDIO
	S_STDLIB
	S_MATH
	S_STRING
	S_ASSERT

feature {ANY} -- routines

	gsl_sf_bessel_Knu (a_nu: REAL_64; a_x: REAL_64): REAL_64
		local
			l_result: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_status: INTEGER_32
			js: JUMP_STATE
		do
			create l_result.make (<<1>>)

			l_status := (gsl_sf_bessel_Knu_e (a_nu, a_x, l_result))
			if l_status /= 0 then
				from
				until
					js.loop_done
				loop
					error_inst.gsl_error (ce_string ("gsl_sf_bessel_Knu_e(nu, x, &result)"), ce_string ("bessel_Knu.c"), 158, l_status)
					Result := (l_result.item.val)
					js.return
					if not js.return_called then
						js.break
					end
				end
			end
			if not js.return_called then
				Result := (l_result.item.val)
			end
		end

	gsl_sf_bessel_Knu_e (a_nu: REAL_64; a_x: REAL_64; a_result: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]): INTEGER_32
		local
			l_b___0: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_stat_K: INTEGER_32
			l_stat_e: INTEGER_32
			l_tmp___1: INTEGER_32
			l_tmp___2: INTEGER_32
		do
			create l_b___0.make (<<1>>)

			l_stat_K := (gsl_sf_bessel_Knu_scaled_e (a_nu, a_x, l_b___0))
			l_stat_e := (exp_inst.gsl_sf_exp_mult_err_e (-a_x, 0.0, l_b___0.item.val, l_b___0.item.err, a_result))
			if l_stat_e /= 0 then
				l_tmp___2 := l_stat_e
			else
				if l_stat_K /= 0 then
					l_tmp___1 := l_stat_K
				else
					l_tmp___1 := 0
				end
				l_tmp___2 := l_tmp___1
			end
			Result := l_tmp___2
		end

	gsl_sf_bessel_Knu_scaled (a_nu: REAL_64; a_x: REAL_64): REAL_64
		local
			l_result: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_status: INTEGER_32
			js: JUMP_STATE
		do
			create l_result.make (<<1>>)

			l_status := (gsl_sf_bessel_Knu_scaled_e (a_nu, a_x, l_result))
			if l_status /= 0 then
				from
				until
					js.loop_done
				loop
					error_inst.gsl_error (ce_string ("gsl_sf_bessel_Knu_scaled_e(nu, x, &result)"), ce_string ("bessel_Knu.c"), 153, l_status)
					Result := (l_result.item.val)
					js.return
					if not js.return_called then
						js.break
					end
				end
			end
			if not js.return_called then
				Result := (l_result.item.val)
			end
		end

	gsl_sf_bessel_Knu_scaled_e (a_nu: REAL_64; a_x: REAL_64; a_result: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]): INTEGER_32
		local
			l_N: INTEGER_32
			l_mu: REAL_64
			l_K_mu: CE_ARRAY [REAL_64]
			l_K_mup1: CE_ARRAY [REAL_64]
			l_Kp_mu: CE_ARRAY [REAL_64]
			l_K_nu: REAL_64
			l_K_nup1: REAL_64
			l_K_num1: REAL_64
			l_n2: INTEGER_32
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			create l_K_mu.make (<<1>>)
			create l_K_mup1.make (<<1>>)
			create l_Kp_mu.make (<<1>>)

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					if not (a_x <= 0.0) then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						l_pos := -1 + 4 -- goto _L
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 7
				when 3 then -- else:
						if not (a_nu < 0.0) then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if: | _L:
						from
						until
							js.loop_done
						loop
							a_result.item.val := (builtin_nanf (ce_string (""))).to_double
							a_result.item.err := (builtin_nanf (ce_string (""))).to_double
							from
							until
								js.loop_done
							loop
								error_inst.gsl_error (ce_string ("domain error"), ce_string ("bessel_Knu.c"), 42, 1)
								Result := 1
								js.return
								if not js.return_called then
									js.break
								end
							end
							if not js.return_called then
								js.break
							end
						end
				when 5 then -- if block done, skip else block:
						l_pos := -1 + 7
				when 6 then -- else:
						l_N := (a_nu + 0.5).truncated_to_integer
						l_mu := a_nu - l_N.to_double
						if a_x < 2.0 then
							bessel_temme_inst.gsl_sf_bessel_K_scaled_temme (l_mu, a_x, l_K_mu, l_K_mup1, l_Kp_mu).do_nothing
						else
							bessel_inst.gsl_sf_bessel_K_scaled_steed_temme_CF2 (l_mu, a_x, l_K_mu, l_K_mup1, l_Kp_mu).do_nothing
						end
						l_K_nu := l_K_mu.item
						l_K_nup1 := l_K_mup1.item
						l_n2 := 0
						from
						until
							not (l_n2 < l_N)
						loop
							l_K_num1 := l_K_nu
							l_K_nu := l_K_nup1
							l_K_nup1 := ((2.0 * ((l_mu + l_n2.to_double) + (1).to_double)) / a_x) * l_K_nu + l_K_num1
							l_n2 := l_n2 + 1
						end
						a_result.item.val := l_K_nu
						a_result.item.err := ((2.0 * 2.2204460492503131e-16) * (l_N.to_double + 4.0)) * (fabs (a_result.item.val))
						Result := 0
						js.return
				when 7 then -- if done
						if not js.return_called then
							Result := 0
						end
				when 8 then
						l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
			end

	gsl_sf_bessel_lnKnu (a_nu: REAL_64; a_x: REAL_64): REAL_64
		local
			l_result: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_status: INTEGER_32
			js: JUMP_STATE
		do
			create l_result.make (<<1>>)

			l_status := (gsl_sf_bessel_lnKnu_e (a_nu, a_x, l_result))
			if l_status /= 0 then
				from
				until
					js.loop_done
				loop
					error_inst.gsl_error (ce_string ("gsl_sf_bessel_lnKnu_e(nu, x, &result)"), ce_string ("bessel_Knu.c"), 163, l_status)
					Result := (l_result.item.val)
					js.return
					if not js.return_called then
						js.break
					end
				end
			end
			if not js.return_called then
				Result := (l_result.item.val)
			end
		end

	gsl_sf_bessel_lnKnu_e (a_nu: REAL_64; a_x: REAL_64; a_result: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]): INTEGER_32
		local
			l_K_scaled: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_tmp___3: REAL_64
			l_ln_bound: REAL_64
			l_lg_nu: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_xi: REAL_64
			l_sum: REAL_64
			l_K_scaled___0: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_tmp___11: REAL_64
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			create l_K_scaled.make (<<1>>)
			create l_lg_nu.make (<<1>>)
			create l_K_scaled___0.make (<<1>>)

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					if not (a_x <= 0.0) then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						l_pos := -1 + 4 -- goto _L
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 7
				when 3 then -- else:
						if not (a_nu < 0.0) then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if: | _L:
						from
						until
							js.loop_done
						loop
							a_result.item.val := (builtin_nanf (ce_string (""))).to_double
							a_result.item.err := (builtin_nanf (ce_string (""))).to_double
							from
							until
								js.loop_done
							loop
								error_inst.gsl_error (ce_string ("domain error"), ce_string ("bessel_Knu.c"), 91, 1)
								Result := 1
								js.return
								if not js.return_called then
									js.break
								end
							end
							if not js.return_called then
								js.break
							end
						end
				when 5 then -- if block done, skip else block:
						l_pos := -1 + 7
				when 6 then -- else:
						if a_nu = 0.0 then
							bessel_k0_inst.gsl_sf_bessel_K0_scaled_e2 (a_x, l_K_scaled).do_nothing
							a_result.item.val := -a_x + (log ((fabs (l_K_scaled.item.val))))
							l_tmp___3 := fabs (a_x)
							a_result.item.err := 2.2204460492503131e-16 * l_tmp___3 + (fabs (l_K_scaled.item.err / l_K_scaled.item.val))
							a_result.item.err := a_result.item.err + (2.2204460492503131e-16 * (fabs (a_result.item.val)))
							Result := 0
							js.return
						elseif a_x < 2.0 then
							if a_nu > 1.0 then
								gamma_inst.gsl_sf_lngamma_e (a_nu, l_lg_nu).do_nothing
								l_ln_bound := (-0.69314718055994530942 - a_nu * (log (0.5 * a_x))) + l_lg_nu.item.val
								if l_ln_bound > 7.0978271289338397e+02 - 20.0 then
									l_xi := (0.25 * a_x) * a_x
									l_sum := 1.0 - l_xi / (a_nu - 1.0)
									if a_nu > 2.0 then
										l_sum := l_sum + ((l_xi / (a_nu - 1.0)) * (l_xi / (a_nu - 2.0)))
									end
									a_result.item.val := l_ln_bound + (log (l_sum))
									a_result.item.err := l_lg_nu.item.err
									a_result.item.err := a_result.item.err + ((2.0 * 2.2204460492503131e-16) * (fabs (a_result.item.val)))
									Result := 0
									js.return
								end
							end
						end
				when 7 then -- if done
						if not js.return_called then
							gsl_sf_bessel_Knu_scaled_e (a_nu, a_x, l_K_scaled___0).do_nothing
							a_result.item.val := -a_x + (log ((fabs (l_K_scaled___0.item.val))))
							l_tmp___11 := fabs (a_x)
							a_result.item.err := 2.2204460492503131e-16 * l_tmp___11 + (fabs (l_K_scaled___0.item.err / l_K_scaled___0.item.val))
							a_result.item.err := a_result.item.err + (2.2204460492503131e-16 * (fabs (a_result.item.val)))
							Result := 0
						end
				when 8 then
						l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
			end

end
