class
	P_BESSEL_Y0

inherit
	LIBGSL_DATA
		redefine
			default_create
		end
	S_GCC_BUILTIN
	S_LIBIO
	S_STDIO
	S_STDLIB
	S_MATH
	S_STRING
	S_ASSERT

feature {NONE} -- Initialization

	default_create
		do
			create by0_cs.make (<<1>>)
			create by0_data.make (<<13>>)
		end

feature {ANY} -- attributes

	by0_cs: CE_ARRAY [PD_CHEB_SERIES_STRUCT]

	by0_data: CE_ARRAY [REAL_64]

feature {ANY} -- routines

	gsl_sf_bessel_Y0 (a_x: REAL_64): REAL_64
		local
			l_result: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_status: INTEGER_32
			js: JUMP_STATE
		do
			create l_result.make (<<1>>)

			l_status := (gsl_sf_bessel_Y0_e (a_x, l_result))
			if l_status /= 0 then
				from
				until
					js.loop_done
				loop
					error_inst.gsl_error (ce_string ("gsl_sf_bessel_Y0_e(x, &result)"), ce_string ("bessel_Y0.c"), 121, l_status)
					Result := (l_result.item.val)
					js.return
					if not js.return_called then
						js.break
					end
				end
			end
			if not js.return_called then
				Result := (l_result.item.val)
			end
		end

	gsl_sf_bessel_Y0_e (a_x: REAL_64; a_result: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]): INTEGER_32
		local
			l_two_over_pi: REAL_64
			l_xmax: REAL_64
			l_J0: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_c___1: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_stat_J0: INTEGER_32
			l_z: REAL_64
			l_c1___2: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_c2___0: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_sp: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_stat_c1: INTEGER_32
			l_stat_c2: INTEGER_32
			l_stat_sp: INTEGER_32
			l_sqrtx: REAL_64
			l_ampl: REAL_64
			l_tmp___8: REAL_64
			l_tmp___11: INTEGER_32
			l_tmp___12: INTEGER_32
			l_tmp___13: INTEGER_32
			js: JUMP_STATE
		do
			create l_J0.make (<<1>>)
			create l_c___1.make (<<1>>)
			create l_c1___2.make (<<1>>)
			create l_c2___0.make (<<1>>)
			create l_sp.make (<<1>>)

			l_two_over_pi := 2.0 / 3.14159265358979323846
			l_xmax := 1.0 / 2.2204460492503131e-16
			if a_x <= 0.0 then
				from
				until
					js.loop_done
				loop
					a_result.item.val := (builtin_nanf (ce_string (""))).to_double
					a_result.item.err := (builtin_nanf (ce_string (""))).to_double
					from
					until
						js.loop_done
					loop
						error_inst.gsl_error (ce_string ("domain error"), ce_string ("bessel_Y0.c"), 80, 1)
						Result := 1
						js.return
						if not js.return_called then
							js.break
						end
					end
					if not js.return_called then
						js.break
					end
				end
			elseif a_x < 4.0 then
				l_stat_J0 := (bessel_j0_inst.gsl_sf_bessel_J0_e2 (a_x, l_J0))
				cheb_eval_inst.cheb_eval_e (by0_cs, (0.125 * a_x) * a_x - 1.0, l_c___1).do_nothing
				a_result.item.val := ((l_two_over_pi * (-0.69314718055994530942 + (log (a_x)))) * l_J0.item.val + 0.375) + l_c___1.item.val
				a_result.item.err := (2.0 * 2.2204460492503131e-16) * (fabs (a_result.item.val)) + l_c___1.item.err
				Result := l_stat_J0
				js.return
			elseif a_x < l_xmax then
				l_z := 32.0 / (a_x * a_x) - 1.0
				l_stat_c1 := (cheb_eval_inst.cheb_eval_e (bessel_amp_phase_inst.gsl_sf_bessel_amp_phase_bm0_cs, l_z, l_c1___2))
				l_stat_c2 := (cheb_eval_inst.cheb_eval_e (bessel_amp_phase_inst.gsl_sf_bessel_amp_phase_bth0_cs, l_z, l_c2___0))
				l_stat_sp := (bessel_inst.gsl_sf_bessel_sin_pi4_e (a_x, l_c2___0.item.val / a_x, l_sp))
				l_sqrtx := (sqrt (a_x))
				l_ampl := (0.75 + l_c1___2.item.val) / l_sqrtx
				a_result.item.val := l_ampl * l_sp.item.val
				l_tmp___8 := fabs (l_sp.item.val)
				a_result.item.err := (l_tmp___8 * l_c1___2.item.err) / l_sqrtx + (fabs (l_ampl)) * l_sp.item.err
				a_result.item.err := a_result.item.err + ((2.0 * 2.2204460492503131e-16) * (fabs (a_result.item.val)))
				if l_stat_sp /= 0 then
					l_tmp___13 := l_stat_sp
				else
					if l_stat_c1 /= 0 then
						l_tmp___12 := l_stat_c1
					else
						if l_stat_c2 /= 0 then
							l_tmp___11 := l_stat_c2
						else
							l_tmp___11 := 0
						end
						l_tmp___12 := l_tmp___11
					end
					l_tmp___13 := l_tmp___12
				end
				Result := l_tmp___13
				js.return
			else
				from
				until
					js.loop_done
				loop
					a_result.item.val := 0.0
					a_result.item.err := 2.2250738585072014e-308
					from
					until
						js.loop_done
					loop
						error_inst.gsl_error (ce_string ("underflow"), ce_string ("bessel_Y0.c"), 110, 15)
						Result := 15
						js.return
						if not js.return_called then
							js.break
						end
					end
					if not js.return_called then
						js.break
					end
				end
			end
			if not js.return_called then
				Result := 0
			end
		end

end
