class
	P_CANONICAL

inherit
	LIBGSL_DATA
	S_GCC_BUILTIN
	S_LIBIO
	S_STDIO
	S_STDLIB
	S_MATH
	S_STRING
	S_ASSERT

feature {ANY} -- routines

	gsl_permutation_canonical_cycles (a_p: CE_POINTER [PD_GSL_PERMUTATION_STRUCT]): NATURAL_32
		local
			l_i: NATURAL_32
			l_count: NATURAL_32
			l_min: NATURAL_32
		do
			l_count := (1).to_natural_32
			l_min := a_p.item.data[0]
			l_i := (0).to_natural_32
			from
			until
				not (l_i < a_p.item.size)
			loop
				if a_p.item.data[l_i.to_integer_32] < l_min then
					l_min := a_p.item.data[l_i.to_integer_32]
					l_count := l_count + 1
				end
				l_i := l_i + 1
			end
			Result := l_count
		end

	gsl_permutation_canonical_to_linear (a_p: CE_POINTER [PD_GSL_PERMUTATION_STRUCT]; a_q___3: CE_POINTER [PD_GSL_PERMUTATION_STRUCT]): INTEGER_32
		local
			l_i: NATURAL_32
			l_k: NATURAL_32
			l_kk: NATURAL_32
			l_first: NATURAL_32
			l_n: NATURAL_32
			l_pp: CE_POINTER [NATURAL_32]
			l_qq: CE_POINTER [NATURAL_32]
			js: JUMP_STATE
		do
			create l_pp.make
			create l_qq.make

			l_n := a_p.item.size
			l_pp := a_p.item.data
			l_qq := a_q___3.item.data
			if a_q___3.item.size /= a_p.item.size then
				from
				until
					js.loop_done
				loop
					error_inst.gsl_error (ce_string ("size of q does not match size of p"), ce_string ("canonical.c"), 95, 4)
					Result := 4
					js.return
					if not js.return_called then
						js.break
					end
				end
			end
			if not js.return_called then
				l_i := (0).to_natural_32
				from
				until
					not (l_i < l_n)
				loop
					l_pp[l_i.to_integer_32] := l_i
					l_i := l_i + 1
				end
				l_k := l_qq[0]
				l_first := l_pp[l_k.to_integer_32]
				l_i := (1).to_natural_32
				from
				until
					not (l_i < l_n)
				loop
					l_kk := l_qq[l_i.to_integer_32]
					if l_kk > l_first then
						l_pp[l_k.to_integer_32] := l_pp[l_kk.to_integer_32]
						l_k := l_kk
					else
						l_pp[l_k.to_integer_32] := l_first
						l_k := l_kk
						l_first := l_pp[l_kk.to_integer_32]
					end
					l_i := l_i + 1
				end
				l_pp[l_k.to_integer_32] := l_first
				Result := 0
			end
		end

	gsl_permutation_inversions (a_p: CE_POINTER [PD_GSL_PERMUTATION_STRUCT]): NATURAL_32
		local
			l_count: NATURAL_32
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_size: NATURAL_32
			js: JUMP_STATE
		do
			l_count := (0).to_natural_32
			l_size := a_p.item.size
			l_i := (0).to_natural_32
			from
			until
				not (not js.loop_done and then (l_i < l_size - {NATURAL_32} 1))
			loop
				l_j := l_i + {NATURAL_32} 1
				from
				until
					not (l_j < l_size)
				loop
					if a_p.item.data[l_i.to_integer_32] > a_p.item.data[l_j.to_integer_32] then
						l_count := l_count + 1
					end
					l_j := l_j + 1
				end
				l_i := l_i + 1
			end
			Result := l_count
		end

	gsl_permutation_linear_cycles (a_p: CE_POINTER [PD_GSL_PERMUTATION_STRUCT]): NATURAL_32
		local
			l_i: NATURAL_32
			l_k: NATURAL_32
			l_count: NATURAL_32
			l_size: NATURAL_32
			js: JUMP_STATE
		do
			l_count := (0).to_natural_32
			l_size := a_p.item.size
			l_i := (0).to_natural_32
			from
			until
				not (not js.loop_done and then (l_i < l_size))
			loop
				l_k := a_p.item.data[l_i.to_integer_32]
				from
				until
					not (l_k > l_i)
				loop
					l_k := a_p.item.data[l_k.to_integer_32]
				end
				if l_k < l_i then
					js.continue
				end
				if not js.loop_jumped then
					l_count := l_count + 1
				end
				if js.loop_increment then
					l_i := l_i + 1
				end
			end
			Result := l_count
		end

	gsl_permutation_linear_to_canonical (a_q___3: CE_POINTER [PD_GSL_PERMUTATION_STRUCT]; a_p: CE_POINTER [PD_GSL_PERMUTATION_STRUCT]): INTEGER_32
		local
			l_n: NATURAL_32
			l_i: NATURAL_32
			l_k: NATURAL_32
			l_s: NATURAL_32
			l_t: NATURAL_32
			l_pp: CE_POINTER [NATURAL_32]
			l_qq: CE_POINTER [NATURAL_32]
			js: JUMP_STATE
		do
			create l_pp.make
			create l_qq.make

			l_n := a_p.item.size
			l_t := l_n
			l_pp := a_p.item.data
			l_qq := a_q___3.item.data
			if a_q___3.item.size /= a_p.item.size then
				from
				until
					js.loop_done
				loop
					error_inst.gsl_error (ce_string ("size of q does not match size of p"), ce_string ("canonical.c"), 42, 4)
					Result := 4
					js.return
					if not js.return_called then
						js.break
					end
				end
			end
			if not js.return_called then
				l_i := (0).to_natural_32
				from
				until
					not (not js.loop_done and then (l_i < l_n))
				loop
					l_k := l_pp[l_i.to_integer_32]
					l_s := (1).to_natural_32
					from
					until
						not (l_k > l_i)
					loop
						l_k := l_pp[l_k.to_integer_32]
						l_s := l_s + 1
					end
					if l_k < l_i then
						js.continue
					end
					if not js.loop_jumped then
						l_t := l_t - l_s
						l_qq[l_t.to_integer_32] := l_i
						l_k := l_pp[l_i.to_integer_32]
						l_s := (1).to_natural_32
						from
						until
							not (l_k > l_i)
						loop
							l_qq[(l_t + l_s).to_integer_32] := l_k
							l_k := l_pp[l_k.to_integer_32]
							l_s := l_s + 1
						end
						if l_t = {NATURAL_32} 0 then
							js.break
						end
					end
					if js.loop_increment then
						l_i := l_i + 1
					end
				end
				Result := 0
			end
		end

end
