class
	P_CLAUSEN

inherit
	LIBGSL_DATA
		redefine
			default_create
		end
	S_GCC_BUILTIN
	S_LIBIO
	S_STDIO
	S_STDLIB
	S_MATH
	S_STRING
	S_ASSERT

feature {NONE} -- Initialization

	default_create
		do
			create aclaus_cs.make (<<1>>)
			create aclaus_data.make (<<15>>)
		end

feature {ANY} -- attributes

	aclaus_cs: CE_ARRAY [PD_CHEB_SERIES_STRUCT]

	aclaus_data: CE_ARRAY [REAL_64]

feature {ANY} -- routines

	gsl_sf_clausen (a_x: REAL_64): REAL_64
		local
			l_result: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_status: INTEGER_32
			js: JUMP_STATE
		do
			create l_result.make (<<1>>)

			l_status := (gsl_sf_clausen_e (a_x, l_result))
			if l_status /= 0 then
				from
				until
					js.loop_done
				loop
					error_inst.gsl_error (ce_string ("gsl_sf_clausen_e(x, &result)"), ce_string ("clausen.c"), 110, l_status)
					Result := (l_result.item.val)
					js.return
					if not js.return_called then
						js.break
					end
				end
			end
			if not js.return_called then
				Result := (l_result.item.val)
			end
		end

	gsl_sf_clausen_e (a_x: REAL_64; a_result: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]): INTEGER_32
		local
			l_x: CE_ARRAY [REAL_64]
			l_x_cut: REAL_64
			l_sgn: REAL_64
			l_status_red: INTEGER_32
			l_p0: REAL_64
			l_p1: REAL_64
			l_t: REAL_64
			l_result_c: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
		do
			create l_x.make (<<1>>)
			create l_result_c.make (<<1>>)

			l_x.item := a_x
			l_x_cut := 3.14159265358979323846 * 1.4901161193847656e-08
			l_sgn := 1.0
			if l_x.item < 0.0 then
				l_x.item := -l_x.item
				l_sgn := -1.0
			end
			l_status_red := trig_inst.gsl_sf_angle_restrict_pos_e (l_x)
			if l_x.item > 3.14159265358979323846 then
				l_p0 := 6.28125
				l_p1 := 0.19353071795864769253e-02
				l_x.item := (l_p0 - l_x.item) + l_p1
				l_sgn := -l_sgn
			end
			if l_x.item = 0.0 then
				a_result.item.val := 0.0
				a_result.item.err := 0.0
			elseif l_x.item < l_x_cut then
				a_result.item.val := l_x.item * (1.0 - (log (l_x.item)))
				a_result.item.err := l_x.item * 2.2204460492503131e-16
			else
				l_t := 2.0 * ((l_x.item * l_x.item) / (3.14159265358979323846 * 3.14159265358979323846) - 0.5)
				cheb_eval_inst.cheb_eval_e (aclaus_cs, l_t, l_result_c).do_nothing
				a_result.item.val := l_x.item * (l_result_c.item.val - (log (l_x.item)))
				a_result.item.err := l_x.item * (l_result_c.item.err + 2.2204460492503131e-16)
			end
			a_result.item.val := a_result.item.val * l_sgn
			Result := l_status_red
		end

end
