class
	P_DAWSON

inherit
	LIBGSL_DATA
		redefine
			default_create
		end
	S_GCC_BUILTIN
	S_LIBIO
	S_STDIO
	S_STDLIB
	S_MATH
	S_STRING
	S_ASSERT

feature {NONE} -- Initialization

	default_create
		do
			create daw2_cs.make (<<1>>)
			create daw2_data.make (<<45>>)
			create daw_cs.make (<<1>>)
			create daw_data.make (<<21>>)
			create dawa_cs.make (<<1>>)
			create dawa_data.make (<<75>>)
		end

feature {ANY} -- attributes

	daw2_cs: CE_ARRAY [PD_CHEB_SERIES_STRUCT]

	daw2_data: CE_ARRAY [REAL_64]

	daw_cs: CE_ARRAY [PD_CHEB_SERIES_STRUCT]

	daw_data: CE_ARRAY [REAL_64]

	dawa_cs: CE_ARRAY [PD_CHEB_SERIES_STRUCT]

	dawa_data: CE_ARRAY [REAL_64]

feature {ANY} -- routines

	gsl_sf_dawson (a_x: REAL_64): REAL_64
		local
			l_result: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_status: INTEGER_32
			js: JUMP_STATE
		do
			create l_result.make (<<1>>)

			l_status := (gsl_sf_dawson_e (a_x, l_result))
			if l_status /= 0 then
				from
				until
					js.loop_done
				loop
					error_inst.gsl_error (ce_string ("gsl_sf_dawson_e(x, &result)"), ce_string ("dawson.c"), 280, l_status)
					Result := (l_result.item.val)
					js.return
					if not js.return_called then
						js.break
					end
				end
			end
			if not js.return_called then
				Result := (l_result.item.val)
			end
		end

	gsl_sf_dawson_e (a_x: REAL_64; a_result: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]): INTEGER_32
		local
			l_xsml: REAL_64
			l_xbig: REAL_64
			l_xmax: REAL_64
			l_y: REAL_64
			l_result_c: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_result_c___0: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_result_c___1: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			js: JUMP_STATE
		do
			create l_result_c.make (<<1>>)
			create l_result_c___0.make (<<1>>)
			create l_result_c___1.make (<<1>>)

			l_xsml := 1.225 * 1.4901161193847656e-08
			l_xbig := 1.0 / (1.41421356237309504880 * 1.4901161193847656e-08)
			l_xmax := 0.1 * 1.7976931348623157e+308
			l_y := (fabs (a_x))
			if l_y < l_xsml then
				a_result.item.val := a_x
				a_result.item.err := 0.0
				Result := 0
				js.return
			elseif l_y < 1.0 then
				cheb_eval_inst.cheb_eval_e (daw_cs, (2.0 * l_y) * l_y - 1.0, l_result_c).do_nothing
				a_result.item.val := a_x * (0.75 + l_result_c.item.val)
				a_result.item.err := l_y * l_result_c.item.err
				a_result.item.err := a_result.item.err + ((2.0 * 2.2204460492503131e-16) * (fabs (a_result.item.val)))
				Result := 0
				js.return
			elseif l_y < 4.0 then
				cheb_eval_inst.cheb_eval_e (daw2_cs, (0.125 * l_y) * l_y - 1.0, l_result_c___0).do_nothing
				a_result.item.val := a_x * (0.25 + l_result_c___0.item.val)
				a_result.item.err := l_y * l_result_c___0.item.err
				a_result.item.err := a_result.item.err + ((2.0 * 2.2204460492503131e-16) * (fabs (a_result.item.val)))
				Result := 0
				js.return
			elseif l_y < l_xbig then
				cheb_eval_inst.cheb_eval_e (dawa_cs, 32.0 / (l_y * l_y) - 1.0, l_result_c___1).do_nothing
				a_result.item.val := (0.5 + l_result_c___1.item.val) / a_x
				a_result.item.err := l_result_c___1.item.err / l_y
				a_result.item.err := a_result.item.err + ((2.0 * 2.2204460492503131e-16) * (fabs (a_result.item.val)))
				Result := 0
				js.return
			elseif l_y < l_xmax then
				a_result.item.val := 0.5 / a_x
				a_result.item.err := (2.0 * 2.2204460492503131e-16) * a_result.item.val
				Result := 0
				js.return
			else
				from
				until
					js.loop_done
				loop
					a_result.item.val := 0.0
					a_result.item.err := 2.2250738585072014e-308
					from
					until
						js.loop_done
					loop
						error_inst.gsl_error (ce_string ("underflow"), ce_string ("dawson.c"), 269, 15)
						Result := 15
						js.return
						if not js.return_called then
							js.break
						end
					end
					if not js.return_called then
						js.break
					end
				end
			end
			if not js.return_called then
				Result := 0
			end
		end

end
