class
	P_EXPONENTIAL

inherit
	LIBGSL_DATA
		redefine
			default_create
		end
	S_GCC_BUILTIN
	S_LIBIO
	S_STDIO
	S_STDLIB
	S_MATH
	S_STRING
	S_ASSERT

feature {NONE} -- Initialization

	default_create
		do
			create mvl_tab.make (<<3, 6>>)
		end

feature {ANY} -- attributes

	mvl_tab: CE_ARRAY [CE_ARRAY [PD_MOLER_VANLOAN_OPTIMAL_SUGGESTION]]

feature {ANY} -- routines

	gsl_cdf_exponential_P (a_x: REAL_64; a_mu: REAL_64): REAL_64
		local
			l_P___2: REAL_64
		do
			if a_x < (0).to_double then
				Result := (0).to_double
			else
				l_P___2 := -(expm1 (-a_x / a_mu))
				Result := l_P___2
			end
		end

	gsl_cdf_exponential_Q (a_x: REAL_64; a_mu: REAL_64): REAL_64
		local
			l_Q___1: REAL_64
		do
			if a_x < (0).to_double then
				Result := (1).to_double
			else
				l_Q___1 := (exp (-a_x / a_mu))
				Result := l_Q___1
			end
		end

	gsl_linalg_exponential_ss (a_A___0: CE_POINTER [PD_GSL_MATRIX]; a_eA: CE_POINTER [PD_GSL_MATRIX]; a_mode: NATURAL_32): INTEGER_32
		local
			l_i: INTEGER_32
			l_sugg: PD_MOLER_VANLOAN_OPTIMAL_SUGGESTION
			l_divisor: REAL_64
			l_reduced_A: CE_POINTER [PD_GSL_MATRIX]
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			create l_sugg.make
			create l_reduced_A.make

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					if a_A___0.item.size1 = a_A___0.item.size2 then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						from
						until
							js.loop_done
						loop
							error_inst.gsl_error (ce_string ("cannot exponentiate a non-square matrix"), ce_string ("exponential.c"), 154, 20)
							Result := 20
							js.return
							if not js.return_called then
								js.break
							end
						end
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 10
				when 3 then -- else:
						if a_A___0.item.size1 = a_eA.item.size1 then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if:
						l_pos := -1 + 7 -- goto _L
				when 5 then -- if block done, skip else block:
						l_pos := -1 + 10
				when 6 then -- else:
						if a_A___0.item.size2 = a_eA.item.size2 then
							l_pos := -1 + 9 -- skip if block
						end
				when 7 then -- if: | _L:
						from
						until
							js.loop_done
						loop
							error_inst.gsl_error (ce_string ("exponential of matrix must have same dimension as matrix"), ce_string ("exponential.c"), 158, 19)
							Result := 19
							js.return
							if not js.return_called then
								js.break
							end
						end
				when 8 then -- if block done, skip else block:
						l_pos := -1 + 10
				when 9 then -- else:
						l_sugg := (obtain_suggestion (a_A___0, a_mode)).twin
						l_divisor := (exp (0.69314718055994530942 * (l_sugg.j).to_double))
						l_reduced_A := (init_source_inst.gsl_matrix_alloc (a_A___0.item.size1, a_A___0.item.size2))
						copy_source_inst.gsl_matrix_memcpy (l_reduced_A, a_A___0).do_nothing
						oper_source_inst.gsl_matrix_scale (l_reduced_A, 1.0 / l_divisor).do_nothing
						matrix_exp_series (l_reduced_A, a_eA, l_sugg.k)
						l_i := 0
						from
						until
							not (l_i < l_sugg.j)
						loop
							blas_inst.gsl_blas_dgemm ({PC_CBLAS_TRANSPOSE}.CblasNoTrans, {PC_CBLAS_TRANSPOSE}.CblasNoTrans, 1.0, a_eA, a_eA, 0.0, l_reduced_A).do_nothing
							copy_source_inst.gsl_matrix_memcpy (a_eA, l_reduced_A).do_nothing
							l_i := l_i + 1
						end
						init_source_inst.gsl_matrix_free (l_reduced_A)
						Result := 0
						js.return
				when 10 then -- if done | if done
						if not js.return_called then
							Result := 0
						end
				when 11 then
						l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
			end

	gsl_ran_exponential (a_r___3: CE_POINTER [PD_GSL_RNG]; a_mu: REAL_64): REAL_64
		local
			l_u: REAL_64
		do
			l_u := (gsl_gsl_rng_inst.gsl_rng_uniform (a_r___3))
			Result := (-a_mu * (log1p (-l_u)))
		end

	gsl_ran_exponential_pdf (a_x: REAL_64; a_mu: REAL_64): REAL_64
		local
			l_p: REAL_64
		do
			if a_x < (0).to_double then
				Result := (0).to_double
			else
				l_p := (exp (-a_x / a_mu)) / a_mu
				Result := l_p
			end
		end

	matrix_exp_series (a_B: CE_POINTER [PD_GSL_MATRIX]; a_eB: CE_POINTER [PD_GSL_MATRIX]; a_number_of_terms: INTEGER_32)
		local
			l_count: INTEGER_32
			l_temp: CE_POINTER [PD_GSL_MATRIX]
		do
			create l_temp.make

			l_temp := (init_source_inst.gsl_matrix_calloc (a_B.item.size1, a_B.item.size2))
			copy_source_inst.gsl_matrix_memcpy (a_eB, a_B).do_nothing
			oper_source_inst.gsl_matrix_scale (a_eB, 1.0 / a_number_of_terms.to_double).do_nothing
			oper_source_inst.gsl_matrix_add_diagonal (a_eB, 1.0).do_nothing
			l_count := a_number_of_terms - 1
			from
			until
				not (l_count >= 1)
			loop
				blas_inst.gsl_blas_dgemm ({PC_CBLAS_TRANSPOSE}.CblasNoTrans, {PC_CBLAS_TRANSPOSE}.CblasNoTrans, 1.0, a_B, a_eB, 0.0, l_temp).do_nothing
				oper_source_inst.gsl_matrix_scale (l_temp, 1.0 / l_count.to_double).do_nothing
				oper_source_inst.gsl_matrix_add_diagonal (l_temp, 1.0).do_nothing
				copy_source_inst.gsl_matrix_memcpy (a_eB, l_temp).do_nothing
				l_count := l_count - 1
			end
			init_source_inst.gsl_matrix_free (l_temp)
		end

	obtain_suggestion (a_A___0: CE_POINTER [PD_GSL_MATRIX]; a_mode: NATURAL_32): PD_MOLER_VANLOAN_OPTIMAL_SUGGESTION
		local
			l_mode_prec: NATURAL_32
			l_norm_A: REAL_64
			l_extra: REAL_64
			l_extra_i: INTEGER_32
			l_s: PD_MOLER_VANLOAN_OPTIMAL_SUGGESTION
		do
			create l_s.make

			l_mode_prec := (gsl_gsl_mode_inst.GSL_MODE_PREC (a_mode))
			l_norm_A := (sup_norm (a_A___0))
			if l_norm_A < 0.01 then
				Result := ((mvl_tab[l_mode_prec.to_integer_32])[0]).twin
			elseif l_norm_A < 0.1 then
				Result := ((mvl_tab[l_mode_prec.to_integer_32])[1]).twin
			elseif l_norm_A < 1.0 then
				Result := ((mvl_tab[l_mode_prec.to_integer_32])[2]).twin
			elseif l_norm_A < 10.0 then
				Result := ((mvl_tab[l_mode_prec.to_integer_32])[3]).twin
			elseif l_norm_A < 100.0 then
				Result := ((mvl_tab[l_mode_prec.to_integer_32])[4]).twin
			elseif l_norm_A < 1000.0 then
				Result := ((mvl_tab[l_mode_prec.to_integer_32])[5]).twin
			else
				l_extra := (log ((1.01 * l_norm_A) / 1000.0)) / 0.69314718055994530942
				l_extra_i := ((ceil (l_extra)).truncated_to_integer_64.to_natural_32).to_integer_32
				l_s := ((mvl_tab[a_mode.to_integer_32])[5]).twin
				l_s.j := l_s.j + l_extra_i
				Result := l_s.twin
			end
		end

	sup_norm (a_A___0: CE_POINTER [PD_GSL_MATRIX]): REAL_64
		local
			l_min: CE_ARRAY [REAL_64]
			l_max: CE_ARRAY [REAL_64]
			l_tmp: REAL_64
		do
			create l_min.make (<<1>>)
			create l_max.make (<<1>>)

			minmax_source_inst.gsl_matrix_minmax (a_A___0, l_min, l_max)
			l_tmp := fabs (l_max.item)
			Result := (gsl_gsl_minmax_inst.GSL_MAX_DBL ((fabs (l_min.item)), l_tmp))
		end

end
