class
	P_GAMMA

inherit
	LIBGSL_DATA
		redefine
			default_create
		end
	S_GCC_BUILTIN
	S_LIBIO
	S_STDIO
	S_STDLIB
	S_MATH
	S_STRING
	S_ASSERT

feature {NONE} -- Initialization

	default_create
		do
			create doub_fact_table.make (<<298>>)
			create fact_table.make (<<171>>)
			create gamma_5_10_cs.make (<<1>>)
			create gamma_5_10_data.make (<<24>>)
			create gstar_a_cs.make (<<1>>)
			create gstar_a_data.make (<<30>>)
			create gstar_b_cs.make (<<1>>)
			create gstar_b_data.make (<<30>>)
			create lanczos_7_c.make (<<9>>)
		end

feature {ANY} -- attributes

	doub_fact_table: CE_ARRAY [PD_DOUB_FACT_TABLE]

	fact_table: CE_ARRAY [PD_FACT_TABLE]

	gamma_5_10_cs: CE_ARRAY [PD_CHEB_SERIES_STRUCT]

	gamma_5_10_data: CE_ARRAY [REAL_64]

	gstar_a_cs: CE_ARRAY [PD_CHEB_SERIES_STRUCT]

	gstar_a_data: CE_ARRAY [REAL_64]

	gstar_b_cs: CE_ARRAY [PD_CHEB_SERIES_STRUCT]

	gstar_b_data: CE_ARRAY [REAL_64]

	lanczos_7_c: CE_ARRAY [REAL_64]

feature {ANY} -- routines

	gamma_frac (a_r___3: CE_POINTER [PD_GSL_RNG]; a_a___5: REAL_64): REAL_64
		local
			l_p: REAL_64
			l_q___3: REAL_64
			l_x: REAL_64
			l_u: REAL_64
			l_v: REAL_64
			js: JUMP_STATE
		do
			l_p := 2.7182818284590452354 / (a_a___5 + 2.7182818284590452354)
			from
			until
				js.loop_done
			loop
				l_u := gsl_gsl_rng_inst.gsl_rng_uniform (a_r___3)
				l_v := gsl_gsl_rng_inst.gsl_rng_uniform_pos (a_r___3)
				if l_u < l_p then
					l_x := exp (((1).to_double / a_a___5) * (log (l_v)))
					l_q___3 := exp (-l_x)
				else
					l_x := (1).to_double - (log (l_v))
					l_q___3 := exp ((a_a___5 - (1).to_double) * (log (l_x)))
				end
				if not ((gsl_gsl_rng_inst.gsl_rng_uniform (a_r___3)) >= l_q___3) then
					js.break
				end
			end
			Result := l_x
		end

	gamma_large (a_r___3: CE_POINTER [PD_GSL_RNG]; a_a___5: REAL_64): REAL_64
		local
			l_sqa: REAL_64
			l_x: REAL_64
			l_y: REAL_64
			l_v: REAL_64
			js: JUMP_STATE
		do
			l_sqa := sqrt ((2).to_double * a_a___5 - (1).to_double)
			from
			until
				js.loop_done
			loop
				from
				until
					js.loop_done
				loop
					l_y := tan (3.14159265358979323846 * (gsl_gsl_rng_inst.gsl_rng_uniform (a_r___3)))
					l_x := (l_sqa * l_y + a_a___5) - (1).to_double
					if not (l_x <= (0).to_double) then
						js.break
					end
				end
				l_v := gsl_gsl_rng_inst.gsl_rng_uniform (a_r___3)
				if not (l_v > ((1).to_double + l_y * l_y) * (exp ((a_a___5 - (1).to_double) * (log (l_x / (a_a___5 - (1).to_double))) - l_sqa * l_y))) then
					js.break
				end
			end
			Result := l_x
		end

	gamma_xgthalf (a_x: REAL_64; a_result: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]): INTEGER_32
		local
			l_n: INTEGER_32
			l_eps: REAL_64
			l_c1___2: REAL_64
			l_c2___0: REAL_64
			l_c3___3: REAL_64
			l_c4___2: REAL_64
			l_c5___1: REAL_64
			l_c6___2: REAL_64
			l_c7___0: REAL_64
			l_eps___0: REAL_64
			l_c1___3: REAL_64
			l_c2___1: REAL_64
			l_c3___4: REAL_64
			l_c4___3: REAL_64
			l_c5___2: REAL_64
			l_c6___3: REAL_64
			l_c7___1: REAL_64
			l_c8___0: REAL_64
			l_lg: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_gamma_8: REAL_64
			l_t: REAL_64
			l_c___1: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_p: REAL_64
			l_e: REAL_64
			l_q___3: REAL_64
			l_pre: REAL_64
			l_gstar: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_stat_gs: INTEGER_32
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			create l_lg.make (<<1>>)
			create l_c___1.make (<<1>>)
			create l_gstar.make (<<1>>)

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					if a_x /= 0.5 then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						a_result.item.val := 1.77245385090551602729817
						a_result.item.err := 2.2204460492503131e-16 * a_result.item.val
						Result := 0
						js.return
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 7
				when 3 then -- else:
						if not (a_x <= (170).to_double + 1.0) then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if:
						if a_x = (floor (a_x)) then
							l_n := (floor (a_x)).truncated_to_integer
							a_result.item.val := fact_table[l_n - 1].f
							a_result.item.err := 2.2204460492503131e-16 * a_result.item.val
							Result := 0
							js.return
						else
							l_pos := -1 + 6 -- goto _L
						end
				when 5 then -- if block done, skip else block:
						l_pos := -1 + 7
				when 6 then -- else: | _L:
						if (fabs (a_x - 1.0)) < 0.01 then
							l_eps := a_x - 1.0
							l_c1___2 := 0.4227843350984671394
							l_c2___0 := -0.01094400467202744461
							l_c3___3 := 0.09252092391911371098
							l_c4___2 := -0.018271913165599812664
							l_c5___1 := 0.018004931096854797895
							l_c6___2 := -0.006850885378723806846
							l_c7___0 := 0.003998239557568466030
							a_result.item.val := 1.0 / a_x + l_eps * (l_c1___2 + l_eps * (l_c2___0 + l_eps * (l_c3___3 + l_eps * (l_c4___2 + l_eps * (l_c5___1 + l_eps * (l_c6___2 + l_eps * l_c7___0))))))
							a_result.item.err := 2.2204460492503131e-16
							Result := 0
							js.return
						elseif (fabs (a_x - 2.0)) < 0.01 then
							l_eps___0 := a_x - 2.0
							l_c1___3 := 0.4227843350984671394
							l_c2___1 := 0.4118403304264396948
							l_c3___4 := 0.08157691924708626638
							l_c4___3 := 0.07424901075351389832
							l_c5___2 := -0.00026698206874501476832
							l_c6___3 := 0.011154045718130991049
							l_c7___1 := -0.002852645821155340816
							l_c8___0 := 0.0021039333406973880085
							a_result.item.val := 1.0 + l_eps___0 * (l_c1___3 + l_eps___0 * (l_c2___1 + l_eps___0 * (l_c3___4 + l_eps___0 * (l_c4___3 + l_eps___0 * (l_c5___2 + l_eps___0 * (l_c6___3 + l_eps___0 * (l_c7___1 + l_eps___0 * l_c8___0)))))))
							a_result.item.err := 2.2204460492503131e-16
							Result := 0
							js.return
						elseif a_x < 5.0 then
							lngamma_lanczos (a_x, l_lg).do_nothing
							a_result.item.val := exp (l_lg.item.val)
							a_result.item.err := a_result.item.val * (l_lg.item.err + 2.0 * 2.2204460492503131e-16)
							Result := 0
							js.return
						elseif a_x < 10.0 then
							l_gamma_8 := 5040.0
							l_t := (2.0 * a_x - 15.0) / 5.0
							cheb_eval_inst.cheb_eval_e (gamma_5_10_cs, l_t, l_c___1).do_nothing
							a_result.item.val := (exp (l_c___1.item.val)) * l_gamma_8
							a_result.item.err := a_result.item.val * l_c___1.item.err
							a_result.item.err := a_result.item.err + ((2.0 * 2.2204460492503131e-16) * a_result.item.val)
							Result := 0
							js.return
						elseif a_x < 171.0 then
							l_p := (pow (a_x, 0.5 * a_x))
							l_e := (exp (-a_x))
							l_q___3 := (l_p * l_e) * l_p
							l_pre := ((1.41421356237309504880 * 1.77245385090551602729816748334) * l_q___3) / (sqrt (a_x))
							l_stat_gs := (gammastar_ser (a_x, l_gstar))
							a_result.item.val := l_pre * l_gstar.item.val
							a_result.item.err := ((a_x + 2.5) * 2.2204460492503131e-16) * a_result.item.val
							Result := l_stat_gs
							js.return
						else
							from
							until
								js.loop_done
							loop
								a_result.item.val := (builtin_inff).to_double
								a_result.item.err := (builtin_inff).to_double
								from
								until
									js.loop_done
								loop
									error_inst.gsl_error (ce_string ("overflow"), ce_string ("gamma.c"), 1_106, 16)
									Result := 16
									js.return
									if not js.return_called then
										js.break
									end
								end
								if not js.return_called then
									js.break
								end
							end
						end
				when 7 then -- if done
						if not js.return_called then
							Result := 0
						end
				when 8 then
						l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
			end

	gammastar_ser (a_x: REAL_64; a_result: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]): INTEGER_32
		local
			l_y: REAL_64
			l_c0___0: REAL_64
			l_c1___2: REAL_64
			l_c2___0: REAL_64
			l_c3___3: REAL_64
			l_c4___2: REAL_64
			l_c5___1: REAL_64
			l_c6___2: REAL_64
			l_c7___0: REAL_64
			l_ser: REAL_64
		do
			l_y := 1.0 / (a_x * a_x)
			l_c0___0 := 1.0 / 12.0
			l_c1___2 := -1.0 / 360.0
			l_c2___0 := 1.0 / 1260.0
			l_c3___3 := -1.0 / 1680.0
			l_c4___2 := 1.0 / 1188.0
			l_c5___1 := -691.0 / 360360.0
			l_c6___2 := 1.0 / 156.0
			l_c7___0 := -3617.0 / 122400.0
			l_ser := l_c0___0 + l_y * (l_c1___2 + l_y * (l_c2___0 + l_y * (l_c3___3 + l_y * (l_c4___2 + l_y * (l_c5___1 + l_y * (l_c6___2 + l_y * l_c7___0))))))
			a_result.item.val := exp (l_ser / a_x)
			a_result.item.err := ((2.0 * 2.2204460492503131e-16) * a_result.item.val) * (gsl_gsl_minmax_inst.GSL_MAX_DBL (1.0, l_ser / a_x))
			Result := 0
		end

	gsl_cdf_gamma_P (a_x: REAL_64; a_a___5: REAL_64; a_b___0: REAL_64): REAL_64
		local
			l_P___2: REAL_64
			l_y: REAL_64
			js: JUMP_STATE
		do
			l_y := a_x / a_b___0
			if a_x <= 0.0 then
				Result := (0.0)
				js.return
			end
			if not js.return_called then
				if l_y > a_a___5 then
					l_P___2 := (1).to_double - (gamma_inc_inst.gsl_sf_gamma_inc_Q (a_a___5, l_y))
				else
					l_P___2 := gamma_inc_inst.gsl_sf_gamma_inc_P (a_a___5, l_y)
				end
				Result := l_P___2
			end
		end

	gsl_cdf_gamma_Q (a_x: REAL_64; a_a___5: REAL_64; a_b___0: REAL_64): REAL_64
		local
			l_Q___1: REAL_64
			l_y: REAL_64
			js: JUMP_STATE
		do
			l_y := a_x / a_b___0
			if a_x <= 0.0 then
				Result := (1.0)
				js.return
			end
			if not js.return_called then
				if l_y < a_a___5 then
					l_Q___1 := (1).to_double - (gamma_inc_inst.gsl_sf_gamma_inc_P (a_a___5, l_y))
				else
					l_Q___1 := gamma_inc_inst.gsl_sf_gamma_inc_Q (a_a___5, l_y)
				end
				Result := l_Q___1
			end
		end

	gsl_ran_gamma (a_r___3: CE_POINTER [PD_GSL_RNG]; a_a___5: REAL_64; a_b___0: REAL_64): REAL_64
		local
			l_u: REAL_64
			l_tmp___0: REAL_64
			l_x: REAL_64
			l_v: REAL_64
			l_u___0: REAL_64
			l_d: REAL_64
			l_c___1: REAL_64
			l_tmp___3: REAL_64
			js: JUMP_STATE
		do
			if a_a___5 < (1).to_double then
				l_u := (gsl_gsl_rng_inst.gsl_rng_uniform_pos (a_r___3))
				l_tmp___0 := gsl_ran_gamma (a_r___3, 1.0 + a_a___5, a_b___0)
				Result := (l_tmp___0 * (pow (l_u, 1.0 / a_a___5)))
				js.return
			end
			if not js.return_called then
				l_d := a_a___5 - 1.0 / 3.0
				l_c___1 := (1.0 / 3.0) / (sqrt (l_d))
				from
				until
					js.loop_done
				loop
					from
					until
						js.loop_done
					loop
						l_x := gausszig_inst.gsl_ran_gaussian_ziggurat (a_r___3, 1.0)
						l_v := 1.0 + l_c___1 * l_x
						if not (l_v <= (0).to_double) then
							js.break
						end
					end
					l_v := (l_v * l_v) * l_v
					l_u___0 := gsl_gsl_rng_inst.gsl_rng_uniform_pos (a_r___3)
					if l_u___0 < (1).to_double - (((0.0331 * l_x) * l_x) * l_x) * l_x then
						js.break
					end
					if not js.loop_jumped then
						l_tmp___3 := log (l_u___0)
						if l_tmp___3 < (0.5 * l_x) * l_x + l_d * (((1).to_double - l_v) + (log (l_v))) then
							js.break
						end
					end
				end
				Result := ((a_b___0 * l_d) * l_v)
			end
		end

	gsl_ran_gamma_int (a_r___3: CE_POINTER [PD_GSL_RNG]; a_a___5: NATURAL_32): REAL_64
		local
			l_i: NATURAL_32
			l_prod: REAL_64
		do
			if a_a___5 < {NATURAL_32} 12 then
				l_prod := (1).to_double
				l_i := {NATURAL_32} 0
				from
				until
					not (l_i < a_a___5)
				loop
					l_prod := l_prod * (gsl_gsl_rng_inst.gsl_rng_uniform_pos (a_r___3))
					l_i := l_i + 1
				end
				Result := (-(log (l_prod)))
			else
				Result := (gamma_large (a_r___3, a_a___5.to_real_64))
			end
		end

	gsl_ran_gamma_knuth (a_r___3: CE_POINTER [PD_GSL_RNG]; a_a___5: REAL_64; a_b___0: REAL_64): REAL_64
		local
			l_na: NATURAL_32
			l_tmp___1: REAL_64
			l_tmp___6: REAL_64
		do
			l_na := (floor (a_a___5)).truncated_to_integer_64.to_natural_32
			if a_a___5 >= ({NATURAL_32} 4_294_967_295).to_real_64 then
				l_tmp___1 := gamma_large (a_r___3, (floor (a_a___5)))
				Result := (a_b___0 * (l_tmp___1 + (gamma_frac (a_r___3, a_a___5 - (floor (a_a___5))))))
			elseif a_a___5 = l_na.to_real_64 then
				Result := (a_b___0 * (gsl_ran_gamma_int (a_r___3, l_na)))
			elseif l_na = {NATURAL_32} 0 then
				Result := (a_b___0 * (gamma_frac (a_r___3, a_a___5)))
			else
				l_tmp___6 := gsl_ran_gamma_int (a_r___3, l_na)
				Result := (a_b___0 * (l_tmp___6 + (gamma_frac (a_r___3, a_a___5 - l_na.to_real_64))))
			end
		end

	gsl_ran_gamma_mt (a_r___3: CE_POINTER [PD_GSL_RNG]; a_a___5: REAL_64; a_b___0: REAL_64): REAL_64
		do
			Result := (gsl_ran_gamma (a_r___3, a_a___5, a_b___0))
		end

	gsl_ran_gamma_pdf (a_x: REAL_64; a_a___5: REAL_64; a_b___0: REAL_64): REAL_64
		local
			l_p: REAL_64
			l_lngamma: REAL_64
		do
			if a_x < (0).to_double then
				Result := (0).to_double
			elseif a_x = (0).to_double then
				if a_a___5 = (1).to_double then
					Result := ((1).to_double / a_b___0)
				else
					Result := (0).to_double
				end
			elseif a_a___5 = (1).to_double then
				Result := ((exp (-a_x / a_b___0)) / a_b___0)
			else
				l_lngamma := (gsl_sf_lngamma (a_a___5))
				l_p := (exp (((a_a___5 - (1).to_double) * (log (a_x / a_b___0)) - a_x / a_b___0) - l_lngamma)) / a_b___0
				Result := l_p
			end
		end

	gsl_sf_choose (a_n: NATURAL_32; a_m___4: NATURAL_32): REAL_64
		local
			l_result: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_status: INTEGER_32
			js: JUMP_STATE
		do
			create l_result.make (<<1>>)

			l_status := (gsl_sf_choose_e (a_n, a_m___4, l_result))
			if l_status /= 0 then
				from
				until
					js.loop_done
				loop
					error_inst.gsl_error (ce_string ("gsl_sf_choose_e(n, m, &result)"), ce_string ("gamma.c"), 1_679, l_status)
					Result := (l_result.item.val)
					js.return
					if not js.return_called then
						js.break
					end
				end
			end
			if not js.return_called then
				Result := (l_result.item.val)
			end
		end

	gsl_sf_choose_e (a_n: NATURAL_32; a_m___4: NATURAL_32; a_result: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]): INTEGER_32
		local
			l_m___4: NATURAL_32
			l_prod: REAL_64
			l_k: NATURAL_32
			l_tk: REAL_64
			l_lc: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_stat_lc: INTEGER_32
			l_stat_e: INTEGER_32
			l_tmp___7: INTEGER_32
			l_tmp___8: INTEGER_32
			js: JUMP_STATE
		do
			create l_lc.make (<<1>>)

			l_m___4 := a_m___4
			if l_m___4 > a_n then
				from
				until
					js.loop_done
				loop
					a_result.item.val := (builtin_nanf (ce_string (""))).to_double
					a_result.item.err := (builtin_nanf (ce_string (""))).to_double
					from
					until
						js.loop_done
					loop
						error_inst.gsl_error (ce_string ("domain error"), ce_string ("gamma.c"), 1_587, 1)
						Result := 1
						js.return
						if not js.return_called then
							js.break
						end
					end
					if not js.return_called then
						js.break
					end
				end
			elseif l_m___4 = a_n then
				a_result.item.val := 1.0
				a_result.item.err := 0.0
				Result := 0
				js.return
			elseif l_m___4 = {NATURAL_32} 0 then
				a_result.item.val := 1.0
				a_result.item.err := 0.0
				Result := 0
				js.return
			elseif a_n <= {NATURAL_32} 170 then
				a_result.item.val := (fact_table[a_n.to_integer_32].f / fact_table[l_m___4.to_integer_32].f) / fact_table[(a_n - l_m___4).to_integer_32].f
				a_result.item.err := (6.0 * 2.2204460492503131e-16) * (fabs (a_result.item.val))
				Result := 0
				js.return
			else
				if l_m___4 * {NATURAL_32} 2 < a_n then
					l_m___4 := a_n - l_m___4
				end
				if a_n - l_m___4 < {NATURAL_32} 64 then
					l_prod := 1.0
					l_k := a_n
					from
					until
						not (not js.loop_done and then (l_k >= l_m___4 + {NATURAL_32} 1))
					loop
						l_tk := l_k.to_real_64 / (l_k - l_m___4).to_real_64
						if l_tk > 1.7976931348623157e+308 / l_prod then
							from
							until
								js.loop_done
							loop
								a_result.item.val := (builtin_inff).to_double
								a_result.item.err := (builtin_inff).to_double
								from
								until
									js.loop_done
								loop
									error_inst.gsl_error (ce_string ("overflow"), ce_string ("gamma.c"), 1_609, 16)
									Result := 16
									js.return
									if not js.return_called then
										js.break
									end
								end
								if not js.return_called then
									js.break
								end
							end
						end
						if not js.return_called then
							l_prod := l_prod * l_tk
							l_k := l_k - 1
						end
					end
					if not js.return_called then
						a_result.item.val := l_prod
						a_result.item.err := ((2.0 * 2.2204460492503131e-16) * l_prod) * (fabs ((a_n - l_m___4).to_real_64))
						Result := 0
						js.return
					end
				else
					l_stat_lc := (gsl_sf_lnchoose_e (a_n, l_m___4, l_lc))
					l_stat_e := (exp_inst.gsl_sf_exp_err_e (l_lc.item.val, l_lc.item.err, a_result))
					if l_stat_lc /= 0 then
						l_tmp___8 := l_stat_lc
					else
						if l_stat_e /= 0 then
							l_tmp___7 := l_stat_e
						else
							l_tmp___7 := 0
						end
						l_tmp___8 := l_tmp___7
					end
					Result := l_tmp___8
					js.return
				end
			end
			if not js.return_called then
				Result := 0
			end
		end

	gsl_sf_doublefact (a_n: NATURAL_32): REAL_64
		local
			l_result: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_status: INTEGER_32
			js: JUMP_STATE
		do
			create l_result.make (<<1>>)

			l_status := (gsl_sf_doublefact_e (a_n, l_result))
			if l_status /= 0 then
				from
				until
					js.loop_done
				loop
					error_inst.gsl_error (ce_string ("gsl_sf_doublefact_e(n, &result)"), ce_string ("gamma.c"), 1_644, l_status)
					Result := (l_result.item.val)
					js.return
					if not js.return_called then
						js.break
					end
				end
			end
			if not js.return_called then
				Result := (l_result.item.val)
			end
		end

	gsl_sf_doublefact_e (a_n: NATURAL_32; a_result: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]): INTEGER_32
		local
			js: JUMP_STATE
		do
			if a_n < {NATURAL_32} 26 then
				a_result.item.val := doub_fact_table[a_n.to_integer_32].f
				a_result.item.err := 0.0
				Result := 0
				js.return
			elseif a_n <= {NATURAL_32} 297 then
				a_result.item.val := doub_fact_table[a_n.to_integer_32].f
				a_result.item.err := (2.0 * 2.2204460492503131e-16) * (fabs (a_result.item.val))
				Result := 0
				js.return
			else
				from
				until
					js.loop_done
				loop
					a_result.item.val := (builtin_inff).to_double
					a_result.item.err := (builtin_inff).to_double
					from
					until
						js.loop_done
					loop
						error_inst.gsl_error (ce_string ("overflow"), ce_string ("gamma.c"), 1_509, 16)
						Result := 16
						js.return
						if not js.return_called then
							js.break
						end
					end
					if not js.return_called then
						js.break
					end
				end
			end
			if not js.return_called then
				Result := 0
			end
		end

	gsl_sf_fact (a_n: NATURAL_32): REAL_64
		local
			l_result: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_status: INTEGER_32
			js: JUMP_STATE
		do
			create l_result.make (<<1>>)

			l_status := (gsl_sf_fact_e (a_n, l_result))
			if l_status /= 0 then
				from
				until
					js.loop_done
				loop
					error_inst.gsl_error (ce_string ("gsl_sf_fact_e(n, &result)"), ce_string ("gamma.c"), 1_634, l_status)
					Result := (l_result.item.val)
					js.return
					if not js.return_called then
						js.break
					end
				end
			end
			if not js.return_called then
				Result := (l_result.item.val)
			end
		end

	gsl_sf_fact_e (a_n: NATURAL_32; a_result: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]): INTEGER_32
		local
			js: JUMP_STATE
		do
			if a_n < {NATURAL_32} 18 then
				a_result.item.val := fact_table[a_n.to_integer_32].f
				a_result.item.err := 0.0
				Result := 0
				js.return
			elseif a_n <= {NATURAL_32} 170 then
				a_result.item.val := fact_table[a_n.to_integer_32].f
				a_result.item.err := (2.0 * 2.2204460492503131e-16) * (fabs (a_result.item.val))
				Result := 0
				js.return
			else
				from
				until
					js.loop_done
				loop
					a_result.item.val := (builtin_inff).to_double
					a_result.item.err := (builtin_inff).to_double
					from
					until
						js.loop_done
					loop
						error_inst.gsl_error (ce_string ("overflow"), ce_string ("gamma.c"), 1_489, 16)
						Result := 16
						js.return
						if not js.return_called then
							js.break
						end
					end
					if not js.return_called then
						js.break
					end
				end
			end
			if not js.return_called then
				Result := 0
			end
		end

	gsl_sf_gamma (a_x: REAL_64): REAL_64
		local
			l_result: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_status: INTEGER_32
			js: JUMP_STATE
		do
			create l_result.make (<<1>>)

			l_status := (gsl_sf_gamma_e (a_x, l_result))
			if l_status /= 0 then
				from
				until
					js.loop_done
				loop
					error_inst.gsl_error (ce_string ("gsl_sf_gamma_e(x, &result)"), ce_string ("gamma.c"), 1_659, l_status)
					Result := (l_result.item.val)
					js.return
					if not js.return_called then
						js.break
					end
				end
			end
			if not js.return_called then
				Result := (l_result.item.val)
			end
		end

	gsl_sf_gamma_e (a_x: REAL_64; a_result: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]): INTEGER_32
		local
			l_rint_x: INTEGER_32
			l_f_x: REAL_64
			l_sgn_gamma: REAL_64
			l_tmp___0: REAL_64
			l_sin_term: REAL_64
			l_g: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_tmp___4: REAL_64
			l_lng: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_sgn: CE_ARRAY [REAL_64]
			l_stat_lng: INTEGER_32
			l_stat_e: INTEGER_32
			l_tmp___10: INTEGER_32
			l_tmp___11: INTEGER_32
			js: JUMP_STATE
		do
			create l_g.make (<<1>>)
			create l_lng.make (<<1>>)
			create l_sgn.make (<<1>>)

			if a_x < 0.5 then
				l_rint_x := (floor (a_x + 0.5)).truncated_to_integer
				l_f_x := a_x - l_rint_x.to_double
				if not ((l_rint_x).bit_and (1)).to_boolean then
					l_tmp___0 := 1.0
				else
					l_tmp___0 := -1.0
				end
				l_sgn_gamma := l_tmp___0
				l_sin_term := (l_sgn_gamma * (sin (3.14159265358979323846 * l_f_x))) / 3.14159265358979323846
				if l_sin_term = 0.0 then
					from
					until
						js.loop_done
					loop
						a_result.item.val := (builtin_nanf (ce_string (""))).to_double
						a_result.item.err := (builtin_nanf (ce_string (""))).to_double
						from
						until
							js.loop_done
						loop
							error_inst.gsl_error (ce_string ("domain error"), ce_string ("gamma.c"), 1_264, 1)
							Result := 1
							js.return
							if not js.return_called then
								js.break
							end
						end
						if not js.return_called then
							js.break
						end
					end
				elseif a_x > -169.0 then
					gamma_xgthalf (1.0 - a_x, l_g).do_nothing
					if ((fabs (l_sin_term)) * l_g.item.val) * 2.2250738585072014e-308 < 1.0 then
						a_result.item.val := 1.0 / (l_sin_term * l_g.item.val)
						l_tmp___4 := fabs (l_g.item.err / l_g.item.val)
						a_result.item.err := l_tmp___4 * (fabs (a_result.item.val))
						a_result.item.err := a_result.item.err + ((2.0 * 2.2204460492503131e-16) * (fabs (a_result.item.val)))
						Result := 0
						js.return
					else
						from
						until
							js.loop_done
						loop
							a_result.item.val := 0.0
							a_result.item.err := 2.2250738585072014e-308
							from
							until
								js.loop_done
							loop
								error_inst.gsl_error (ce_string ("underflow"), ce_string ("gamma.c"), 1_276, 15)
								Result := 15
								js.return
								if not js.return_called then
									js.break
								end
							end
							if not js.return_called then
								js.break
							end
						end
					end
				else
					l_stat_lng := (gsl_sf_lngamma_sgn_e (a_x, l_lng, l_sgn))
					l_stat_e := (exp_inst.gsl_sf_exp_mult_err_e (l_lng.item.val, l_lng.item.err, l_sgn.item, 0.0, a_result))
					if l_stat_e /= 0 then
						l_tmp___11 := l_stat_e
					else
						if l_stat_lng /= 0 then
							l_tmp___10 := l_stat_lng
						else
							l_tmp___10 := 0
						end
						l_tmp___11 := l_tmp___10
					end
					Result := l_tmp___11
					js.return
				end
			else
				Result := (gamma_xgthalf (a_x, a_result))
				js.return
			end
			if not js.return_called then
				Result := 0
			end
		end

	gsl_sf_gammainv (a_x: REAL_64): REAL_64
		local
			l_result: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_status: INTEGER_32
			js: JUMP_STATE
		do
			create l_result.make (<<1>>)

			l_status := (gsl_sf_gammainv_e (a_x, l_result))
			if l_status /= 0 then
				from
				until
					js.loop_done
				loop
					error_inst.gsl_error (ce_string ("gsl_sf_gammainv_e(x, &result)"), ce_string ("gamma.c"), 1_669, l_status)
					Result := (l_result.item.val)
					js.return
					if not js.return_called then
						js.break
					end
				end
			end
			if not js.return_called then
				Result := (l_result.item.val)
			end
		end

	gsl_sf_gammainv_e (a_x: REAL_64; a_result: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]): INTEGER_32
		local
			l_lng: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_sgn: CE_ARRAY [REAL_64]
			l_stat_lng: INTEGER_32
			l_g: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_stat_g: INTEGER_32
			l_tmp___2: REAL_64
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			create l_lng.make (<<1>>)
			create l_sgn.make (<<1>>)
			create l_g.make (<<1>>)

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					if not (a_x <= 0.0) then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						if a_x = (floor (a_x)) then
							a_result.item.val := 0.0
							a_result.item.err := 0.0
							Result := 0
							js.return
						else
							l_pos := -1 + 3 -- goto _L
						end
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 4
				when 3 then -- else: | _L:
						if a_x < 0.5 then
							l_stat_lng := (gsl_sf_lngamma_sgn_e (a_x, l_lng, l_sgn))
							if l_stat_lng = 1 then
								a_result.item.val := 0.0
								a_result.item.err := 0.0
								Result := 0
								js.return
							elseif l_stat_lng /= 0 then
								a_result.item.val := 0.0
								a_result.item.err := 0.0
								Result := l_stat_lng
								js.return
							else
								Result := (exp_inst.gsl_sf_exp_mult_err_e (-l_lng.item.val, l_lng.item.err, l_sgn.item, 0.0, a_result))
								js.return
							end
						else
							l_stat_g := (gamma_xgthalf (a_x, l_g))
							if l_stat_g = 16 then
								from
								until
									js.loop_done
								loop
									a_result.item.val := 0.0
									a_result.item.err := 2.2250738585072014e-308
									from
									until
										js.loop_done
									loop
										error_inst.gsl_error (ce_string ("underflow"), ce_string ("gamma.c"), 1_379, 15)
										Result := 15
										js.return
										if not js.return_called then
											js.break
										end
									end
									if not js.return_called then
										js.break
									end
								end
							else
								a_result.item.val := 1.0 / l_g.item.val
								l_tmp___2 := fabs (l_g.item.err / l_g.item.val)
								a_result.item.err := l_tmp___2 * (fabs (a_result.item.val))
								a_result.item.err := a_result.item.err + ((2.0 * 2.2204460492503131e-16) * (fabs (a_result.item.val)))
								if (fabs (a_result.item.val)) < 2.2250738585072014e-308 then
									from
									until
										js.loop_done
									loop
										error_inst.gsl_error (ce_string ("underflow"), ce_string ("gamma.c"), 1_385, 15)
										Result := 15
										js.return
										if not js.return_called then
											js.break
										end
									end
								end
								if not js.return_called then
									Result := 0
									js.return
								end
							end
						end
				when 4 then
					if not js.return_called then
						Result := 0
					end
				when 5 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	gsl_sf_gammastar (a_x: REAL_64): REAL_64
		local
			l_result: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_status: INTEGER_32
			js: JUMP_STATE
		do
			create l_result.make (<<1>>)

			l_status := (gsl_sf_gammastar_e (a_x, l_result))
			if l_status /= 0 then
				from
				until
					js.loop_done
				loop
					error_inst.gsl_error (ce_string ("gsl_sf_gammastar_e(x, &result)"), ce_string ("gamma.c"), 1_664, l_status)
					Result := (l_result.item.val)
					js.return
					if not js.return_called then
						js.break
					end
				end
			end
			if not js.return_called then
				Result := (l_result.item.val)
			end
		end

	gsl_sf_gammastar_e (a_x: REAL_64; a_result: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]): INTEGER_32
		local
			l_lg: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_stat_lg: INTEGER_32
			l_lx: REAL_64
			l_c___1: REAL_64
			l_lnr_val: REAL_64
			l_lnr_err: REAL_64
			l_stat_e: INTEGER_32
			l_tmp___5: INTEGER_32
			l_tmp___6: INTEGER_32
			l_t: REAL_64
			l_t___0: REAL_64
			l_c___2: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_xi: REAL_64
			js: JUMP_STATE
		do
			create l_lg.make (<<1>>)
			create l_c___2.make (<<1>>)

			if a_x <= 0.0 then
				from
				until
					js.loop_done
				loop
					a_result.item.val := (builtin_nanf (ce_string (""))).to_double
					a_result.item.err := (builtin_nanf (ce_string (""))).to_double
					from
					until
						js.loop_done
					loop
						error_inst.gsl_error (ce_string ("domain error"), ce_string ("gamma.c"), 1_304, 1)
						Result := 1
						js.return
						if not js.return_called then
							js.break
						end
					end
					if not js.return_called then
						js.break
					end
				end
			elseif a_x < 0.5 then
				l_stat_lg := (gsl_sf_lngamma_e (a_x, l_lg))
				l_lx := (log (a_x))
				l_c___1 := 0.5 * (0.69314718055994530942 + 1.14472988584940017414342735135)
				l_lnr_val := ((l_lg.item.val - (a_x - 0.5) * l_lx) + a_x) - l_c___1
				l_lnr_err := l_lg.item.err + (2.0 * 2.2204460492503131e-16) * ((a_x + 0.5) * (fabs (l_lx)) + l_c___1)
				l_stat_e := (exp_inst.gsl_sf_exp_err_e (l_lnr_val, l_lnr_err, a_result))
				if l_stat_lg /= 0 then
					l_tmp___6 := l_stat_lg
				else
					if l_stat_e /= 0 then
						l_tmp___5 := l_stat_e
					else
						l_tmp___5 := 0
					end
					l_tmp___6 := l_tmp___5
				end
				Result := l_tmp___6
				js.return
			elseif a_x < 2.0 then
				l_t := (4.0 / 3.0) * (a_x - 0.5) - 1.0
				Result := (cheb_eval_inst.cheb_eval_e (gstar_a_cs, l_t, a_result))
				js.return
			elseif a_x < 10.0 then
				l_t___0 := 0.25 * (a_x - 2.0) - 1.0
				cheb_eval_inst.cheb_eval_e (gstar_b_cs, l_t___0, l_c___2).do_nothing
				a_result.item.val := (l_c___2.item.val / (a_x * a_x) + 1.0) + 1.0 / (12.0 * a_x)
				a_result.item.err := l_c___2.item.err / (a_x * a_x)
				a_result.item.err := a_result.item.err + ((2.0 * 2.2204460492503131e-16) * (fabs (a_result.item.val)))
				Result := 0
				js.return
			elseif a_x < 1.0 / 1.2207031250000000e-04 then
				Result := (gammastar_ser (a_x, a_result))
				js.return
			elseif a_x < 1.0 / 2.2204460492503131e-16 then
				l_xi := 1.0 / a_x
				a_result.item.val := 1.0 + (l_xi / 12.0) * (1.0 + (l_xi / 24.0) * (1.0 - l_xi * (139.0 / 180.0 + (571.0 / 8640.0) * l_xi)))
				a_result.item.err := (2.0 * 2.2204460492503131e-16) * (fabs (a_result.item.val))
				Result := 0
				js.return
			else
				a_result.item.val := 1.0
				a_result.item.err := 1.0 / a_x
				Result := 0
				js.return
			end
			if not js.return_called then
				Result := 0
			end
		end

	gsl_sf_lnchoose (a_n: NATURAL_32; a_m___4: NATURAL_32): REAL_64
		local
			l_result: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_status: INTEGER_32
			js: JUMP_STATE
		do
			create l_result.make (<<1>>)

			l_status := (gsl_sf_lnchoose_e (a_n, a_m___4, l_result))
			if l_status /= 0 then
				from
				until
					js.loop_done
				loop
					error_inst.gsl_error (ce_string ("gsl_sf_lnchoose_e(n, m, &result)"), ce_string ("gamma.c"), 1_684, l_status)
					Result := (l_result.item.val)
					js.return
					if not js.return_called then
						js.break
					end
				end
			end
			if not js.return_called then
				Result := (l_result.item.val)
			end
		end

	gsl_sf_lnchoose_e (a_n: NATURAL_32; a_m___4: NATURAL_32; a_result: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]): INTEGER_32
		local
			l_m___4: NATURAL_32
			l_nf: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_mf: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_nmmf: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			js: JUMP_STATE
		do
			create l_nf.make (<<1>>)
			create l_mf.make (<<1>>)
			create l_nmmf.make (<<1>>)

			l_m___4 := a_m___4
			if l_m___4 > a_n then
				from
				until
					js.loop_done
				loop
					a_result.item.val := (builtin_nanf (ce_string (""))).to_double
					a_result.item.err := (builtin_nanf (ce_string (""))).to_double
					from
					until
						js.loop_done
					loop
						error_inst.gsl_error (ce_string ("domain error"), ce_string ("gamma.c"), 1_561, 1)
						Result := 1
						js.return
						if not js.return_called then
							js.break
						end
					end
					if not js.return_called then
						js.break
					end
				end
			elseif l_m___4 = a_n then
				a_result.item.val := 0.0
				a_result.item.err := 0.0
				Result := 0
				js.return
			elseif l_m___4 = {NATURAL_32} 0 then
				a_result.item.val := 0.0
				a_result.item.err := 0.0
				Result := 0
				js.return
			else
				if l_m___4 * {NATURAL_32} 2 > a_n then
					l_m___4 := a_n - l_m___4
				end
				gsl_sf_lnfact_e (a_n, l_nf).do_nothing
				gsl_sf_lnfact_e (l_m___4, l_mf).do_nothing
				gsl_sf_lnfact_e (a_n - l_m___4, l_nmmf).do_nothing
				a_result.item.val := (l_nf.item.val - l_mf.item.val) - l_nmmf.item.val
				a_result.item.err := (l_nf.item.err + l_mf.item.err) + l_nmmf.item.err
				a_result.item.err := a_result.item.err + ((2.0 * 2.2204460492503131e-16) * (fabs (a_result.item.val)))
				Result := 0
				js.return
			end
			if not js.return_called then
				Result := 0
			end
		end

	gsl_sf_lndoublefact (a_n: NATURAL_32): REAL_64
		local
			l_result: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_status: INTEGER_32
			js: JUMP_STATE
		do
			create l_result.make (<<1>>)

			l_status := (gsl_sf_lndoublefact_e (a_n, l_result))
			if l_status /= 0 then
				from
				until
					js.loop_done
				loop
					error_inst.gsl_error (ce_string ("gsl_sf_lndoublefact_e(n, &result)"), ce_string ("gamma.c"), 1_649, l_status)
					Result := (l_result.item.val)
					js.return
					if not js.return_called then
						js.break
					end
				end
			end
			if not js.return_called then
				Result := (l_result.item.val)
			end
		end

	gsl_sf_lndoublefact_e (a_n: NATURAL_32; a_result: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]): INTEGER_32
		local
			l_lg: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_lg___0: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
		do
			create l_lg.make (<<1>>)
			create l_lg___0.make (<<1>>)

			if a_n <= {NATURAL_32} 297 then
				a_result.item.val := log (doub_fact_table[a_n.to_integer_32].f)
				a_result.item.err := (2.0 * 2.2204460492503131e-16) * (fabs (a_result.item.val))
				Result := 0
			elseif ((a_n).bit_and ({NATURAL_32} 1)).to_boolean then
				gsl_sf_lngamma_e (0.5 * (a_n.to_real_64 + 2.0), l_lg).do_nothing
				a_result.item.val := ((0.5 * (a_n.to_real_64 + 1.0)) * 0.69314718055994530942 - 0.5 * 1.14472988584940017414342735135) + l_lg.item.val
				a_result.item.err := (2.0 * 2.2204460492503131e-16) * (fabs (a_result.item.val)) + l_lg.item.err
				Result := 0
			else
				gsl_sf_lngamma_e (0.5 * a_n.to_real_64 + 1.0, l_lg___0).do_nothing
				a_result.item.val := (0.5 * a_n.to_real_64) * 0.69314718055994530942 + l_lg___0.item.val
				a_result.item.err := (2.0 * 2.2204460492503131e-16) * (fabs (a_result.item.val)) + l_lg___0.item.err
				Result := 0
			end
		end

	gsl_sf_lnfact (a_n: NATURAL_32): REAL_64
		local
			l_result: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_status: INTEGER_32
			js: JUMP_STATE
		do
			create l_result.make (<<1>>)

			l_status := (gsl_sf_lnfact_e (a_n, l_result))
			if l_status /= 0 then
				from
				until
					js.loop_done
				loop
					error_inst.gsl_error (ce_string ("gsl_sf_lnfact_e(n, &result)"), ce_string ("gamma.c"), 1_639, l_status)
					Result := (l_result.item.val)
					js.return
					if not js.return_called then
						js.break
					end
				end
			end
			if not js.return_called then
				Result := (l_result.item.val)
			end
		end

	gsl_sf_lnfact_e (a_n: NATURAL_32; a_result: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]): INTEGER_32
		do
			if a_n <= {NATURAL_32} 170 then
				a_result.item.val := log (fact_table[a_n.to_integer_32].f)
				a_result.item.err := (2.0 * 2.2204460492503131e-16) * (fabs (a_result.item.val))
				Result := 0
			else
				gsl_sf_lngamma_e (a_n.to_real_64 + 1.0, a_result).do_nothing
				Result := 0
			end
		end

	gsl_sf_lngamma (a_x: REAL_64): REAL_64
		local
			l_result: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_status: INTEGER_32
			js: JUMP_STATE
		do
			create l_result.make (<<1>>)

			l_status := (gsl_sf_lngamma_e (a_x, l_result))
			if l_status /= 0 then
				from
				until
					js.loop_done
				loop
					error_inst.gsl_error (ce_string ("gsl_sf_lngamma_e(x, &result)"), ce_string ("gamma.c"), 1_654, l_status)
					Result := (l_result.item.val)
					js.return
					if not js.return_called then
						js.break
					end
				end
			end
			if not js.return_called then
				Result := (l_result.item.val)
			end
		end

	gsl_sf_lngamma_complex_e (a_zr: REAL_64; a_zi: REAL_64; a_lnr: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]; a_arg: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]): INTEGER_32
		local
			l_x: REAL_64
			l_y: REAL_64
			l_a___5: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_b___0: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_lnsin_r: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_lnsin_i: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_stat_l: INTEGER_32
			l_stat_s: INTEGER_32
			l_stat_r: INTEGER_32
			l_tmp___3: INTEGER_32
			l_tmp___4: INTEGER_32
			js: JUMP_STATE
		do
			create l_a___5.make (<<1>>)
			create l_b___0.make (<<1>>)
			create l_lnsin_r.make (<<1>>)
			create l_lnsin_i.make (<<1>>)

			if a_zr <= 0.5 then
				l_x := 1.0 - a_zr
				l_y := -a_zi
				l_stat_l := (lngamma_lanczos_complex (l_x, l_y, l_a___5, l_b___0))
				l_stat_s := (trig_inst.gsl_sf_complex_logsin_e (3.14159265358979323846 * a_zr, 3.14159265358979323846 * a_zi, l_lnsin_r, l_lnsin_i))
				if l_stat_s = 0 then
					a_lnr.item.val := (1.14472988584940017414342735135 - l_lnsin_r.item.val) - l_a___5.item.val
					a_lnr.item.err := (l_lnsin_r.item.err + l_a___5.item.err) + (2.0 * 2.2204460492503131e-16) * (fabs (a_lnr.item.val))
					a_arg.item.val := -l_lnsin_i.item.val - l_b___0.item.val
					a_arg.item.err := (l_lnsin_i.item.err + l_b___0.item.err) + (2.0 * 2.2204460492503131e-16) * (fabs (a_arg.item.val))
					l_stat_r := trig_inst.gsl_sf_angle_restrict_symm_e (create {CE_POINTER [REAL_64]}.make_shared (a_arg.c_pointer + (create {PD_GSL_SF_RESULT_STRUCT}.make).offset_of ("val")))
					if l_stat_r /= 0 then
						l_tmp___4 := l_stat_r
					else
						if l_stat_l /= 0 then
							l_tmp___3 := l_stat_l
						else
							l_tmp___3 := 0
						end
						l_tmp___4 := l_tmp___3
					end
					Result := l_tmp___4
					js.return
				else
					from
					until
						js.loop_done
					loop
						a_lnr.item.val := (builtin_nanf (ce_string (""))).to_double
						a_lnr.item.err := (builtin_nanf (ce_string (""))).to_double
						a_arg.item.val := (builtin_nanf (ce_string (""))).to_double
						a_arg.item.err := (builtin_nanf (ce_string (""))).to_double
						from
						until
							js.loop_done
						loop
							error_inst.gsl_error (ce_string ("domain error"), ce_string ("gamma.c"), 1_417, 1)
							Result := 1
							js.return
							if not js.return_called then
								js.break
							end
						end
						if not js.return_called then
							js.break
						end
					end
				end
			else
				Result := (lngamma_lanczos_complex (a_zr, a_zi, a_lnr, a_arg))
				js.return
			end
			if not js.return_called then
				Result := 0
			end
		end

	gsl_sf_lngamma_e (a_x: REAL_64; a_result: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]): INTEGER_32
		local
			l_stat: INTEGER_32
			l_stat___0: INTEGER_32
			l_sgn: CE_ARRAY [REAL_64]
			l_z: REAL_64
			l_s: REAL_64
			l_as: REAL_64
			l_N: INTEGER_32
			l_eps: REAL_64
			l_sgn___0: CE_ARRAY [REAL_64]
			l_lg_z: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			js: JUMP_STATE
		do
			create l_sgn.make (<<1>>)
			create l_sgn___0.make (<<1>>)
			create l_lg_z.make (<<1>>)

			if (fabs (a_x - 1.0)) < 0.01 then
				l_stat := (lngamma_1_pade (a_x - 1.0, a_result))
				a_result.item.err := a_result.item.err * (1.0 / (2.2204460492503131e-16 + (fabs (a_x - 1.0))))
				Result := l_stat
				js.return
			elseif (fabs (a_x - 2.0)) < 0.01 then
				l_stat___0 := (lngamma_2_pade (a_x - 2.0, a_result))
				a_result.item.err := a_result.item.err * (1.0 / (2.2204460492503131e-16 + (fabs (a_x - 2.0))))
				Result := l_stat___0
				js.return
			elseif a_x >= 0.5 then
				Result := (lngamma_lanczos (a_x, a_result))
				js.return
			elseif a_x = 0.0 then
				from
				until
					js.loop_done
				loop
					a_result.item.val := (builtin_nanf (ce_string (""))).to_double
					a_result.item.err := (builtin_nanf (ce_string (""))).to_double
					from
					until
						js.loop_done
					loop
						error_inst.gsl_error (ce_string ("domain error"), ce_string ("gamma.c"), 1_138, 1)
						Result := 1
						js.return
						if not js.return_called then
							js.break
						end
					end
					if not js.return_called then
						js.break
					end
				end
			elseif (fabs (a_x)) < 0.02 then
				Result := (lngamma_sgn_0 (a_x, a_result, l_sgn))
				js.return
			elseif a_x > -0.5 / (2.2204460492503131e-16 * 3.14159265358979323846) then
				l_z := 1.0 - a_x
				l_s := (sin (3.14159265358979323846 * l_z))
				l_as := (fabs (l_s))
				if l_s = 0.0 then
					from
					until
						js.loop_done
					loop
						a_result.item.val := (builtin_nanf (ce_string (""))).to_double
						a_result.item.err := (builtin_nanf (ce_string (""))).to_double
						from
						until
							js.loop_done
						loop
							error_inst.gsl_error (ce_string ("domain error"), ce_string ("gamma.c"), 1_152, 1)
							Result := 1
							js.return
							if not js.return_called then
								js.break
							end
						end
						if not js.return_called then
							js.break
						end
					end
				elseif l_as < 3.14159265358979323846 * 0.015 then
					if a_x < (-0x7fffffff - 1).to_double + 2.0 then
						a_result.item.val := 0.0
						a_result.item.err := 0.0
						from
						until
							js.loop_done
						loop
							error_inst.gsl_error (ce_string ("error"), ce_string ("gamma.c"), 1_159, 18)
							Result := 18
							js.return
							if not js.return_called then
								js.break
							end
						end
					else
						l_N := -(a_x - 0.5).truncated_to_integer
						l_eps := a_x + l_N.to_double
						Result := (lngamma_sgn_sing (l_N, l_eps, a_result, l_sgn___0))
						js.return
					end
				else
					lngamma_lanczos (l_z, l_lg_z).do_nothing
					a_result.item.val := 1.14472988584940017414342735135 - ((log (l_as)) + l_lg_z.item.val)
					a_result.item.err := (2.0 * 2.2204460492503131e-16) * (fabs (a_result.item.val)) + l_lg_z.item.err
					Result := 0
					js.return
				end
			else
				a_result.item.val := 0.0
				a_result.item.err := 0.0
				from
				until
					js.loop_done
				loop
					error_inst.gsl_error (ce_string ("error"), ce_string ("gamma.c"), 1_180, 18)
					Result := 18
					js.return
					if not js.return_called then
						js.break
					end
				end
			end
			if not js.return_called then
				Result := 0
			end
		end

	gsl_sf_lngamma_sgn_e (a_x: REAL_64; a_result_lg: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]; a_sgn: CE_POINTER [REAL_64]): INTEGER_32
		local
			l_stat: INTEGER_32
			l_stat___0: INTEGER_32
			l_z: REAL_64
			l_s: REAL_64
			l_as: REAL_64
			l_N: INTEGER_32
			l_eps: REAL_64
			l_lg_z: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			js: JUMP_STATE
		do
			create l_lg_z.make (<<1>>)

			if (fabs (a_x - 1.0)) < 0.01 then
				l_stat := (lngamma_1_pade (a_x - 1.0, a_result_lg))
				a_result_lg.item.err := a_result_lg.item.err * (1.0 / (2.2204460492503131e-16 + (fabs (a_x - 1.0))))
				a_sgn.item := 1.0
				Result := l_stat
				js.return
			elseif (fabs (a_x - 2.0)) < 0.01 then
				l_stat___0 := (lngamma_2_pade (a_x - 2.0, a_result_lg))
				a_result_lg.item.err := a_result_lg.item.err * (1.0 / (2.2204460492503131e-16 + (fabs (a_x - 2.0))))
				a_sgn.item := 1.0
				Result := l_stat___0
				js.return
			elseif a_x >= 0.5 then
				a_sgn.item := 1.0
				Result := (lngamma_lanczos (a_x, a_result_lg))
				js.return
			elseif a_x = 0.0 then
				a_sgn.item := 0.0
				from
				until
					js.loop_done
				loop
					a_result_lg.item.val := (builtin_nanf (ce_string (""))).to_double
					a_result_lg.item.err := (builtin_nanf (ce_string (""))).to_double
					from
					until
						js.loop_done
					loop
						error_inst.gsl_error (ce_string ("domain error"), ce_string ("gamma.c"), 1_205, 1)
						Result := 1
						js.return
						if not js.return_called then
							js.break
						end
					end
					if not js.return_called then
						js.break
					end
				end
			elseif (fabs (a_x)) < 0.02 then
				Result := (lngamma_sgn_0 (a_x, a_result_lg, a_sgn))
				js.return
			elseif a_x > -0.5 / (2.2204460492503131e-16 * 3.14159265358979323846) then
				l_z := 1.0 - a_x
				l_s := (sin (3.14159265358979323846 * a_x))
				l_as := (fabs (l_s))
				if l_s = 0.0 then
					a_sgn.item := 0.0
					from
					until
						js.loop_done
					loop
						a_result_lg.item.val := (builtin_nanf (ce_string (""))).to_double
						a_result_lg.item.err := (builtin_nanf (ce_string (""))).to_double
						from
						until
							js.loop_done
						loop
							error_inst.gsl_error (ce_string ("domain error"), ce_string ("gamma.c"), 1_219, 1)
							Result := 1
							js.return
							if not js.return_called then
								js.break
							end
						end
						if not js.return_called then
							js.break
						end
					end
				elseif l_as < 3.14159265358979323846 * 0.015 then
					if a_x < (-0x7fffffff - 1).to_double + 2.0 then
						a_result_lg.item.val := 0.0
						a_result_lg.item.err := 0.0
						a_sgn.item := 0.0
						from
						until
							js.loop_done
						loop
							error_inst.gsl_error (ce_string ("error"), ce_string ("gamma.c"), 1_227, 18)
							Result := 18
							js.return
							if not js.return_called then
								js.break
							end
						end
					else
						l_N := -(a_x - 0.5).truncated_to_integer
						l_eps := a_x + l_N.to_double
						Result := (lngamma_sgn_sing (l_N, l_eps, a_result_lg, a_sgn))
						js.return
					end
				else
					lngamma_lanczos (l_z, l_lg_z).do_nothing
					if l_s > 0.0 then
						a_sgn.item := 1.0
					else
						a_sgn.item := -1.0
					end
					a_result_lg.item.val := 1.14472988584940017414342735135 - ((log (l_as)) + l_lg_z.item.val)
					a_result_lg.item.err := (2.0 * 2.2204460492503131e-16) * (fabs (a_result_lg.item.val)) + l_lg_z.item.err
					Result := 0
					js.return
				end
			else
				a_result_lg.item.val := 0.0
				a_result_lg.item.err := 0.0
				a_sgn.item := 0.0
				from
				until
					js.loop_done
				loop
					error_inst.gsl_error (ce_string ("x too large to extract fraction part"), ce_string ("gamma.c"), 1_249, 18)
					Result := 18
					js.return
					if not js.return_called then
						js.break
					end
				end
			end
			if not js.return_called then
				Result := 0
			end
		end

	gsl_sf_taylorcoeff (a_n: INTEGER_32; a_x: REAL_64): REAL_64
		local
			l_result: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_status: INTEGER_32
			js: JUMP_STATE
		do
			create l_result.make (<<1>>)

			l_status := (gsl_sf_taylorcoeff_e (a_n, a_x, l_result))
			if l_status /= 0 then
				from
				until
					js.loop_done
				loop
					error_inst.gsl_error (ce_string ("gsl_sf_taylorcoeff_e(n, x, &result)"), ce_string ("gamma.c"), 1_674, l_status)
					Result := (l_result.item.val)
					js.return
					if not js.return_called then
						js.break
					end
				end
			end
			if not js.return_called then
				Result := (l_result.item.val)
			end
		end

	gsl_sf_taylorcoeff_e (a_n: INTEGER_32; a_x: REAL_64; a_result: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]): INTEGER_32
		local
			l_log2pi: REAL_64
			l_ln_test: REAL_64
			l_tmp___1: REAL_64
			l_product: REAL_64
			l_k: INTEGER_32
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					if not (a_x < 0.0) then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						l_pos := -1 + 4 -- goto _L
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 7
				when 3 then -- else:
						if not (a_n < 0) then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if: | _L:
						from
						until
							js.loop_done
						loop
							a_result.item.val := (builtin_nanf (ce_string (""))).to_double
							a_result.item.err := (builtin_nanf (ce_string (""))).to_double
							from
							until
								js.loop_done
							loop
								error_inst.gsl_error (ce_string ("domain error"), ce_string ("gamma.c"), 1_432, 1)
								Result := 1
								js.return
								if not js.return_called then
									js.break
								end
							end
							if not js.return_called then
								js.break
							end
						end
				when 5 then -- if block done, skip else block:
						l_pos := -1 + 7
				when 6 then -- else:
						if a_n = 0 then
							a_result.item.val := 1.0
							a_result.item.err := 0.0
							Result := 0
							js.return
						elseif a_n = 1 then
							a_result.item.val := a_x
							a_result.item.err := 0.0
							Result := 0
							js.return
						elseif a_x = 0.0 then
							a_result.item.val := 0.0
							a_result.item.err := 0.0
							Result := 0
							js.return
						else
							l_log2pi := 1.14472988584940017414342735135 + 0.69314718055994530942
							l_tmp___1 := log (a_x)
							l_ln_test := ((a_n.to_double * (l_tmp___1 + 1.0) + 1.0) - (a_n.to_double + 0.5) * (log (a_n.to_double + 1.0))) + 0.5 * l_log2pi
							if l_ln_test < -7.0839641853226408e+02 + 1.0 then
								from
								until
									js.loop_done
								loop
									a_result.item.val := 0.0
									a_result.item.err := 2.2250738585072014e-308
									from
									until
										js.loop_done
									loop
										error_inst.gsl_error (ce_string ("underflow"), ce_string ("gamma.c"), 1_454, 15)
										Result := 15
										js.return
										if not js.return_called then
											js.break
										end
									end
									if not js.return_called then
										js.break
									end
								end
							elseif l_ln_test > 7.0978271289338397e+02 - 1.0 then
								from
								until
									js.loop_done
								loop
									a_result.item.val := (builtin_inff).to_double
									a_result.item.err := (builtin_inff).to_double
									from
									until
										js.loop_done
									loop
										error_inst.gsl_error (ce_string ("overflow"), ce_string ("gamma.c"), 1_457, 16)
										Result := 16
										js.return
										if not js.return_called then
											js.break
										end
									end
									if not js.return_called then
										js.break
									end
								end
							else
								l_product := 1.0
								l_k := 1
								from
								until
									not (l_k <= a_n)
								loop
									l_product := l_product * (a_x / l_k.to_double)
									l_k := l_k + 1
								end
								a_result.item.val := l_product
								a_result.item.err := (a_n.to_double * 2.2204460492503131e-16) * l_product
								if (fabs (a_result.item.val)) < 2.2250738585072014e-308 then
									from
									until
										js.loop_done
									loop
										error_inst.gsl_error (ce_string ("underflow"), ce_string ("gamma.c"), 1_467, 15)
										Result := 15
										js.return
										if not js.return_called then
											js.break
										end
									end
								end
								if not js.return_called then
									Result := 0
									js.return
								end
							end
						end
				when 7 then -- if done
						if not js.return_called then
							Result := 0
						end
				when 8 then
						l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
			end

	lngamma_1_pade (a_eps: REAL_64; a_result: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]): INTEGER_32
		local
			l_n1: REAL_64
			l_n2: REAL_64
			l_d1: REAL_64
			l_d2: REAL_64
			l_num: REAL_64
			l_den: REAL_64
			l_pade: REAL_64
			l_c0___0: REAL_64
			l_c1___2: REAL_64
			l_c2___0: REAL_64
			l_c3___3: REAL_64
			l_c4___2: REAL_64
			l_eps5: REAL_64
			l_corr: REAL_64
		do
			l_n1 := -1.0017419282349508699871138440
			l_n2 := 1.7364839209922879823280541733
			l_d1 := 1.2433006018858751556055436011
			l_d2 := 5.0456274100274010152489597514
			l_num := (a_eps + l_n1) * (a_eps + l_n2)
			l_den := (a_eps + l_d1) * (a_eps + l_d2)
			l_pade := (2.0816265188662692474880210318 * l_num) / l_den
			l_c0___0 := 0.004785324257581753
			l_c1___2 := -0.01192457083645441
			l_c2___0 := 0.01931961413960498
			l_c3___3 := -0.02594027398725020
			l_c4___2 := 0.03141928755021455
			l_eps5 := (((a_eps * a_eps) * a_eps) * a_eps) * a_eps
			l_corr := l_eps5 * (l_c0___0 + a_eps * (l_c1___2 + a_eps * (l_c2___0 + a_eps * (l_c3___3 + l_c4___2 * a_eps))))
			a_result.item.val := a_eps * (l_pade + l_corr)
			a_result.item.err := (2.0 * 2.2204460492503131e-16) * (fabs (a_result.item.val))
			Result := 0
		end

	lngamma_2_pade (a_eps: REAL_64; a_result: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]): INTEGER_32
		local
			l_n1: REAL_64
			l_n2: REAL_64
			l_d1: REAL_64
			l_d2: REAL_64
			l_num: REAL_64
			l_den: REAL_64
			l_pade: REAL_64
			l_c0___0: REAL_64
			l_c1___2: REAL_64
			l_c2___0: REAL_64
			l_c3___3: REAL_64
			l_c4___2: REAL_64
			l_eps5: REAL_64
			l_corr: REAL_64
		do
			l_n1 := 1.000895834786669227164446568
			l_n2 := 4.209376735287755081642901277
			l_d1 := 2.618851904903217274682578255
			l_d2 := 10.85766559900983515322922936
			l_num := (a_eps + l_n1) * (a_eps + l_n2)
			l_den := (a_eps + l_d1) * (a_eps + l_d2)
			l_pade := (2.85337998765781918463568869 * l_num) / l_den
			l_c0___0 := 0.0001139406357036744
			l_c1___2 := -0.0001365435269792533
			l_c2___0 := 0.0001067287169183665
			l_c3___3 := -0.0000693271800931282
			l_c4___2 := 0.0000407220927867950
			l_eps5 := (((a_eps * a_eps) * a_eps) * a_eps) * a_eps
			l_corr := l_eps5 * (l_c0___0 + a_eps * (l_c1___2 + a_eps * (l_c2___0 + a_eps * (l_c3___3 + l_c4___2 * a_eps))))
			a_result.item.val := a_eps * (l_pade + l_corr)
			a_result.item.err := (2.0 * 2.2204460492503131e-16) * (fabs (a_result.item.val))
			Result := 0
		end

	lngamma_lanczos (a_x: REAL_64; a_result: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]): INTEGER_32
		local
			l_x: REAL_64
			l_k: INTEGER_32
			l_Ag: REAL_64
			l_term1: REAL_64
			l_term2: REAL_64
			l_tmp___1: REAL_64
		do
			l_x := a_x
			l_x := l_x - (1.0)
			l_Ag := lanczos_7_c[0]
			l_k := 1
			from
			until
				not (l_k <= 8)
			loop
				l_Ag := l_Ag + (lanczos_7_c[l_k] / (l_x + l_k.to_double))
				l_k := l_k + 1
			end
			l_term1 := (l_x + 0.5) * (log ((l_x + 7.5) / 2.7182818284590452354))
			l_term2 := 0.9189385332046727418 + (log (l_Ag))
			a_result.item.val := l_term1 + (l_term2 - 7.0)
			l_tmp___1 := fabs (l_term1)
			a_result.item.err := (2.0 * 2.2204460492503131e-16) * ((l_tmp___1 + (fabs (l_term2))) + 7.0)
			a_result.item.err := a_result.item.err + (2.2204460492503131e-16 * (fabs (a_result.item.val)))
			Result := 0
		end

	lngamma_lanczos_complex (a_zr: REAL_64; a_zi: REAL_64; a_yr: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]; a_yi: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]): INTEGER_32
		local
			l_zr: REAL_64
			l_k: INTEGER_32
			l_log1_r: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_log1_i: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_logAg_r: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_logAg_i: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_Ag_r: REAL_64
			l_Ag_i: REAL_64
			l_yi_tmp_val: REAL_64
			l_yi_tmp_err: REAL_64
			l_R: REAL_64
			l_I: REAL_64
			l_a___5: REAL_64
		do
			create l_log1_r.make (<<1>>)
			create l_log1_i.make (<<1>>)
			create l_logAg_r.make (<<1>>)
			create l_logAg_i.make (<<1>>)

			l_zr := a_zr
			l_zr := l_zr - (1.0)
			l_Ag_r := lanczos_7_c[0]
			l_Ag_i := 0.0
			l_k := 1
			from
			until
				not (l_k <= 8)
			loop
				l_R := l_zr + l_k.to_double
				l_I := a_zi
				l_a___5 := lanczos_7_c[l_k] / (l_R * l_R + l_I * l_I)
				l_Ag_r := l_Ag_r + (l_a___5 * l_R)
				l_Ag_i := l_Ag_i - (l_a___5 * l_I)
				l_k := l_k + 1
			end
			log_inst.gsl_sf_complex_log_e (l_zr + 7.5, a_zi, l_log1_r, l_log1_i).do_nothing
			log_inst.gsl_sf_complex_log_e (l_Ag_r, l_Ag_i, l_logAg_r, l_logAg_i).do_nothing
			a_yr.item.val := ((((l_zr + 0.5) * l_log1_r.item.val - a_zi * l_log1_i.item.val) - (l_zr + 7.5)) + 0.9189385332046727418) + l_logAg_r.item.val
			a_yi.item.val := ((a_zi * l_log1_r.item.val + (l_zr + 0.5) * l_log1_i.item.val) - a_zi) + l_logAg_i.item.val
			a_yr.item.err := (4.0 * 2.2204460492503131e-16) * (fabs (a_yr.item.val))
			a_yi.item.err := (4.0 * 2.2204460492503131e-16) * (fabs (a_yi.item.val))
			l_yi_tmp_val := a_yi.item.val
			l_yi_tmp_err := a_yi.item.err
			trig_inst.gsl_sf_angle_restrict_symm_err_e (l_yi_tmp_val, a_yi).do_nothing
			a_yi.item.err := a_yi.item.err + l_yi_tmp_err
			Result := 0
		end

	lngamma_sgn_0 (a_eps: REAL_64; a_lng: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]; a_sgn: CE_POINTER [REAL_64]): INTEGER_32
		local
			l_c1___2: REAL_64
			l_c2___0: REAL_64
			l_c3___3: REAL_64
			l_c4___2: REAL_64
			l_c5___1: REAL_64
			l_c6___2: REAL_64
			l_c7___0: REAL_64
			l_c8___0: REAL_64
			l_c9: REAL_64
			l_c10: REAL_64
			l_g6: REAL_64
			l_g: REAL_64
			l_gee: REAL_64
		do
			l_c1___2 := -0.07721566490153286061
			l_c2___0 := -0.01094400467202744461
			l_c3___3 := 0.09252092391911371098
			l_c4___2 := -0.01827191316559981266
			l_c5___1 := 0.01800493109685479790
			l_c6___2 := -0.00685088537872380685
			l_c7___0 := 0.00399823955756846603
			l_c8___0 := -0.00189430621687107802
			l_c9 := 0.00097473237804513221
			l_c10 := -0.00048434392722255893
			l_g6 := l_c6___2 + a_eps * (l_c7___0 + a_eps * (l_c8___0 + a_eps * (l_c9 + a_eps * l_c10)))
			l_g := a_eps * (l_c1___2 + a_eps * (l_c2___0 + a_eps * (l_c3___3 + a_eps * (l_c4___2 + a_eps * (l_c5___1 + a_eps * l_g6)))))
			l_gee := (l_g + 1.0 / (1.0 + a_eps)) + 0.5 * a_eps
			a_lng.item.val := log (l_gee / (fabs (a_eps)))
			a_lng.item.err := (4.0 * 2.2204460492503131e-16) * (fabs (a_lng.item.val))
			if a_eps >= 0.0 then
				a_sgn.item := (1).to_double
			else
				a_sgn.item := (-1).to_double
			end
			Result := 0
		end

	lngamma_sgn_sing (a_N: INTEGER_32; a_eps: REAL_64; a_lng: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]; a_sgn: CE_POINTER [REAL_64]): INTEGER_32
		local
			l_c0___0: REAL_64
			l_c1___2: REAL_64
			l_c2___0: REAL_64
			l_c3___3: REAL_64
			l_c4___2: REAL_64
			l_c5___1: REAL_64
			l_c6___2: REAL_64
			l_c7___0: REAL_64
			l_c8___0: REAL_64
			l_c9: REAL_64
			l_g5: REAL_64
			l_g: REAL_64
			l_gam_e: REAL_64
			l_tmp: REAL_64
			l_g___0: REAL_64
			l_cs1: REAL_64
			l_cs2: REAL_64
			l_cs3: REAL_64
			l_cs4: REAL_64
			l_cs5: REAL_64
			l_e2: REAL_64
			l_sin_ser: REAL_64
			l_aeps: REAL_64
			l_c1___3: REAL_64
			l_c2___1: REAL_64
			l_c3___4: REAL_64
			l_c4___3: REAL_64
			l_c5___2: REAL_64
			l_c6___3: REAL_64
			l_c7___1: REAL_64
			l_lng_ser: REAL_64
			l_c0___1: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_psi_0: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_psi_1: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_psi_2: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_psi_3: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_psi_4: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_psi_5: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_psi_6: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_tmp___6: REAL_64
			l_tmp___8: REAL_64
			l_tmp___9: REAL_64
			js: JUMP_STATE
		do
			create l_c0___1.make (<<1>>)
			create l_psi_0.make (<<1>>)
			create l_psi_1.make (<<1>>)
			create l_psi_2.make (<<1>>)
			create l_psi_3.make (<<1>>)
			create l_psi_4.make (<<1>>)
			create l_psi_5.make (<<1>>)
			create l_psi_6.make (<<1>>)

			if a_eps = 0.0 then
				a_lng.item.val := 0.0
				a_lng.item.err := 0.0
				a_sgn.item := 0.0
				from
				until
					js.loop_done
				loop
					error_inst.gsl_error (ce_string ("error"), ce_string ("gamma.c"), 769, 1)
					Result := 1
					js.return
					if not js.return_called then
						js.break
					end
				end
			elseif a_N = 1 then
				l_c0___0 := 0.07721566490153286061
				l_c1___2 := 0.08815966957356030521
				l_c2___0 := -0.00436125434555340577
				l_c3___3 := 0.01391065882004640689
				l_c4___2 := -0.00409427227680839100
				l_c5___1 := 0.00275661310191541584
				l_c6___2 := -0.00124162645565305019
				l_c7___0 := 0.00065267976121802783
				l_c8___0 := -0.00032205261682710437
				l_c9 := 0.00016229131039545456
				l_g5 := l_c5___1 + a_eps * (l_c6___2 + a_eps * (l_c7___0 + a_eps * (l_c8___0 + a_eps * l_c9)))
				l_g := a_eps * (l_c0___0 + a_eps * (l_c1___2 + a_eps * (l_c2___0 + a_eps * (l_c3___3 + a_eps * (l_c4___2 + a_eps * l_g5)))))
				l_gam_e := (l_g - 1.0) - ((0.5 * a_eps) * (1.0 + 3.0 * a_eps)) / (1.0 - a_eps * a_eps)
				l_tmp := fabs (l_gam_e)
				a_lng.item.val := log (l_tmp / (fabs (a_eps)))
				a_lng.item.err := (2.0 * 2.2204460492503131e-16) * (fabs (a_lng.item.val))
				if a_eps > 0.0 then
					a_sgn.item := -1.0
				else
					a_sgn.item := 1.0
				end
				Result := 0
				js.return
			else
				l_cs1 := -1.6449340668482264365
				l_cs2 := 0.8117424252833536436
				l_cs3 := -0.1907518241220842137
				l_cs4 := 0.0261478478176548005
				l_cs5 := -0.0023460810354558236
				l_e2 := a_eps * a_eps
				l_sin_ser := 1.0 + l_e2 * (l_cs1 + l_e2 * (l_cs2 + l_e2 * (l_cs3 + l_e2 * (l_cs4 + l_e2 * l_cs5))))
				l_aeps := (fabs (a_eps))
				l_psi_2.item.val := 0.0
				l_psi_3.item.val := 0.0
				l_psi_4.item.val := 0.0
				l_psi_5.item.val := 0.0
				l_psi_6.item.val := 0.0
				gsl_sf_lnfact_e (a_N.to_natural_32, l_c0___1).do_nothing
				psi_inst.gsl_sf_psi_int_e (a_N + 1, l_psi_0).do_nothing
				psi_inst.gsl_sf_psi_1_int_e (a_N + 1, l_psi_1).do_nothing
				if l_aeps > 0.00001 then
					psi_inst.gsl_sf_psi_n_e (2, a_N.to_double + 1.0, l_psi_2).do_nothing
				end
				if l_aeps > 0.0002 then
					psi_inst.gsl_sf_psi_n_e (3, a_N.to_double + 1.0, l_psi_3).do_nothing
				end
				if l_aeps > 0.001 then
					psi_inst.gsl_sf_psi_n_e (4, a_N.to_double + 1.0, l_psi_4).do_nothing
				end
				if l_aeps > 0.005 then
					psi_inst.gsl_sf_psi_n_e (5, a_N.to_double + 1.0, l_psi_5).do_nothing
				end
				if l_aeps > 0.01 then
					psi_inst.gsl_sf_psi_n_e (6, a_N.to_double + 1.0, l_psi_6).do_nothing
				end
				l_c1___3 := l_psi_0.item.val
				l_c2___1 := l_psi_1.item.val / 2.0
				l_c3___4 := l_psi_2.item.val / 6.0
				l_c4___3 := l_psi_3.item.val / 24.0
				l_c5___2 := l_psi_4.item.val / 120.0
				l_c6___3 := l_psi_5.item.val / 720.0
				l_c7___1 := l_psi_6.item.val / 5040.0
				l_lng_ser := l_c0___1.item.val - a_eps * (l_c1___3 - a_eps * (l_c2___1 - a_eps * (l_c3___4 - a_eps * (l_c4___3 - a_eps * (l_c5___2 - a_eps * (l_c6___3 - a_eps * l_c7___1))))))
				l_g___0 := -l_lng_ser - (log (l_sin_ser))
				a_lng.item.val := l_g___0 - (log ((fabs (a_eps))))
				l_tmp___6 := fabs (l_g___0)
				a_lng.item.err := l_c0___1.item.err + (2.0 * 2.2204460492503131e-16) * (l_tmp___6 + (fabs (a_lng.item.val)))
				if ((a_N).bit_and (1)).to_boolean then
					l_tmp___8 := -1.0
				else
					l_tmp___8 := 1.0
				end
				if a_eps > 0.0 then
					l_tmp___9 := 1.0
				else
					l_tmp___9 := -1.0
				end
				a_sgn.item := l_tmp___8 * l_tmp___9
				Result := 0
				js.return
			end
			if not js.return_called then
				Result := 0
			end
		end

end
