class
	P_GENSYMM

inherit
	LIBGSL_DATA
	S_GCC_BUILTIN
	S_LIBIO
	S_STDIO
	S_STDLIB
	S_MATH
	S_STRING
	S_ASSERT

feature {ANY} -- routines

	gsl_eigen_gensymm (a_A___0: CE_POINTER [PD_GSL_MATRIX]; a_B: CE_POINTER [PD_GSL_MATRIX]; a_eval: CE_POINTER [PD_GSL_VECTOR]; a_w: CE_POINTER [PD_GSL_EIGEN_GENSYMM_WORKSPACE]): INTEGER_32
		local
			l_N: NATURAL_32
			l_s: INTEGER_32
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_N := a_A___0.item.size1
					if l_N = a_A___0.item.size2 then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						from
						until
							js.loop_done
						loop
							error_inst.gsl_error (ce_string ("matrix must be square to compute eigenvalues"), ce_string ("gensymm.c"), 120, 20)
							Result := 20
							js.return
							if not js.return_called then
								js.break
							end
						end
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 10
				when 3 then -- else:
						if l_N = a_B.item.size1 then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if:
						l_pos := -1 + 7 -- goto _L
				when 5 then -- if block done, skip else block:
						l_pos := -1 + 10
				when 6 then -- else:
						if l_N = a_B.item.size2 then
							l_pos := -1 + 9 -- skip if block
						end
				when 7 then -- if: | _L:
						from
						until
							js.loop_done
						loop
							error_inst.gsl_error (ce_string ("B matrix dimensions must match A"), ce_string ("gensymm.c"), 124, 19)
							Result := 19
							js.return
							if not js.return_called then
								js.break
							end
						end
				when 8 then -- if block done, skip else block:
						l_pos := -1 + 10
				when 9 then -- else:
						if a_eval.item.size /= l_N then
							from
							until
								js.loop_done
							loop
								error_inst.gsl_error (ce_string ("eigenvalue vector must match matrix size"), ce_string ("gensymm.c"), 128, 19)
								Result := 19
								js.return
								if not js.return_called then
									js.break
								end
							end
						elseif a_w.item.size /= l_N then
							from
							until
								js.loop_done
							loop
								error_inst.gsl_error (ce_string ("matrix size does not match workspace"), ce_string ("gensymm.c"), 132, 19)
								Result := 19
								js.return
								if not js.return_called then
									js.break
								end
							end
						else
							l_s := cholesky_inst.gsl_linalg_cholesky_decomp (a_B)
							if l_s /= 0 then
								Result := l_s
								js.return
							end
							if not js.return_called then
								gsl_eigen_gensymm_standardize (a_A___0, a_B).do_nothing
								l_s := symm_inst.gsl_eigen_symm (a_A___0, a_eval, a_w.item.symm_workspace_p)
								Result := l_s
								js.return
							end
						end
				when 10 then -- if done | if done
						if not js.return_called then
							Result := 0
						end
				when 11 then
						l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
			end

	gsl_eigen_gensymm_alloc (a_n: NATURAL_32): CE_POINTER [PD_GSL_EIGEN_GENSYMM_WORKSPACE]
		local
			l_w: CE_POINTER [PD_GSL_EIGEN_GENSYMM_WORKSPACE]
			js: JUMP_STATE
		do
			create l_w.make

			if a_n = {NATURAL_32} 0 then
				from
				until
					js.loop_done
				loop
					error_inst.gsl_error (ce_string ("matrix dimension must be positive integer"), ce_string ("gensymm.c"), 55, 4)
					Result := (create {CE_POINTER [PD_GSL_EIGEN_GENSYMM_WORKSPACE]}.make_shared (default_pointer))
					js.return
					if not js.return_called then
						js.break
					end
				end
			end
			if not js.return_called then
				l_w := create {CE_POINTER [PD_GSL_EIGEN_GENSYMM_WORKSPACE]}.make_cast ((calloc ((1).to_natural_32, (create {PD_GSL_EIGEN_GENSYMM_WORKSPACE}.make).structure_size.to_natural_32)))
				if l_w.is_void then
					from
					until
						js.loop_done
					loop
						error_inst.gsl_error (ce_string ("failed to allocate space for workspace"), ce_string ("gensymm.c"), 62, 8)
						Result := (create {CE_POINTER [PD_GSL_EIGEN_GENSYMM_WORKSPACE]}.make_shared (default_pointer))
						js.return
						if not js.return_called then
							js.break
						end
					end
				end
			end
			if not js.return_called then
				l_w.item.size := a_n
				l_w.item.symm_workspace_p := symm_inst.gsl_eigen_symm_alloc (a_n)
				if (l_w.item.symm_workspace_p).is_void then
					gsl_eigen_gensymm_free (l_w)
					from
					until
						js.loop_done
					loop
						error_inst.gsl_error (ce_string ("failed to allocate space for symm workspace"), ce_string ("gensymm.c"), 71, 8)
						Result := (create {CE_POINTER [PD_GSL_EIGEN_GENSYMM_WORKSPACE]}.make_shared (default_pointer))
						js.return
						if not js.return_called then
							js.break
						end
					end
				end
			end
			if not js.return_called then
				Result := l_w
			end
		end

	gsl_eigen_gensymm_free (a_w: CE_POINTER [PD_GSL_EIGEN_GENSYMM_WORKSPACE])
		local
			js: JUMP_STATE
		do
			if a_w.is_void then
				js.return
			end
			if not js.return_called then
				if (not (a_w.item.symm_workspace_p).is_void) then
					symm_inst.gsl_eigen_symm_free (a_w.item.symm_workspace_p)
				end
				free (create {CE_POINTER [ANY]}.make_cast (a_w))
			end
		end

	gsl_eigen_gensymm_standardize (a_A___0: CE_POINTER [PD_GSL_MATRIX]; a_B: CE_POINTER [PD_GSL_MATRIX]): INTEGER_32
		local
			l_N: NATURAL_32
			l_i: NATURAL_32
			l_a___5: REAL_64
			l_b___0: REAL_64
			l_c___1: REAL_64
			l_ai: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_ma: CE_ARRAY [PD_GSL_MATRIX_VIEW]
			l_bi: CE_ARRAY [PD_GSL_VECTOR_CONST_VIEW]
			l_mb: CE_ARRAY [PD_GSL_MATRIX_CONST_VIEW]
		do
			create l_ai.make (<<1>>)
			create l_ma.make (<<1>>)
			create l_bi.make (<<1>>)
			create l_mb.make (<<1>>)

			l_N := a_A___0.item.size1
			l_i := (0).to_natural_32
			from
			until
				not (l_i < l_N)
			loop
				l_a___5 := gsl_gsl_matrix_double_inst.gsl_matrix_get (a_A___0, l_i, l_i)
				l_b___0 := gsl_gsl_matrix_double_inst.gsl_matrix_get (a_B, l_i, l_i)
				l_a___5 := l_a___5 / (l_b___0 * l_b___0)
				gsl_gsl_matrix_double_inst.gsl_matrix_set (a_A___0, l_i, l_i, l_a___5)
				if l_i < l_N - {NATURAL_32} 1 then
					l_ai.item := (rowcol_source_inst.gsl_matrix_subcolumn (a_A___0, l_i, l_i + {NATURAL_32} 1, (l_N - l_i) - {NATURAL_32} 1)).twin
					l_ma.item := (submatrix_source_inst.gsl_matrix_submatrix (a_A___0, l_i + {NATURAL_32} 1, l_i + {NATURAL_32} 1, (l_N - l_i) - {NATURAL_32} 1, (l_N - l_i) - {NATURAL_32} 1)).twin
					l_bi.item := (rowcol_source_inst.gsl_matrix_const_subcolumn (a_B, l_i, l_i + {NATURAL_32} 1, (l_N - l_i) - {NATURAL_32} 1)).twin
					l_mb.item := (submatrix_source_inst.gsl_matrix_const_submatrix (a_B, l_i + {NATURAL_32} 1, l_i + {NATURAL_32} 1, (l_N - l_i) - {NATURAL_32} 1, (l_N - l_i) - {NATURAL_32} 1)).twin
					blas_inst.gsl_blas_dscal (1.0 / l_b___0, create {CE_POINTER [PD_GSL_VECTOR]}.make_shared (l_ai.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")))
					l_c___1 := -0.5 * l_a___5
					blas_inst.gsl_blas_daxpy (l_c___1, (create {CE_POINTER [PD_GSL_VECTOR]}.make_shared (l_bi.item.c_pointer + (create {PD_GSL_VECTOR_CONST_VIEW}.make).offset_of ("vector"))), create {CE_POINTER [PD_GSL_VECTOR]}.make_shared (l_ai.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).do_nothing
					blas_inst.gsl_blas_dsyr2 ({PC_CBLAS_UPLO}.CblasLower, -1.0, (create {CE_POINTER [PD_GSL_VECTOR]}.make_shared (l_ai.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))), (create {CE_POINTER [PD_GSL_VECTOR]}.make_shared (l_bi.item.c_pointer + (create {PD_GSL_VECTOR_CONST_VIEW}.make).offset_of ("vector"))), create {CE_POINTER [PD_GSL_MATRIX]}.make_shared (l_ma.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).do_nothing
					blas_inst.gsl_blas_daxpy (l_c___1, (create {CE_POINTER [PD_GSL_VECTOR]}.make_shared (l_bi.item.c_pointer + (create {PD_GSL_VECTOR_CONST_VIEW}.make).offset_of ("vector"))), create {CE_POINTER [PD_GSL_VECTOR]}.make_shared (l_ai.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).do_nothing
					blas_inst.gsl_blas_dtrsv ({PC_CBLAS_UPLO}.CblasLower, {PC_CBLAS_TRANSPOSE}.CblasNoTrans, {PC_CBLAS_DIAG}.CblasNonUnit, (create {CE_POINTER [PD_GSL_MATRIX]}.make_shared (l_mb.item.c_pointer + (create {PD_GSL_MATRIX_CONST_VIEW}.make).offset_of ("matrix"))), create {CE_POINTER [PD_GSL_VECTOR]}.make_shared (l_ai.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).do_nothing
				end
				l_i := l_i + 1
			end
			Result := 0
		end

end
