class
	P_HC_UNPACK

inherit
	LIBGSL_DATA
	S_GCC_BUILTIN
	S_LIBIO
	S_STDIO
	S_STDLIB
	S_MATH
	S_STRING
	S_ASSERT

feature {ANY} -- routines

	gsl_fft_halfcomplex_float_radix2_unpack (a_halfcomplex_coefficient: CE_POINTER [REAL_32]; a_complex_coefficient: CE_POINTER [REAL_32]; a_stride: NATURAL_32; a_n: NATURAL_32): INTEGER_32
		local
			l_i: NATURAL_32
			l_hc_real: REAL_32
			l_hc_imag: REAL_32
			js: JUMP_STATE
		do
			if a_n = {NATURAL_32} 0 then
				from
				until
					js.loop_done
				loop
					error_inst.gsl_error (ce_string ("length n must be positive integer"), ce_string ("hc_unpack.c"), 65, 1)
					Result := 1
					js.return
					if not js.return_called then
						js.break
					end
				end
			end
			if not js.return_called then
				a_complex_coefficient[({NATURAL_32} 0).to_integer_32] := a_halfcomplex_coefficient[0]
				a_complex_coefficient[({NATURAL_32} 1).to_integer_32] := (0.0).truncated_to_real
				l_i := (1).to_natural_32
				from
				until
					not (l_i < a_n - l_i)
				loop
					l_hc_real := a_halfcomplex_coefficient[(l_i * a_stride).to_integer_32]
					l_hc_imag := a_halfcomplex_coefficient[((a_n - l_i) * a_stride).to_integer_32]
					a_complex_coefficient[(({NATURAL_32} 2 * a_stride) * l_i).to_integer_32] := l_hc_real
					a_complex_coefficient[(({NATURAL_32} 2 * a_stride) * l_i + {NATURAL_32} 1).to_integer_32] := l_hc_imag
					a_complex_coefficient[(({NATURAL_32} 2 * a_stride) * (a_n - l_i)).to_integer_32] := l_hc_real
					a_complex_coefficient[(({NATURAL_32} 2 * a_stride) * (a_n - l_i) + {NATURAL_32} 1).to_integer_32] := -l_hc_imag
					l_i := l_i + 1
				end
				if l_i = a_n - l_i then
					a_complex_coefficient[(({NATURAL_32} 2 * a_stride) * l_i).to_integer_32] := a_halfcomplex_coefficient[(l_i * a_stride).to_integer_32]
					a_complex_coefficient[(({NATURAL_32} 2 * a_stride) * l_i + {NATURAL_32} 1).to_integer_32] := (0.0).truncated_to_real
				end
				Result := 0
			end
		end

	gsl_fft_halfcomplex_float_unpack (a_halfcomplex_coefficient: CE_POINTER [REAL_32]; a_complex_coefficient: CE_POINTER [REAL_32]; a_stride: NATURAL_32; a_n: NATURAL_32): INTEGER_32
		local
			l_i: NATURAL_32
			l_hc_real: REAL_32
			l_hc_imag: REAL_32
			js: JUMP_STATE
		do
			if a_n = {NATURAL_32} 0 then
				from
				until
					js.loop_done
				loop
					error_inst.gsl_error (ce_string ("length n must be positive integer"), ce_string ("hc_unpack.c"), 29, 1)
					Result := 1
					js.return
					if not js.return_called then
						js.break
					end
				end
			end
			if not js.return_called then
				a_complex_coefficient[({NATURAL_32} 0).to_integer_32] := a_halfcomplex_coefficient[0]
				a_complex_coefficient[({NATURAL_32} 1).to_integer_32] := (0.0).truncated_to_real
				l_i := (1).to_natural_32
				from
				until
					not (l_i < a_n - l_i)
				loop
					l_hc_real := a_halfcomplex_coefficient[(({NATURAL_32} 2 * l_i - {NATURAL_32} 1) * a_stride).to_integer_32]
					l_hc_imag := a_halfcomplex_coefficient[(({NATURAL_32} 2 * l_i) * a_stride).to_integer_32]
					a_complex_coefficient[(({NATURAL_32} 2 * a_stride) * l_i).to_integer_32] := l_hc_real
					a_complex_coefficient[(({NATURAL_32} 2 * a_stride) * l_i + {NATURAL_32} 1).to_integer_32] := l_hc_imag
					a_complex_coefficient[(({NATURAL_32} 2 * a_stride) * (a_n - l_i)).to_integer_32] := l_hc_real
					a_complex_coefficient[(({NATURAL_32} 2 * a_stride) * (a_n - l_i) + {NATURAL_32} 1).to_integer_32] := -l_hc_imag
					l_i := l_i + 1
				end
				if l_i = a_n - l_i then
					a_complex_coefficient[(({NATURAL_32} 2 * a_stride) * l_i).to_integer_32] := a_halfcomplex_coefficient[((a_n - {NATURAL_32} 1) * a_stride).to_integer_32]
					a_complex_coefficient[(({NATURAL_32} 2 * a_stride) * l_i + {NATURAL_32} 1).to_integer_32] := (0.0).truncated_to_real
				end
				Result := 0
			end
		end

	gsl_fft_halfcomplex_radix2_unpack (a_halfcomplex_coefficient: CE_POINTER [REAL_64]; a_complex_coefficient: CE_POINTER [REAL_64]; a_stride: NATURAL_32; a_n: NATURAL_32): INTEGER_32
		local
			l_i: NATURAL_32
			l_hc_real: REAL_64
			l_hc_imag: REAL_64
			js: JUMP_STATE
		do
			if a_n = {NATURAL_32} 0 then
				from
				until
					js.loop_done
				loop
					error_inst.gsl_error (ce_string ("length n must be positive integer"), ce_string ("hc_unpack.c"), 65, 1)
					Result := 1
					js.return
					if not js.return_called then
						js.break
					end
				end
			end
			if not js.return_called then
				a_complex_coefficient[({NATURAL_32} 0).to_integer_32] := a_halfcomplex_coefficient[0]
				a_complex_coefficient[({NATURAL_32} 1).to_integer_32] := 0.0
				l_i := (1).to_natural_32
				from
				until
					not (l_i < a_n - l_i)
				loop
					l_hc_real := a_halfcomplex_coefficient[(l_i * a_stride).to_integer_32]
					l_hc_imag := a_halfcomplex_coefficient[((a_n - l_i) * a_stride).to_integer_32]
					a_complex_coefficient[(({NATURAL_32} 2 * a_stride) * l_i).to_integer_32] := l_hc_real
					a_complex_coefficient[(({NATURAL_32} 2 * a_stride) * l_i + {NATURAL_32} 1).to_integer_32] := l_hc_imag
					a_complex_coefficient[(({NATURAL_32} 2 * a_stride) * (a_n - l_i)).to_integer_32] := l_hc_real
					a_complex_coefficient[(({NATURAL_32} 2 * a_stride) * (a_n - l_i) + {NATURAL_32} 1).to_integer_32] := -l_hc_imag
					l_i := l_i + 1
				end
				if l_i = a_n - l_i then
					a_complex_coefficient[(({NATURAL_32} 2 * a_stride) * l_i).to_integer_32] := a_halfcomplex_coefficient[(l_i * a_stride).to_integer_32]
					a_complex_coefficient[(({NATURAL_32} 2 * a_stride) * l_i + {NATURAL_32} 1).to_integer_32] := 0.0
				end
				Result := 0
			end
		end

	gsl_fft_halfcomplex_unpack (a_halfcomplex_coefficient: CE_POINTER [REAL_64]; a_complex_coefficient: CE_POINTER [REAL_64]; a_stride: NATURAL_32; a_n: NATURAL_32): INTEGER_32
		local
			l_i: NATURAL_32
			l_hc_real: REAL_64
			l_hc_imag: REAL_64
			js: JUMP_STATE
		do
			if a_n = {NATURAL_32} 0 then
				from
				until
					js.loop_done
				loop
					error_inst.gsl_error (ce_string ("length n must be positive integer"), ce_string ("hc_unpack.c"), 29, 1)
					Result := 1
					js.return
					if not js.return_called then
						js.break
					end
				end
			end
			if not js.return_called then
				a_complex_coefficient[({NATURAL_32} 0).to_integer_32] := a_halfcomplex_coefficient[0]
				a_complex_coefficient[({NATURAL_32} 1).to_integer_32] := 0.0
				l_i := (1).to_natural_32
				from
				until
					not (l_i < a_n - l_i)
				loop
					l_hc_real := a_halfcomplex_coefficient[(({NATURAL_32} 2 * l_i - {NATURAL_32} 1) * a_stride).to_integer_32]
					l_hc_imag := a_halfcomplex_coefficient[(({NATURAL_32} 2 * l_i) * a_stride).to_integer_32]
					a_complex_coefficient[(({NATURAL_32} 2 * a_stride) * l_i).to_integer_32] := l_hc_real
					a_complex_coefficient[(({NATURAL_32} 2 * a_stride) * l_i + {NATURAL_32} 1).to_integer_32] := l_hc_imag
					a_complex_coefficient[(({NATURAL_32} 2 * a_stride) * (a_n - l_i)).to_integer_32] := l_hc_real
					a_complex_coefficient[(({NATURAL_32} 2 * a_stride) * (a_n - l_i) + {NATURAL_32} 1).to_integer_32] := -l_hc_imag
					l_i := l_i + 1
				end
				if l_i = a_n - l_i then
					a_complex_coefficient[(({NATURAL_32} 2 * a_stride) * l_i).to_integer_32] := a_halfcomplex_coefficient[((a_n - {NATURAL_32} 1) * a_stride).to_integer_32]
					a_complex_coefficient[(({NATURAL_32} 2 * a_stride) * l_i + {NATURAL_32} 1).to_integer_32] := 0.0
				end
				Result := 0
			end
		end

end
