class
	P_SIMAN

inherit
	LIBGSL_DATA
	S_GCC_BUILTIN
	S_LIBIO
	S_STDIO
	S_STDLIB
	S_MATH
	S_STRING
	S_ASSERT

feature {ANY} -- routines

	boltzmann (a_E: REAL_64; a_new_E: REAL_64; a_T: REAL_64; a_params: CE_POINTER [PD_GSL_SIMAN_PARAMS_T]): REAL_64
		local
			l_x: REAL_64
			l_tmp___0: REAL_64
		do
			l_x := -(a_new_E - a_E) / (a_params.item.k * a_T)
			if l_x < -7.0839641853226408e+02 then
				l_tmp___0 := 0.0
			else
				l_tmp___0 := (exp (l_x))
			end
			Result := l_tmp___0
		end

	copy_state (a_src: CE_POINTER [ANY]; a_dst: CE_POINTER [ANY]; a_size: NATURAL_32; a_copyfunc: CE_ROUTINE)
		do
			if (not a_copyfunc.is_void) then
				a_copyfunc.call ([a_src, a_dst])
			else
				memcpy (a_dst, a_src, a_size).do_nothing
			end
		end

	gsl_siman_solve (a_r___3: CE_POINTER [PD_GSL_RNG]; a_x0_p: CE_POINTER [ANY]; a_Ef: CE_ROUTINE; a_take_step___2: CE_ROUTINE; a_distance: CE_ROUTINE; a_print_position: CE_ROUTINE; a_copyfunc: CE_ROUTINE; a_copy_constructor: CE_ROUTINE; a_destructor: CE_ROUTINE; a_element_size: NATURAL_32; a_params: PD_GSL_SIMAN_PARAMS_T)
		local
			l_params: CE_ARRAY [PD_GSL_SIMAN_PARAMS_T]
			l_distance: CE_ROUTINE
			l_x: CE_POINTER [ANY]
			l_new_x: CE_POINTER [ANY]
			l_best_x: CE_POINTER [ANY]
			l_E: REAL_64
			l_new_E: REAL_64
			l_best_E: REAL_64
			l_i: INTEGER_32
			l_T: REAL_64
			l_T_factor: REAL_64
			l_n_evals: INTEGER_32
			l_n_iter: INTEGER_32
			l_n_accepts: INTEGER_32
			l_n_rejects: INTEGER_32
			l_n_eless: INTEGER_32
			l_tmp___2: REAL_64
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			create l_params.make (<<1>>)
			create l_distance.make
			create l_x.make
			create l_new_x.make
			create l_best_x.make

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_params.item := a_params.twin
					l_distance := a_distance
					l_n_evals := 1
					l_n_iter := 0
					if a_copyfunc.is_void then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						if not a_copy_constructor.is_void then
							if a_destructor.is_void then
								l_pos := -1 + 3 -- goto _L___0
							end
						else
							l_pos := -1 + 3 -- goto _L___0
						end
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 4
				when 3 then -- else: | _L___0:
						if a_element_size = {NATURAL_32} 0 then
							assert_fail (ce_string ("(copyfunc != ((void *)0) && copy_constructor != ((void *)0) && destructor != ((void *)0)) || (element_size != 0)"), ce_string ("siman.c"), {NATURAL_32} 73, ce_string ("gsl_siman_solve"))
						end
				when 4 then
					l_distance := create {CE_ROUTINE}.make_shared (default_pointer)
					l_E := a_Ef.real_64_item ([a_x0_p])
					if (not a_copyfunc.is_void) then
						l_x := create {CE_POINTER [ANY]}.make_cast (a_copy_constructor.pointer_item ([a_x0_p]))
						l_new_x := create {CE_POINTER [ANY]}.make_cast (a_copy_constructor.pointer_item ([a_x0_p]))
						l_best_x := create {CE_POINTER [ANY]}.make_cast (a_copy_constructor.pointer_item ([a_x0_p]))
					else
						l_x := (malloc (a_element_size))
						memcpy (l_x, a_x0_p, a_element_size).do_nothing
						l_new_x := (malloc (a_element_size))
						l_best_x := (malloc (a_element_size))
						memcpy (l_best_x, a_x0_p, a_element_size).do_nothing
					end
					l_best_E := l_E
					l_T := l_params.item.t_initial
					l_T_factor := 1.0 / l_params.item.mu_t
					if (not a_print_position.is_void) then
						Io.put_string ("#-iter  #-evals   temperature     position   energy%N")
					end
					from
					until
						js.loop_done
					loop
						l_n_accepts := 0
						l_n_rejects := 0
						l_n_eless := 0
						l_i := 0
						from
						until
							not (l_i < l_params.item.iters_fixed_T)
						loop
							copy_state (l_x, l_new_x, a_element_size, a_copyfunc)
							a_take_step___2.call ([a_r___3, l_new_x, l_params.item.step_size])
							l_new_E := a_Ef.real_64_item ([l_new_x])
							if l_new_E <= l_best_E then
								if (not a_copyfunc.is_void) then
									a_copyfunc.call ([l_new_x, l_best_x])
								else
									memcpy (l_best_x, l_new_x, a_element_size).do_nothing
								end
								l_best_E := l_new_E
							end
							l_n_evals := l_n_evals + 1
							if l_new_E < l_E then
								if l_new_E < l_best_E then
									copy_state (l_new_x, l_best_x, a_element_size, a_copyfunc)
									l_best_E := l_new_E
								end
								copy_state (l_new_x, l_x, a_element_size, a_copyfunc)
								l_E := l_new_E
								l_n_eless := l_n_eless + 1
							else
								l_tmp___2 := gsl_gsl_rng_inst.gsl_rng_uniform (a_r___3)
								if l_tmp___2 < (boltzmann (l_E, l_new_E, l_T, l_params)) then
									copy_state (l_new_x, l_x, a_element_size, a_copyfunc)
									l_E := l_new_E
									l_n_accepts := l_n_accepts + 1
								else
									l_n_rejects := l_n_rejects + 1
								end
							end
							l_i := l_i + 1
						end
						if (not a_print_position.is_void) then
							printf ([ce_string ("%%5d   %%7d  %%12g"), l_n_iter, l_n_evals, l_T]).do_nothing
							a_print_position.call ([l_x])
							printf ([ce_string ("  %%12g  %%12g%N"), l_E, l_best_E]).do_nothing
						end
						l_T := l_T * l_T_factor
						l_n_iter := l_n_iter + 1
						if l_T < l_params.item.t_min then
							js.break
						end
					end
					copy_state (l_best_x, a_x0_p, a_element_size, a_copyfunc)
					if (not a_copyfunc.is_void) then
						a_destructor.call ([l_x])
						a_destructor.call ([l_new_x])
						a_destructor.call ([l_best_x])
					else
						free (l_x)
						free (l_new_x)
						free (l_best_x)
					end
				when 5 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	gsl_siman_solve_many (a_r___3: CE_POINTER [PD_GSL_RNG]; a_x0_p: CE_POINTER [ANY]; a_Ef: CE_ROUTINE; a_take_step___2: CE_ROUTINE; a_distance: CE_ROUTINE; a_print_position: CE_ROUTINE; a_element_size: NATURAL_32; a_params: PD_GSL_SIMAN_PARAMS_T)
		local
			l_params: CE_ARRAY [PD_GSL_SIMAN_PARAMS_T]
			l_x: CE_POINTER [ANY]
			l_new_x: CE_POINTER [ANY]
			l_energies: CE_POINTER [REAL_64]
			l_probs: CE_POINTER [REAL_64]
			l_sum_probs: CE_POINTER [REAL_64]
			l_Ex: REAL_64
			l_T: REAL_64
			l_T_factor: REAL_64
			l_i: INTEGER_32
			l_u: REAL_64
			l_n_iter: INTEGER_32
			js: JUMP_STATE
		do
			create l_params.make (<<1>>)
			create l_x.make
			create l_new_x.make
			create l_energies.make
			create l_probs.make
			create l_sum_probs.make

			l_params.item := a_params.twin
			if (not a_print_position.is_void) then
				Io.put_string ("#-iter    temperature       position")
				Io.put_string ("         delta_pos        energy%N")
			end
			l_x := (malloc ((l_params.item.n_tries).to_natural_32 * a_element_size))
			l_new_x := (malloc ((l_params.item.n_tries).to_natural_32 * a_element_size))
			l_energies := create {CE_POINTER [REAL_64]}.make_cast ((malloc ((l_params.item.n_tries).to_natural_32 * {PLATFORM}.Real_64_bytes.to_natural_32)))
			l_probs := create {CE_POINTER [REAL_64]}.make_cast ((malloc ((l_params.item.n_tries).to_natural_32 * {PLATFORM}.Real_64_bytes.to_natural_32)))
			l_sum_probs := create {CE_POINTER [REAL_64]}.make_cast ((malloc ((l_params.item.n_tries).to_natural_32 * {PLATFORM}.Real_64_bytes.to_natural_32)))
			l_T := l_params.item.t_initial
			l_T_factor := 1.0 / l_params.item.mu_t
			memcpy (l_x, a_x0_p, a_element_size).do_nothing
			l_n_iter := 0
			from
			until
				js.loop_done
			loop
				l_Ex := a_Ef.real_64_item ([l_x])
				l_i := 0
				from
				until
					not (l_i < l_params.item.n_tries - 1)
				loop
					l_sum_probs[l_i] := (0).to_double
					memcpy (create {CE_POINTER [ANY]}.make_cast ((create {CE_POINTER [INTEGER_8]}.make_cast (l_new_x) + (l_i.to_natural_32 * a_element_size).to_integer_32)), l_x, a_element_size).do_nothing
					a_take_step___2.call ([a_r___3, create {CE_POINTER [ANY]}.make_cast ((create {CE_POINTER [INTEGER_8]}.make_cast (l_new_x) + (l_i.to_natural_32 * a_element_size).to_integer_32)), l_params.item.step_size])
					l_energies[l_i] := a_Ef.real_64_item ([create {CE_POINTER [ANY]}.make_cast ((create {CE_POINTER [INTEGER_8]}.make_cast (l_new_x) + (l_i.to_natural_32 * a_element_size).to_integer_32))])
					l_probs[l_i] := boltzmann (l_Ex, l_energies[l_i], l_T, l_params)
					l_i := l_i + 1
				end
				memcpy (create {CE_POINTER [ANY]}.make_cast ((create {CE_POINTER [INTEGER_8]}.make_cast (l_new_x) + ((l_params.item.n_tries - 1).to_natural_32 * a_element_size).to_integer_32)), l_x, a_element_size).do_nothing
				l_energies[(l_params.item.n_tries - 1)] := l_Ex
				l_probs[(l_params.item.n_tries - 1)] := boltzmann (l_Ex, l_energies[l_i], l_T, l_params)
				l_sum_probs[0] := l_probs[0]
				l_i := 1
				from
				until
					not (l_i < l_params.item.n_tries)
				loop
					l_sum_probs[l_i] := l_sum_probs[(l_i - 1)] + l_probs[l_i]
					l_i := l_i + 1
				end
				l_u := (gsl_gsl_rng_inst.gsl_rng_uniform (a_r___3)) * l_sum_probs[(l_params.item.n_tries - 1)]
				l_i := 0
				from
				until
					not (not js.loop_done and then (l_i < l_params.item.n_tries))
				loop
					if l_u < l_sum_probs[l_i] then
						memcpy (l_x, create {CE_POINTER [ANY]}.make_cast ((create {CE_POINTER [INTEGER_8]}.make_cast (l_new_x) + (l_i.to_natural_32 * a_element_size).to_integer_32)), a_element_size).do_nothing
						js.break
					end
					if not js.loop_jumped then
						l_i := l_i + 1
					end
				end
				if (not a_print_position.is_void) then
					printf ([ce_string ("%%5d%T%%12g%T"), l_n_iter, l_T]).do_nothing
					a_print_position.call ([l_x])
					printf ([ce_string ("%T%%12g%T%%12g%N"), (a_distance.real_64_item ([l_x, a_x0_p])), l_Ex]).do_nothing
				end
				l_T := l_T * l_T_factor
				l_n_iter := l_n_iter + 1
				if l_T < l_params.item.t_min then
					js.break
				end
			end
			memcpy (a_x0_p, l_x, a_element_size).do_nothing
			free (l_x)
			free (l_new_x)
			free (create {CE_POINTER [ANY]}.make_cast (l_energies))
			free (create {CE_POINTER [ANY]}.make_cast (l_probs))
			free (create {CE_POINTER [ANY]}.make_cast (l_sum_probs))
		end

end
