class
	P_STEFFENSON

inherit
	LIBGSL_DATA
		redefine
			default_create
		end
	S_GCC_BUILTIN
	S_LIBIO
	S_STDIO
	S_STDLIB
	S_MATH
	S_STRING
	S_ASSERT

feature {NONE} -- Initialization

	default_create
		do
			create gsl_root_fdfsolver_steffenson.make
			create steffenson_type.make (<<1>>)
		end

feature {ANY} -- attributes

	gsl_root_fdfsolver_steffenson: CE_POINTER [PD_GSL_ROOT_FDFSOLVER_TYPE] assign set_gsl_root_fdfsolver_steffenson

	set_gsl_root_fdfsolver_steffenson (a_gsl_root_fdfsolver_steffenson: CE_POINTER [PD_GSL_ROOT_FDFSOLVER_TYPE])
		do
			gsl_root_fdfsolver_steffenson := a_gsl_root_fdfsolver_steffenson
		end

	steffenson_type: CE_ARRAY [PD_GSL_ROOT_FDFSOLVER_TYPE]

feature {ANY} -- routines

	steffenson_init (a_vstate: CE_POINTER [ANY]; a_fdf: CE_POINTER [PD_GSL_FUNCTION_FDF_STRUCT]; a_root: CE_POINTER [REAL_64]): INTEGER_32
		local
			l_state: CE_POINTER [PD_STEFFENSON_STATE_T]
			l_x: REAL_64
		do
			create l_state.make

			l_state := create {CE_POINTER [PD_STEFFENSON_STATE_T]}.make_cast (a_vstate)
			l_x := a_root.item
			l_state.item.f := (a_fdf.item.f).real_64_item ([l_x, a_fdf.item.params])
			l_state.item.df := (a_fdf.item.df).real_64_item ([l_x, a_fdf.item.params])
			l_state.item.x := l_x
			l_state.item.x_1 := 0.0
			l_state.item.x_2 := 0.0
			l_state.item.count := 1
			Result := 0
		end

	steffenson_init_address: CE_ROUTINE
		once ("OBJECT")
			create Result.make_agent (agent steffenson_init)
		end

	steffenson_iterate (a_vstate: CE_POINTER [ANY]; a_fdf: CE_POINTER [PD_GSL_FUNCTION_FDF_STRUCT]; a_root: CE_POINTER [REAL_64]): INTEGER_32
		local
			l_state: CE_POINTER [PD_STEFFENSON_STATE_T]
			l_x_new: REAL_64
			l_f_new: CE_ARRAY [REAL_64]
			l_df_new: CE_ARRAY [REAL_64]
			l_x_1: REAL_64
			l_x: REAL_64
			l_u: REAL_64
			l_v: REAL_64
			js: JUMP_STATE
		do
			create l_state.make
			create l_f_new.make (<<1>>)
			create l_df_new.make (<<1>>)

			l_state := create {CE_POINTER [PD_STEFFENSON_STATE_T]}.make_cast (a_vstate)
			l_x_1 := l_state.item.x_1
			l_x := l_state.item.x
			if l_state.item.df = 0.0 then
				from
				until
					js.loop_done
				loop
					error_inst.gsl_error (ce_string ("derivative is zero"), ce_string ("steffenson.c"), 94, 12)
					Result := 12
					js.return
					if not js.return_called then
						js.break
					end
				end
			end
			if not js.return_called then
				l_x_new := l_x - l_state.item.f / l_state.item.df;
				(a_fdf.item.fdf).call ([l_x_new, a_fdf.item.params, l_f_new, l_df_new])
				l_state.item.x_2 := l_x_1
				l_state.item.x_1 := l_x
				l_state.item.x := l_x_new
				l_state.item.f := l_f_new.item
				l_state.item.df := l_df_new.item
				if not (infnan_inst.gsl_finite (l_f_new.item)).to_boolean then
					from
					until
						js.loop_done
					loop
						error_inst.gsl_error (ce_string ("function value is not finite"), ce_string ("steffenson.c"), 110, 9)
						Result := 9
						js.return
						if not js.return_called then
							js.break
						end
					end
				end
			end
			if not js.return_called then
				if l_state.item.count < 3 then
					a_root.item := l_x_new
					l_state.item.count := (l_state.item.count) + 1
				else
					l_u := l_x - l_x_1
					l_v := (l_x_new - (2).to_double * l_x) + l_x_1
					if l_v = (0).to_double then
						a_root.item := l_x_new
					else
						a_root.item := l_x_1 - (l_u * l_u) / l_v
					end
				end
				if not (infnan_inst.gsl_finite (l_df_new.item)).to_boolean then
					from
					until
						js.loop_done
					loop
						error_inst.gsl_error (ce_string ("derivative value is not finite"), ce_string ("steffenson.c"), 131, 9)
						Result := 9
						js.return
						if not js.return_called then
							js.break
						end
					end
				end
			end
			if not js.return_called then
				Result := 0
			end
		end

	steffenson_iterate_address: CE_ROUTINE
		once ("OBJECT")
			create Result.make_agent (agent steffenson_iterate)
		end

end
