class
	P_UNI32

inherit
	LIBGSL_DATA
		redefine
			default_create
		end
	S_GCC_BUILTIN
	S_LIBIO
	S_STDIO
	S_STDLIB
	S_MATH
	S_STRING
	S_ASSERT

feature {NONE} -- Initialization

	default_create
		do
			create gsl_rng_uni32.make
			create uni32_type.make (<<1>>)
		end

feature {ANY} -- attributes

	gsl_rng_uni32: CE_POINTER [PD_GSL_RNG_TYPE] assign set_gsl_rng_uni32

	set_gsl_rng_uni32 (a_gsl_rng_uni32: CE_POINTER [PD_GSL_RNG_TYPE])
		do
			gsl_rng_uni32 := a_gsl_rng_uni32
		end

	m1: NATURAL_32 assign set_m1

	set_m1 (a_m1: NATURAL_32)
		do
			m1 := a_m1
		end

	m2: NATURAL_32 assign set_m2

	set_m2 (a_m2: NATURAL_32)
		do
			m2 := a_m2
		end

	uni32_type: CE_ARRAY [PD_GSL_RNG_TYPE]

feature {ANY} -- routines

	uni32_get (a_vstate: CE_POINTER [ANY]): NATURAL_32
		local
			l_state: CE_POINTER [PD_UNI32_STATE_T]
			l_i: INTEGER_32
			l_j: INTEGER_32
			l_k: INTEGER_32
		do
			create l_state.make

			l_state := create {CE_POINTER [PD_UNI32_STATE_T]}.make_cast (a_vstate)
			l_i := (l_state.item.i).to_integer_32
			l_j := (l_state.item.j).to_integer_32
			l_k := (l_state.item.m[l_i] - l_state.item.m[l_j]).to_integer_32
			if l_k < 0 then
				l_k := (l_k.to_natural_32 + m1).to_integer_32
			end
			l_state.item.m[l_j] := l_k.to_natural_32
			if l_i = 0 then
				l_state.item.i := 16
			else
				l_state.item.i := (l_state.item.i) - 1
			end
			if l_j = 0 then
				l_state.item.j := 16
			else
				l_state.item.j := (l_state.item.j) - 1
			end
			Result := l_k.to_natural_32
		end

	uni32_get_address: CE_ROUTINE
		once ("OBJECT")
			create Result.make_agent (agent uni32_get)
		end

	uni32_get_double (a_vstate: CE_POINTER [ANY]): REAL_64
		do
			Result := ((uni32_get (a_vstate)).to_real_64 / 2147483647.0)
		end

	uni32_get_double_address: CE_ROUTINE
		once ("OBJECT")
			create Result.make_agent (agent uni32_get_double)
		end

	uni32_set (a_vstate: CE_POINTER [ANY]; a_s: NATURAL_32)
		local
			l_seed: INTEGER_32
			l_k0: INTEGER_32
			l_k1: INTEGER_32
			l_j0: INTEGER_32
			l_j1: INTEGER_32
			l_i: INTEGER_32
			l_state: CE_POINTER [PD_UNI32_STATE_T]
			l_tmp: INTEGER_32
		do
			create l_state.make

			l_state := create {CE_POINTER [PD_UNI32_STATE_T]}.make_cast (a_vstate)
			if a_s < m1 then
				l_seed := a_s.to_integer_32
			else
				l_seed := m1.to_integer_32
			end
			if l_seed \\ 2 = 0 then
				l_tmp := 1
			else
				l_tmp := 0
			end
			l_seed := l_seed - l_tmp.to_integer_32
			l_k0 := ({NATURAL_32} 9_069 \\ m2).to_integer_32
			l_k1 := ({NATURAL_32} 9_069 // m2).to_integer_32
			l_j0 := (l_seed.to_natural_32 \\ m2).to_integer_32
			l_j1 := (l_seed.to_natural_32 // m2).to_integer_32
			l_i := 0
			from
			until
				not (l_i < 17)
			loop
				l_seed := l_j0 * l_k0
				l_j1 := (((l_seed.to_natural_32 // m2 + (l_j0 * l_k1).to_natural_32) + (l_j1 * l_k0).to_natural_32) \\ (m2 // {NATURAL_32} 2)).to_integer_32
				l_j0 := (l_seed.to_natural_32 \\ m2).to_integer_32
				l_state.item.m[l_i] := (l_j0.to_natural_32 + m2 * l_j1.to_natural_32)
				l_i := l_i + 1
			end
			l_state.item.i := 4
			l_state.item.j := 16
		end

	uni32_set_address: CE_ROUTINE
		once ("OBJECT")
			create Result.make_agent (agent uni32_set)
		end

end
