class
	P_WSKEW_SOURCE

inherit
	LIBGSL_DATA
	S_GCC_BUILTIN
	S_LIBIO
	S_STDIO
	S_STDLIB
	S_MATH
	S_STRING
	S_ASSERT

feature {ANY} -- routines

	gsl_stats_float_wskew (a_w: CE_POINTER [REAL_32]; a_wstride: NATURAL_32; a_data: CE_POINTER [REAL_32]; a_stride: NATURAL_32; a_n: NATURAL_32): REAL_64
		local
			l_wmean: REAL_64
			l_wsd: REAL_64
		do
			l_wmean := (wmean_source_inst.gsl_stats_float_wmean (a_w, a_wstride, a_data, a_stride, a_n))
			l_wsd := (wvariance_source_inst.gsl_stats_float_wsd_m (a_w, a_wstride, a_data, a_stride, a_n, l_wmean))
			Result := (gsl_stats_float_wskew_m_sd (a_w, a_wstride, a_data, a_stride, a_n, l_wmean, l_wsd))
		end

	gsl_stats_float_wskew_m_sd (a_w: CE_POINTER [REAL_32]; a_wstride: NATURAL_32; a_data: CE_POINTER [REAL_32]; a_stride: NATURAL_32; a_n: NATURAL_32; a_wmean: REAL_64; a_wsd: REAL_64): REAL_64
		local
			l_wskew: REAL_96
			l_W: REAL_96
			l_i: NATURAL_32
			l_wi: REAL_32
			l_x: REAL_96
		do
			l_wskew := real_96_from_integer_32 (0)
			l_W := real_96_from_integer_32 (0)
			l_i := (0).to_natural_32
			from
			until
				not (l_i < a_n)
			loop
				l_wi := a_w[(l_i * a_wstride).to_integer_32]
				if l_wi > (0).to_real then
					l_x := real_96_from_real_64 ((((a_data[(l_i * a_stride).to_integer_32]).to_double - a_wmean) / a_wsd))
					l_W := l_W + real_96_from_real_32 (l_wi)
					l_wskew := l_wskew + (((l_x * l_x) * l_x - l_wskew) * (real_96_from_real_32 (l_wi) / l_W))
				end
				l_i := l_i + 1
			end
			Result := l_wskew.truncated_to_real_64
		end

	gsl_stats_long_double_wskew (a_w: CE_POINTER [REAL_96]; a_wstride: NATURAL_32; a_data: CE_POINTER [REAL_96]; a_stride: NATURAL_32; a_n: NATURAL_32): REAL_64
		local
			l_wmean: REAL_64
			l_wsd: REAL_64
		do
			l_wmean := (wmean_source_inst.gsl_stats_long_double_wmean (a_w, a_wstride, a_data, a_stride, a_n))
			l_wsd := (wvariance_source_inst.gsl_stats_long_double_wsd_m (a_w, a_wstride, a_data, a_stride, a_n, l_wmean))
			Result := (gsl_stats_long_double_wskew_m_sd (a_w, a_wstride, a_data, a_stride, a_n, l_wmean, l_wsd))
		end

	gsl_stats_long_double_wskew_m_sd (a_w: CE_POINTER [REAL_96]; a_wstride: NATURAL_32; a_data: CE_POINTER [REAL_96]; a_stride: NATURAL_32; a_n: NATURAL_32; a_wmean: REAL_64; a_wsd: REAL_64): REAL_64
		local
			l_wskew: REAL_96
			l_W: REAL_96
			l_i: NATURAL_32
			l_wi: REAL_96
			l_x: REAL_96
		do
			l_wskew := real_96_from_integer_32 (0)
			l_W := real_96_from_integer_32 (0)
			l_i := (0).to_natural_32
			from
			until
				not (l_i < a_n)
			loop
				l_wi := a_w[(l_i * a_wstride).to_integer_32]
				if l_wi > real_96_from_integer_32 (0) then
					l_x := ((a_data[(l_i * a_stride).to_integer_32] - real_96_from_real_64 (a_wmean)) / real_96_from_real_64 (a_wsd))
					l_W := l_W + l_wi
					l_wskew := l_wskew + (((l_x * l_x) * l_x - l_wskew) * (l_wi / l_W))
				end
				l_i := l_i + 1
			end
			Result := l_wskew.truncated_to_real_64
		end

	gsl_stats_wskew (a_w: CE_POINTER [REAL_64]; a_wstride: NATURAL_32; a_data: CE_POINTER [REAL_64]; a_stride: NATURAL_32; a_n: NATURAL_32): REAL_64
		local
			l_wmean: REAL_64
			l_wsd: REAL_64
		do
			l_wmean := (wmean_source_inst.gsl_stats_wmean (a_w, a_wstride, a_data, a_stride, a_n))
			l_wsd := (wvariance_source_inst.gsl_stats_wsd_m (a_w, a_wstride, a_data, a_stride, a_n, l_wmean))
			Result := (gsl_stats_wskew_m_sd (a_w, a_wstride, a_data, a_stride, a_n, l_wmean, l_wsd))
		end

	gsl_stats_wskew_m_sd (a_w: CE_POINTER [REAL_64]; a_wstride: NATURAL_32; a_data: CE_POINTER [REAL_64]; a_stride: NATURAL_32; a_n: NATURAL_32; a_wmean: REAL_64; a_wsd: REAL_64): REAL_64
		local
			l_wskew: REAL_96
			l_W: REAL_96
			l_i: NATURAL_32
			l_wi: REAL_64
			l_x: REAL_96
		do
			l_wskew := real_96_from_integer_32 (0)
			l_W := real_96_from_integer_32 (0)
			l_i := (0).to_natural_32
			from
			until
				not (l_i < a_n)
			loop
				l_wi := a_w[(l_i * a_wstride).to_integer_32]
				if l_wi > (0).to_double then
					l_x := real_96_from_real_64 (((a_data[(l_i * a_stride).to_integer_32] - a_wmean) / a_wsd))
					l_W := l_W + real_96_from_real_64 (l_wi)
					l_wskew := l_wskew + (((l_x * l_x) * l_x - l_wskew) * (real_96_from_real_64 (l_wi) / l_W))
				end
				l_i := l_i + 1
			end
			Result := l_wskew.truncated_to_real_64
		end

end
